/*
 * $Id: //depot/prod/newport_news/main/modules/nn-platform/src/main/java/com/netapp/common/execution/StackedShellCommand.java#4 $
 *
 * Copyright (c) 2007 Network Appliance, Inc.
 * All rights reserved.
 */
package com.netapp.common.execution;

import java.io.File;
import java.util.LinkedList;
import java.util.List;

public class StackedShellCommand implements ShellRunnable {

    private ShellRunnable parentCommand;
    private Command childCommand;

    /**
     * @param parentCommand The command spawning the child command
     * @param childCommand The command being spawned by the parent command
     */
    public StackedShellCommand(ShellRunnable parentCommand, Command childCommand) {
        this.parentCommand = parentCommand;
        this.childCommand = childCommand;
    }

    @Override
    public List<String> getArgumentList() {
        return this.parentCommand.getArgumentList();
    }

    @Override
    public String getUserFacingCommand() {
        return this.childCommand.getUserFacingCommand();
    }

    @Override
    public String toString() {
        return this.getUserFacingCommand();
    }

    /**
     * Produces out put like the following
     *
     * 0: parent command
     * 1: child command
     */
    @Override
    public String getLogFacingCommand() {
        StringBuilder stringBuilder = new StringBuilder();
        int i = 0;
        for (Command command : this.commandStack()) {
            stringBuilder.append("\n");
            // stringBuilder.append(Utils.repeat("\t"));
            stringBuilder.append(i++ + ":");
            stringBuilder.append(command.getUserFacingCommand());
        }
        return stringBuilder.toString();
    }

    private List<Command> commandStack() {
        List<Command> stack = new LinkedList<Command>();

        stack.add(this.parentCommand);

        if (this.childCommand instanceof StackedShellCommand) {
            stack.addAll(((StackedShellCommand) this.childCommand).commandStack());
        } else {
            stack.add(this.childCommand);
        }

        return stack;
    }

    @Override
    public String getCommandString() {
        return this.parentCommand.getCommandString();
    }

    @Override
    public File getWorkingDirectory() {
        /*
         * This is ugly but provides backwards compatibility for uses where childCommand is not a ShellRunnable
         */
        if (childCommand instanceof ShellRunnable) {
            return ((ShellRunnable)childCommand).getWorkingDirectory();
        }
        return null;
    }

    @Override
    public ShellRunnable transformInto(String... argumentList) {
        return this.parentCommand.transformInto(argumentList);
    }

}
