/*
 * $Id: //depot/prod/newport_news/main/modules/nn-core/src/main/java/com/netapp/common/graph/DisplayGraph.java#5 $
 *
 * Copyright (c) 2008 Network Appliance, Inc.
 * All rights reserved.
 */
package com.netapp.common.graph;

import java.io.StringWriter;

import com.google.common.base.Function;
import org.apache.commons.lang.ClassUtils;
import org.apache.commons.lang.builder.EqualsBuilder;
import org.apache.commons.lang.builder.HashCodeBuilder;

import com.netapp.common.graph.GraphvizEncoder.RankDir;

/**
 * A {@link Graph} of {@link DisplayNode}s and a name.
 * A {@link DisplayGraph} is capable of being encoded as Graphviz (via {@link GraphvizEncoder}).
 *
 * @author sirianni
 * @version $Revision: #5 $ $Date: 2011/06/24 $
 */
public class DisplayGraph {

    private String name;
    private Graph<DisplayNode> graph;
    private DisplayNode entry;

    private DisplayGraph(String name, Graph<DisplayNode> graph) {
        this.name = name;
        this.graph = graph;
        this.entry = new DisplayNode(null, "ENTRY", null);
    }

    public DisplayGraph(String name) {
        this(name, new Graph<DisplayNode>());
    }

    public static DisplayGraph fromGraph(GraphRepresentable graphRepresentable) {
        return fromGraph(ClassUtils.getShortClassName(graphRepresentable.getClass()), graphRepresentable.getGraph());
    }

    @SuppressWarnings({"unchecked", "rawtypes"})
    public static DisplayGraph fromGraph(String name, Graph<?> graph) {
        return fromGraph(name, graph, (Function) BasicDisplayNodeFunction.INSTANCE);
    }

    public static <T> DisplayGraph fromGraph(String name, Graph<T> graph, Function<T, DisplayNode> function) {
        return new DisplayGraph(
            name,
            graph.apply(function)
        );
    }

    public String getName() {
        return this.name;
    }

    public DisplayGraph withName(String name) {
        this.name = name;
        return this;
    }

    public Graph<DisplayNode> getGraph() {
        return this.graph;
    }

    public DisplayNode getEntry() {
        return this.entry;
    }

    @Override
    public int hashCode() {
        return new HashCodeBuilder()
            .append(this.name)
            .append(this.graph)
            .toHashCode();
    }

    @Override
    public boolean equals(Object obj) {
        if (obj instanceof DisplayGraph) {
            if (this == obj) {
                return true;
            }

            DisplayGraph other = (DisplayGraph) obj;
            return new EqualsBuilder()
                .append(this.name, other.name)
                .append(this.graph, other.graph)
                .isEquals();
        } else {
            return false;
        }
    }

    /**
     * Encode this graph in Graphviz DOT format.
     */
    public String toGraphviz() {
        return this.toGraphviz(RankDir.TOP_TO_BOTTOM);
    }

    /**
     * Encode this graph in Graphviz DOT format.
     */
    public String toGraphviz(RankDir rankDir) {
        StringWriter stringWriter = new StringWriter();
        new GraphvizEncoder(stringWriter).encode(this, rankDir);
        return stringWriter.toString();
    }

}
