package com.netapp.common.number;

import java.math.BigInteger;
import org.fest.assertions.Assertions;
import org.fest.assertions.StringAssert;
import org.junit.Test;

public class NumberAbbreviationUtilsTest
{
  @Test
  public void testAbbreviateBytes()
  {
    Assertions.assertThat(NumberAbbreviationUtils.abbreviateBytes((Long)null)).isNull();

    Assertions.assertThat(NumberAbbreviationUtils.abbreviateBytes(Long.valueOf(50L))).isEqualTo("50 bytes");
    Assertions.assertThat(NumberAbbreviationUtils.abbreviateBytes(Long.valueOf(500L))).isEqualTo("500 bytes");
    Assertions.assertThat(NumberAbbreviationUtils.abbreviateBytes(Long.valueOf(5000L))).isEqualTo("4.88 KB");
    Assertions.assertThat(NumberAbbreviationUtils.abbreviateBytes(Long.valueOf(50000L))).isEqualTo("48.83 KB");
    Assertions.assertThat(NumberAbbreviationUtils.abbreviateBytes(Long.valueOf(500000L))).isEqualTo("488.28 KB");
    Assertions.assertThat(NumberAbbreviationUtils.abbreviateBytes(Long.valueOf(5000000L))).isEqualTo("4.77 MB");
    Assertions.assertThat(NumberAbbreviationUtils.abbreviateBytes(Long.valueOf(50000000L))).isEqualTo("47.68 MB");
    Assertions.assertThat(NumberAbbreviationUtils.abbreviateBytes(Long.valueOf(500000000L))).isEqualTo("476.84 MB");
    Assertions.assertThat(NumberAbbreviationUtils.abbreviateBytes(Long.valueOf(5000000000L))).isEqualTo("4.66 GB");
    Assertions.assertThat(NumberAbbreviationUtils.abbreviateBytes(Long.valueOf(50000000000L))).isEqualTo("46.57 GB");
    Assertions.assertThat(NumberAbbreviationUtils.abbreviateBytes(Long.valueOf(500000000000L))).isEqualTo("465.66 GB");
    Assertions.assertThat(NumberAbbreviationUtils.abbreviateBytes(Long.valueOf(5000000000000L))).isEqualTo("4.55 TB");
    Assertions.assertThat(NumberAbbreviationUtils.abbreviateBytes(Long.valueOf(50000000000000L))).isEqualTo("45.47 TB");
    Assertions.assertThat(NumberAbbreviationUtils.abbreviateBytes(Long.valueOf(500000000000000L))).isEqualTo("454.75 TB");
    Assertions.assertThat(NumberAbbreviationUtils.abbreviateBytes(Long.valueOf(5000000000000000L))).isEqualTo("4.44 PB");
    Assertions.assertThat(NumberAbbreviationUtils.abbreviateBytes(Long.valueOf(50000000000000000L))).isEqualTo("44.41 PB");
    Assertions.assertThat(NumberAbbreviationUtils.abbreviateBytes(Long.valueOf(500000000000000000L))).isEqualTo("444.09 PB");
    Assertions.assertThat(NumberAbbreviationUtils.abbreviateBytes(Long.valueOf(5000000000000000000L))).isEqualTo("4.34 EB");

    Assertions.assertThat(NumberAbbreviationUtils.abbreviateBytes(Long.valueOf(-50L))).isEqualTo("-50 bytes");
    Assertions.assertThat(NumberAbbreviationUtils.abbreviateBytes(Long.valueOf(-500L))).isEqualTo("-500 bytes");
    Assertions.assertThat(NumberAbbreviationUtils.abbreviateBytes(Long.valueOf(-5000L))).isEqualTo("-4.88 KB");
    Assertions.assertThat(NumberAbbreviationUtils.abbreviateBytes(Long.valueOf(-50000L))).isEqualTo("-48.83 KB");
    Assertions.assertThat(NumberAbbreviationUtils.abbreviateBytes(Long.valueOf(-500000L))).isEqualTo("-488.28 KB");
    Assertions.assertThat(NumberAbbreviationUtils.abbreviateBytes(Long.valueOf(-5000000L))).isEqualTo("-4.77 MB");
    Assertions.assertThat(NumberAbbreviationUtils.abbreviateBytes(Long.valueOf(-50000000L))).isEqualTo("-47.68 MB");
    Assertions.assertThat(NumberAbbreviationUtils.abbreviateBytes(Long.valueOf(-500000000L))).isEqualTo("-476.84 MB");
    Assertions.assertThat(NumberAbbreviationUtils.abbreviateBytes(Long.valueOf(-5000000000L))).isEqualTo("-4.66 GB");
    Assertions.assertThat(NumberAbbreviationUtils.abbreviateBytes(Long.valueOf(-50000000000L))).isEqualTo("-46.57 GB");
    Assertions.assertThat(NumberAbbreviationUtils.abbreviateBytes(Long.valueOf(-500000000000L))).isEqualTo("-465.66 GB");
    Assertions.assertThat(NumberAbbreviationUtils.abbreviateBytes(Long.valueOf(-5000000000000L))).isEqualTo("-4.55 TB");
    Assertions.assertThat(NumberAbbreviationUtils.abbreviateBytes(Long.valueOf(-50000000000000L))).isEqualTo("-45.47 TB");
    Assertions.assertThat(NumberAbbreviationUtils.abbreviateBytes(Long.valueOf(-500000000000000L))).isEqualTo("-454.75 TB");
    Assertions.assertThat(NumberAbbreviationUtils.abbreviateBytes(Long.valueOf(-5000000000000000L))).isEqualTo("-4.44 PB");
    Assertions.assertThat(NumberAbbreviationUtils.abbreviateBytes(Long.valueOf(-50000000000000000L))).isEqualTo("-44.41 PB");
    Assertions.assertThat(NumberAbbreviationUtils.abbreviateBytes(Long.valueOf(-500000000000000000L))).isEqualTo("-444.09 PB");
    Assertions.assertThat(NumberAbbreviationUtils.abbreviateBytes(Long.valueOf(-5000000000000000000L))).isEqualTo("-4.34 EB");
  }

  @Test
  public void testAbbreviateBigIntegerBytes() {
    Assertions.assertThat(NumberAbbreviationUtils.abbreviateBytes((BigInteger)null)).isNull();

    Assertions.assertThat(NumberAbbreviationUtils.abbreviateBytes(new BigInteger("50"))).isEqualTo("50 bytes");
    Assertions.assertThat(NumberAbbreviationUtils.abbreviateBytes(new BigInteger("500"))).isEqualTo("500 bytes");
    Assertions.assertThat(NumberAbbreviationUtils.abbreviateBytes(new BigInteger("5000"))).isEqualTo("4.88 KB");
    Assertions.assertThat(NumberAbbreviationUtils.abbreviateBytes(new BigInteger("50000"))).isEqualTo("48.83 KB");
    Assertions.assertThat(NumberAbbreviationUtils.abbreviateBytes(new BigInteger("500000"))).isEqualTo("488.28 KB");
    Assertions.assertThat(NumberAbbreviationUtils.abbreviateBytes(new BigInteger("5000000"))).isEqualTo("4.77 MB");
    Assertions.assertThat(NumberAbbreviationUtils.abbreviateBytes(new BigInteger("50000000"))).isEqualTo("47.68 MB");
    Assertions.assertThat(NumberAbbreviationUtils.abbreviateBytes(new BigInteger("500000000"))).isEqualTo("476.84 MB");
    Assertions.assertThat(NumberAbbreviationUtils.abbreviateBytes(new BigInteger("5000000000"))).isEqualTo("4.66 GB");
    Assertions.assertThat(NumberAbbreviationUtils.abbreviateBytes(new BigInteger("50000000000"))).isEqualTo("46.57 GB");
    Assertions.assertThat(NumberAbbreviationUtils.abbreviateBytes(new BigInteger("500000000000"))).isEqualTo("465.66 GB");
    Assertions.assertThat(NumberAbbreviationUtils.abbreviateBytes(new BigInteger("5000000000000"))).isEqualTo("4.55 TB");
    Assertions.assertThat(NumberAbbreviationUtils.abbreviateBytes(new BigInteger("50000000000000"))).isEqualTo("45.47 TB");
    Assertions.assertThat(NumberAbbreviationUtils.abbreviateBytes(new BigInteger("500000000000000"))).isEqualTo("454.75 TB");
    Assertions.assertThat(NumberAbbreviationUtils.abbreviateBytes(new BigInteger("5000000000000000"))).isEqualTo("4.44 PB");
    Assertions.assertThat(NumberAbbreviationUtils.abbreviateBytes(new BigInteger("50000000000000000"))).isEqualTo("44.41 PB");
    Assertions.assertThat(NumberAbbreviationUtils.abbreviateBytes(new BigInteger("500000000000000000"))).isEqualTo("444.09 PB");
    Assertions.assertThat(NumberAbbreviationUtils.abbreviateBytes(new BigInteger("5000000000000000000"))).isEqualTo("4.34 EB");
    Assertions.assertThat(NumberAbbreviationUtils.abbreviateBytes(new BigInteger("10000000000000000000"))).isEqualTo("8.67 EB");
    Assertions.assertThat(NumberAbbreviationUtils.abbreviateBytes(new BigInteger("50000000000000000000"))).isEqualTo("43.4 EB");
    Assertions.assertThat(NumberAbbreviationUtils.abbreviateBytes(new BigInteger("500000000000000000000"))).isEqualTo("434 EB");
    Assertions.assertThat(NumberAbbreviationUtils.abbreviateBytes(new BigInteger("5000000000000000000000"))).isEqualTo("4337 EB");

    Assertions.assertThat(NumberAbbreviationUtils.abbreviateBytes(new BigInteger("-50"))).isEqualTo("-50 bytes");
    Assertions.assertThat(NumberAbbreviationUtils.abbreviateBytes(new BigInteger("-500"))).isEqualTo("-500 bytes");
    Assertions.assertThat(NumberAbbreviationUtils.abbreviateBytes(new BigInteger("-5000"))).isEqualTo("-4.88 KB");
    Assertions.assertThat(NumberAbbreviationUtils.abbreviateBytes(new BigInteger("-50000"))).isEqualTo("-48.83 KB");
    Assertions.assertThat(NumberAbbreviationUtils.abbreviateBytes(new BigInteger("-500000"))).isEqualTo("-488.28 KB");
    Assertions.assertThat(NumberAbbreviationUtils.abbreviateBytes(new BigInteger("-5000000"))).isEqualTo("-4.77 MB");
    Assertions.assertThat(NumberAbbreviationUtils.abbreviateBytes(new BigInteger("-50000000"))).isEqualTo("-47.68 MB");
    Assertions.assertThat(NumberAbbreviationUtils.abbreviateBytes(new BigInteger("-500000000"))).isEqualTo("-476.84 MB");
    Assertions.assertThat(NumberAbbreviationUtils.abbreviateBytes(new BigInteger("-5000000000"))).isEqualTo("-4.66 GB");
    Assertions.assertThat(NumberAbbreviationUtils.abbreviateBytes(new BigInteger("-50000000000"))).isEqualTo("-46.57 GB");
    Assertions.assertThat(NumberAbbreviationUtils.abbreviateBytes(new BigInteger("-500000000000"))).isEqualTo("-465.66 GB");
    Assertions.assertThat(NumberAbbreviationUtils.abbreviateBytes(new BigInteger("-5000000000000"))).isEqualTo("-4.55 TB");
    Assertions.assertThat(NumberAbbreviationUtils.abbreviateBytes(new BigInteger("-50000000000000"))).isEqualTo("-45.47 TB");
    Assertions.assertThat(NumberAbbreviationUtils.abbreviateBytes(new BigInteger("-500000000000000"))).isEqualTo("-454.75 TB");
    Assertions.assertThat(NumberAbbreviationUtils.abbreviateBytes(new BigInteger("-5000000000000000"))).isEqualTo("-4.44 PB");
    Assertions.assertThat(NumberAbbreviationUtils.abbreviateBytes(new BigInteger("-50000000000000000"))).isEqualTo("-44.41 PB");
    Assertions.assertThat(NumberAbbreviationUtils.abbreviateBytes(new BigInteger("-500000000000000000"))).isEqualTo("-444.09 PB");
    Assertions.assertThat(NumberAbbreviationUtils.abbreviateBytes(new BigInteger("-5000000000000000000"))).isEqualTo("-4.34 EB");
    Assertions.assertThat(NumberAbbreviationUtils.abbreviateBytes(new BigInteger("-10000000000000000000"))).isEqualTo("-8.67 EB");
    Assertions.assertThat(NumberAbbreviationUtils.abbreviateBytes(new BigInteger("-50000000000000000000"))).isEqualTo("-43.4 EB");
    Assertions.assertThat(NumberAbbreviationUtils.abbreviateBytes(new BigInteger("-500000000000000000000"))).isEqualTo("-434 EB");
    Assertions.assertThat(NumberAbbreviationUtils.abbreviateBytes(new BigInteger("-5000000000000000000000"))).isEqualTo("-4337 EB");
  }

  @Test
  public void testAbbreviateNumber() {
    Assertions.assertThat(NumberAbbreviationUtils.abbreviateNumber((Long)null)).isNull();

    Assertions.assertThat(NumberAbbreviationUtils.abbreviateNumber(Long.valueOf(512L))).isEqualTo("512");
    Assertions.assertThat(NumberAbbreviationUtils.abbreviateNumber(Long.valueOf(5123L))).isEqualTo("5123");
    Assertions.assertThat(NumberAbbreviationUtils.abbreviateNumber(Long.valueOf(5123000L))).isEqualTo("5.12 million");
    Assertions.assertThat(NumberAbbreviationUtils.abbreviateNumber(Long.valueOf(51230000L))).isEqualTo("51.23 million");
    Assertions.assertThat(NumberAbbreviationUtils.abbreviateNumber(Long.valueOf(512300000L))).isEqualTo("512.30 million");
    Assertions.assertThat(NumberAbbreviationUtils.abbreviateNumber(Long.valueOf(5123000000L))).isEqualTo("5.12 billion");
    Assertions.assertThat(NumberAbbreviationUtils.abbreviateNumber(Long.valueOf(51230000000L))).isEqualTo("51.23 billion");
    Assertions.assertThat(NumberAbbreviationUtils.abbreviateNumber(Long.valueOf(512300000000L))).isEqualTo("512.30 billion");
    Assertions.assertThat(NumberAbbreviationUtils.abbreviateNumber(Long.valueOf(5123000000000L))).isEqualTo("5.12 trillion");
    Assertions.assertThat(NumberAbbreviationUtils.abbreviateNumber(Long.valueOf(51230000000000L))).isEqualTo("51.23 trillion");
    Assertions.assertThat(NumberAbbreviationUtils.abbreviateNumber(Long.valueOf(512300000000000L))).isEqualTo("512.30 trillion");
    Assertions.assertThat(NumberAbbreviationUtils.abbreviateNumber(Long.valueOf(5123000000000000L))).isEqualTo("5.12 quadrillion");
    Assertions.assertThat(NumberAbbreviationUtils.abbreviateNumber(Long.valueOf(51230000000000000L))).isEqualTo("51.23 quadrillion");
    Assertions.assertThat(NumberAbbreviationUtils.abbreviateNumber(Long.valueOf(512300000000000000L))).isEqualTo("512.30 quadrillion");
    Assertions.assertThat(NumberAbbreviationUtils.abbreviateNumber(Long.valueOf(5123000000000000000L))).isEqualTo("5.12 quintillion");

    Assertions.assertThat(NumberAbbreviationUtils.abbreviateNumber(Long.valueOf(-512L))).isEqualTo("-512");
    Assertions.assertThat(NumberAbbreviationUtils.abbreviateNumber(Long.valueOf(-5123L))).isEqualTo("-5123");
    Assertions.assertThat(NumberAbbreviationUtils.abbreviateNumber(Long.valueOf(-5123000L))).isEqualTo("-5.12 million");
    Assertions.assertThat(NumberAbbreviationUtils.abbreviateNumber(Long.valueOf(-51230000L))).isEqualTo("-51.23 million");
    Assertions.assertThat(NumberAbbreviationUtils.abbreviateNumber(Long.valueOf(-512300000L))).isEqualTo("-512.30 million");
    Assertions.assertThat(NumberAbbreviationUtils.abbreviateNumber(Long.valueOf(-5123000000L))).isEqualTo("-5.12 billion");
    Assertions.assertThat(NumberAbbreviationUtils.abbreviateNumber(Long.valueOf(-51230000000L))).isEqualTo("-51.23 billion");
    Assertions.assertThat(NumberAbbreviationUtils.abbreviateNumber(Long.valueOf(-512300000000L))).isEqualTo("-512.30 billion");
    Assertions.assertThat(NumberAbbreviationUtils.abbreviateNumber(Long.valueOf(-5123000000000L))).isEqualTo("-5.12 trillion");
    Assertions.assertThat(NumberAbbreviationUtils.abbreviateNumber(Long.valueOf(-51230000000000L))).isEqualTo("-51.23 trillion");
    Assertions.assertThat(NumberAbbreviationUtils.abbreviateNumber(Long.valueOf(-512300000000000L))).isEqualTo("-512.30 trillion");
    Assertions.assertThat(NumberAbbreviationUtils.abbreviateNumber(Long.valueOf(-5123000000000000L))).isEqualTo("-5.12 quadrillion");
    Assertions.assertThat(NumberAbbreviationUtils.abbreviateNumber(Long.valueOf(-51230000000000000L))).isEqualTo("-51.23 quadrillion");
    Assertions.assertThat(NumberAbbreviationUtils.abbreviateNumber(Long.valueOf(-512300000000000000L))).isEqualTo("-512.30 quadrillion");
    Assertions.assertThat(NumberAbbreviationUtils.abbreviateNumber(Long.valueOf(-5123000000000000000L))).isEqualTo("-5.12 quintillion");
  }
}