/*
 * $Id: //depot/prod/newport_news/main/modules/nn-platform/src/main/java/com/netapp/common/platform/linux/AbstractDeviceMapperMultipathHandler.java#3 $
 *
 * Copyright (c) 2008 NetApp, Inc.
 * All rights reserved.
 */
package com.netapp.common.platform.linux;

import java.io.File;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import com.netapp.common.execution.ShellCommand;
import com.netapp.common.execution.ShellResult;
import com.netapp.common.platform.DeviceType;

/**
 * @author sunday
 * @version $Revision: #3 $ $Date: 2011/06/24 $
 */
public abstract class AbstractDeviceMapperMultipathHandler extends AbstractBlockDeviceHandler {

    /**
     * Matches "253:8:MY_DISKp1"
     */
    private static final Pattern DMSETUP_ENTRY_PATTERN = Pattern.compile("(\\d+):(\\d+):(.*)");
    private static final Pattern DEVICE_PATTERN = Pattern.compile("/dev/(mapper|mpath)/(.*)");
    private static final int DEVICE_PARTITION_GROUP = 3;
    private static final int DEVICE_SHORT_NAME = 2;
    public AbstractDeviceMapperMultipathHandler(OperatingSystemImpl operatingSystem) {
        super(operatingSystem);
    }

    /**
     *
     * @see com.netapp.common.platform.linux.BlockDeviceHandler#driverName()
     */
    @Override
    public String driverName() {
        return "device-mapper";
    }

    @Override
    public DeviceType determineDeviceType(File file) {
        Matcher m = DEVICE_PATTERN.matcher(file.getAbsolutePath());
        if (m.matches()) {
            ShellCommand cmd = new ShellCommand("/sbin/dmsetup", "info",
                "--noheadings",
                "-c",
                "-o", "uuid",
                m.group(DEVICE_SHORT_NAME));
            ShellResult dmsetupResult = getShellExecutor().execute(cmd);
            for (String entry : dmsetupResult.getStdOut()) {
                if (entry.matches("(mapper|mpath)-.*")) {
                    return DeviceType.BASE_DEVICE;
                } else if (entry.matches("part\\d+-(mapper|mpath)-.*")) {
                    return DeviceType.PARTITION;
                }
            }
        }
        return DeviceType.NON_DEVICE;
    }

    /**
     * @see com.netapp.common.platform.linux.BlockDeviceHandler#deviceFromMajorMinor(int, int)
     */
    @Override
    public File deviceFromMajorMinor(int major, int minor) {
        // Strictly speaking, we don't need the major and minor, but it's easier
        // to differentiate from an error condition if we have a little more to match
        // on than the "name" column itself.
        // This can be done with JNI against the device nodes in /dev/mapper too.
        ShellCommand cmd = new ShellCommand("/sbin/dmsetup", "info",
            "--noheadings",
            "-c",
            "-o", "major,minor,name",
            "-j", Integer.toString(major),
            "-m", Integer.toString(minor));
        ShellResult dmsetupResult = getShellExecutor().execute(cmd);
        final List<String> stdOutput = dmsetupResult.getStdOut();
        for (String entry : stdOutput) {
            Matcher matcher = DMSETUP_ENTRY_PATTERN.matcher(entry);
            if (matcher.matches()) {
                return new File("/dev/mapper/" + matcher.group(DEVICE_PARTITION_GROUP));
            }
        }
        return null;
    }
}
