/*
 * $Id: //depot/prod/newport_news/main/modules/nn-platform/src/main/java/com/netapp/common/platform/linux/RH5MultipathHandler.java#3 $
 *
 * Copyright (c) 2008 NetApp, Inc.
 * All rights reserved.
 */
package com.netapp.common.platform.linux;

import java.io.File;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * Manages device-mapper-multipath devices from RHEL 5.
 * @author sunday
 * @version $Revision: #3 $ $Date: 2012/11/05 $
 */
public class RH5MultipathHandler extends AbstractDeviceMapperMultipathHandler {

    /**
     * Matches "(/dev/mapper/LUN)(p1)"
     */
    private static final Pattern MPIO_PATTERN = Pattern.compile("(/dev/(mapper|mpath)/\\S+?)(?:p(\\d+))?");
    private static final int MPIO_DISK_GROUP = 1;
    private static final int MPIO_PARTITION_GROUP = 3;

    public RH5MultipathHandler(OperatingSystemImpl operatingSystem) {
        super(operatingSystem);
    }

    @Override
    public File deviceForPartition(File disk, String partition) {
        Matcher matcher = MPIO_PATTERN.matcher(disk.getAbsolutePath());
        if (matcher.matches()) {
            return new File(disk.getAbsolutePath().replace("mpath", "mapper") + "p" + partition);
        }
        return null;
    }

    @Override
    public String extractFullDiskPathElement(File file) {
        Matcher matcher = MPIO_PATTERN.matcher(file.getAbsolutePath().replace("mpath", "mapper"));
        if (matcher.matches()) {
            return matcher.group(MPIO_DISK_GROUP);
        }
        return null;
    }

    @Override
    public String extractPartitionPathElement(File file) {
        Matcher matcher = MPIO_PATTERN.matcher(file.getAbsolutePath());
        if (matcher.matches()) {
            return matcher.group(MPIO_PARTITION_GROUP);
        }
        return null;
    }
}
