/*
 * $Id: //depot/prod/newport_news/main/modules/nn-core/src/main/java/com/netapp/common/util/ChangeSet.java#2 $
 *
 * Copyright (c) 2008 NetApp, Inc.
 * All rights reserved.
 */
package com.netapp.common.util;

import java.util.List;

/**
 * A collection of state change notifications. Each collection also has a tag representing
 * generation number of the newest entry, using this tag in a subsequent call to
 * {@link com.netapp.common.util.ChangeTracker#getChanges(long)} will return the next set of
 * changes newer than this set.
 *
 * @author sunday
 * @version $Revision: #2 $ $Date: 2011/06/24 $
 */
public class ChangeSet<T> {
    private long tag;
    private List<T> changes;

    public ChangeSet(long tag, List<T> changes) {
        this.tag = tag;
        this.changes = changes;
    }

    /**
     * The generation number of the newest member of this set.
     * @return generation number of this set
     */
    public long getTag() {
        return this.tag;
    }

    /**
     * The list of change notifications for this set. Entries are always
     * in order with the oldest changes first.
     * @return list of notifications in this set
     */
    public List<T> getChanges() {
        return this.changes;
    }
}
