/*
 * $Id: //depot/prod/newport_news/main/modules/nn-core/src/main/java/com/netapp/common/util/MulticastProxy.java#3 $
 *
 * Copyright (c) 2008 NetApp, Inc.
 * All rights reserved.
 */
package com.netapp.common.util;

import java.lang.reflect.InvocationHandler;
import java.lang.reflect.Method;
import java.lang.reflect.Proxy;

import com.netapp.common.logging.NALogger;

/**
 * Create a proxy class that multicasts messages to multiple targets.
 * If you want a weakly-bound version of this that maintains the collection
 * of listeners, see {@link ListenerProxy} instead.
 * @author sunday
 * @version $Revision: #3 $ $Date: 2011/06/24 $
 */
public class MulticastProxy<T> implements InvocationHandler {
    private Iterable<? extends T> targets;

    private MulticastProxy(Iterable<? extends T> targets) {
        this.targets = targets;
    }

    /**
     * Create a proxy implementation.
     * @param clazz Interface proxy to create
     * @param targets Iterable of targets to send messages to.
     *
     * @return New proxy implementing clazz that will be sent to all targets
     */
    public static <T> T createMulticastProxy(Class<T> clazz, Iterable<? extends T> targets) {
        MulticastProxy<T> h = new MulticastProxy<T>(targets);
        return clazz.cast(Proxy.newProxyInstance(clazz.getClassLoader(), new Class<?>[] {clazz}, h));
    }

    @Override
    public Object invoke(Object proxy, Method method, Object[] args) throws Throwable {
        if (method.getDeclaringClass() == Object.class && method.getName().equals("finalize")) {
            return null;
        }

        for (T listener : this.targets) {
            try {
                method.invoke(listener, args);
            } catch (Throwable t) {
                // TODO: Add a handler here
                // Important: This logger is loaded late to prevent a circular reference with NALogger itself.
                NALogger.getLogger(MulticastProxy.class).internal("Exception caught in MulticastProxy for %s: %s", t, method.toString(), t.getLocalizedMessage());
            }
        }
        return null;
    }
}
