/*
 * $Id: //depot/prod/newport_news/main/modules/nn-core/src/main/java/com/netapp/common/util/ProcessOutputReader.java#4 $
 *
 * Copyright (c) 2007 Network Appliance, Inc.
 * All rights reserved.
 *
 */
package com.netapp.common.util;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.List;

import com.netapp.common.logging.NALogger;

/**
 * Read the input stream of a process and stores the newline separated lines in a list of strings.
 *
 * @author debert
 * @version $Revision: #4 $ $Date: 2012/11/13 $
 */
public class ProcessOutputReader extends Thread {

    private static final NALogger logger = NALogger.getLogger(ProcessOutputReader.class);

    private final InputStream is;
    private final List<String> result;

    /**
     * Default constructor
     * <p>
     * Reads from an InputStream and stores the entries in a list of strings
     *
     * @param is InputStream to be read
     * @param result A list of strings that represents the converted, newline separated InputStream
     */
    public ProcessOutputReader(final InputStream is, final List<String> result) {
        this.is = is;
        this.result = result;
    }

    /*
     * (non-Javadoc)
     *
     * @see java.lang.Thread#run()
     */
    @Override
    @edu.umd.cs.findbugs.annotations.SuppressWarnings("DM_DEFAULT_ENCODING")
    public void run() {
        final BufferedReader br = new BufferedReader(new InputStreamReader(this.is));

        try {
            String s;
            while ((s = br.readLine()) != null) {
                this.result.add(s);
            }
        } catch (final IOException ex) {
            ProcessOutputReader.logger.internal(ex.toString());
        } finally {
            try {
                br.close();
            } catch (final IOException ex) {
                // do nothing
            }
        }
    }
}
