/*
 * $Id: //depot/prod/newport_news/main/modules/nn-core/src/main/java/com/netapp/common/xml/SimpleNamespaceContext.java#2 $
 *
 * Copyright (c) 2009 NetApp, Inc.
 * All rights reserved.
 */
package com.netapp.common.xml;

import java.util.Iterator;

import javax.xml.namespace.NamespaceContext;

import org.apache.commons.collections.BidiMap;
import org.apache.commons.collections.bidimap.DualHashBidiMap;

/**
 * A simple implementation of the {@link NamespaceContext} interface.
 *
 * Uses a bi-directional HashMap (via apache collections) to bind prefixes to namespace URIs.
 *
 * @author sirianni
 * @version $Revision: #2 $ $Date: 2011/06/24 $
 */
public class SimpleNamespaceContext implements NamespaceContext {

    private BidiMap namespaceMap = new DualHashBidiMap();

    public void bind(String prefix, String namespaceURI) {
        this.namespaceMap.put(prefix, namespaceURI);
    }

    public SimpleNamespaceContext withBinding(String prefix, String namespaceURI) {
        this.bind(prefix, namespaceURI);
        return this;
    }

    @Override
    public String getNamespaceURI(String prefix) {
        return (String) this.namespaceMap.get(prefix);
    }

    @Override
    public String getPrefix(String namespaceURI) {
        return (String) this.namespaceMap.getKey(namespaceURI);
    }

    @Override
    public Iterator<?> getPrefixes(String namespaceURI) {
        /*
         * This method is not called by the vast majority of the clients of the NamespaceContext interface.
         * It is not supported by most other NamespaceContext implementations either.
         */
        throw new UnsupportedOperationException();
    }

}
