/*
 * $Id: //depot/prod/newport_news/main/modules/nn-core/src/main/java/com/netapp/common/xml/XMLTransformChainer.java#3 $
 *
 * Copyright (c) 2009 NetApp, Inc.
 * All rights reserved.
 */
package com.netapp.common.xml;

import java.io.IOException;
import java.io.PipedReader;
import java.io.PipedWriter;
import java.io.Reader;
import java.io.Writer;

import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerException;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.stream.StreamSource;

/**
 * Utility class for chaining a series of XML {@link Transformer}s together.
 *
 * @author sirianni
 * @version $Revision: #3 $ $Date: 2011/06/24 $
 */
public class XMLTransformChainer {

    private XMLTransformChainer() {
    }

    /**
     * Apply a series of XML transformations to the input reader,
     * writing the result to the output writer.
     *
     * @param reader Input reader (must be XML)
     * @param writer Output writer
     * @param transformers List of transformers to apply
     */
    public static void applyTransforms(Reader reader, Writer writer, Transformer... transformers) {
        if (transformers.length == 0) {
            throw new IllegalArgumentException();
        }
        try {
            Reader currentReader = reader;

            for (int i = 0; i < transformers.length - 1; i++) {
                final Transformer transformer = transformers[i];

                PipedWriter pipedWriter = new PipedWriter();
                PipedReader pipedReader = new PipedReader(pipedWriter);

                new TransformerThread(i, transformer, currentReader, pipedWriter).start();

                currentReader = pipedReader;
            }

            Transformer lastTransformer = transformers[transformers.length - 1];
            lastTransformer.transform(new StreamSource(currentReader), new StreamResult(writer));
        } catch (IOException e) {
            throw new IllegalStateException(e);
        } catch (TransformerException e) {
            throw new IllegalStateException(e);
        }
    }

    private static class TransformerThread extends Thread {

        private Transformer transformer;
        private Reader reader;
        private Writer writer;

        private TransformerThread(int number, Transformer transformer, Reader reader, Writer writer) {
            super("Transformer-Thread-" + number);
            this.transformer = transformer;
            this.reader = reader;
            this.writer = writer;
        }

        @Override
        public void run() {
            try {
                this.transformer.transform(new StreamSource(this.reader), new StreamResult(this.writer));
            } catch (TransformerException e) {
                throw new RuntimeException(e);
            } finally {
                try {
                    this.writer.close();
                } catch (IOException e) {
                    // Ignore
                }
            }
        }
    }

}
