package com.vmware.vcloud.sdk;

import com.vmware.vcloud.api.rest.schema.ApiDefinitionType;
import com.vmware.vcloud.api.rest.schema.LinkType;
import com.vmware.vcloud.api.rest.schema.QueryResultFileDescriptorRecordType;
import com.vmware.vcloud.api.rest.schema.ReferenceType;
import com.vmware.vcloud.sdk.constants.SdkMessage;
import com.vmware.vcloud.sdk.constants.query.FormatType;
import java.util.logging.Level;
import java.util.logging.Logger;

public class ApiDefinition extends VcloudEntity<ApiDefinitionType>
{
  private static Logger logger = Logger.getLogger("com.vmware.vcloud.sdk");
  private ReferenceType serviceRef;

  ApiDefinition(VcloudClient client, ApiDefinitionType apiDefinitionParams)
  {
    super(client, apiDefinitionParams);
    sortApiDefinitionRefs();
  }

  private void sortApiDefinitionRefs()
  {
    if (getResource() != null)
      for (LinkType link : ((ApiDefinitionType)getResource()).getLink())
        if ((link.getRel().equals("up")) && (link.getType().equals("application/vnd.vmware.vcloud.service+xml")))
        {
          this.serviceRef = link;
        }
  }

  public static ApiDefinition getApiDefinitionByReference(VcloudClient client, ReferenceType reference)
    throws VCloudException
  {
    logger.log(Level.INFO, SdkUtil.getI18nString(SdkMessage.GET_URL_MSG) + " - " + reference.getHref());

    return new ApiDefinition(client, (ApiDefinitionType)getResourceByReference(client, reference));
  }

  public static ApiDefinition getApiDefinitionById(VcloudClient client, String vCloudId)
    throws VCloudException
  {
    logger.log(Level.INFO, SdkUtil.getI18nString(SdkMessage.GET_VCLOUD_ID_MSG) + " - " + vCloudId);

    return new ApiDefinition(client, (ApiDefinitionType)getEntityById(client, vCloudId, "application/vnd.vmware.vcloud.apidefinition+xml"));
  }

  public RecordResult<QueryResultFileDescriptorRecordType> getFileDescriptorRecords()
    throws VCloudException
  {
    String query = getReference().getHref() + "/files" + "?" + getVcloudClient().getQueryService().buildQuery(FormatType.RECORD_VIEW);

    return (RecordResult)getVcloudClient().getQueryService().executeQuery(query);
  }

  public ReferenceType getServiceReference()
    throws VCloudException
  {
    if (this.serviceRef != null)
      return this.serviceRef;
    throw new VCloudException(SdkUtil.getI18nString(SdkMessage.REFERENCE_NOT_FOUND_MSG));
  }
}