package com.vmware.vcloud.sdk;

import java.io.ByteArrayOutputStream;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.io.UnsupportedEncodingException;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBElement;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Marshaller;
import javax.xml.bind.Unmarshaller;
import javax.xml.transform.stream.StreamSource;

public class JAXBUtil
{
  private static Logger logger = Logger.getLogger("com.vmware.vcloud.sdk");
  private static JAXBContext jaxbContexts;

  public static void addPackages(List<String> packageNames)
  {
    String packages = "com.vmware.vcloud.api.rest.schema.versioning:com.vmware.vcloud.api.rest.schema.ovf:com.vmware.vcloud.api.rest.schema:com.vmware.vcloud.api.rest.schema.extension:com.vmware.vcloud.api.rest.schema.ovf.environment:com.vmware.vcloud.api.rest.schema.ovf.vmware:";

    for (String packageName : packageNames)
      packages = packages + packageName + ":";
    try
    {
      if ((packages != null) && (!packages.isEmpty()))
        jaxbContexts = JAXBContext.newInstance(packages);
      else
        throw new VCloudRuntimeException("No Packages Found");
    }
    catch (JAXBException e) {
      logger.log(Level.SEVERE, null, e);
      throw new VCloudRuntimeException(e.getMessage());
    }
  }

  public static <Type> Type unmarshallResource(InputStream is)
  {
    JAXBElement element = null;
    try {
      Unmarshaller unmarshaller = jaxbContexts.createUnmarshaller();

      element = (JAXBElement)unmarshaller.unmarshal(new StreamSource(new InputStreamReader(is, "UTF-8")));
    }
    catch (JAXBException ex)
    {
      logger.log(Level.SEVERE, null, ex);
      throw new VCloudRuntimeException(ex.getMessage());
    } catch (UnsupportedEncodingException e) {
      logger.log(Level.SEVERE, null, e);
      throw new VCloudRuntimeException(e.getMessage());
    }
    if (element == null) {
      return null;
    }
    return element.getValue();
  }

  public static String marshal(JAXBElement<?> element)
  {
    try
    {
      Marshaller marshaller = jaxbContexts.createMarshaller();

      marshaller.setProperty("jaxb.encoding", "UTF-8");

      marshaller.setProperty("jaxb.formatted.output", Boolean.TRUE);

      OutputStream os = new ByteArrayOutputStream();
      marshaller.marshal(element, os);
      return os.toString();
    }
    catch (JAXBException e) {
      e.printStackTrace();
    }
    return null;
  }

  static
  {
    if (jaxbContexts == null)
      try {
        jaxbContexts = JAXBContext.newInstance("com.vmware.vcloud.api.rest.schema.versioning:com.vmware.vcloud.api.rest.schema.ovf:com.vmware.vcloud.api.rest.schema:com.vmware.vcloud.api.rest.schema.extension:com.vmware.vcloud.api.rest.schema.ovf.environment:com.vmware.vcloud.api.rest.schema.ovf.vmware");
      }
      catch (JAXBException e)
      {
        logger.log(Level.SEVERE, e.getMessage());
      }
  }
}