package com.vmware.vcloud.sdk;

import com.vmware.vcloud.sdk.constants.MetadataDomain;
import com.vmware.vcloud.sdk.constants.query.QueryField;
import com.vmware.vcloud.sdk.constants.query.SortType;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Set;

public class QueryParams<T extends QueryField>
{
  private LinkedHashMap<T, SortType> sortFields;
  private Set<T> fields;
  private List<String> metadataFields;
  private Integer page;
  private Integer pageSize;
  private Integer offset;
  private Filter filter;

  List<String> getMetadataFields()
  {
    if (this.metadataFields == null) {
      this.metadataFields = new ArrayList();
    }
    return this.metadataFields;
  }

  public void addMetadataField(String metadataKey, MetadataDomain domain)
  {
    if ((metadataKey != null) && (domain != null))
      getMetadataFields().add("metadata@" + domain.value() + ":" + metadataKey);
  }

  public void addMetadataField(String metadataKey)
  {
    if (metadataKey != null)
      getMetadataFields().add("metadata:" + metadataKey);
  }

  public Integer getOffset()
  {
    return this.offset;
  }

  public void setOffset(Integer offset)
  {
    this.offset = offset;
  }

  public Set<T> getFields()
  {
    if (this.fields == null) {
      this.fields = new HashSet();
    }
    return this.fields;
  }

  public void setFields(Set<T> fields)
  {
    this.fields = fields;
  }

  public Filter getFilter()
  {
    return this.filter;
  }

  public void setFilter(Filter filter)
  {
    this.filter = filter;
  }

  public LinkedHashMap<T, SortType> getSortFields()
  {
    if (this.sortFields == null) {
      this.sortFields = new LinkedHashMap();
    }
    return this.sortFields;
  }

  public void setSortFields(LinkedHashMap<T, SortType> sortFields)
  {
    this.sortFields = sortFields;
  }

  public Integer getPage()
  {
    return this.page;
  }

  public void setPage(Integer page)
  {
    this.page = page;
  }

  public Integer getPageSize()
  {
    return this.pageSize;
  }

  public void setPageSize(Integer pageSize)
  {
    this.pageSize = pageSize;
  }
}