package com.vmware.vcloud.sdk.admin;

import com.vmware.vcloud.api.rest.schema.AdminCatalogType;
import com.vmware.vcloud.api.rest.schema.AdminOrgType;
import com.vmware.vcloud.api.rest.schema.AdminVdcType;
import com.vmware.vcloud.api.rest.schema.CatalogsListType;
import com.vmware.vcloud.api.rest.schema.CertificateUpdateParamsType;
import com.vmware.vcloud.api.rest.schema.CertificateUploadSocketType;
import com.vmware.vcloud.api.rest.schema.CreateVdcParamsType;
import com.vmware.vcloud.api.rest.schema.EventType;
import com.vmware.vcloud.api.rest.schema.GroupType;
import com.vmware.vcloud.api.rest.schema.GroupsListType;
import com.vmware.vcloud.api.rest.schema.KeystoreUpdateParamsType;
import com.vmware.vcloud.api.rest.schema.KeystoreUploadSocketType;
import com.vmware.vcloud.api.rest.schema.LinkType;
import com.vmware.vcloud.api.rest.schema.NetworksType;
import com.vmware.vcloud.api.rest.schema.ObjectFactory;
import com.vmware.vcloud.api.rest.schema.OrgEmailSettingsType;
import com.vmware.vcloud.api.rest.schema.OrgFederationSettingsType;
import com.vmware.vcloud.api.rest.schema.OrgGeneralSettingsType;
import com.vmware.vcloud.api.rest.schema.OrgGuestPersonalizationSettingsType;
import com.vmware.vcloud.api.rest.schema.OrgLdapSettingsType;
import com.vmware.vcloud.api.rest.schema.OrgLeaseSettingsType;
import com.vmware.vcloud.api.rest.schema.OrgNetworkType;
import com.vmware.vcloud.api.rest.schema.OrgOperationLimitsSettingsType;
import com.vmware.vcloud.api.rest.schema.OrgPasswordPolicySettingsType;
import com.vmware.vcloud.api.rest.schema.OrgSettingsType;
import com.vmware.vcloud.api.rest.schema.OrgVAppTemplateLeaseSettingsType;
import com.vmware.vcloud.api.rest.schema.ReferenceType;
import com.vmware.vcloud.api.rest.schema.SspiKeytabUpdateParamsType;
import com.vmware.vcloud.api.rest.schema.SspiKeytabUploadSocketType;
import com.vmware.vcloud.api.rest.schema.UserType;
import com.vmware.vcloud.api.rest.schema.UsersListType;
import com.vmware.vcloud.api.rest.schema.VdcsType;
import com.vmware.vcloud.sdk.JAXBUtil;
import com.vmware.vcloud.sdk.SdkUtil;
import com.vmware.vcloud.sdk.VCloudException;
import com.vmware.vcloud.sdk.VcloudClient;
import com.vmware.vcloud.sdk.VcloudEntity;
import com.vmware.vcloud.sdk.constants.SdkMessage;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.xml.bind.JAXBElement;

public class AdminOrganization extends VcloudEntity<AdminOrgType>
{
  private static Logger logger = Logger.getLogger("com.vmware.vcloud.sdk");
  private HashMap<String, ReferenceType> userRefsByName;
  private Collection<ReferenceType> catalogRefs;
  private HashMap<String, ReferenceType> groupRefsByName;
  private HashMap<String, ReferenceType> adminVdcRefsByName;
  private HashMap<String, ReferenceType> orgNetworkRefsByName;
  private ReferenceType orgReference;
  private ReferenceType tasksListRef;

  AdminOrganization(VcloudClient client, AdminOrgType adminOrgType_v1_5)
  {
    super(client, adminOrgType_v1_5);
    sortAdminOrgRefs_v1_5();
  }

  public static AdminOrganization getAdminOrgByReference(VcloudClient client, ReferenceType orgRef)
    throws VCloudException
  {
    logger.log(Level.INFO, SdkUtil.getI18nString(SdkMessage.GET_URL_MSG) + " - " + orgRef.getHref());

    return new AdminOrganization(client, (AdminOrgType)getResourceByReference(client, orgRef));
  }

  public static AdminOrganization getAdminOrgById(VcloudClient client, String vCloudId)
    throws VCloudException
  {
    logger.log(Level.INFO, SdkUtil.getI18nString(SdkMessage.GET_VCLOUD_ID_MSG) + " - " + vCloudId);

    return new AdminOrganization(client, (AdminOrgType)getEntityById(client, vCloudId, "application/vnd.vmware.admin.organization+xml"));
  }

  public OrgSettingsType getOrgSettings()
  {
    return ((AdminOrgType)getResource()).getSettings();
  }

  public static OrgSettingsType getOrgSettings(VcloudClient client, ReferenceType adminOrgReference)
    throws VCloudException
  {
    String adminOrgSettingsUrl = adminOrgReference.getHref() + "/settings";

    return (OrgSettingsType)SdkUtil.get(client, adminOrgSettingsUrl, 200);
  }

  public OrgSettingsType updateOrgSettings(OrgSettingsType orgSettingsType)
    throws VCloudException
  {
    String adminOrgSettingsUpdateUrl = getReference().getHref() + "/settings";

    ObjectFactory objectFactory_v1_5 = new ObjectFactory();
    String adminOrgSettingsUpdateXml = JAXBUtil.marshal(objectFactory_v1_5.createOrgSettings(orgSettingsType));

    return (OrgSettingsType)SdkUtil.put(getVcloudClient(), adminOrgSettingsUpdateUrl, adminOrgSettingsUpdateXml, "application/vnd.vmware.admin.orgSettings+xml", 200);
  }

  public AdminCatalog createCatalog(AdminCatalogType adminCatalogType)
    throws VCloudException
  {
    ObjectFactory of = new ObjectFactory();
    JAXBElement catalogParams = of.createAdminCatalog(adminCatalogType);

    String requestBodyXml = JAXBUtil.marshal(catalogParams);
    String postURL = getReference().getHref() + "/catalogs";

    return new AdminCatalog(getVcloudClient(), (AdminCatalogType)SdkUtil.post(getVcloudClient(), postURL, requestBodyXml, "application/vnd.vmware.admin.catalog+xml", 201));
  }

  public User createUser(UserType userType)
    throws VCloudException
  {
    ObjectFactory of = new ObjectFactory();
    JAXBElement userParams = of.createUser(userType);
    String requestBodyXml = JAXBUtil.marshal(userParams);
    String postURL = getReference().getHref() + "/users";

    return new User(getVcloudClient(), (UserType)SdkUtil.post(getVcloudClient(), postURL, requestBodyXml, "application/vnd.vmware.admin.user+xml", 201));
  }

  public Group importGroup(GroupType groupType)
    throws VCloudException
  {
    ObjectFactory of = new ObjectFactory();
    JAXBElement groupParams = of.createGroup(groupType);
    String requestBodyXml = JAXBUtil.marshal(groupParams);
    String postURL = getReference().getHref() + "/groups";

    return new Group(getVcloudClient(), (GroupType)SdkUtil.post(getVcloudClient(), postURL, requestBodyXml, "application/vnd.vmware.admin.group+xml", 201));
  }

  /** @deprecated */
  public AdminVdc createAdminVdc(AdminVdcType adminVdcType)
    throws VCloudException
  {
    ObjectFactory of = new ObjectFactory();
    JAXBElement adminVdcParams = of.createAdminVdc(adminVdcType);

    String requestBodyXml = JAXBUtil.marshal(adminVdcParams);
    String postURL = getReference().getHref() + "/vdcs";

    return new AdminVdc(getVcloudClient(), (AdminVdcType)SdkUtil.post(getVcloudClient(), postURL, requestBodyXml, "application/vnd.vmware.admin.vdc+xml", 201));
  }

  public AdminVdc createAdminVdc(CreateVdcParamsType vdcParams)
    throws VCloudException
  {
    ObjectFactory of = new ObjectFactory();
    JAXBElement adminVdcParams = of.createCreateVdcParams(vdcParams);

    String requestBodyXml = JAXBUtil.marshal(adminVdcParams);
    String postURL = getReference().getHref() + "/vdcsparams";

    return new AdminVdc(getVcloudClient(), (AdminVdcType)SdkUtil.post(getVcloudClient(), postURL, requestBodyXml, "application/vnd.vmware.admin.createVdcParams+xml", 201));
  }

  /** @deprecated */
  public AdminOrgNetwork createOrgNetwork(OrgNetworkType orgNetworkType)
    throws VCloudException
  {
    ObjectFactory of = new ObjectFactory();
    JAXBElement orgNetworkParams = of.createOrgNetwork(orgNetworkType);

    String requestBodyXml = JAXBUtil.marshal(orgNetworkParams);
    String postURL = getReference().getHref() + "/networks";

    return new AdminOrgNetwork(getVcloudClient(), (OrgNetworkType)SdkUtil.post(getVcloudClient(), postURL, requestBodyXml, "application/vnd.vmware.admin.orgNetwork+xml", 201));
  }

  public ReferenceType getOrgReference()
    throws VCloudException
  {
    if (this.orgReference != null)
      return this.orgReference;
    throw new VCloudException(SdkUtil.getI18nString(SdkMessage.REFERENCE_NOT_FOUND_MSG));
  }

  public ReferenceType getTaskListReference()
    throws VCloudException
  {
    if (this.tasksListRef != null)
      return this.tasksListRef;
    throw new VCloudException(SdkUtil.getI18nString(SdkMessage.REFERENCE_NOT_FOUND_MSG));
  }

  private void sortAdminOrgRefs_v1_5()
  {
    this.userRefsByName = new HashMap();
    this.groupRefsByName = new HashMap();
    this.catalogRefs = new ArrayList();
    this.adminVdcRefsByName = new HashMap();
    this.orgNetworkRefsByName = new HashMap();

    if (((AdminOrgType)getResource()).getUsers() != null) {
      for (ReferenceType userRef : ((AdminOrgType)getResource()).getUsers().getUserReference())
      {
        this.userRefsByName.put(userRef.getName(), userRef);
      }
    }
    if (((AdminOrgType)getResource()).getGroups() != null) {
      for (ReferenceType groupRef : ((AdminOrgType)getResource()).getGroups().getGroupReference())
      {
        this.groupRefsByName.put(groupRef.getName(), groupRef);
      }
    }
    if (((AdminOrgType)getResource()).getCatalogs() != null) {
      for (ReferenceType catalogRef : ((AdminOrgType)getResource()).getCatalogs().getCatalogReference())
      {
        this.catalogRefs.add(catalogRef);
      }
    }
    if (((AdminOrgType)getResource()).getVdcs() != null) {
      for (ReferenceType vdcRef : ((AdminOrgType)getResource()).getVdcs().getVdc()) {
        this.adminVdcRefsByName.put(vdcRef.getName(), vdcRef);
      }
    }
    if (((AdminOrgType)getResource()).getNetworks() != null) {
      for (ReferenceType networkRef : ((AdminOrgType)getResource()).getNetworks().getNetwork())
      {
        this.orgNetworkRefsByName.put(networkRef.getName(), networkRef);
      }
    }
    for (LinkType adminOrgLink : ((AdminOrgType)getResource()).getLink()) {
      if ((adminOrgLink.getRel().equals("alternate")) && (adminOrgLink.getType().equals("application/vnd.vmware.vcloud.org+xml")))
      {
        this.orgReference = adminOrgLink;
      }if ((adminOrgLink.getRel().equals("down")) && (adminOrgLink.getType().equals("application/vnd.vmware.vcloud.tasksList+xml")))
      {
        this.tasksListRef = adminOrgLink;
      }
    }
  }

  public static OrgVAppTemplateLeaseSettingsType getvAppTemplateLeaseSettings(VcloudClient client, ReferenceType adminOrgReference)
    throws VCloudException
  {
    String adminOrgvAppTemplateLeaseSettingsUrl = adminOrgReference.getHref() + "/settings" + "/vAppTemplateLeaseSettings";

    return (OrgVAppTemplateLeaseSettingsType)SdkUtil.get(client, adminOrgvAppTemplateLeaseSettingsUrl, 200);
  }

  public OrgVAppTemplateLeaseSettingsType getvAppTemplateLeaseSettings()
    throws VCloudException
  {
    return ((AdminOrgType)getResource()).getSettings().getVAppTemplateLeaseSettings();
  }

  public OrgVAppTemplateLeaseSettingsType updatevAppTemplateLeaseSettings(OrgVAppTemplateLeaseSettingsType orgVAppTemplateLeaseSettingsType)
    throws VCloudException
  {
    String adminOrgvAppTemplateLeaseSettingsUpdateUrl = getReference().getHref() + "/settings" + "/vAppTemplateLeaseSettings";

    ObjectFactory objectFactory101 = new ObjectFactory();
    String adminOrgvAppTemplateLeaseSettingsUpdateXml = JAXBUtil.marshal(objectFactory101.createOrgVAppTemplateLeaseSettings(orgVAppTemplateLeaseSettingsType));

    return (OrgVAppTemplateLeaseSettingsType)SdkUtil.put(getVcloudClient(), adminOrgvAppTemplateLeaseSettingsUpdateUrl, adminOrgvAppTemplateLeaseSettingsUpdateXml, "application/vnd.vmware.admin.vAppTemplateLeaseSettings+xml", 200);
  }

  public static OrgLeaseSettingsType getvAppLeaseSettings(VcloudClient client, ReferenceType adminOrgReference)
    throws VCloudException
  {
    String adminOrgvAppLeaseSettingsUrl = adminOrgReference.getHref() + "/settings" + "/vAppLeaseSettings";

    return (OrgLeaseSettingsType)SdkUtil.get(client, adminOrgvAppLeaseSettingsUrl, 200);
  }

  public OrgLeaseSettingsType getvAppLeaseSettings()
    throws VCloudException
  {
    return ((AdminOrgType)getResource()).getSettings().getVAppLeaseSettings();
  }

  public OrgLeaseSettingsType updatevAppLeaseSettings(OrgLeaseSettingsType orgVAppLeaseSettingsType)
    throws VCloudException
  {
    String adminOrgvAppLeaseSettingsUpdateUrl = getReference().getHref() + "/settings" + "/vAppLeaseSettings";

    ObjectFactory objectFactory_v1_5 = new ObjectFactory();
    String adminOrgvAppLeaseSettingsUpdateXml = JAXBUtil.marshal(objectFactory_v1_5.createVAppLeaseSettings(orgVAppLeaseSettingsType));

    return (OrgLeaseSettingsType)SdkUtil.put(getVcloudClient(), adminOrgvAppLeaseSettingsUpdateUrl, adminOrgvAppLeaseSettingsUpdateXml, "application/vnd.vmware.admin.vAppLeaseSettings+xml", 200);
  }

  public static OrgFederationSettingsType getFederationSettings(VcloudClient client, ReferenceType adminOrgReference)
    throws VCloudException
  {
    String adminOrgEmailSettingsUrl = adminOrgReference.getHref() + "/settings" + "/federation";

    return (OrgFederationSettingsType)SdkUtil.get(client, adminOrgEmailSettingsUrl, 200);
  }

  public OrgFederationSettingsType getFederationSettings()
  {
    return ((AdminOrgType)getResource()).getSettings().getOrgFederationSettings();
  }

  public OrgFederationSettingsType updateFederationSettings(OrgFederationSettingsType orgFederationSettingsType)
    throws VCloudException
  {
    String adminOrgFederationSettingsUpdateUrl = getReference().getHref() + "/settings" + "/federation";

    ObjectFactory objectFactory_v1_5 = new ObjectFactory();
    String adminOrgFederationSettingsUpdateXml = JAXBUtil.marshal(objectFactory_v1_5.createOrgFederationSettings(orgFederationSettingsType));

    return (OrgFederationSettingsType)SdkUtil.put(getVcloudClient(), adminOrgFederationSettingsUpdateUrl, adminOrgFederationSettingsUpdateXml, "application/vnd.vmware.admin.organizationFederationSettings+xml", 200);
  }

  public static OrgEmailSettingsType getEmailSettings(VcloudClient client, ReferenceType adminOrgReference)
    throws VCloudException
  {
    String adminOrgEmailSettingsUrl = adminOrgReference.getHref() + "/settings" + "/email";

    return (OrgEmailSettingsType)SdkUtil.get(client, adminOrgEmailSettingsUrl, 200);
  }

  public OrgEmailSettingsType getEmailSettings()
  {
    return ((AdminOrgType)getResource()).getSettings().getOrgEmailSettings();
  }

  public OrgEmailSettingsType updateEmailSettings(OrgEmailSettingsType orgEmailSettingsType)
    throws VCloudException
  {
    String adminOrgEmailSettingsUpdateUrl = getReference().getHref() + "/settings" + "/email";

    ObjectFactory objectFactory_v1_5 = new ObjectFactory();
    String adminOrgEmailSettingsUpdateXml = JAXBUtil.marshal(objectFactory_v1_5.createOrgEmailSettings(orgEmailSettingsType));

    return (OrgEmailSettingsType)SdkUtil.put(getVcloudClient(), adminOrgEmailSettingsUpdateUrl, adminOrgEmailSettingsUpdateXml, "application/vnd.vmware.admin.organizationEmailSettings+xml", 200);
  }

  public static OrgGeneralSettingsType getGeneralSettings(VcloudClient client, ReferenceType adminOrgReference)
    throws VCloudException
  {
    String adminOrgGeneralSettingsUrl = adminOrgReference.getHref() + "/settings" + "/general";

    return (OrgGeneralSettingsType)SdkUtil.get(client, adminOrgGeneralSettingsUrl, 200);
  }

  public OrgGeneralSettingsType getGeneralSettings()
  {
    return ((AdminOrgType)getResource()).getSettings().getOrgGeneralSettings();
  }

  public OrgGeneralSettingsType updateGeneralSettings(OrgGeneralSettingsType orgGeneralSettingsType)
    throws VCloudException
  {
    String adminOrgGeneralSettingsUpdateUrl = getReference().getHref() + "/settings" + "/general";

    ObjectFactory objectFactory_v1_5 = new ObjectFactory();
    String adminOrgGeneralSettingsUpdateXml = JAXBUtil.marshal(objectFactory_v1_5.createGeneralOrgSettings(orgGeneralSettingsType));

    return (OrgGeneralSettingsType)SdkUtil.put(getVcloudClient(), adminOrgGeneralSettingsUpdateUrl, adminOrgGeneralSettingsUpdateXml, "application/vnd.vmware.admin.organizationGeneralSettings+xml", 200);
  }

  public static OrgPasswordPolicySettingsType getPasswordPolicySettings(VcloudClient client, ReferenceType adminOrgReference)
    throws VCloudException
  {
    String adminOrgPasswordPolicySettingsUrl = adminOrgReference.getHref() + "/settings" + "/passwordPolicy";

    return (OrgPasswordPolicySettingsType)SdkUtil.get(client, adminOrgPasswordPolicySettingsUrl, 200);
  }

  public OrgPasswordPolicySettingsType getPasswordPolicySettings()
  {
    return ((AdminOrgType)getResource()).getSettings().getOrgPasswordPolicySettings();
  }

  public OrgPasswordPolicySettingsType updatePasswordPolicySettings(OrgPasswordPolicySettingsType orgPasswordPolicySettingsType)
    throws VCloudException
  {
    String adminOrgPasswordPolicySettingsUpdateUrl = getReference().getHref() + "/settings" + "/passwordPolicy";

    ObjectFactory objectFactory_v1_5 = new ObjectFactory();
    String adminOrgPasswordPolicySettingsUpdateXml = JAXBUtil.marshal(objectFactory_v1_5.createOrgPasswordPolicySettings(orgPasswordPolicySettingsType));

    return (OrgPasswordPolicySettingsType)SdkUtil.put(getVcloudClient(), adminOrgPasswordPolicySettingsUpdateUrl, adminOrgPasswordPolicySettingsUpdateXml, "application/vnd.vmware.admin.organizationPasswordPolicySettings+xml", 200);
  }

  public AdminOrganization updateAdminOrg(AdminOrgType adminOrgType)
    throws VCloudException
  {
    ObjectFactory of = new ObjectFactory();
    String adminOrgUpdateUrl = getReference().getHref();
    String adminOrgUpdateRequestXmlString = JAXBUtil.marshal(of.createAdminOrg(adminOrgType));

    return new AdminOrganization(getVcloudClient(), (AdminOrgType)SdkUtil.put(getVcloudClient(), adminOrgUpdateUrl, adminOrgUpdateRequestXmlString, "application/vnd.vmware.admin.organization+xml", 200));
  }

  public void delete()
    throws VCloudException
  {
    String adminOrgUrl = getReference().getHref();
    deleteAdminOrg(getVcloudClient(), adminOrgUrl);
  }

  public static void delete(VcloudClient client, ReferenceType adminOrgRef)
    throws VCloudException
  {
    deleteAdminOrg(client, adminOrgRef.getHref());
  }

  private static void deleteAdminOrg(VcloudClient client, String adminOrgUrl)
    throws VCloudException
  {
    SdkUtil.delete(client, adminOrgUrl, 204);
  }

  public void enable()
    throws VCloudException
  {
    String enableURL = getReference().getHref() + "/action/enable";

    executeOrgAction(getVcloudClient(), enableURL, null, null, 204);
  }

  public static void enable(VcloudClient client, ReferenceType orgRef)
    throws VCloudException
  {
    String enableURL = orgRef.getHref() + "/action/enable";

    executeOrgAction(client, enableURL, null, null, 204);
  }

  public void disable()
    throws VCloudException
  {
    String disableUrl = getReference().getHref() + "/action/disable";

    executeOrgAction(getVcloudClient(), disableUrl, null, null, 204);
  }

  public static void disable(VcloudClient client, ReferenceType adminOrgRef)
    throws VCloudException
  {
    String enableURL = adminOrgRef.getHref() + "/action/disable";

    executeOrgAction(client, enableURL, null, null, 204);
  }

  private static void executeOrgAction(VcloudClient client, String orgActionUrl, String content, String contentType, int statusCode)
    throws VCloudException
  {
    logger.log(Level.INFO, SdkUtil.getI18nString(SdkMessage.POST_URL_MSG) + " - " + orgActionUrl);

    SdkUtil.post(client, orgActionUrl, content, contentType, statusCode);
  }

  public HashMap<String, ReferenceType> getUserRefsByName()
  {
    return this.userRefsByName;
  }

  public Collection<ReferenceType> getUserRefs()
  {
    return this.userRefsByName.values();
  }

  public ReferenceType getUserRefByName(String name)
  {
    return (ReferenceType)this.userRefsByName.get(name);
  }

  public HashMap<String, ReferenceType> getGroupRefsByName()
  {
    return this.groupRefsByName;
  }

  public Collection<ReferenceType> getGroupRefs()
  {
    return this.groupRefsByName.values();
  }

  public ReferenceType getGroupRefByName(String name)
  {
    return (ReferenceType)this.groupRefsByName.get(name);
  }

  public Collection<ReferenceType> getCatalogRefs()
  {
    return this.catalogRefs;
  }

  public HashMap<String, ReferenceType> getAdminVdcRefsByName()
  {
    return this.adminVdcRefsByName;
  }

  public Collection<ReferenceType> getAdminVdcRefs()
  {
    return this.adminVdcRefsByName.values();
  }

  public ReferenceType getAdminVdcRefByName(String name)
  {
    return (ReferenceType)this.adminVdcRefsByName.get(name);
  }

  /** @deprecated */
  public HashMap<String, ReferenceType> getAdminOrgNetworkRefsByName()
  {
    return this.orgNetworkRefsByName;
  }

  /** @deprecated */
  public Collection<ReferenceType> getAdminOrgNetworkRefs()
  {
    return this.orgNetworkRefsByName.values();
  }

  /** @deprecated */
  public ReferenceType getAdminOrgNetworkRefByName(String name)
  {
    return (ReferenceType)this.orgNetworkRefsByName.get(name);
  }

  public void createEvent(EventType eventType)
    throws VCloudException
  {
    String createEventUrl = getReference().getHref() + "/events";

    String adminOrgSettingsUpdateXml = JAXBUtil.marshal(new ObjectFactory().createEvent(eventType));

    SdkUtil.post(getVcloudClient(), createEventUrl, adminOrgSettingsUpdateXml, "application/vnd.vmware.admin.event+xml", 204);
  }

  public OrgGuestPersonalizationSettingsType getGuestPersonalizationSettings()
    throws VCloudException
  {
    String adminOrgSettingsUrl = getReference().getHref() + "/settings" + "/guestPersonalizationSettings";

    return (OrgGuestPersonalizationSettingsType)SdkUtil.get(getVcloudClient(), adminOrgSettingsUrl, 200);
  }

  public OrgOperationLimitsSettingsType getOperationLimitsSettings()
    throws VCloudException
  {
    String adminOrgSettingsUrl = getReference().getHref() + "/settings" + "/operationLimitsSettings";

    return (OrgOperationLimitsSettingsType)SdkUtil.get(getVcloudClient(), adminOrgSettingsUrl, 200);
  }

  public OrgGuestPersonalizationSettingsType updateGuestPersonalizationSettings(OrgGuestPersonalizationSettingsType orgGuestPersonalizationSettingsType)
    throws VCloudException
  {
    String adminOrgSettingsUpdateUrl = getReference().getHref() + "/settings" + "/guestPersonalizationSettings";

    ObjectFactory objectFactory_v1_5 = new ObjectFactory();
    String adminOrgSettingsUpdateXml = JAXBUtil.marshal(objectFactory_v1_5.createOrgGuestPersonalizationSettings(orgGuestPersonalizationSettingsType));

    return (OrgGuestPersonalizationSettingsType)SdkUtil.put(getVcloudClient(), adminOrgSettingsUpdateUrl, adminOrgSettingsUpdateXml, "application/vnd.vmware.admin.guestPersonalizationSettings+xml", 200);
  }

  public OrgOperationLimitsSettingsType updateOperationLimitsSettings(OrgOperationLimitsSettingsType operationLimitsSettingsType)
    throws VCloudException
  {
    String adminOrgSettingsUpdateUrl = getReference().getHref() + "/settings" + "/operationLimitsSettings";

    ObjectFactory objectFactory_v1_5 = new ObjectFactory();
    String adminOrgSettingsUpdateXml = JAXBUtil.marshal(objectFactory_v1_5.createOrgOperationLimitsSettings(operationLimitsSettingsType));

    return (OrgOperationLimitsSettingsType)SdkUtil.put(getVcloudClient(), adminOrgSettingsUpdateUrl, adminOrgSettingsUpdateXml, "application/vnd.vmware.admin.operationLimitsSettings+xml", 200);
  }

  /** @deprecated */
  public OrgLdapSettingsType getLdapSettings()
    throws VCloudException
  {
    return ((AdminOrgType)getResource()).getSettings().getOrgLdapSettings();
  }

  /** @deprecated */
  public static OrgLdapSettingsType getLdapSettings(VcloudClient client, ReferenceType adminOrgReference)
    throws VCloudException
  {
    String adminOrgLdapSettingsUrl = adminOrgReference.getHref() + "/settings" + "/ldap";

    return (OrgLdapSettingsType)SdkUtil.get(client, adminOrgLdapSettingsUrl, 200);
  }

  /** @deprecated */
  public OrgLdapSettingsType UpdateLdapSettings(OrgLdapSettingsType orgLdapSettingsType)
    throws VCloudException
  {
    String adminOrgLdapSettingsUpdateUrl = getReference().getHref() + "/settings" + "/ldap";

    ObjectFactory objectFactory_v1_5 = new ObjectFactory();
    String adminOrgLdapSettingsUpdateXml = JAXBUtil.marshal(objectFactory_v1_5.createOrgLdapSettings(orgLdapSettingsType));

    return (OrgLdapSettingsType)SdkUtil.put(getVcloudClient(), adminOrgLdapSettingsUpdateUrl, adminOrgLdapSettingsUpdateXml, "application/vnd.vmware.admin.organizationLdapSettings+xml", 200);
  }

  public void resetLdapCertificate()
    throws VCloudException
  {
    String resetLdapCertificateUrl = getReference().getHref() + "/settings" + "/ldap" + "/action/resetLdapCertificate";

    SdkUtil.post(getVcloudClient(), resetLdapCertificateUrl, null, null, 204);
  }

  public void resetLdapKeyStore()
    throws VCloudException
  {
    String resetLdapKeyStore = getReference().getHref() + "/settings" + "/ldap" + "/action/resetLdapKeyStore";

    SdkUtil.post(getVcloudClient(), resetLdapKeyStore, null, null, 204);
  }

  public void resetLdapSspiKeytab()
    throws VCloudException
  {
    String resetLdapSspiKeytabUrl = getReference().getHref() + "/settings" + "/ldap" + "/action/resetLdapSspiKeytab";

    SdkUtil.post(getVcloudClient(), resetLdapSspiKeytabUrl, null, null, 204);
  }

  public CertificateUploadSocketType updateLdapCertificate(CertificateUpdateParamsType certificateUpdateParamsType)
    throws VCloudException
  {
    String adminOrgLdapCertificate = getReference().getHref() + "/settings" + "/ldap" + "/action/updateLdapCertificate";

    ObjectFactory objectFactory_v5_1 = new ObjectFactory();
    String requestBodyXml = JAXBUtil.marshal(objectFactory_v5_1.createCertificateUpdateParams(certificateUpdateParamsType));

    return (CertificateUploadSocketType)SdkUtil.post(getVcloudClient(), adminOrgLdapCertificate, requestBodyXml, "application/vnd.vmware.admin.certificateUpdateParams+xml", 200);
  }

  public KeystoreUploadSocketType updateLdapKeyStore(KeystoreUpdateParamsType keystoreUpdateParamsType)
    throws VCloudException
  {
    String adminOrgLdapKeyStore = getReference().getHref() + "/settings" + "/ldap" + "/action/updateLdapKeyStore";

    ObjectFactory objectFactory_v5_1 = new ObjectFactory();
    String requestBodyXml = JAXBUtil.marshal(objectFactory_v5_1.createKeystoreUpdateParams(keystoreUpdateParamsType));

    return (KeystoreUploadSocketType)SdkUtil.post(getVcloudClient(), adminOrgLdapKeyStore, requestBodyXml, "application/vnd.vmware.admin.keystoreUpdateParams+xml", 200);
  }

  public SspiKeytabUploadSocketType updateLdapSspiKeytab(SspiKeytabUpdateParamsType sspiKeytabUpdateParamsType)
    throws VCloudException
  {
    String adminOrgLdapSspiKeytab = getReference().getHref() + "/settings" + "/ldap" + "/action/updateLdapSspiKeytab";

    ObjectFactory objectFactory_v5_1 = new ObjectFactory();
    String requestBodyXml = JAXBUtil.marshal(objectFactory_v5_1.createSspiKeytabUpdateParams(sspiKeytabUpdateParamsType));

    return (SspiKeytabUploadSocketType)SdkUtil.post(getVcloudClient(), adminOrgLdapSspiKeytab, requestBodyXml, "application/vnd.vmware.admin.sspiKeytabUpdateParams+xml", 200);
  }
}