package com.vmware.vcloud.sdk.admin;

import com.vmware.vcloud.api.rest.schema.AdminVdcType;
import com.vmware.vcloud.api.rest.schema.AvailableNetworksType;
import com.vmware.vcloud.api.rest.schema.GatewayType;
import com.vmware.vcloud.api.rest.schema.LinkType;
import com.vmware.vcloud.api.rest.schema.ObjectFactory;
import com.vmware.vcloud.api.rest.schema.OrgVdcNetworkType;
import com.vmware.vcloud.api.rest.schema.ReferenceType;
import com.vmware.vcloud.api.rest.schema.ResourceEntitiesType;
import com.vmware.vcloud.api.rest.schema.ResourceReferenceType;
import com.vmware.vcloud.api.rest.schema.TaskType;
import com.vmware.vcloud.api.rest.schema.UpdateVdcStorageProfilesType;
import com.vmware.vcloud.api.rest.schema.VCloudExtensionType;
import com.vmware.vcloud.api.rest.schema.VdcStorageProfileParamsType;
import com.vmware.vcloud.api.rest.schema.VdcStorageProfilesType;
import com.vmware.vcloud.api.rest.schema.extension.OrganizationResourcePoolSetType;
import com.vmware.vcloud.api.rest.schema.extension.VimObjectRefType;
import com.vmware.vcloud.sdk.JAXBUtil;
import com.vmware.vcloud.sdk.QueryService;
import com.vmware.vcloud.sdk.ReferenceResult;
import com.vmware.vcloud.sdk.SdkUtil;
import com.vmware.vcloud.sdk.Task;
import com.vmware.vcloud.sdk.VCloudException;
import com.vmware.vcloud.sdk.VcloudClient;
import com.vmware.vcloud.sdk.VcloudEntity;
import com.vmware.vcloud.sdk.constants.SdkMessage;
import com.vmware.vcloud.sdk.constants.VimObjectTypeEnum;
import com.vmware.vcloud.sdk.constants.query.FormatType;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.xml.bind.JAXBElement;

public class AdminVdc extends VcloudEntity<AdminVdcType>
{
  private static Logger logger = Logger.getLogger("com.vmware.vcloud.sdk");
  private Collection<ReferenceType> vappTemplateRefs;
  private Collection<ReferenceType> mediaRefs;
  private Collection<ReferenceType> storageProfileRefs;
  private Collection<ReferenceType> diskRefs;
  private HashMap<String, ReferenceType> vappRefsByName;
  private HashMap<String, ReferenceType> orgNetworkRefsByName;
  private ReferenceType adminOrgReference;
  private ReferenceType vdcReference;
  private VimObjectRefType primaryResourcePoolVimRef;
  private ReferenceType orgVdcResourcePoolsRef;

  AdminVdc(VcloudClient client, AdminVdcType adminVdcType_v1_5)
  {
    super(client, adminVdcType_v1_5);
    sortEntityAndNetworkRefs_v1_5();
  }

  public static AdminVdc getAdminVdcByReference(VcloudClient client, ReferenceType vdcRef)
    throws VCloudException
  {
    logger.log(Level.INFO, SdkUtil.getI18nString(SdkMessage.GET_URL_MSG) + " - " + vdcRef.getHref());

    return new AdminVdc(client, (AdminVdcType)getResourceByReference(client, vdcRef));
  }

  public static AdminVdc getAdminVdcById(VcloudClient client, String vCloudId)
    throws VCloudException
  {
    logger.log(Level.INFO, SdkUtil.getI18nString(SdkMessage.GET_VCLOUD_ID_MSG) + " - " + vCloudId);

    return new AdminVdc(client, (AdminVdcType)getEntityById(client, vCloudId, "application/vnd.vmware.admin.vdc+xml"));
  }

  public Task updateAdminVdc(AdminVdcType adminVdcType)
    throws VCloudException
  {
    ObjectFactory of = new ObjectFactory();
    String adminVdcUpdateUrl = getReference().getHref();
    String adminVdcUpdateRequestXmlString = JAXBUtil.marshal(of.createAdminVdc(adminVdcType));

    logger.log(Level.INFO, SdkUtil.getI18nString(SdkMessage.PUT_URL_MSG) + " - " + adminVdcUpdateUrl);

    return new Task(getVcloudClient(), (TaskType)SdkUtil.put(getVcloudClient(), adminVdcUpdateUrl, adminVdcUpdateRequestXmlString, "application/vnd.vmware.admin.vdc+xml", 202));
  }

  public Task delete()
    throws VCloudException
  {
    String adminVdcUrl = getReference().getHref();
    return deleteAdminVdc(getVcloudClient(), adminVdcUrl);
  }

  public static Task delete(VcloudClient client, ReferenceType adminVdcRef)
    throws VCloudException
  {
    return deleteAdminVdc(client, adminVdcRef.getHref());
  }

  private static Task deleteAdminVdc(VcloudClient client, String adminVdcUrl)
    throws VCloudException
  {
    return new Task(client, (TaskType)SdkUtil.delete(client, adminVdcUrl, 202));
  }

  public void enable()
    throws VCloudException
  {
    String enableURL = getReference().getHref() + "/action/enable";

    executeVdcAction(getVcloudClient(), enableURL, null, null, 204);
  }

  public static void enable(VcloudClient client, ReferenceType adminVdcRef)
    throws VCloudException
  {
    String enableURL = adminVdcRef.getHref() + "/action/enable";

    executeVdcAction(client, enableURL, null, null, 204);
  }

  public void disable()
    throws VCloudException
  {
    String disableUrl = getReference().getHref() + "/action/disable";

    executeVdcAction(getVcloudClient(), disableUrl, null, null, 204);
  }

  public static void disable(VcloudClient client, ReferenceType adminVdcRef)
    throws VCloudException
  {
    String enableURL = adminVdcRef.getHref() + "/action/disable";

    executeVdcAction(client, enableURL, null, null, 204);
  }

  public Task enableFastProvisioning()
    throws VCloudException
  {
    AdminVdcType adminVdcType = (AdminVdcType)getResource();
    adminVdcType.setUsesFastProvisioning(Boolean.valueOf(true));
    return updateAdminVdc(adminVdcType);
  }

  public Task disableFastProvisioning()
    throws VCloudException
  {
    AdminVdcType adminVdcType = (AdminVdcType)getResource();
    adminVdcType.setUsesFastProvisioning(Boolean.valueOf(false));
    return updateAdminVdc(adminVdcType);
  }

  private static void executeVdcAction(VcloudClient client, String orgActionUrl, String content, String contentType, int statusCode)
    throws VCloudException
  {
    SdkUtil.post(client, orgActionUrl, content, contentType, statusCode);
  }

  public ReferenceType getAdminOrganizationReference()
    throws VCloudException
  {
    if (this.adminOrgReference != null)
      return this.adminOrgReference;
    throw new VCloudException(SdkUtil.getI18nString(SdkMessage.REFERENCE_NOT_FOUND_MSG));
  }

  public ReferenceType getVdcReference()
    throws VCloudException
  {
    if (this.vdcReference != null)
      return this.vdcReference;
    throw new VCloudException(SdkUtil.getI18nString(SdkMessage.REFERENCE_NOT_FOUND_MSG));
  }

  public VimObjectRefType getResourcePoolVimRef()
    throws VCloudException
  {
    if (this.primaryResourcePoolVimRef != null) {
      return this.primaryResourcePoolVimRef;
    }
    throw new VCloudException(SdkUtil.getI18nString(SdkMessage.VIM_OBJECT_REF_NOT_FOUND_MSG));
  }

  public HashMap<String, VimObjectRefType> getResourcePoolVimRefsByMoref()
    throws VCloudException
  {
    if (this.orgVdcResourcePoolsRef == null) {
      throw new VCloudException(SdkUtil.getI18nString(SdkMessage.REFERENCE_NOT_FOUND_MSG));
    }

    OrganizationResourcePoolSetType organizationResourcePoolSet = (OrganizationResourcePoolSetType)SdkUtil.get(getVcloudClient(), this.orgVdcResourcePoolsRef.getHref(), 200);

    HashMap orgVdcResourcePoolVimRefsByMoref = new HashMap();
    for (VimObjectRefType resourcePoolVimObjectRef : organizationResourcePoolSet.getResourcePoolVimObjectRef())
    {
      orgVdcResourcePoolVimRefsByMoref.put(resourcePoolVimObjectRef.getMoRef(), resourcePoolVimObjectRef);
    }

    return orgVdcResourcePoolVimRefsByMoref;
  }

  private void sortEntityAndNetworkRefs_v1_5()
  {
    this.vappTemplateRefs = new ArrayList();
    this.mediaRefs = new ArrayList();
    this.vappRefsByName = new HashMap();
    this.orgNetworkRefsByName = new HashMap();
    this.storageProfileRefs = new ArrayList();
    this.diskRefs = new ArrayList();

    if (((AdminVdcType)getResource()).getResourceEntities() != null) {
      ResourceEntitiesType resEntities = ((AdminVdcType)getResource()).getResourceEntities();

      List entities = resEntities.getResourceEntity();

      if (entities != null) {
        for (ResourceReferenceType entity : entities) {
          if (entity.getType().equals("application/vnd.vmware.vcloud.vAppTemplate+xml"))
          {
            this.vappTemplateRefs.add(entity);
          } else if (entity.getType().equals("application/vnd.vmware.vcloud.media+xml"))
          {
            this.mediaRefs.add(entity);
          } else if (entity.getType().equals("application/vnd.vmware.vcloud.vApp+xml"))
          {
            this.vappRefsByName.put(entity.getName(), entity);
          } else if (entity.getType().equals("application/vnd.vmware.vcloud.disk+xml"))
          {
            this.diskRefs.add(entity);
          }
          else logger.log(Level.WARNING, SdkUtil.getI18nString(SdkMessage.UNKNOWN_REF_TYPE_MSG), entity.getType());

        }

      }

    }

    if (((AdminVdcType)getResource()).getAvailableNetworks() != null) {
      AvailableNetworksType availableNetworksType = ((AdminVdcType)getResource()).getAvailableNetworks();

      if (availableNetworksType != null) {
        for (ReferenceType networkRef : availableNetworksType.getNetwork())
        {
          this.orgNetworkRefsByName.put(networkRef.getName(), networkRef);
        }
      }
    }
    for (LinkType adminVdcLink : ((AdminVdcType)getResource()).getLink()) {
      if ((adminVdcLink.getRel().equals("up")) && (adminVdcLink.getType().equals("application/vnd.vmware.admin.organization+xml")))
      {
        this.adminOrgReference = adminVdcLink;
      }if ((adminVdcLink.getRel().equals("alternate")) && (adminVdcLink.getType().equals("application/vnd.vmware.vcloud.vdc+xml")))
      {
        this.vdcReference = adminVdcLink;
      }if ((adminVdcLink.getRel().equals("down")) && (adminVdcLink.getType().equals("application/vnd.vmware.admin.OrganizationVdcResourcePoolSet+xml")))
      {
        this.orgVdcResourcePoolsRef = adminVdcLink;
      }
    }

    for (VCloudExtensionType vcloudExtension : ((AdminVdcType)getResource()).getVCloudExtension())
    {
      for (i$ = vcloudExtension.getAny().iterator(); i$.hasNext(); ) { Object jaxbObject = i$.next();
        if (((((JAXBElement)jaxbObject).getValue() instanceof VimObjectRefType)) && 
          (((VimObjectRefType)((JAXBElement)jaxbObject).getValue()).getVimObjectType().equals(VimObjectTypeEnum.RESOURCE_POOL.value())))
        {
          this.primaryResourcePoolVimRef = ((VimObjectRefType)((JAXBElement)jaxbObject).getValue());
        }
      }
    }
    Iterator i$;
    if (((AdminVdcType)getResource()).getVdcStorageProfiles() != null)
      for (ReferenceType ref : ((AdminVdcType)getResource()).getVdcStorageProfiles().getVdcStorageProfile())
      {
        this.storageProfileRefs.add(ref);
      }
  }

  /** @deprecated */
  public HashMap<String, ReferenceType> getAdminOrgNetworkRefsByName()
  {
    return this.orgNetworkRefsByName;
  }

  /** @deprecated */
  public Collection<ReferenceType> getAdminOrgNetworkRefs()
  {
    return this.orgNetworkRefsByName.values();
  }

  /** @deprecated */
  public ReferenceType getAdminOrgNetworkRefByName(String name)
  {
    return (ReferenceType)this.orgNetworkRefsByName.get(name);
  }

  public Collection<ReferenceType> getVappTemplateRefs()
  {
    return this.vappTemplateRefs;
  }

  public Collection<ReferenceType> getMediaRefs()
  {
    return this.mediaRefs;
  }

  public HashMap<String, ReferenceType> getVappRefsByName()
  {
    return this.vappRefsByName;
  }

  public Collection<ReferenceType> getVappRefs()
  {
    return this.vappRefsByName.values();
  }

  public ReferenceType getVappRefByName(String name)
  {
    return (ReferenceType)this.vappRefsByName.get(name);
  }

  public ReferenceType getProviderVdcRef()
  {
    return ((AdminVdcType)getResource()).getProviderVdcReference();
  }

  public ReferenceType getNetworkPoolRef()
    throws VCloudException
  {
    if (((AdminVdcType)getResource()).getNetworkPoolReference() != null) {
      return ((AdminVdcType)getResource()).getNetworkPoolReference();
    }
    throw new VCloudException(SdkUtil.getI18nString(SdkMessage.REFERENCE_NOT_FOUND_MSG));
  }

  public Collection<ReferenceType> getAdminVdcStorageProfileRefs()
  {
    return this.storageProfileRefs;
  }

  public Collection<ReferenceType> getDiskRefs()
  {
    return this.diskRefs;
  }

  public Task updateAdminVdcStorageProfiles(List<VdcStorageProfileParamsType> addStorageProfiles, List<ReferenceType> removeStorageProfiles)
    throws VCloudException
  {
    ObjectFactory of = new ObjectFactory();
    UpdateVdcStorageProfilesType updateStorageProfilesType = new UpdateVdcStorageProfilesType();
    updateStorageProfilesType.getAddStorageProfile().addAll(addStorageProfiles);
    updateStorageProfilesType.getRemoveStorageProfile().addAll(removeStorageProfiles);

    String updateVdcStprageProfilesUrl = getReference().getHref() + "/vdcStorageProfiles";

    String adminVdcUpdateRequestXmlString = JAXBUtil.marshal(of.createUpdateVdcStorageProfiles(updateStorageProfilesType));

    logger.log(Level.INFO, SdkUtil.getI18nString(SdkMessage.POST_URL_MSG) + " - " + updateVdcStprageProfilesUrl);

    return new Task(getVcloudClient(), (TaskType)SdkUtil.post(getVcloudClient(), updateVdcStprageProfilesUrl, adminVdcUpdateRequestXmlString, "application/vnd.vmware.admin.updateVdcStorageProfiles+xml", 202));
  }

  public EdgeGateway createEdgeGateway(GatewayType gatewayParams)
    throws VCloudException
  {
    String requestXml = JAXBUtil.marshal(new ObjectFactory().createEdgeGateway(gatewayParams));

    return new EdgeGateway(getVcloudClient(), (GatewayType)SdkUtil.post(getVcloudClient(), ((AdminVdcType)getResource()).getHref() + "/edgeGateways", requestXml, "application/vnd.vmware.admin.edgeGateway+xml", 201));
  }

  public ReferenceResult getEdgeGatewayRefs()
    throws VCloudException
  {
    String query = getReference().getHref() + "/edgeGateways" + "?" + getVcloudClient().getQueryService().buildQuery(FormatType.REFERENCE_VIEW);

    return (ReferenceResult)getVcloudClient().getQueryService().executeQuery(query);
  }

  public ReferenceResult getOrgVdcNetworkRefs()
    throws VCloudException
  {
    String query = getReference().getHref() + "/networks" + "?" + getVcloudClient().getQueryService().buildQuery(FormatType.REFERENCE_VIEW);

    return (ReferenceResult)getVcloudClient().getQueryService().executeQuery(query);
  }

  public AdminOrgVdcNetwork createOrgVdcNetwork(OrgVdcNetworkType orgVdcNetworkParams)
    throws VCloudException
  {
    String requestXml = JAXBUtil.marshal(new ObjectFactory().createOrgVdcNetwork(orgVdcNetworkParams));

    return new AdminOrgVdcNetwork(getVcloudClient(), (OrgVdcNetworkType)SdkUtil.post(getVcloudClient(), ((AdminVdcType)getResource()).getHref() + "/networks", requestXml, "application/vnd.vmware.vcloud.orgVdcNetwork+xml", 201));
  }
}