package com.vmware.vcloud.sdk.admin;

import com.vmware.vcloud.api.rest.schema.ExternalNetworkType;
import com.vmware.vcloud.api.rest.schema.ReferenceType;
import com.vmware.vcloud.api.rest.schema.TaskType;
import com.vmware.vcloud.sdk.SdkUtil;
import com.vmware.vcloud.sdk.Task;
import com.vmware.vcloud.sdk.VCloudException;
import com.vmware.vcloud.sdk.VcloudClient;
import com.vmware.vcloud.sdk.VcloudEntity;
import com.vmware.vcloud.sdk.constants.SdkMessage;
import java.util.logging.Level;
import java.util.logging.Logger;

public class ExternalNetwork extends VcloudEntity<ExternalNetworkType>
{
  private static Logger logger = Logger.getLogger("com.vmware.vcloud.sdk");

  ExternalNetwork(VcloudClient client, ExternalNetworkType externalNetworkType_1_5)
  {
    super(client, externalNetworkType_1_5);
  }

  public static ExternalNetwork getExternalNetworkByReference(VcloudClient client, ReferenceType externalNetworkRef)
    throws VCloudException
  {
    logger.log(Level.INFO, SdkUtil.getI18nString(SdkMessage.GET_URL_MSG) + " - " + externalNetworkRef.getHref());

    return new ExternalNetwork(client, (ExternalNetworkType)getResourceByReference(client, externalNetworkRef));
  }

  public static ExternalNetwork getExternalNetworkById(VcloudClient client, String vCloudId)
    throws VCloudException
  {
    logger.log(Level.INFO, SdkUtil.getI18nString(SdkMessage.GET_VCLOUD_ID_MSG) + " - " + vCloudId);

    return new ExternalNetwork(client, (ExternalNetworkType)getEntityById(client, vCloudId, "application/vnd.vmware.admin.network+xml"));
  }

  /** @deprecated */
  public Task reset()
    throws VCloudException
  {
    String taskCanelActionUrl = getReference().getHref() + "/action/reset";

    return executeAction(getVcloudClient(), taskCanelActionUrl, null, null, 202);
  }

  /** @deprecated */
  public static Task reset(VcloudClient client, ReferenceType externalNetworkReference)
    throws VCloudException
  {
    return executeAction(client, externalNetworkReference.getHref() + "/action/reset", null, null, 202);
  }

  private static Task executeAction(VcloudClient client, String externalNetworkActionUrl, String content, String contentType, int statusCode)
    throws VCloudException
  {
    return new Task(client, (TaskType)SdkUtil.post(client, externalNetworkActionUrl, content, contentType, statusCode));
  }
}