package com.vmware.vcloud.sdk.admin.extensions;

import com.vmware.vcloud.api.rest.schema.QueryResultAdminOrgNetworkRecordType;
import com.vmware.vcloud.api.rest.schema.QueryResultAdminVAppRecordType;
import com.vmware.vcloud.api.rest.schema.QueryResultAdminVdcRecordType;
import com.vmware.vcloud.api.rest.schema.QueryResultDatastoreRecordType;
import com.vmware.vcloud.api.rest.schema.QueryResultHostRecordType;
import com.vmware.vcloud.api.rest.schema.QueryResultNetworkPoolRecordType;
import com.vmware.vcloud.api.rest.schema.QueryResultNetworkRecordType;
import com.vmware.vcloud.api.rest.schema.QueryResultVMWProviderVdcRecordType;
import com.vmware.vcloud.api.rest.schema.QueryResultVirtualCenterRecordType;
import com.vmware.vcloud.sdk.QueryParams;
import com.vmware.vcloud.sdk.RecordResult;
import com.vmware.vcloud.sdk.ReferenceResult;
import com.vmware.vcloud.sdk.VCloudException;
import com.vmware.vcloud.sdk.VcloudClient;
import com.vmware.vcloud.sdk.admin.AdminQueryService;
import com.vmware.vcloud.sdk.constants.query.FormatType;
import com.vmware.vcloud.sdk.constants.query.QueryAdminOrgNetworkField;
import com.vmware.vcloud.sdk.constants.query.QueryAdminVAppField;
import com.vmware.vcloud.sdk.constants.query.QueryAdminVdcField;
import com.vmware.vcloud.sdk.constants.query.QueryDatastoreField;
import com.vmware.vcloud.sdk.constants.query.QueryHostField;
import com.vmware.vcloud.sdk.constants.query.QueryNetworkField;
import com.vmware.vcloud.sdk.constants.query.QueryNetworkPoolField;
import com.vmware.vcloud.sdk.constants.query.QueryVMWProviderVdcField;
import com.vmware.vcloud.sdk.constants.query.QueryVirtualCenterField;

public class ExtensionQueryService extends AdminQueryService
{
  ExtensionQueryService(VcloudClient client)
  {
    super(client);
  }

  public ReferenceResult queryHostReferences()
    throws VCloudException
  {
    String query = this.client.getVcloudApiURL() + "/admin/extension/hostReferences/query" + "?" + buildQuery(FormatType.REFERENCE_VIEW);

    return (ReferenceResult)executeQuery(query);
  }

  public RecordResult<QueryResultHostRecordType> queryHostRecords()
    throws VCloudException
  {
    String query = this.client.getVcloudApiURL() + "/admin/extension/hostReferences/query" + "?" + buildQuery(FormatType.RECORD_VIEW);

    return (RecordResult)executeQuery(query);
  }

  public RecordResult<QueryResultHostRecordType> queryHostRecords(QueryParams<QueryHostField> queryParams)
    throws VCloudException
  {
    String query = this.client.getVcloudApiURL() + "/admin/extension/hostReferences/query" + "?" + buildQuery(queryParams) + buildQuery(FormatType.RECORD_VIEW);

    return (RecordResult)executeQuery(query);
  }

  public RecordResult<QueryResultHostRecordType> queryHostIdRecords()
    throws VCloudException
  {
    String query = this.client.getVcloudApiURL() + "/admin/extension/hostReferences/query" + "?" + buildQuery(FormatType.ID_RECORD_VIEW);

    return (RecordResult)executeQuery(query);
  }

  public RecordResult<QueryResultHostRecordType> queryHostIdRecords(QueryParams<QueryHostField> queryParams)
    throws VCloudException
  {
    String query = this.client.getVcloudApiURL() + "/admin/extension/hostReferences/query" + "?" + buildQuery(queryParams) + buildQuery(FormatType.ID_RECORD_VIEW);

    return (RecordResult)executeQuery(query);
  }

  public ReferenceResult queryHostReferences(QueryParams<QueryHostField> queryParams)
    throws VCloudException
  {
    String query = this.client.getVcloudApiURL() + "/admin/extension/hostReferences/query" + "?" + buildQuery(queryParams) + buildQuery(FormatType.REFERENCE_VIEW);

    return (ReferenceResult)executeQuery(query);
  }

  public ReferenceResult queryDatastoreReferences()
    throws VCloudException
  {
    String query = this.client.getVcloudApiURL() + "/admin/extension/datastores/query" + "?" + buildQuery(FormatType.REFERENCE_VIEW);

    return (ReferenceResult)executeQuery(query);
  }

  public RecordResult<QueryResultDatastoreRecordType> queryDatastoreRecords()
    throws VCloudException
  {
    String query = this.client.getVcloudApiURL() + "/admin/extension/datastores/query" + "?" + buildQuery(FormatType.RECORD_VIEW);

    return (RecordResult)executeQuery(query);
  }

  public RecordResult<QueryResultDatastoreRecordType> queryDatastoreRecords(QueryParams<QueryDatastoreField> queryParams)
    throws VCloudException
  {
    String query = this.client.getVcloudApiURL() + "/admin/extension/datastores/query" + "?" + buildQuery(queryParams) + buildQuery(FormatType.RECORD_VIEW);

    return (RecordResult)executeQuery(query);
  }

  public RecordResult<QueryResultDatastoreRecordType> queryDatastoreIdRecords()
    throws VCloudException
  {
    String query = this.client.getVcloudApiURL() + "/admin/extension/datastores/query" + "?" + buildQuery(FormatType.ID_RECORD_VIEW);

    return (RecordResult)executeQuery(query);
  }

  public RecordResult<QueryResultDatastoreRecordType> queryDatastoreIdRecords(QueryParams<QueryDatastoreField> queryParams)
    throws VCloudException
  {
    String query = this.client.getVcloudApiURL() + "/admin/extension/datastores/query" + "?" + buildQuery(queryParams) + buildQuery(FormatType.ID_RECORD_VIEW);

    return (RecordResult)executeQuery(query);
  }

  public ReferenceResult queryDatastoreReferences(QueryParams<QueryDatastoreField> queryParams)
    throws VCloudException
  {
    String query = this.client.getVcloudApiURL() + "/admin/extension/datastores/query" + "?" + buildQuery(queryParams) + buildQuery(FormatType.REFERENCE_VIEW);

    return (ReferenceResult)executeQuery(query);
  }

  public ReferenceResult queryExternalNetworkReferences()
    throws VCloudException
  {
    String query = this.client.getVcloudApiURL() + "/admin/extension/externalNetworkReferences/query" + "?" + buildQuery(FormatType.REFERENCE_VIEW);

    return (ReferenceResult)executeQuery(query);
  }

  public RecordResult<QueryResultNetworkRecordType> queryExternalNetworkRecords()
    throws VCloudException
  {
    String query = this.client.getVcloudApiURL() + "/admin/extension/externalNetworkReferences/query" + "?" + buildQuery(FormatType.RECORD_VIEW);

    return (RecordResult)executeQuery(query);
  }

  public RecordResult<QueryResultNetworkRecordType> queryExternalNetworkRecords(QueryParams<QueryNetworkField> queryParams)
    throws VCloudException
  {
    String query = this.client.getVcloudApiURL() + "/admin/extension/externalNetworkReferences/query" + "?" + buildQuery(queryParams) + buildQuery(FormatType.RECORD_VIEW);

    return (RecordResult)executeQuery(query);
  }

  public RecordResult<QueryResultNetworkRecordType> queryExternalNetworkIdRecords()
    throws VCloudException
  {
    String query = this.client.getVcloudApiURL() + "/admin/extension/externalNetworkReferences/query" + "?" + buildQuery(FormatType.ID_RECORD_VIEW);

    return (RecordResult)executeQuery(query);
  }

  public RecordResult<QueryResultNetworkRecordType> queryExternalNetworkIdRecords(QueryParams<QueryNetworkField> queryParams)
    throws VCloudException
  {
    String query = this.client.getVcloudApiURL() + "/admin/extension/externalNetworkReferences/query" + "?" + buildQuery(queryParams) + buildQuery(FormatType.ID_RECORD_VIEW);

    return (RecordResult)executeQuery(query);
  }

  public ReferenceResult queryExternalNetworkReferences(QueryParams<QueryNetworkField> queryParams)
    throws VCloudException
  {
    String query = this.client.getVcloudApiURL() + "/admin/extension/externalNetworkReferences/query" + "?" + buildQuery(queryParams) + buildQuery(FormatType.REFERENCE_VIEW);

    return (ReferenceResult)executeQuery(query);
  }

  public ReferenceResult queryNetworkPoolReferences()
    throws VCloudException
  {
    String query = this.client.getVcloudApiURL() + "/admin/extension/networkPoolReferences/query" + "?" + buildQuery(FormatType.REFERENCE_VIEW);

    return (ReferenceResult)executeQuery(query);
  }

  public RecordResult<QueryResultNetworkPoolRecordType> queryNetworkPoolRecords()
    throws VCloudException
  {
    String query = this.client.getVcloudApiURL() + "/admin/extension/networkPoolReferences/query" + "?" + buildQuery(FormatType.RECORD_VIEW);

    return (RecordResult)executeQuery(query);
  }

  public RecordResult<QueryResultNetworkPoolRecordType> queryNetworkPoolRecords(QueryParams<QueryNetworkPoolField> queryParams)
    throws VCloudException
  {
    String query = this.client.getVcloudApiURL() + "/admin/extension/networkPoolReferences/query" + "?" + buildQuery(queryParams) + buildQuery(FormatType.RECORD_VIEW);

    return (RecordResult)executeQuery(query);
  }

  public RecordResult<QueryResultNetworkPoolRecordType> queryNetworkPoolIdRecords()
    throws VCloudException
  {
    String query = this.client.getVcloudApiURL() + "/admin/extension/networkPoolReferences/query" + "?" + buildQuery(FormatType.ID_RECORD_VIEW);

    return (RecordResult)executeQuery(query);
  }

  public RecordResult<QueryResultNetworkPoolRecordType> queryNetworkPoolIdRecords(QueryParams<QueryNetworkPoolField> queryParams)
    throws VCloudException
  {
    String query = this.client.getVcloudApiURL() + "/admin/extension/networkPoolReferences/query" + "?" + buildQuery(queryParams) + buildQuery(FormatType.ID_RECORD_VIEW);

    return (RecordResult)executeQuery(query);
  }

  public ReferenceResult queryNetworkPoolReferences(QueryParams<QueryNetworkPoolField> queryParams)
    throws VCloudException
  {
    String query = this.client.getVcloudApiURL() + "/admin/extension/networkPoolReferences/query" + "?" + buildQuery(queryParams) + buildQuery(FormatType.REFERENCE_VIEW);

    return (ReferenceResult)executeQuery(query);
  }

  public ReferenceResult queryProviderVdcReferences()
    throws VCloudException
  {
    String query = this.client.getVcloudApiURL() + "/admin/extension/providerVdcReferences/query" + "?" + buildQuery(FormatType.REFERENCE_VIEW);

    return (ReferenceResult)executeQuery(query);
  }

  public RecordResult<QueryResultVMWProviderVdcRecordType> queryProviderVdcRecords()
    throws VCloudException
  {
    String query = this.client.getVcloudApiURL() + "/admin/extension/providerVdcReferences/query" + "?" + buildQuery(FormatType.RECORD_VIEW);

    return (RecordResult)executeQuery(query);
  }

  public RecordResult<QueryResultVMWProviderVdcRecordType> queryProviderVdcRecords(QueryParams<QueryVMWProviderVdcField> queryParams)
    throws VCloudException
  {
    String query = this.client.getVcloudApiURL() + "/admin/extension/providerVdcReferences/query" + "?" + buildQuery(queryParams) + buildQuery(FormatType.RECORD_VIEW);

    return (RecordResult)executeQuery(query);
  }

  public RecordResult<QueryResultVMWProviderVdcRecordType> queryProviderVdcIdRecords()
    throws VCloudException
  {
    String query = this.client.getVcloudApiURL() + "/admin/extension/providerVdcReferences/query" + "?" + buildQuery(FormatType.ID_RECORD_VIEW);

    return (RecordResult)executeQuery(query);
  }

  public RecordResult<QueryResultVMWProviderVdcRecordType> queryProviderVdcIdRecords(QueryParams<QueryVMWProviderVdcField> queryParams)
    throws VCloudException
  {
    String query = this.client.getVcloudApiURL() + "/admin/extension/providerVdcReferences/query" + "?" + buildQuery(queryParams) + buildQuery(FormatType.ID_RECORD_VIEW);

    return (RecordResult)executeQuery(query);
  }

  public ReferenceResult queryProviderVdcReferences(QueryParams<QueryVMWProviderVdcField> queryParams)
    throws VCloudException
  {
    String query = this.client.getVcloudApiURL() + "/admin/extension/providerVdcReferences/query" + "?" + buildQuery(queryParams) + buildQuery(FormatType.REFERENCE_VIEW);

    return (ReferenceResult)executeQuery(query);
  }

  public ReferenceResult queryVimServerReferences()
    throws VCloudException
  {
    String query = this.client.getVcloudApiURL() + "/admin/extension/vimServerReferences/query" + "?" + buildQuery(FormatType.REFERENCE_VIEW);

    return (ReferenceResult)executeQuery(query);
  }

  public RecordResult<QueryResultVirtualCenterRecordType> queryVimServerRecords()
    throws VCloudException
  {
    String query = this.client.getVcloudApiURL() + "/admin/extension/vimServerReferences/query" + "?" + buildQuery(FormatType.RECORD_VIEW);

    return (RecordResult)executeQuery(query);
  }

  public RecordResult<QueryResultVirtualCenterRecordType> queryVimServerRecords(QueryParams<QueryVirtualCenterField> queryParams)
    throws VCloudException
  {
    String query = this.client.getVcloudApiURL() + "/admin/extension/vimServerReferences/query" + "?" + buildQuery(queryParams) + buildQuery(FormatType.RECORD_VIEW);

    return (RecordResult)executeQuery(query);
  }

  public RecordResult<QueryResultVirtualCenterRecordType> queryVimServerIdRecords()
    throws VCloudException
  {
    String query = this.client.getVcloudApiURL() + "/admin/extension/vimServerReferences/query" + "?" + buildQuery(FormatType.ID_RECORD_VIEW);

    return (RecordResult)executeQuery(query);
  }

  public RecordResult<QueryResultVirtualCenterRecordType> queryVimServerIdRecords(QueryParams<QueryVirtualCenterField> queryParams)
    throws VCloudException
  {
    String query = this.client.getVcloudApiURL() + "/admin/extension/vimServerReferences/query" + "?" + buildQuery(queryParams) + buildQuery(FormatType.ID_RECORD_VIEW);

    return (RecordResult)executeQuery(query);
  }

  public ReferenceResult queryVimServerReferences(QueryParams<QueryVirtualCenterField> queryParams)
    throws VCloudException
  {
    String query = this.client.getVcloudApiURL() + "/admin/extension/vimServerReferences/query" + "?" + buildQuery(queryParams) + buildQuery(FormatType.REFERENCE_VIEW);

    return (ReferenceResult)executeQuery(query);
  }

  /** @deprecated */
  public ReferenceResult queryAllOrgNetworkReferences()
    throws VCloudException
  {
    String query = this.client.getVcloudApiURL() + "/admin/extension/orgNetworks/query" + "?" + buildQuery(FormatType.REFERENCE_VIEW);

    return (ReferenceResult)executeQuery(query);
  }

  /** @deprecated */
  public RecordResult<QueryResultAdminOrgNetworkRecordType> queryAllOrgNetworkRecords()
    throws VCloudException
  {
    String query = this.client.getVcloudApiURL() + "/admin/extension/orgNetworks/query" + "?" + buildQuery(FormatType.RECORD_VIEW);

    return (RecordResult)executeQuery(query);
  }

  /** @deprecated */
  public RecordResult<QueryResultAdminOrgNetworkRecordType> queryAllOrgNetworkRecords(QueryParams<QueryAdminOrgNetworkField> queryParams)
    throws VCloudException
  {
    String query = this.client.getVcloudApiURL() + "/admin/extension/orgNetworks/query" + "?" + buildQuery(queryParams) + buildQuery(FormatType.RECORD_VIEW);

    return (RecordResult)executeQuery(query);
  }

  /** @deprecated */
  public RecordResult<QueryResultAdminOrgNetworkRecordType> queryAllOrgNetworkIdRecords()
    throws VCloudException
  {
    String query = this.client.getVcloudApiURL() + "/admin/extension/orgNetworks/query" + "?" + buildQuery(FormatType.ID_RECORD_VIEW);

    return (RecordResult)executeQuery(query);
  }

  /** @deprecated */
  public RecordResult<QueryResultAdminOrgNetworkRecordType> queryAllOrgNetworkIdRecords(QueryParams<QueryAdminOrgNetworkField> queryParams)
    throws VCloudException
  {
    String query = this.client.getVcloudApiURL() + "/admin/extension/orgNetworks/query" + "?" + buildQuery(queryParams) + buildQuery(FormatType.ID_RECORD_VIEW);

    return (RecordResult)executeQuery(query);
  }

  /** @deprecated */
  public ReferenceResult queryAllOrgNetworkReferences(QueryParams<QueryAdminOrgNetworkField> queryParams)
    throws VCloudException
  {
    String query = this.client.getVcloudApiURL() + "/admin/extension/orgNetworks/query" + "?" + buildQuery(queryParams) + buildQuery(FormatType.REFERENCE_VIEW);

    return (ReferenceResult)executeQuery(query);
  }

  public ReferenceResult queryAllOrgVdcReferences()
    throws VCloudException
  {
    String query = this.client.getVcloudApiURL() + "/admin/extension/orgVdcs/query" + "?" + buildQuery(FormatType.REFERENCE_VIEW);

    return (ReferenceResult)executeQuery(query);
  }

  public RecordResult<QueryResultAdminVdcRecordType> queryAllOrgVdcRecords()
    throws VCloudException
  {
    String query = this.client.getVcloudApiURL() + "/admin/extension/orgVdcs/query" + "?" + buildQuery(FormatType.RECORD_VIEW);

    return (RecordResult)executeQuery(query);
  }

  public RecordResult<QueryResultAdminVdcRecordType> queryAllOrgVdcRecords(QueryParams<QueryAdminVdcField> queryParams)
    throws VCloudException
  {
    String query = this.client.getVcloudApiURL() + "/admin/extension/orgVdcs/query" + "?" + buildQuery(queryParams) + buildQuery(FormatType.RECORD_VIEW);

    return (RecordResult)executeQuery(query);
  }

  public RecordResult<QueryResultAdminVdcRecordType> queryAllOrgVdcIdRecords()
    throws VCloudException
  {
    String query = this.client.getVcloudApiURL() + "/admin/extension/orgVdcs/query" + "?" + buildQuery(FormatType.ID_RECORD_VIEW);

    return (RecordResult)executeQuery(query);
  }

  public RecordResult<QueryResultAdminVdcRecordType> queryAllOrgVdcIdRecords(QueryParams<QueryAdminVdcField> queryParams)
    throws VCloudException
  {
    String query = this.client.getVcloudApiURL() + "/admin/extension/orgVdcs/query" + "?" + buildQuery(queryParams) + buildQuery(FormatType.ID_RECORD_VIEW);

    return (RecordResult)executeQuery(query);
  }

  public ReferenceResult queryAllOrgVdcReferences(QueryParams<QueryAdminVdcField> queryParams)
    throws VCloudException
  {
    String query = this.client.getVcloudApiURL() + "/admin/extension/orgVdcs/query" + "?" + buildQuery(queryParams) + buildQuery(FormatType.REFERENCE_VIEW);

    return (ReferenceResult)executeQuery(query);
  }

  public ReferenceResult queryAllVappReferences()
    throws VCloudException
  {
    String query = this.client.getVcloudApiURL() + "/admin/extension/vapps/query" + "?" + buildQuery(FormatType.REFERENCE_VIEW);

    return (ReferenceResult)executeQuery(query);
  }

  public RecordResult<QueryResultAdminVAppRecordType> queryAllVappRecords()
    throws VCloudException
  {
    String query = this.client.getVcloudApiURL() + "/admin/extension/vapps/query" + "?" + buildQuery(FormatType.RECORD_VIEW);

    return (RecordResult)executeQuery(query);
  }

  public RecordResult<QueryResultAdminVAppRecordType> queryAllVappRecords(QueryParams<QueryAdminVAppField> queryParams)
    throws VCloudException
  {
    String query = this.client.getVcloudApiURL() + "/admin/extension/vapps/query" + "?" + buildQuery(queryParams) + buildQuery(FormatType.RECORD_VIEW);

    return (RecordResult)executeQuery(query);
  }

  public RecordResult<QueryResultAdminVAppRecordType> queryAllVappIdRecords()
    throws VCloudException
  {
    String query = this.client.getVcloudApiURL() + "/admin/extension/vapps/query" + "?" + buildQuery(FormatType.ID_RECORD_VIEW);

    return (RecordResult)executeQuery(query);
  }

  public RecordResult<QueryResultAdminVAppRecordType> queryAllVappIdRecords(QueryParams<QueryAdminVAppField> queryParams)
    throws VCloudException
  {
    String query = this.client.getVcloudApiURL() + "/admin/extension/vapps/query" + "?" + buildQuery(queryParams) + buildQuery(FormatType.ID_RECORD_VIEW);

    return (RecordResult)executeQuery(query);
  }

  public ReferenceResult queryAllVappReferences(QueryParams<QueryAdminVAppField> queryParams)
    throws VCloudException
  {
    String query = this.client.getVcloudApiURL() + "/admin/extension/vapps/query" + "?" + buildQuery(queryParams) + buildQuery(FormatType.REFERENCE_VIEW);

    return (ReferenceResult)executeQuery(query);
  }
}