package com.vmware.vcloud.sdk.admin.extensions.service;

import com.vmware.vcloud.api.rest.schema.LinkType;
import com.vmware.vcloud.api.rest.schema.ReferenceType;
import com.vmware.vcloud.api.rest.schema.extension.ObjectFactory;
import com.vmware.vcloud.api.rest.schema.extension.ResourceClassActionType;
import com.vmware.vcloud.api.rest.schema.extension.ResourceClassType;
import com.vmware.vcloud.api.rest.schema.extension.ServiceResourceType;
import com.vmware.vcloud.sdk.JAXBUtil;
import com.vmware.vcloud.sdk.QueryService;
import com.vmware.vcloud.sdk.ReferenceResult;
import com.vmware.vcloud.sdk.SdkUtil;
import com.vmware.vcloud.sdk.VCloudException;
import com.vmware.vcloud.sdk.VcloudClient;
import com.vmware.vcloud.sdk.VcloudEntity;
import com.vmware.vcloud.sdk.constants.SdkMessage;
import com.vmware.vcloud.sdk.constants.query.FormatType;
import java.util.logging.Level;
import java.util.logging.Logger;

public class AdminResourceClass extends VcloudEntity<ResourceClassType>
{
  private static Logger logger = Logger.getLogger("com.vmware.vcloud.sdk");
  private ReferenceType adminServiceRef;

  AdminResourceClass(VcloudClient client, ResourceClassType resourceClassParams)
  {
    super(client, resourceClassParams);
    sortAdminResourceClassLinkRefs();
  }

  private void sortAdminResourceClassLinkRefs()
  {
    if (getResource() != null)
      for (LinkType link : ((ResourceClassType)getResource()).getLink())
        if ((link.getRel().equals("up")) && (link.getType().equals("application/vnd.vmware.admin.service+xml")))
        {
          this.adminServiceRef = link;
        }
  }

  public static AdminResourceClass getAdminResourceClassByReference(VcloudClient client, ReferenceType reference)
    throws VCloudException
  {
    logger.log(Level.INFO, SdkUtil.getI18nString(SdkMessage.GET_URL_MSG) + " - " + reference.getHref());

    return new AdminResourceClass(client, (ResourceClassType)getResourceByReference(client, reference));
  }

  public static AdminResourceClass getAdminResourceClassById(VcloudClient client, String vCloudId)
    throws VCloudException
  {
    logger.log(Level.INFO, SdkUtil.getI18nString(SdkMessage.GET_VCLOUD_ID_MSG) + " - " + vCloudId);

    return new AdminResourceClass(client, (ResourceClassType)getEntityById(client, vCloudId, "application/vnd.vmware.admin.resourceClass+xml"));
  }

  public void delete()
    throws VCloudException
  {
    SdkUtil.delete(getVcloudClient(), getReference().getHref(), 204);
  }

  public ReferenceType getAdminServiceReference()
    throws VCloudException
  {
    if (this.adminServiceRef != null)
      return this.adminServiceRef;
    throw new VCloudException(SdkUtil.getI18nString(SdkMessage.REFERENCE_NOT_FOUND_MSG));
  }

  public AdminResourceClassAction registerAdminResourceClassAction(ResourceClassActionType resourceClasaActionParams)
    throws VCloudException
  {
    String requestXML = JAXBUtil.marshal(new ObjectFactory().createResourceClassAction(resourceClasaActionParams));

    return new AdminResourceClassAction(getVcloudClient(), (ResourceClassActionType)SdkUtil.post(getVcloudClient(), ((ResourceClassType)getResource()).getHref() + "/resourceclassactions", requestXML, "application/vnd.vmware.admin.resourceClassAction+xml", 201));
  }

  public ReferenceResult getAdminResourceClassActionRefs()
    throws VCloudException
  {
    String query = getReference().getHref() + "/resourceclassactions" + "?" + getVcloudClient().getQueryService().buildQuery(FormatType.REFERENCE_VIEW);

    return (ReferenceResult)getVcloudClient().getQueryService().executeQuery(query);
  }

  public AdminServiceResource registerAdminServiceResource(ServiceResourceType serviceResParams)
    throws VCloudException
  {
    String requestXML = JAXBUtil.marshal(new ObjectFactory().createServiceResource(serviceResParams));

    return new AdminServiceResource(getVcloudClient(), (ServiceResourceType)SdkUtil.post(getVcloudClient(), ((ResourceClassType)getResource()).getHref() + "/serviceresources", requestXML, "application/vnd.vmware.admin.serviceResource+xml", 201));
  }

  public ReferenceResult getAdminServiceResourceRefs()
    throws VCloudException
  {
    String query = getReference().getHref() + "/serviceresources" + "?" + getVcloudClient().getQueryService().buildQuery(FormatType.REFERENCE_VIEW);

    return (ReferenceResult)getVcloudClient().getQueryService().executeQuery(query);
  }
}