package com.vmware.vcloud.sdk.admin.extensions.service;

import com.vmware.vcloud.api.rest.schema.ReferenceType;
import com.vmware.vcloud.api.rest.schema.RightType;
import com.vmware.vcloud.api.rest.schema.extension.AdminApiDefinitionType;
import com.vmware.vcloud.api.rest.schema.extension.AdminServiceLinkType;
import com.vmware.vcloud.api.rest.schema.extension.AdminServiceType;
import com.vmware.vcloud.api.rest.schema.extension.ApiFilterType;
import com.vmware.vcloud.api.rest.schema.extension.AuthorizationCheckParamsType;
import com.vmware.vcloud.api.rest.schema.extension.AuthorizationCheckResponseType;
import com.vmware.vcloud.api.rest.schema.extension.ResourceClassType;
import com.vmware.vcloud.api.rest.schema.extension.RightRefsType;
import com.vmware.vcloud.sdk.JAXBUtil;
import com.vmware.vcloud.sdk.QueryService;
import com.vmware.vcloud.sdk.ReferenceResult;
import com.vmware.vcloud.sdk.SdkUtil;
import com.vmware.vcloud.sdk.VCloudException;
import com.vmware.vcloud.sdk.VcloudClient;
import com.vmware.vcloud.sdk.VcloudEntity;
import com.vmware.vcloud.sdk.admin.Right;
import com.vmware.vcloud.sdk.constants.SdkMessage;
import com.vmware.vcloud.sdk.constants.query.FormatType;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;

public class AdminService extends VcloudEntity<AdminServiceType>
{
  private static Logger logger = Logger.getLogger("com.vmware.vcloud.sdk");

  public AdminService(VcloudClient client, AdminServiceType serviceResource)
  {
    super(client, serviceResource);
  }

  public static AdminService getAdminServiceByReference(VcloudClient client, ReferenceType reference)
    throws VCloudException
  {
    logger.log(Level.INFO, SdkUtil.getI18nString(SdkMessage.GET_URL_MSG) + " - " + reference.getHref());

    return new AdminService(client, (AdminServiceType)getResourceByReference(client, reference));
  }

  public static AdminService getAdminServiceById(VcloudClient client, String vCloudId)
    throws VCloudException
  {
    logger.log(Level.INFO, SdkUtil.getI18nString(SdkMessage.GET_VCLOUD_ID_MSG) + " - " + vCloudId);

    return new AdminService(client, (AdminServiceType)getEntityById(client, vCloudId, "application/vnd.vmware.admin.service+xml"));
  }

  public AdminService updateAdminService(AdminServiceType serviceParams)
    throws VCloudException
  {
    String requestXML = JAXBUtil.marshal(new com.vmware.vcloud.api.rest.schema.extension.ObjectFactory().createService(serviceParams));

    return new AdminService(getVcloudClient(), (AdminServiceType)SdkUtil.put(getVcloudClient(), ((AdminServiceType)getResource()).getHref(), requestXML, "application/vnd.vmware.admin.service+xml", 200));
  }

  public void delete()
    throws VCloudException
  {
    SdkUtil.delete(getVcloudClient(), getReference().getHref(), 204);
  }

  public Right createRight(RightType rightType)
    throws VCloudException
  {
    String requestXML = JAXBUtil.marshal(new com.vmware.vcloud.api.rest.schema.ObjectFactory().createRight(rightType));

    return new Right(getVcloudClient(), (RightType)SdkUtil.post(getVcloudClient(), ((AdminServiceType)getResource()).getHref() + "/rights", requestXML, "application/vnd.vmware.admin.right+xml", 201));
  }

  public AdminServiceLink createAdminLink(AdminServiceLinkType serviceLinkParams)
    throws VCloudException
  {
    String requestXML = JAXBUtil.marshal(new com.vmware.vcloud.api.rest.schema.extension.ObjectFactory().createServiceLink(serviceLinkParams));

    return new AdminServiceLink(getVcloudClient(), (AdminServiceLinkType)SdkUtil.post(getVcloudClient(), ((AdminServiceType)getResource()).getHref() + "/links", requestXML, "application/vnd.vmware.admin.serviceLink+xml", 201));
  }

  public AdminApiDefinition createAdminApiDefinition(AdminApiDefinitionType apiDefinitionParams)
    throws VCloudException
  {
    String requestXML = JAXBUtil.marshal(new com.vmware.vcloud.api.rest.schema.extension.ObjectFactory().createApiDefinition(apiDefinitionParams));

    return new AdminApiDefinition(getVcloudClient(), (AdminApiDefinitionType)SdkUtil.post(getVcloudClient(), ((AdminServiceType)getResource()).getHref() + "/apidefinitions", requestXML, "application/vnd.vmware.admin.apiDefinition+xml", 201));
  }

  public AdminResourceClass registerAdminResourceClass(ResourceClassType resourceClassParams)
    throws VCloudException
  {
    String requestXML = JAXBUtil.marshal(new com.vmware.vcloud.api.rest.schema.extension.ObjectFactory().createResourceClass(resourceClassParams));

    return new AdminResourceClass(getVcloudClient(), (ResourceClassType)SdkUtil.post(getVcloudClient(), ((AdminServiceType)getResource()).getHref() + "/resourceclasses", requestXML, "application/vnd.vmware.admin.resourceClass+xml", 201));
  }

  public AdminApiFilter createAdminApiFilter(ApiFilterType apiFilterParams)
    throws VCloudException
  {
    String requestXML = JAXBUtil.marshal(new com.vmware.vcloud.api.rest.schema.extension.ObjectFactory().createApiFilter(apiFilterParams));

    return new AdminApiFilter(getVcloudClient(), (ApiFilterType)SdkUtil.post(getVcloudClient(), ((AdminServiceType)getResource()).getHref() + "/apifilters", requestXML, "application/vnd.vmware.admin.apiFilter+xml", 201));
  }

  public List<ReferenceType> getRightRefs()
    throws VCloudException
  {
    return ((RightRefsType)SdkUtil.get(getVcloudClient(), ((AdminServiceType)getResource()).getHref() + "/rights", 200)).getRight();
  }

  public ReferenceResult getAdminLinkRefs()
    throws VCloudException
  {
    String query = getReference().getHref() + "/links" + "?" + getVcloudClient().getQueryService().buildQuery(FormatType.REFERENCE_VIEW);

    return (ReferenceResult)getVcloudClient().getQueryService().executeQuery(query);
  }

  public ReferenceResult getAdminApiDefinitionRefs()
    throws VCloudException
  {
    String query = getReference().getHref() + "/apidefinitions" + "?" + getVcloudClient().getQueryService().buildQuery(FormatType.REFERENCE_VIEW);

    return (ReferenceResult)getVcloudClient().getQueryService().executeQuery(query);
  }

  public ReferenceResult getAdminResourceClassRefs()
    throws VCloudException
  {
    String query = getReference().getHref() + "/resourceclasses" + "?" + getVcloudClient().getQueryService().buildQuery(FormatType.REFERENCE_VIEW);

    return (ReferenceResult)getVcloudClient().getQueryService().executeQuery(query);
  }

  public ReferenceResult getAdminApiFilterRefs()
    throws VCloudException
  {
    String query = getReference().getHref() + "/apifilters" + "?" + getVcloudClient().getQueryService().buildQuery(FormatType.REFERENCE_VIEW);

    return (ReferenceResult)getVcloudClient().getQueryService().executeQuery(query);
  }

  public boolean isAuthorized(AuthorizationCheckParamsType authorizationCheckParamsType)
    throws VCloudException
  {
    String checkAuthorizationUri = ((AdminServiceType)getResource()).getHref() + "/authorizationcheck";

    com.vmware.vcloud.api.rest.schema.extension.ObjectFactory objectFactory_v5_1 = new com.vmware.vcloud.api.rest.schema.extension.ObjectFactory();
    String requestBodyXml = JAXBUtil.marshal(objectFactory_v5_1.createAuthorizationCheckParams(authorizationCheckParamsType));
    return ((AuthorizationCheckResponseType)SdkUtil.post(getVcloudClient(), checkAuthorizationUri, requestBodyXml, "application/vnd.vmware.admin.authorizationCheckParams+xml", 200)).isIsAuthorized().booleanValue();
  }
}