package com.vmware.vcloud.sdk_v1_0;

import com.vmware.vcloud.api.rest.schema_v1_0.ErrorType;
import java.io.ByteArrayInputStream;
import java.io.InputStream;
import java.io.UnsupportedEncodingException;
import java.util.logging.Level;
import java.util.logging.Logger;

public class Response
{
  private static Logger logger = Logger.getLogger(RestUtil.class.getSimpleName());
  private String responseXml;
  private int responseStatusCode;

  String getResponseXml()
  {
    return this.responseXml;
  }

  void setResponseXml(String responseXml)
  {
    this.responseXml = responseXml;
  }

  int getResonseStatusCode()
  {
    return this.responseStatusCode;
  }

  void setResonseStatusCode(int resonseStatucCode)
  {
    this.responseStatusCode = resonseStatucCode;
  }

  public boolean isExpected(int expectedStatusCode)
  {
    if (getResonseStatusCode() != expectedStatusCode) {
      logger.log(Level.INFO, "Expected Status Code: " + expectedStatusCode);

      logger.log(Level.INFO, "Actual Status Code: " + getResonseStatusCode());

      return false;
    }
    return true;
  }

  public Object getResource()
    throws VCloudException
  {
    if ((getResponseXml() == null) || (getResponseXml().isEmpty()))
    {
      throw new VCloudException("NO Response received");
    }
    InputStream is = null;
    try {
      is = new ByteArrayInputStream(getResponseXml().getBytes("UTF-8"));
    }
    catch (UnsupportedEncodingException e) {
      logger.log(Level.SEVERE, e.getMessage());
      throw new VCloudRuntimeException(e);
    }
    return JAXBUtil.unmarshallResource(is);
  }

  public void handleUnExpectedResponse()
    throws VCloudException
  {
    if ((getResponseXml() != null) && (!getResponseXml().isEmpty()))
    {
      InputStream is = null;
      try {
        is = new ByteArrayInputStream(getResponseXml().getBytes("UTF-8"));
      }
      catch (UnsupportedEncodingException e) {
        logger.log(Level.SEVERE, e.getMessage());
        throw new VCloudRuntimeException(e);
      }
      ErrorType err = (ErrorType)JAXBUtil.unmarshallResource(is);
      throw new VCloudException(err);
    }

    throw new VCloudException("NO Error Report: " + getResonseStatusCode());
  }
}