package com.vmware.vcloud.sdk_v1_0;

import com.vmware.vcloud.api.rest.schema_v1_0.ErrorType;
import com.vmware.vcloud.api.rest.schema_v1_0.FileType;
import com.vmware.vcloud.api.rest.schema_v1_0.LinkType;
import com.vmware.vcloud.api.rest.schema_v1_0.RelationType;
import java.io.BufferedInputStream;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.UnsupportedEncodingException;
import java.util.Iterator;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;
import org.apache.commons.httpclient.Header;
import org.apache.commons.httpclient.HttpClient;
import org.apache.commons.httpclient.HttpException;
import org.apache.commons.httpclient.HttpMethod;
import org.apache.commons.httpclient.HttpStatus;
import org.apache.commons.httpclient.URI;
import org.apache.commons.httpclient.methods.DeleteMethod;
import org.apache.commons.httpclient.methods.GetMethod;
import org.apache.commons.httpclient.methods.InputStreamRequestEntity;
import org.apache.commons.httpclient.methods.PostMethod;
import org.apache.commons.httpclient.methods.PutMethod;
import org.apache.commons.httpclient.methods.StringRequestEntity;

public class RestUtil
{
  private static Logger logger = Logger.getLogger(RestUtil.class.getSimpleName());

  public static Response delete(VcloudClient client, String url)
    throws VCloudException
  {
    Response response = new Response();
    DeleteMethod deleteMethod = null;
    String resp = null;
    int statusCode = 0;
    try
    {
      deleteMethod = new DeleteMethod(url);
      deleteMethod.setRequestHeader(client.getVCloudAuthenticationHeaderName(), client.getVCloudAuthenticationHeaderValue());

      logger.log(Level.INFO, new StringBuilder().append("delete URL: ").append(url).toString());
      client.getClient().executeMethod(deleteMethod);
      if ((deleteMethod.getStatusCode() == 301) || (deleteMethod.getStatusCode() == 302))
      {
        deleteMethod.releaseConnection();
        String redirectedUrl = redirectURL(deleteMethod);
        deleteMethod.setURI(new URI(redirectedUrl));
        client.getClient().executeMethod(deleteMethod);
      }
      statusCode = deleteMethod.getStatusCode();
      response.setResonseStatusCode(statusCode);
      logger.log(Level.INFO, new StringBuilder().append("status Code: ").append(statusCode).toString());
      logger.log(Level.INFO, new StringBuilder().append("Status Line: ").append(deleteMethod.getStatusLine()).toString());

      logger.log(Level.INFO, new StringBuilder().append("Status: ").append(deleteMethod.getStatusText()).toString());
      resp = getContent(deleteMethod.getResponseBodyAsStream());
      response.setResponseXml(resp);
      logger.log(Level.INFO, new StringBuilder().append("Response: ").append(resp).toString());
    }
    catch (HttpException e) {
      logger.log(Level.SEVERE, e.getMessage());
      throw new VCloudRuntimeException(e);
    } catch (UnsupportedEncodingException e) {
      logger.log(Level.SEVERE, e.getMessage());
      throw new VCloudRuntimeException(e);
    } catch (IOException e) {
      logger.log(Level.SEVERE, e.getMessage());
      throw new VCloudRuntimeException(e);
    } finally {
      if (deleteMethod != null) {
        deleteMethod.releaseConnection();
      }
    }

    return response;
  }

  public static Response put(VcloudClient client, String url, String content, String contentType, String charset)
    throws VCloudException
  {
    Response response = new Response();
    String resp = null;

    logger.log(Level.INFO, new StringBuilder().append("---> Put: ").append(url).toString());
    PutMethod putMethod = null;
    try {
      putMethod = new PutMethod(url);
      putMethod.setRequestHeader(client.getVCloudAuthenticationHeaderName(), client.getVCloudAuthenticationHeaderValue());

      if ((content != null) && (content.length() > 0) && (contentType != null) && (contentType.length() > 0))
      {
        StringRequestEntity req = new StringRequestEntity(content, contentType, charset);

        logger.log(Level.INFO, new StringBuilder().append("PUT Request Body: ").append(content).toString());
        logger.log(Level.INFO, new StringBuilder().append("PUT  Request Content-Type: ").append(contentType).toString());

        logger.log(Level.INFO, new StringBuilder().append("PUT Request charset: ").append(charset).toString());
        putMethod.setRequestEntity(req);
      }
      client.getClient().executeMethod(putMethod);
      if ((putMethod.getStatusCode() == 301) || (putMethod.getStatusCode() == 302))
      {
        putMethod.releaseConnection();
        String redirectedUrl = redirectURL(putMethod);
        putMethod.setURI(new URI(redirectedUrl));
        client.getClient().executeMethod(putMethod);
      }
      int statusCode = putMethod.getStatusCode();
      response.setResonseStatusCode(statusCode);
      logger.log(Level.INFO, new StringBuilder().append("status Code: ").append(statusCode).toString());
      logger.log(Level.INFO, new StringBuilder().append("Status Line: ").append(putMethod.getStatusLine()).toString());
      logger.log(Level.INFO, new StringBuilder().append("Status: ").append(putMethod.getStatusText()).toString());
      resp = getContent(putMethod.getResponseBodyAsStream());
      response.setResponseXml(resp);
      logger.log(Level.INFO, new StringBuilder().append("Response: ").append(resp).toString());
    }
    catch (HttpException e) {
      logger.log(Level.SEVERE, e.getMessage());
      throw new VCloudRuntimeException(e);
    } catch (UnsupportedEncodingException e) {
      logger.log(Level.SEVERE, e.getMessage());
      throw new VCloudRuntimeException(e);
    } catch (IOException e) {
      logger.log(Level.SEVERE, e.getMessage());
      throw new VCloudRuntimeException(e);
    } finally {
      if (putMethod != null) {
        putMethod.releaseConnection();
      }
    }
    return response;
  }

  static String redirectURL(HttpMethod method)
  {
    String redirectedUrl = method.getResponseHeader("Location").getValue();
    logger.log(Level.INFO, new StringBuilder().append("---> Redirected URL: ").append(redirectedUrl).toString());
    return redirectedUrl;
  }

  static Response login(VcloudClient client, String url, String authHeaderValue)
    throws VCloudException
  {
    Response response = new Response();
    String resp = null;

    logger.log(Level.INFO, new StringBuilder().append("---> Login: ").append(url).toString());
    PostMethod postMethod = null;
    try {
      postMethod = new PostMethod(url);
      postMethod.addRequestHeader("Authorization", authHeaderValue);
      client.getClient().executeMethod(postMethod);
      if (postMethod.getStatusCode() == 302) {
        postMethod.releaseConnection();
        String redirectedUrl = redirectURL(postMethod);
        postMethod.setURI(new URI(redirectedUrl));
        client.getClient().executeMethod(postMethod);

        String vCloudAPIURL = redirectedUrl.replace("/login", "");

        client.setVCloudApiURL(vCloudAPIURL);
      }
      if (postMethod.getResponseHeader("x-vcloud-authorization") != null) {
        client.setVCloudAuthenticationHeaderName("x-vcloud-authorization");

        client.setVCloudAuthenticationHeaderValue(postMethod.getResponseHeader("x-vcloud-authorization").getValue());
      }

      int statusCode = postMethod.getStatusCode();
      response.setResonseStatusCode(statusCode);
      logger.log(Level.INFO, new StringBuilder().append("status Code: ").append(statusCode).toString());
      logger.log(Level.INFO, new StringBuilder().append("Status Line: ").append(postMethod.getStatusLine()).toString());

      logger.log(Level.INFO, new StringBuilder().append("Status: ").append(postMethod.getStatusText()).toString());
      resp = getContent(postMethod.getResponseBodyAsStream());
      response.setResponseXml(resp);
      logger.log(Level.INFO, new StringBuilder().append("Response: ").append(resp).toString());
    }
    catch (HttpException e) {
      logger.log(Level.SEVERE, e.getMessage());
      throw new VCloudRuntimeException(e);
    } catch (UnsupportedEncodingException e) {
      logger.log(Level.SEVERE, e.getMessage());
      throw new VCloudRuntimeException(e);
    } catch (IOException e) {
      logger.log(Level.SEVERE, e.getMessage());
      throw new VCloudRuntimeException(e);
    } finally {
      if (postMethod != null) {
        postMethod.releaseConnection();
      }
    }
    return response;
  }

  public static Response post(VcloudClient client, String url, String content, String contentType, String charset)
    throws VCloudException
  {
    Response response = new Response();
    String resp = null;

    logger.log(Level.INFO, new StringBuilder().append("---> Post: ").append(url).toString());
    PostMethod postMethod = null;
    try {
      postMethod = new PostMethod(url);
      postMethod.setRequestHeader(client.getVCloudAuthenticationHeaderName(), client.getVCloudAuthenticationHeaderValue());

      if ((content != null) && (content.length() > 0) && (contentType != null) && (contentType.length() > 0))
      {
        StringRequestEntity req = new StringRequestEntity(content, contentType, charset);

        logger.log(Level.INFO, new StringBuilder().append("POST Request Body: ").append(content).toString());
        logger.log(Level.INFO, new StringBuilder().append("POST Request Content-Type: ").append(contentType).toString());

        logger.log(Level.INFO, new StringBuilder().append("POST Request charset: ").append(charset).toString());
        postMethod.setRequestEntity(req);
      }
      client.getClient().executeMethod(postMethod);
      if ((postMethod.getStatusCode() == 301) || (postMethod.getStatusCode() == 302))
      {
        postMethod.releaseConnection();
        String redirectedUrl = redirectURL(postMethod);
        postMethod.setURI(new URI(redirectedUrl));
        client.getClient().executeMethod(postMethod);
      }
      int statusCode = postMethod.getStatusCode();
      response.setResonseStatusCode(statusCode);
      logger.log(Level.INFO, new StringBuilder().append("status Code: ").append(statusCode).toString());
      logger.log(Level.INFO, new StringBuilder().append("Status Line: ").append(postMethod.getStatusLine()).toString());

      logger.log(Level.INFO, new StringBuilder().append("Status: ").append(postMethod.getStatusText()).toString());
      resp = getContent(postMethod.getResponseBodyAsStream());
      response.setResponseXml(resp);
      logger.log(Level.INFO, new StringBuilder().append("Response: ").append(resp).toString());
    }
    catch (HttpException e) {
      logger.log(Level.SEVERE, e.getMessage());
      throw new VCloudRuntimeException(e);
    } catch (UnsupportedEncodingException e) {
      logger.log(Level.SEVERE, e.getMessage());
      throw new VCloudRuntimeException(e);
    } catch (IOException e) {
      logger.log(Level.SEVERE, e.getMessage());
      throw new VCloudRuntimeException(e);
    } finally {
      if (postMethod != null) {
        postMethod.releaseConnection();
      }
    }
    return response;
  }

  static Response getSupportedVersions(VcloudClient client, String url)
    throws VCloudException
  {
    Response response = new Response();

    String resp = null;
    GetMethod getMethod = null;
    try {
      getMethod = new GetMethod(url);
      logger.log(Level.INFO, new StringBuilder().append("GET: URL ").append(url).toString());
      client.getClient().executeMethod(getMethod);
      int statusCode = getMethod.getStatusCode();
      response.setResonseStatusCode(statusCode);
      logger.log(Level.INFO, new StringBuilder().append("GET: status Code: ").append(statusCode).toString());
      logger.log(Level.INFO, new StringBuilder().append("Status Line: ").append(getMethod.getStatusLine()).toString());
      resp = getContent(getMethod.getResponseBodyAsStream());
      response.setResponseXml(resp);
      logger.log(Level.INFO, new StringBuilder().append("Response : \n").append(resp).toString());
    } catch (HttpException e) {
      logger.log(Level.SEVERE, new StringBuilder().append("HTTP GET Failed: ").append(url).toString());
      throw new VCloudRuntimeException(e);
    } catch (IOException e) {
      logger.log(Level.SEVERE, null, e);
      throw new VCloudRuntimeException(e);
    } finally {
      getMethod.releaseConnection();
    }
    return response;
  }

  public static Response get(VcloudClient client, String url)
    throws VCloudException
  {
    Response response = new Response();

    String resp = null;
    GetMethod getMethod = null;
    try {
      getMethod = new GetMethod(url);
      getMethod.setRequestHeader(client.getVCloudAuthenticationHeaderName(), client.getVCloudAuthenticationHeaderValue());

      logger.log(Level.INFO, new StringBuilder().append("GET: URL ").append(url).toString());
      client.getClient().executeMethod(getMethod);
      if ((getMethod.getStatusCode() == 301) || (getMethod.getStatusCode() == 302))
      {
        getMethod.releaseConnection();
        String redirectedUrl = redirectURL(getMethod);
        getMethod.setURI(new URI(redirectedUrl));
        client.getClient().executeMethod(getMethod);
      }
      int statusCode = getMethod.getStatusCode();
      response.setResonseStatusCode(statusCode);
      logger.log(Level.INFO, new StringBuilder().append("GET: status Code: ").append(statusCode).toString());
      logger.log(Level.INFO, new StringBuilder().append("Status Line: ").append(getMethod.getStatusLine()).toString());
      resp = getContent(getMethod.getResponseBodyAsStream());
      response.setResponseXml(resp);
      logger.log(Level.INFO, new StringBuilder().append("Response : \n").append(resp).toString());
    } catch (HttpException e) {
      logger.log(Level.SEVERE, new StringBuilder().append("HTTP GET Failed: ").append(url).toString());
      throw new VCloudRuntimeException(e);
    } catch (IOException e) {
      logger.log(Level.SEVERE, null, e);
      throw new VCloudRuntimeException(e);
    } finally {
      getMethod.releaseConnection();
    }
    return response;
  }

  private static String getContent(InputStream is)
    throws IOException
  {
    if (is != null) {
      StringBuilder sb = new StringBuilder();
      try
      {
        BufferedReader reader = new BufferedReader(new InputStreamReader(is, "UTF-8"));
        String line;
        while ((line = reader.readLine()) != null)
          sb.append(new StringBuilder().append(line).append("\n").toString());
      }
      finally {
        is.close();
      }
      return sb.toString();
    }
    return "";
  }

  public static void uploadFile(VcloudClient client, FileType file, InputStream is, long size, long startByte, long endByte)
    throws VCloudException
  {
    String fileURL = getDefaultUploadURL(file);
    logger.log(Level.INFO, new StringBuilder().append("Upload File Size: ").append(file.getName()).append("\t").append(size).toString());

    PutMethod put = new PutMethod(fileURL);
    put.setRequestHeader(client.getVCloudAuthenticationHeaderName(), client.getVCloudAuthenticationHeaderValue());

    BufferedInputStream bufferedInputStream = new BufferedInputStream(is);

    InputStreamRequestEntity inputStreamRequestEntity = new InputStreamRequestEntity(bufferedInputStream, size);

    put.setRequestHeader("Content-Length", Long.toString(size));
    if (endByte > startByte) {
      inputStreamRequestEntity = new InputStreamRequestEntity(bufferedInputStream, endByte - startByte + 1L);

      put.setRequestHeader("Content-Length", Long.toString(endByte - startByte + 1L));

      put.setRequestHeader("Content-Range", new StringBuilder().append("bytes ").append(Long.toString(startByte)).append("-").append(Long.toString(endByte)).append("/").append(Long.toString(size)).toString());
    }

    put.setRequestEntity(inputStreamRequestEntity);
    try
    {
      int status = client.getClient().executeMethod(put);

      if (status == 200) {
        logger.log(Level.INFO, new StringBuilder().append("Upload complete: ").append(file.getName()).toString());
      }
      else {
        logger.log(Level.SEVERE, new StringBuilder().append("Upload failed: ").append(fileURL).append("\n").append("response=").append(HttpStatus.getStatusText(status)).toString());

        logger.log(Level.INFO, getContent(put.getResponseBodyAsStream()));

        logger.log(Level.SEVERE, new StringBuilder().append("File upload failed").append(fileURL).toString());
      }
    } catch (HttpException e) {
      logger.log(Level.SEVERE, new StringBuilder().append("HTTP Exception while uploading: ").append(fileURL).toString());

      throw new VCloudException(e.getMessage());
    } catch (IOException e) {
      logger.log(Level.SEVERE, new StringBuilder().append("IO Exception while uploading: ").append(fileURL).toString());

      throw new VCloudException(e.getMessage());
    } finally {
      put.releaseConnection();
    }
  }

  private static String getDefaultUploadURL(FileType file)
  {
    String fileHref = null;

    List links = file.getLink();
    Iterator linkIt = links.iterator();
    while (linkIt.hasNext()) {
      LinkType ln = (LinkType)linkIt.next();
      RelationType rel = ln.getRel();
      if (rel.compareTo(RelationType.UPLOAD_DEFAULT) == 0) {
        fileHref = ln.getHref();
        break;
      }
    }
    return fileHref;
  }

  public static InputStream downloadFile(VcloudClient client, String downloadURL)
    throws VCloudException
  {
    DownloadInputStream inputStream = null;

    GetMethod getMethod = new GetMethod(downloadURL);
    getMethod.setRequestHeader(client.getVCloudAuthenticationHeaderName(), client.getVCloudAuthenticationHeaderValue());
    int statusCode;
    try
    {
      logger.log(Level.INFO, new StringBuilder().append("Download: URL ").append(downloadURL).toString());
      client.getClient().executeMethod(getMethod);
      statusCode = getMethod.getStatusCode();
      logger.log(Level.INFO, new StringBuilder().append("GET: status Code: ").append(statusCode).toString());
      logger.log(Level.INFO, new StringBuilder().append("Status Line: ").append(getMethod.getStatusLine()).toString());
      void tmp140_137 = new RestUtil(); tmp140_137.getClass(); inputStream = new DownloadInputStream(getMethod);
    }
    catch (HttpException e) {
      logger.log(Level.SEVERE, new StringBuilder().append("HTTP GET Failed: ").append(downloadURL).toString());
      throw new VCloudRuntimeException(e);
    } catch (IOException e) {
      logger.log(Level.SEVERE, null, e);
      throw new VCloudRuntimeException(e);
    }

    if (statusCode != 200) {
      if (inputStream != null)
      {
        ErrorType err = (ErrorType)JAXBUtil.unmarshallResource(inputStream);
        throw new VCloudException(err);
      }

      throw new VCloudException(new StringBuilder().append("Download Failed: ").append(downloadURL).toString());
    }

    return inputStream;
  }

  private class DownloadInputStream extends InputStream
  {
    private InputStream inputStream = null;
    private GetMethod getMethod = null;

    public DownloadInputStream(GetMethod getMethod) throws IOException {
      this.getMethod = getMethod;
      this.inputStream = getMethod.getResponseBodyAsStream();
    }

    public void close()
      throws IOException
    {
      this.getMethod.abort();

      this.getMethod.releaseConnection();

      this.inputStream.close();
    }

    public int read()
      throws IOException
    {
      return this.inputStream.read();
    }
  }
}