package com.vmware.vcloud.sdk_v1_0;

import com.vmware.vcloud.api.rest.schema_v1_0.AvailableNetworksType;
import com.vmware.vcloud.api.rest.schema_v1_0.CaptureVAppParamsType;
import com.vmware.vcloud.api.rest.schema_v1_0.CloneMediaParamsType;
import com.vmware.vcloud.api.rest.schema_v1_0.CloneVAppParamsType;
import com.vmware.vcloud.api.rest.schema_v1_0.CloneVAppTemplateParamsType;
import com.vmware.vcloud.api.rest.schema_v1_0.ComposeVAppParamsType;
import com.vmware.vcloud.api.rest.schema_v1_0.InstantiateVAppTemplateParamsType;
import com.vmware.vcloud.api.rest.schema_v1_0.LinkType;
import com.vmware.vcloud.api.rest.schema_v1_0.MediaType;
import com.vmware.vcloud.api.rest.schema_v1_0.ObjectFactory;
import com.vmware.vcloud.api.rest.schema_v1_0.ReferenceType;
import com.vmware.vcloud.api.rest.schema_v1_0.RelationType;
import com.vmware.vcloud.api.rest.schema_v1_0.ResourceEntitiesType;
import com.vmware.vcloud.api.rest.schema_v1_0.ResourceReferenceType;
import com.vmware.vcloud.api.rest.schema_v1_0.UploadVAppTemplateParamsType;
import com.vmware.vcloud.api.rest.schema_v1_0.VAppTemplateType;
import com.vmware.vcloud.api.rest.schema_v1_0.VAppType;
import com.vmware.vcloud.api.rest.schema_v1_0.VCloudExtensionType;
import com.vmware.vcloud.api.rest.schema_v1_0.VdcType;
import com.vmware.vcloud.api.rest.schema_v1_0.extension.VimObjectRefType;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.xml.bind.JAXBElement;

public class Vdc extends VcloudEntity<VdcType>
{
  private static Logger logger = Logger.getLogger(Vdc.class.getSimpleName());
  private Collection<ResourceReferenceType> vappTemplateRefs;
  private Collection<ResourceReferenceType> mediaRefs;
  private HashMap<String, ResourceReferenceType> vappRefsByName;
  private HashMap<String, ReferenceType> availableNetworkRefsByName;
  private ReferenceType orgReference;
  private VimObjectRefType vimObjectRefType;

  Vdc(VcloudClient client, VdcType vdcType)
  {
    super(client, vdcType);
    sortEntityRefs();
  }

  public static Vdc getVdcByReference(VcloudClient client, ReferenceType vdcRef)
    throws VCloudException
  {
    String url = vdcRef.getHref();
    return getVdc(client, url);
  }

  private static Vdc getVdc(VcloudClient client, String url)
    throws VCloudException
  {
    Response response = RestUtil.get(client, url);
    VdcType vdcType = null;

    if (!response.isExpected(200))
    {
      response.handleUnExpectedResponse();
    }
    vdcType = (VdcType)response.getResource();
    Vdc vdc = new Vdc(client, vdcType);
    return vdc;
  }

  public VimObjectRefType getVimObjectRef()
    throws VCloudException
  {
    if (this.vimObjectRefType != null)
      return this.vimObjectRefType;
    throw new VCloudException("Vim Object Reference Type Not Found");
  }

  public ReferenceType getOrgReference()
    throws VCloudException
  {
    if (this.orgReference != null)
      return this.orgReference;
    throw new VCloudException("Reference Not Found");
  }

  private void sortEntityRefs()
  {
    this.vappTemplateRefs = new ArrayList();
    this.mediaRefs = new ArrayList();
    this.vappRefsByName = new HashMap();
    this.availableNetworkRefsByName = new HashMap();

    if (((VdcType)getResource()).getResourceEntities() != null) {
      ResourceEntitiesType resEntities = ((VdcType)getResource()).getResourceEntities();

      List entities = resEntities.getResourceEntity();

      for (ResourceReferenceType entt : entities) {
        if (entt.getType().equals("application/vnd.vmware.vcloud.vAppTemplate+xml"))
        {
          this.vappTemplateRefs.add(entt);
        } else if (entt.getType().equals("application/vnd.vmware.vcloud.media+xml"))
        {
          this.mediaRefs.add(entt);
        } else if (entt.getType().equals("application/vnd.vmware.vcloud.vApp+xml"))
        {
          this.vappRefsByName.put(entt.getName(), entt);
        }
        else logger.log(Level.WARNING, "Unknown Reference Type: ", entt.getType());

      }

    }

    if (((VdcType)getResource()).getAvailableNetworks() != null) {
      AvailableNetworksType availableNetworksType = ((VdcType)getResource()).getAvailableNetworks();

      for (ReferenceType networkRef : availableNetworksType.getNetwork()) {
        this.availableNetworkRefsByName.put(networkRef.getName(), networkRef);
      }

    }

    for (VCloudExtensionType vcloudExtension : ((VdcType)getResource()).getVCloudExtension())
    {
      for (i$ = vcloudExtension.getAny().iterator(); i$.hasNext(); ) { Object jaxbObject = i$.next();
        this.vimObjectRefType = ((VimObjectRefType)((JAXBElement)jaxbObject).getValue());
      }
    }
    Iterator i$;
    for (LinkType vdcLink : ((VdcType)getResource()).getLink())
      if ((vdcLink.getRel().equals(RelationType.UP)) && (vdcLink.getType().equals("application/vnd.vmware.vcloud.org+xml")))
      {
        this.orgReference = vdcLink;
      }
  }

  public HashMap<String, ReferenceType> getAvailableNetworkRefsByName()
  {
    return this.availableNetworkRefsByName;
  }

  public ReferenceType getAvailableNetworkRefByName(String name)
  {
    return (ReferenceType)this.availableNetworkRefsByName.get(name);
  }

  public Collection<ReferenceType> getAvailableNetworkRefs()
  {
    return this.availableNetworkRefsByName.values();
  }

  public Collection<ResourceReferenceType> getVappTemplateRefs()
  {
    return this.vappTemplateRefs;
  }

  public Collection<ResourceReferenceType> getMediaRefs()
  {
    return this.mediaRefs;
  }

  public HashMap<String, ResourceReferenceType> getVappRefsByName()
  {
    return this.vappRefsByName;
  }

  public ReferenceType getVappRefByName(String name)
  {
    return (ReferenceType)this.vappRefsByName.get(name);
  }

  public Collection<ResourceReferenceType> getVappRefs()
  {
    return this.vappRefsByName.values();
  }

  public VappTemplate createVappTemplate(UploadVAppTemplateParamsType vappTemplParams)
    throws VCloudException
  {
    String url = ((VdcType)getResource()).getHref() + "/action/uploadVAppTemplate";

    JAXBElement params = new ObjectFactory().createUploadVAppTemplateParams(vappTemplParams);

    String reqPayload = JAXBUtil.marshal(params);

    return executeVappTemplUpload(getVcloudClient(), url, reqPayload, "application/vnd.vmware.vcloud.uploadVAppTemplateParams+xml");
  }

  public Media createMedia(MediaType mediaParams)
    throws VCloudException
  {
    String url = ((VdcType)getResource()).getHref() + "/media/";

    JAXBElement params = new ObjectFactory().createMedia(mediaParams);

    String reqPayload = JAXBUtil.marshal(params);

    Response response = RestUtil.post(getVcloudClient(), url, reqPayload, "application/vnd.vmware.vcloud.media+xml", null);

    MediaType mediaType = null;

    if (!response.isExpected(201))
    {
      response.handleUnExpectedResponse();
    }
    mediaType = (MediaType)response.getResource();
    Media media = new Media(getVcloudClient(), mediaType);

    return media;
  }

  private static VappTemplate executeVappTemplUpload(VcloudClient client, String vdcActionUrl, String reqPayload, String contentType)
    throws VCloudException
  {
    Response response = RestUtil.post(client, vdcActionUrl, reqPayload, contentType, null);

    VAppTemplateType vAppTemplateType = null;

    if (!response.isExpected(201))
    {
      response.handleUnExpectedResponse();
    }
    vAppTemplateType = (VAppTemplateType)response.getResource();
    VappTemplate vappTemplate = new VappTemplate(client, vAppTemplateType);

    return vappTemplate;
  }

  public Vapp composeVapp(ComposeVAppParamsType composeVappParamsType)
    throws VCloudException
  {
    ObjectFactory of = new ObjectFactory();

    JAXBElement composeVappParams = of.createComposeVAppParams(composeVappParamsType);

    String requestBodyXml = JAXBUtil.marshal(composeVappParams);

    String postURL = ((VdcType)getResource()).getHref() + "/action/composeVApp";

    Response response = RestUtil.post(getVcloudClient(), postURL, requestBodyXml, "application/vnd.vmware.vcloud.composeVAppParams+xml", null);

    VAppType vAppType = null;

    if (!response.isExpected(201))
    {
      response.handleUnExpectedResponse();
    }
    vAppType = (VAppType)response.getResource();
    Vapp vapp = new Vapp(getVcloudClient(), vAppType);

    return vapp;
  }

  public VappTemplate captureVapp(CaptureVAppParamsType captureVappParamsType)
    throws VCloudException
  {
    ObjectFactory of = new ObjectFactory();

    JAXBElement captureVappParams = of.createCaptureVAppParams(captureVappParamsType);

    String requestBodyXml = JAXBUtil.marshal(captureVappParams);

    String postURL = ((VdcType)getResource()).getHref() + "/action/captureVApp";

    Response response = RestUtil.post(getVcloudClient(), postURL, requestBodyXml, "application/vnd.vmware.vcloud.captureVAppParams+xml", null);

    VAppTemplateType vAppTemplateType = null;

    if (!response.isExpected(201))
    {
      response.handleUnExpectedResponse();
    }
    vAppTemplateType = (VAppTemplateType)response.getResource();
    VappTemplate vappTemplate = new VappTemplate(getVcloudClient(), vAppTemplateType);

    return vappTemplate;
  }

  public Vapp instantiateVappTemplate(InstantiateVAppTemplateParamsType instVappTemplParamsType)
    throws VCloudException
  {
    ObjectFactory of = new ObjectFactory();

    JAXBElement instVappParams = of.createInstantiateVAppTemplateParams(instVappTemplParamsType);

    String requestBodyXml = JAXBUtil.marshal(instVappParams);

    String postURL = ((VdcType)getResource()).getHref() + "/action/instantiateVAppTemplate";

    Response response = RestUtil.post(getVcloudClient(), postURL, requestBodyXml, "application/vnd.vmware.vcloud.instantiateVAppTemplateParams+xml", null);

    VAppType vAppType = null;

    if (!response.isExpected(201))
    {
      response.handleUnExpectedResponse();
    }
    vAppType = (VAppType)response.getResource();
    Vapp vapp = new Vapp(getVcloudClient(), vAppType);

    return vapp;
  }

  public Vapp cloneVapp(CloneVAppParamsType cloneVappParamsType)
    throws VCloudException
  {
    ObjectFactory of = new ObjectFactory();
    JAXBElement cloneVappParams = of.createCloneVAppParams(cloneVappParamsType);

    String requestBodyXml = JAXBUtil.marshal(cloneVappParams);
    String postURL = ((VdcType)getResource()).getHref() + "/action/cloneVApp";

    Response response = RestUtil.post(getVcloudClient(), postURL, requestBodyXml, "application/vnd.vmware.vcloud.cloneVAppParams+xml", null);

    VAppType vAppType = null;

    if (!response.isExpected(201))
    {
      response.handleUnExpectedResponse();
    }
    vAppType = (VAppType)response.getResource();
    Vapp vapp = new Vapp(getVcloudClient(), vAppType);

    return vapp;
  }

  public VappTemplate cloneVappTemplate(CloneVAppTemplateParamsType cloneVappTemplateParamsType)
    throws VCloudException
  {
    ObjectFactory of = new ObjectFactory();
    JAXBElement cloneVappParams = of.createCloneVAppTemplateParams(cloneVappTemplateParamsType);

    String requestBodyXml = JAXBUtil.marshal(cloneVappParams);
    String postURL = ((VdcType)getResource()).getHref() + "/action/cloneVAppTemplate";

    Response response = RestUtil.post(getVcloudClient(), postURL, requestBodyXml, "application/vnd.vmware.vcloud.cloneVAppTemplateParams+xml", null);

    VAppTemplateType vAppTemplateType = null;

    if (!response.isExpected(201))
    {
      response.handleUnExpectedResponse();
    }
    vAppTemplateType = (VAppTemplateType)response.getResource();
    VappTemplate vappTemplate = new VappTemplate(getVcloudClient(), vAppTemplateType);

    return vappTemplate;
  }

  public Media cloneMedia(CloneMediaParamsType cloneMediaParamsType)
    throws VCloudException
  {
    ObjectFactory of = new ObjectFactory();
    JAXBElement cloneMediaParams = of.createCloneMediaParams(cloneMediaParamsType);

    String requestBodyXml = JAXBUtil.marshal(cloneMediaParams);
    String postURL = ((VdcType)getResource()).getHref() + "/action/cloneMedia";

    Response response = RestUtil.post(getVcloudClient(), postURL, requestBodyXml, "application/vnd.vmware.vcloud.cloneMediaParams+xml", null);

    MediaType mediaType = null;

    if (!response.isExpected(201))
    {
      response.handleUnExpectedResponse();
    }
    mediaType = (MediaType)response.getResource();
    Media media = new Media(getVcloudClient(), mediaType);

    return media;
  }
}