package com.vmware.vcloud.sdk_v1_0.admin;

import com.vmware.vcloud.api.rest.schema_v1_0.AdminVdcType;
import com.vmware.vcloud.api.rest.schema_v1_0.AvailableNetworksType;
import com.vmware.vcloud.api.rest.schema_v1_0.LinkType;
import com.vmware.vcloud.api.rest.schema_v1_0.ObjectFactory;
import com.vmware.vcloud.api.rest.schema_v1_0.ReferenceType;
import com.vmware.vcloud.api.rest.schema_v1_0.RelationType;
import com.vmware.vcloud.api.rest.schema_v1_0.ResourceEntitiesType;
import com.vmware.vcloud.api.rest.schema_v1_0.ResourceReferenceType;
import com.vmware.vcloud.api.rest.schema_v1_0.TaskType;
import com.vmware.vcloud.api.rest.schema_v1_0.VCloudExtensionType;
import com.vmware.vcloud.api.rest.schema_v1_0.extension.VimObjectRefType;
import com.vmware.vcloud.sdk_v1_0.JAXBUtil;
import com.vmware.vcloud.sdk_v1_0.Response;
import com.vmware.vcloud.sdk_v1_0.RestUtil;
import com.vmware.vcloud.sdk_v1_0.Task;
import com.vmware.vcloud.sdk_v1_0.VCloudException;
import com.vmware.vcloud.sdk_v1_0.VcloudClient;
import com.vmware.vcloud.sdk_v1_0.VcloudEntity;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.xml.bind.JAXBElement;

public class AdminVdc extends VcloudEntity<AdminVdcType>
{
  private static Logger logger = Logger.getLogger(AdminVdc.class.getSimpleName());
  private Collection<ResourceReferenceType> vappTemplateRefs;
  private Collection<ResourceReferenceType> mediaRefs;
  private HashMap<String, ResourceReferenceType> vappRefsByName;
  private HashMap<String, ReferenceType> orgNetworkRefsByName;
  private ReferenceType adminOrgReference;
  private ReferenceType vdcReference;
  private VimObjectRefType vimObjectRefType;

  AdminVdc(VcloudClient client, AdminVdcType adminVdcType)
  {
    super(client, adminVdcType);
    sortEntityAndNetworkRefs();
  }

  public static AdminVdc getAdminVdcByReference(VcloudClient client, ReferenceType vdcRef)
    throws VCloudException
  {
    String url = vdcRef.getHref();
    return get(client, url);
  }

  private static AdminVdc get(VcloudClient client, String url)
    throws VCloudException
  {
    Response response = RestUtil.get(client, url);
    AdminVdcType adminVdcType = null;
    if (response.isExpected(200))
      adminVdcType = (AdminVdcType)response.getResource();
    else
      response.handleUnExpectedResponse();
    AdminVdc vdc = new AdminVdc(client, adminVdcType);
    return vdc;
  }

  public AdminVdc updateAdminVdc(AdminVdcType adminVdcType)
    throws VCloudException
  {
    ObjectFactory of = new ObjectFactory();
    String adminVdcUpdateUrl = ((AdminVdcType)getResource()).getHref();
    String adminVdcUpdateRequestXmlString = JAXBUtil.marshal(of.createAdminVdc(adminVdcType));

    logger.log(Level.INFO, "Update ADMIN VDC URL : " + adminVdcUpdateUrl);

    Response response = RestUtil.put(getVcloudClient(), adminVdcUpdateUrl, adminVdcUpdateRequestXmlString, "application/vnd.vmware.admin.vdc+xml", null);

    AdminVdcType updatedAdminVdcType = null;
    if (response.isExpected(200))
      updatedAdminVdcType = (AdminVdcType)response.getResource();
    else {
      response.handleUnExpectedResponse();
    }
    return new AdminVdc(getVcloudClient(), updatedAdminVdcType);
  }

  public Task delete()
    throws VCloudException
  {
    String adminVdcUrl = ((AdminVdcType)getResource()).getHref();
    return deleteAdminVdc(getVcloudClient(), adminVdcUrl);
  }

  public static Task delete(VcloudClient client, ReferenceType adminVdcRef)
    throws VCloudException
  {
    return deleteAdminVdc(client, adminVdcRef.getHref());
  }

  private static Task deleteAdminVdc(VcloudClient client, String adminVdcUrl)
    throws VCloudException
  {
    Response response = RestUtil.delete(client, adminVdcUrl);
    TaskType taskType = null;
    if (response.isExpected(202)) {
      taskType = (TaskType)response.getResource();
    }
    else {
      response.handleUnExpectedResponse();
    }
    Task task = new Task(client, taskType);

    return task;
  }

  public void enable()
    throws VCloudException
  {
    String enableURL = ((AdminVdcType)getResource()).getHref() + "/action/enable";

    executeVdcAction(getVcloudClient(), enableURL, null, null, null, 204);
  }

  public static void enable(VcloudClient client, ReferenceType adminVdcRef)
    throws VCloudException
  {
    String enableURL = adminVdcRef.getHref() + "/action/enable";

    executeVdcAction(client, enableURL, null, null, null, 204);
  }

  public void disable()
    throws VCloudException
  {
    String disableUrl = ((AdminVdcType)getResource()).getHref() + "/action/disable";

    executeVdcAction(getVcloudClient(), disableUrl, null, null, null, 204);
  }

  public static void disable(VcloudClient client, ReferenceType adminVdcRef)
    throws VCloudException
  {
    String enableURL = adminVdcRef.getHref() + "/action/disable";

    executeVdcAction(client, enableURL, null, null, null, 204);
  }

  private static Task executeVdcAction(VcloudClient client, String orgActionUrl, String content, String contentType, String charset, int statusCode)
    throws VCloudException
  {
    logger.log(Level.INFO, "POST: Org Action URL " + orgActionUrl);
    Response response = RestUtil.post(client, orgActionUrl, content, contentType, charset);

    boolean expectedStatus = response.isExpected(statusCode);
    if (statusCode == 204)
      if (!expectedStatus)
        response.handleUnExpectedResponse();
      else
        return null;
    if (!expectedStatus)
      response.handleUnExpectedResponse();
    TaskType taskType = (TaskType)response.getResource();
    Task task = new Task(client, taskType);

    return task;
  }

  public ReferenceType getAdminOrganizationReference()
    throws VCloudException
  {
    if (this.adminOrgReference != null)
      return this.adminOrgReference;
    throw new VCloudException("Reference Not Found");
  }

  public ReferenceType getVdcReference()
    throws VCloudException
  {
    if (this.vdcReference != null)
      return this.vdcReference;
    throw new VCloudException("Reference Not Found");
  }

  public VimObjectRefType getVimObjectRef()
    throws VCloudException
  {
    if (this.vimObjectRefType != null)
      return this.vimObjectRefType;
    throw new VCloudException("Vim Object Reference Type Not Found");
  }

  private void sortEntityAndNetworkRefs()
  {
    this.vappTemplateRefs = new ArrayList();
    this.mediaRefs = new ArrayList();
    this.vappRefsByName = new HashMap();
    this.orgNetworkRefsByName = new HashMap();

    if (((AdminVdcType)getResource()).getResourceEntities() != null) {
      ResourceEntitiesType resEntities = ((AdminVdcType)getResource()).getResourceEntities();

      List entities = resEntities.getResourceEntity();

      if (entities != null) {
        for (ResourceReferenceType entt : entities) {
          if (entt.getType().equals("application/vnd.vmware.vcloud.vAppTemplate+xml"))
          {
            this.vappTemplateRefs.add(entt);
          } else if (entt.getType().equals("application/vnd.vmware.vcloud.media+xml"))
          {
            this.mediaRefs.add(entt);
          } else if (entt.getType().equals("application/vnd.vmware.vcloud.vApp+xml"))
          {
            this.vappRefsByName.put(entt.getName(), entt);
          }
          else logger.log(Level.WARNING, "Unknown Reference Type: ", entt.getType());
        }
      }

    }

    if (((AdminVdcType)getResource()).getAvailableNetworks() != null) {
      AvailableNetworksType availableNetworksType = ((AdminVdcType)getResource()).getAvailableNetworks();

      if (availableNetworksType != null) {
        for (ReferenceType networkRef : availableNetworksType.getNetwork())
        {
          this.orgNetworkRefsByName.put(networkRef.getName(), networkRef);
        }
      }
    }
    for (LinkType adminVdcLink : ((AdminVdcType)getResource()).getLink()) {
      if ((adminVdcLink.getRel().equals(RelationType.UP)) && (adminVdcLink.getType().equals("application/vnd.vmware.admin.organization+xml")))
      {
        this.adminOrgReference = adminVdcLink;
      }if ((adminVdcLink.getRel().equals(RelationType.ALTERNATE)) && (adminVdcLink.getType().equals("application/vnd.vmware.vcloud.vdc+xml")))
      {
        this.vdcReference = adminVdcLink;
      }
    }

    for (VCloudExtensionType vcloudExtension : ((AdminVdcType)getResource()).getVCloudExtension())
    {
      for (i$ = vcloudExtension.getAny().iterator(); i$.hasNext(); ) { Object jaxbObject = i$.next();
        this.vimObjectRefType = ((VimObjectRefType)((JAXBElement)jaxbObject).getValue());
      }
    }
    Iterator i$;
  }

  public HashMap<String, ReferenceType> getAdminOrgNetworkRefsByName()
  {
    return this.orgNetworkRefsByName;
  }

  public Collection<ReferenceType> getAdminOrgNetworkRefs()
  {
    return this.orgNetworkRefsByName.values();
  }

  public ReferenceType getAdminOrgNetworkRefByName(String name)
  {
    return (ReferenceType)this.orgNetworkRefsByName.get(name);
  }

  public Collection<ResourceReferenceType> getVappTemplateRefs()
  {
    return this.vappTemplateRefs;
  }

  public Collection<ResourceReferenceType> getMediaRefs()
  {
    return this.mediaRefs;
  }

  public HashMap<String, ResourceReferenceType> getVappRefsByName()
  {
    return this.vappRefsByName;
  }

  public Collection<ResourceReferenceType> getVappRefs()
  {
    return this.vappRefsByName.values();
  }

  public ReferenceType getVappRefByName(String name)
  {
    return (ReferenceType)this.vappRefsByName.get(name);
  }

  public ReferenceType getProviderVdcRef()
  {
    return ((AdminVdcType)getResource()).getProviderVdcReference();
  }

  public ReferenceType getNetworkPoolRef()
  {
    return ((AdminVdcType)getResource()).getNetworkPoolReference();
  }
}