package com.vmware.vcloud.sdk_v1_0.admin;

import com.vmware.vcloud.api.rest.schema_v1_0.ObjectFactory;
import com.vmware.vcloud.api.rest.schema_v1_0.ReferenceType;
import com.vmware.vcloud.api.rest.schema_v1_0.RoleType;
import com.vmware.vcloud.sdk_v1_0.JAXBUtil;
import com.vmware.vcloud.sdk_v1_0.Response;
import com.vmware.vcloud.sdk_v1_0.RestUtil;
import com.vmware.vcloud.sdk_v1_0.VCloudException;
import com.vmware.vcloud.sdk_v1_0.VcloudClient;
import com.vmware.vcloud.sdk_v1_0.VcloudEntity;
import java.util.logging.Level;
import java.util.logging.Logger;

public class Role extends VcloudEntity<RoleType>
{
  private static Logger logger = Logger.getLogger(Role.class.getSimpleName());

  Role(VcloudClient client, RoleType roleType)
  {
    super(client, roleType);
  }

  public static Role getRoleByReference(VcloudClient client, ReferenceType roleRef)
    throws VCloudException
  {
    return get(client, roleRef.getHref());
  }

  private static Role get(VcloudClient client, String url)
    throws VCloudException
  {
    logger.log(Level.INFO, "GET: Role " + url);

    Response response = RestUtil.get(client, url);
    RoleType roleType = null;
    if (response.isExpected(200))
      roleType = (RoleType)response.getResource();
    else
      response.handleUnExpectedResponse();
    Role role = new Role(client, roleType);
    return role;
  }

  public Role updateRole(RoleType roleType)
    throws VCloudException
  {
    ObjectFactory of = new ObjectFactory();
    String roleUpdateUrl = ((RoleType)getResource()).getHref();
    String roleUpdateRequestXmlString = JAXBUtil.marshal(of.createRole(roleType));

    logger.log(Level.INFO, "Update Role URL : " + roleUpdateUrl);

    Response response = RestUtil.put(getVcloudClient(), roleUpdateUrl, roleUpdateRequestXmlString, "application/vnd.vmware.admin.role+xml", null);

    RoleType updatedRoleType = null;
    if (response.isExpected(200))
      updatedRoleType = (RoleType)response.getResource();
    else {
      response.handleUnExpectedResponse();
    }
    return new Role(getVcloudClient(), updatedRoleType);
  }

  public void delete()
    throws VCloudException
  {
    String roleUrl = ((RoleType)getResource()).getHref();
    deleteRole(getVcloudClient(), roleUrl);
  }

  public static void delete(VcloudClient client, ReferenceType roleRef)
    throws VCloudException
  {
    deleteRole(client, roleRef.getHref());
  }

  private static void deleteRole(VcloudClient client, String roleUrl)
    throws VCloudException
  {
    Response response = RestUtil.delete(client, roleUrl);
    if (!response.isExpected(204))
      response.handleUnExpectedResponse();
  }
}