package com.vmware.vcloud.sdk_v1_0.admin.extensions;

import com.vmware.vcloud.api.rest.schema_v1_0.ReferenceType;
import com.vmware.vcloud.api.rest.schema_v1_0.extension.ObjectFactory;
import com.vmware.vcloud.api.rest.schema_v1_0.extension.VMWNetworkPoolType;
import com.vmware.vcloud.sdk_v1_0.JAXBUtil;
import com.vmware.vcloud.sdk_v1_0.Response;
import com.vmware.vcloud.sdk_v1_0.RestUtil;
import com.vmware.vcloud.sdk_v1_0.VCloudException;
import com.vmware.vcloud.sdk_v1_0.VcloudClient;
import com.vmware.vcloud.sdk_v1_0.VcloudEntity;
import java.util.logging.Level;
import java.util.logging.Logger;

public class VMWNetworkPool extends VcloudEntity<VMWNetworkPoolType>
{
  private static Logger logger = Logger.getLogger(VMWNetworkPool.class.getSimpleName());

  VMWNetworkPool(VcloudClient client, VMWNetworkPoolType vmwNetworkPoolType)
  {
    super(client, vmwNetworkPoolType);
  }

  public static VMWNetworkPool getVMWNetworkPoolByReference(VcloudClient client, ReferenceType referenceType)
    throws VCloudException
  {
    return getVMWNetworkPool(client, referenceType.getHref());
  }

  private static VMWNetworkPool getVMWNetworkPool(VcloudClient client, String url)
    throws VCloudException
  {
    logger.log(Level.INFO, "GET: VMWNetworkPool " + url);
    Response response = RestUtil.get(client, url);
    VMWNetworkPoolType vmwNetworkPoolType = null;

    if (response.isExpected(200))
      vmwNetworkPoolType = (VMWNetworkPoolType)response.getResource();
    else {
      response.handleUnExpectedResponse();
    }
    VMWNetworkPool vmwNetworkPool = new VMWNetworkPool(client, vmwNetworkPoolType);

    return vmwNetworkPool;
  }

  public VMWNetworkPool updateVMWNetworkPool(VMWNetworkPoolType vmwNetworkPoolType)
    throws VCloudException
  {
    ObjectFactory ofFactory = new ObjectFactory();
    String vmwNetworkPoolUpdateUrl = ((VMWNetworkPoolType)getResource()).getHref();
    String vmwNetworkPoolUpdateRequest = JAXBUtil.marshal(ofFactory.createVMWNetworkPool(vmwNetworkPoolType));

    logger.log(Level.INFO, "Update VMWNetworkPool URL : " + vmwNetworkPoolUpdateUrl);

    Response response = RestUtil.put(getVcloudClient(), vmwNetworkPoolUpdateUrl, vmwNetworkPoolUpdateRequest, "application/vnd.vmware.admin.networkPool+xml", null);

    VMWNetworkPoolType updatedVMWNetworkPoolType = null;

    if (response.isExpected(200)) {
      updatedVMWNetworkPoolType = (VMWNetworkPoolType)response.getResource();
    }
    else {
      response.handleUnExpectedResponse();
    }
    return new VMWNetworkPool(getVcloudClient(), updatedVMWNetworkPoolType);
  }

  public void delete()
    throws VCloudException
  {
    String vmwNetworkPoolUrl = ((VMWNetworkPoolType)getResource()).getHref();
    deleteVMWNetworkPool(getVcloudClient(), vmwNetworkPoolUrl);
  }

  public static void delete(VcloudClient client, ReferenceType vmwNetworkPoolRef)
    throws VCloudException
  {
    deleteVMWNetworkPool(client, vmwNetworkPoolRef.getHref());
  }

  private static void deleteVMWNetworkPool(VcloudClient client, String vmwNetworkPoolUrl)
    throws VCloudException
  {
    Response response = RestUtil.delete(client, vmwNetworkPoolUrl);
    if (!response.isExpected(204))
      response.handleUnExpectedResponse();
  }
}