package com.vmware.vcloud.sdk_v1_0.admin.extensions;

import com.vmware.vcloud.api.rest.schema_v1_0.ReferenceType;
import com.vmware.vcloud.api.rest.schema_v1_0.extension.LicensingReportListType;
import com.vmware.vcloud.api.rest.schema_v1_0.extension.ObjectFactory;
import com.vmware.vcloud.api.rest.schema_v1_0.extension.RegisterVimServerParamsType;
import com.vmware.vcloud.api.rest.schema_v1_0.extension.VMWExtensionType;
import com.vmware.vcloud.api.rest.schema_v1_0.extension.VMWExternalNetworkReferencesType;
import com.vmware.vcloud.api.rest.schema_v1_0.extension.VMWExternalNetworkType;
import com.vmware.vcloud.api.rest.schema_v1_0.extension.VMWHostReferencesType;
import com.vmware.vcloud.api.rest.schema_v1_0.extension.VMWNetworkPoolReferencesType;
import com.vmware.vcloud.api.rest.schema_v1_0.extension.VMWNetworkPoolType;
import com.vmware.vcloud.api.rest.schema_v1_0.extension.VMWProviderVdcReferencesType;
import com.vmware.vcloud.api.rest.schema_v1_0.extension.VMWProviderVdcType;
import com.vmware.vcloud.api.rest.schema_v1_0.extension.VMWVimServerReferencesType;
import com.vmware.vcloud.sdk_v1_0.JAXBUtil;
import com.vmware.vcloud.sdk_v1_0.Response;
import com.vmware.vcloud.sdk_v1_0.RestUtil;
import com.vmware.vcloud.sdk_v1_0.VCloudException;
import com.vmware.vcloud.sdk_v1_0.VcloudClient;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.logging.Level;
import java.util.logging.Logger;

public class VcloudAdminExtension
{
  private VMWExtensionType vmwExtensionResource;
  private static Logger logger = Logger.getLogger(VcloudAdminExtension.class.getSimpleName());
  private VcloudClient client;

  public VcloudAdminExtension(VcloudClient client)
    throws VCloudException
  {
    this.client = client;

    Response response = RestUtil.get(client, client.getVcloudApiURL() + "/admin/extension");

    if (response.isExpected(200))
      this.vmwExtensionResource = ((VMWExtensionType)response.getResource());
    else
      response.handleUnExpectedResponse();
  }

  public VMWExtensionType getResource()
  {
    return this.vmwExtensionResource;
  }

  public HashMap<String, ReferenceType> getVMWProviderVdcRefsByName()
    throws VCloudException
  {
    HashMap vmwProviderVdcRefs = new HashMap();
    String url = this.client.getVcloudApiURL() + "/admin/extension" + "/providerVdcReferences";

    logger.log(Level.INFO, "GET Extension Provider Vdc References : " + url);

    Response response = RestUtil.get(this.client, url);
    VMWProviderVdcReferencesType vmwProviderVdcReferencesType = null;

    if (!response.isExpected(200)) {
      response.handleUnExpectedResponse();
    }
    vmwProviderVdcReferencesType = (VMWProviderVdcReferencesType)response.getResource();

    for (ReferenceType vmwProviderVdcRef : vmwProviderVdcReferencesType.getProviderVdcReference())
    {
      vmwProviderVdcRefs.put(vmwProviderVdcRef.getName(), vmwProviderVdcRef);
    }

    return vmwProviderVdcRefs;
  }

  public HashMap<String, ReferenceType> getVMWExternalNetworkRefsByName()
    throws VCloudException
  {
    HashMap vmwExternalNetworkRefs = new HashMap();
    String url = this.client.getVcloudApiURL() + "/admin/extension" + "/externalNetworkReferences";

    logger.log(Level.INFO, "GET Extension ExternalNetwork References : " + url);

    Response response = RestUtil.get(this.client, url);
    VMWExternalNetworkReferencesType vmwExternalNetworkReferencesType = null;

    if (!response.isExpected(200)) {
      response.handleUnExpectedResponse();
    }
    vmwExternalNetworkReferencesType = (VMWExternalNetworkReferencesType)response.getResource();

    for (ReferenceType vmwExternalNetworkRef : vmwExternalNetworkReferencesType.getExternalNetworkReference())
    {
      vmwExternalNetworkRefs.put(vmwExternalNetworkRef.getName(), vmwExternalNetworkRef);
    }

    return vmwExternalNetworkRefs;
  }

  public HashMap<String, ReferenceType> getVMWNetworkPoolRefsByName()
    throws VCloudException
  {
    HashMap vmwNetworkPoolRefs = new HashMap();
    String url = this.client.getVcloudApiURL() + "/admin/extension" + "/networkPoolReferences";

    logger.log(Level.INFO, "GET Extension NetworkPool References : " + url);
    Response response = RestUtil.get(this.client, url);
    VMWNetworkPoolReferencesType vmwNetworkPoolReferencesType = null;

    if (!response.isExpected(200)) {
      response.handleUnExpectedResponse();
    }
    vmwNetworkPoolReferencesType = (VMWNetworkPoolReferencesType)response.getResource();

    for (ReferenceType vmwNetworkPoolRef : vmwNetworkPoolReferencesType.getNetworkPoolReference())
    {
      vmwNetworkPoolRefs.put(vmwNetworkPoolRef.getName(), vmwNetworkPoolRef);
    }

    return vmwNetworkPoolRefs;
  }

  public HashMap<String, ReferenceType> getVMWVimServerRefsByName()
    throws VCloudException
  {
    HashMap vmwVimServerRefs = new HashMap();
    String url = this.client.getVcloudApiURL() + "/admin/extension" + "/vimServerReferences";

    logger.log(Level.INFO, "GET Extension VimServer References : " + url);
    Response response = RestUtil.get(this.client, url);
    VMWVimServerReferencesType vmwVimServerReferencesType = null;

    if (!response.isExpected(200)) {
      response.handleUnExpectedResponse();
    }
    vmwVimServerReferencesType = (VMWVimServerReferencesType)response.getResource();

    for (ReferenceType vmwVimServerRef : vmwVimServerReferencesType.getVimServerReference())
    {
      vmwVimServerRefs.put(vmwVimServerRef.getName(), vmwVimServerRef);
    }
    return vmwVimServerRefs;
  }

  public HashMap<String, ReferenceType> getVMWHostRefsByName()
    throws VCloudException
  {
    HashMap vmwHostRefs = new HashMap();
    String url = this.client.getVcloudApiURL() + "/admin/extension" + "/hostReferences";

    logger.log(Level.INFO, "GET Extension Host References : " + url);
    Response response = RestUtil.get(this.client, url);
    VMWHostReferencesType vmwHostReferencesType = null;

    if (!response.isExpected(200)) {
      response.handleUnExpectedResponse();
    }
    vmwHostReferencesType = (VMWHostReferencesType)response.getResource();

    for (ReferenceType vmwHostRef : vmwHostReferencesType.getHostReference())
    {
      vmwHostRefs.put(vmwHostRef.getName(), vmwHostRef);
    }
    return vmwHostRefs;
  }

  public Collection<ReferenceType> getLicensingReportRefs()
    throws VCloudException
  {
    Collection licenseReportRefs = new ArrayList();
    String url = this.client.getVcloudApiURL() + "/admin/extension" + "/licensing/" + "reports";

    logger.log(Level.INFO, "GET Extension Host References : " + url);
    Response response = RestUtil.get(this.client, url);
    LicensingReportListType licensingReportListType = null;

    if (!response.isExpected(200)) {
      response.handleUnExpectedResponse();
    }
    licensingReportListType = (LicensingReportListType)response.getResource();

    for (ReferenceType licenseReportRef : licensingReportListType.getReport())
    {
      licenseReportRefs.add(licenseReportRef);
    }
    return licenseReportRefs;
  }

  public RegisterVimServerParamsType registerVMWVimServer(RegisterVimServerParamsType registerVimServerParamsType)
    throws VCloudException
  {
    String url = this.client.getVcloudApiURL() + "/admin/extension" + "/action/registervimserver";

    ObjectFactory extensionFactory = new ObjectFactory();
    String registerVimServerRequestString = JAXBUtil.marshal(extensionFactory.createRegisterVimServerParams(registerVimServerParamsType));

    logger.log(Level.INFO, "Register provider vdc URL" + url);
    Response response = RestUtil.post(this.client, url, registerVimServerRequestString, "application/vnd.vmware.admin.registerVimServerParams+xml", null);

    RegisterVimServerParamsType registeredVimServerParamsType = null;

    if (response.isExpected(200)) {
      registeredVimServerParamsType = (RegisterVimServerParamsType)response.getResource();
    }
    else {
      response.handleUnExpectedResponse();
    }
    return registeredVimServerParamsType;
  }

  public VMWProviderVdc createVMWProviderVdc(VMWProviderVdcType vmwProviderVdcType)
    throws VCloudException
  {
    String url = this.client.getVcloudApiURL() + "/admin/extension" + "/providervdcs";

    ObjectFactory extensionFactory = new ObjectFactory();
    String createVMWProviderVdcRequestString = JAXBUtil.marshal(extensionFactory.createVMWProviderVdc(vmwProviderVdcType));

    logger.log(Level.INFO, "Create VMW Provider vdc URL" + url);
    Response response = RestUtil.post(this.client, url, createVMWProviderVdcRequestString, "application/vnd.vmware.admin.vmwprovidervdc+xml", null);

    VMWProviderVdcType createdVMWProviderVdcType = null;

    if (response.isExpected(201)) {
      createdVMWProviderVdcType = (VMWProviderVdcType)response.getResource();
    }
    else
    {
      response.handleUnExpectedResponse();
    }
    return new VMWProviderVdc(this.client, createdVMWProviderVdcType);
  }

  public VMWExternalNetwork createVMWExternalNetwork(VMWExternalNetworkType vmwExternalNetworkType)
    throws VCloudException
  {
    String url = this.client.getVcloudApiURL() + "/admin/extension" + "/externalnets";

    ObjectFactory extensionFactory = new ObjectFactory();
    String createVMWExternalNetworkRequestString = JAXBUtil.marshal(extensionFactory.createVMWExternalNetwork(vmwExternalNetworkType));

    logger.log(Level.INFO, "Create VMW ExternalNetwork URL" + url);
    Response response = RestUtil.post(this.client, url, createVMWExternalNetworkRequestString, "application/vnd.vmware.admin.vmwexternalnet+xml", null);

    VMWExternalNetworkType createdVMWExternalNetworkType = null;

    if (response.isExpected(201)) {
      createdVMWExternalNetworkType = (VMWExternalNetworkType)response.getResource();
    }
    else
    {
      response.handleUnExpectedResponse();
    }
    return new VMWExternalNetwork(this.client, createdVMWExternalNetworkType);
  }

  public VMWNetworkPool createVMWNetworkPool(VMWNetworkPoolType vmwNetworkPoolType)
    throws VCloudException
  {
    String url = this.client.getVcloudApiURL() + "/admin/extension" + "/networkPools";

    ObjectFactory extensionFactory = new ObjectFactory();
    String createVMWNetworkPoolRequestString = JAXBUtil.marshal(extensionFactory.createVMWNetworkPool(vmwNetworkPoolType));

    logger.log(Level.INFO, "Create VMW Network Pool URL" + url);
    Response response = RestUtil.post(this.client, url, createVMWNetworkPoolRequestString, "application/vnd.vmware.admin.networkPool+xml", null);

    VMWNetworkPoolType createdVMWNetworkPoolType = null;

    if (response.isExpected(201)) {
      createdVMWNetworkPoolType = (VMWNetworkPoolType)response.getResource();
    }
    else
    {
      response.handleUnExpectedResponse();
    }
    return new VMWNetworkPool(this.client, createdVMWNetworkPoolType);
  }
}