package org.eclipse.jdt.internal.compiler.ast;

import org.eclipse.jdt.core.compiler.CharOperation;
import org.eclipse.jdt.internal.compiler.ASTVisitor;
import org.eclipse.jdt.internal.compiler.env.AccessRestriction;
import org.eclipse.jdt.internal.compiler.impl.CompilerOptions;
import org.eclipse.jdt.internal.compiler.lookup.AnnotationBinding;
import org.eclipse.jdt.internal.compiler.lookup.ArrayBinding;
import org.eclipse.jdt.internal.compiler.lookup.Binding;
import org.eclipse.jdt.internal.compiler.lookup.BlockScope;
import org.eclipse.jdt.internal.compiler.lookup.FieldBinding;
import org.eclipse.jdt.internal.compiler.lookup.InvocationSite;
import org.eclipse.jdt.internal.compiler.lookup.LocalVariableBinding;
import org.eclipse.jdt.internal.compiler.lookup.LookupEnvironment;
import org.eclipse.jdt.internal.compiler.lookup.MethodBinding;
import org.eclipse.jdt.internal.compiler.lookup.PackageBinding;
import org.eclipse.jdt.internal.compiler.lookup.ParameterizedGenericMethodBinding;
import org.eclipse.jdt.internal.compiler.lookup.ReferenceBinding;
import org.eclipse.jdt.internal.compiler.lookup.Scope;
import org.eclipse.jdt.internal.compiler.lookup.TypeBinding;
import org.eclipse.jdt.internal.compiler.lookup.TypeConstants;
import org.eclipse.jdt.internal.compiler.lookup.TypeIds;
import org.eclipse.jdt.internal.compiler.lookup.WildcardBinding;
import org.eclipse.jdt.internal.compiler.problem.ProblemReporter;

public abstract class ASTNode
  implements TypeConstants, TypeIds
{
  public int sourceStart;
  public int sourceEnd;
  public static final int Bit1 = 1;
  public static final int Bit2 = 2;
  public static final int Bit3 = 4;
  public static final int Bit4 = 8;
  public static final int Bit5 = 16;
  public static final int Bit6 = 32;
  public static final int Bit7 = 64;
  public static final int Bit8 = 128;
  public static final int Bit9 = 256;
  public static final int Bit10 = 512;
  public static final int Bit11 = 1024;
  public static final int Bit12 = 2048;
  public static final int Bit13 = 4096;
  public static final int Bit14 = 8192;
  public static final int Bit15 = 16384;
  public static final int Bit16 = 32768;
  public static final int Bit17 = 65536;
  public static final int Bit18 = 131072;
  public static final int Bit19 = 262144;
  public static final int Bit20 = 524288;
  public static final int Bit21 = 1048576;
  public static final int Bit22 = 2097152;
  public static final int Bit23 = 4194304;
  public static final int Bit24 = 8388608;
  public static final int Bit25 = 16777216;
  public static final int Bit26 = 33554432;
  public static final int Bit27 = 67108864;
  public static final int Bit28 = 134217728;
  public static final int Bit29 = 268435456;
  public static final int Bit30 = 536870912;
  public static final int Bit31 = 1073741824;
  public static final int Bit32 = -2147483648;
  public static final long Bit32L = 2147483648L;
  public static final long Bit33L = 4294967296L;
  public static final long Bit34L = 8589934592L;
  public static final long Bit35L = 17179869184L;
  public static final long Bit36L = 34359738368L;
  public static final long Bit37L = 68719476736L;
  public static final long Bit38L = 137438953472L;
  public static final long Bit39L = 274877906944L;
  public static final long Bit40L = 549755813888L;
  public static final long Bit41L = 1099511627776L;
  public static final long Bit42L = 2199023255552L;
  public static final long Bit43L = 4398046511104L;
  public static final long Bit44L = 8796093022208L;
  public static final long Bit45L = 17592186044416L;
  public static final long Bit46L = 35184372088832L;
  public static final long Bit47L = 70368744177664L;
  public static final long Bit48L = 140737488355328L;
  public static final long Bit49L = 281474976710656L;
  public static final long Bit50L = 562949953421312L;
  public static final long Bit51L = 1125899906842624L;
  public static final long Bit52L = 2251799813685248L;
  public static final long Bit53L = 4503599627370496L;
  public static final long Bit54L = 9007199254740992L;
  public static final long Bit55L = 18014398509481984L;
  public static final long Bit56L = 36028797018963968L;
  public static final long Bit57L = 72057594037927936L;
  public static final long Bit58L = 144115188075855872L;
  public static final long Bit59L = 288230376151711744L;
  public static final long Bit60L = 576460752303423488L;
  public static final long Bit61L = 1152921504606846976L;
  public static final long Bit62L = 2305843009213693952L;
  public static final long Bit63L = 4611686018427387904L;
  public static final long Bit64L = -9223372036854775808L;
  public int bits = -2147483648;
  public static final int ReturnTypeIDMASK = 15;
  public static final int OperatorSHIFT = 6;
  public static final int OperatorMASK = 4032;
  public static final int IsReturnedValue = 16;
  public static final int UnnecessaryCast = 16384;
  public static final int DisableUnnecessaryCastCheck = 32;
  public static final int GenerateCheckcast = 64;
  public static final int UnsafeCast = 128;
  public static final int RestrictiveFlagMASK = 7;
  public static final int FirstAssignmentToLocal = 8;
  public static final int IsImplicitThis = 4;
  public static final int DepthSHIFT = 5;
  public static final int DepthMASK = 8160;
  public static final int IsReachable = -2147483648;
  public static final int LabelUsed = 64;
  public static final int DocumentedFallthrough = 536870912;
  public static final int IsLocalDeclarationReachable = 1073741824;
  public static final int IsSubRoutineEscaping = 16384;
  public static final int IsTryBlockExiting = 536870912;
  public static final int ContainsAssertion = 1;
  public static final int IsLocalType = 256;
  public static final int IsAnonymousType = 512;
  public static final int IsMemberType = 1024;
  public static final int HasAbstractMethods = 2048;
  public static final int IsSecondaryType = 4096;
  public static final int HasBeenGenerated = 8192;
  public static final int HasLocalType = 2;
  public static final int HasBeenResolved = 16;
  public static final int ParenthesizedSHIFT = 21;
  public static final int ParenthesizedMASK = 534773760;
  public static final int IgnoreNoEffectAssignCheck = 536870912;
  public static final int IsStrictlyAssigned = 8192;
  public static final int IsCompoundAssigned = 65536;
  public static final int DiscardEnclosingInstance = 8192;
  public static final int IsUsefulEmptyStatement = 1;
  public static final int UndocumentedEmptyBlock = 8;
  public static final int OverridingMethodWithSupercall = 16;
  public static final int ErrorInSignature = 32;
  public static final int NeedFreeReturn = 64;
  public static final int IsDefaultConstructor = 128;
  public static final int HasAllMethodBodies = 16;
  public static final int IsImplicitUnit = 1;
  public static final int InsideJavadoc = 32768;
  public static final int SuperAccess = 16384;
  public static final int Empty = 262144;
  public static final int IsElseIfStatement = 536870912;
  public static final int ThenExit = 1073741824;
  public static final int IsSuperType = 16;
  public static final int IsVarArgs = 16384;
  public static final int IgnoreRawTypeCheck = 1073741824;
  public static final int IsAnnotationDefaultValue = 1;
  public static final int IsNonNull = 131072;
  public static final int NeededScope = 536870912;
  public static final int OnDemand = 131072;
  public static final int Used = 2;
  public static final int DidResolve = 262144;
  public static final int IsAnySubRoutineEscaping = 536870912;
  public static final int IsSynchronized = 1073741824;
  public static final int BlockExit = 536870912;
  public static final int INVOCATION_ARGUMENT_OK = 0;
  public static final int INVOCATION_ARGUMENT_UNCHECKED = 1;
  public static final int INVOCATION_ARGUMENT_WILDCARD = 2;

  private static int checkInvocationArgument(BlockScope scope, Expression argument, TypeBinding parameterType, TypeBinding argumentType, TypeBinding originalParameterType)
  {
    argument.computeConversion(scope, parameterType, argumentType);

    if ((argumentType != TypeBinding.NULL) && (parameterType.isWildcard())) {
      WildcardBinding wildcard = (WildcardBinding)parameterType;
      if ((wildcard.boundKind != 2) && (wildcard.otherBounds == null))
        return 2;
    }
    TypeBinding checkedParameterType = originalParameterType == null ? parameterType : originalParameterType;
    if ((argumentType != checkedParameterType) && (argumentType.needsUncheckedConversion(checkedParameterType))) {
      scope.problemReporter().unsafeTypeConversion(argument, argumentType, checkedParameterType);
      return 1;
    }
    return 0;
  }
  public static void checkInvocationArguments(BlockScope scope, Expression receiver, TypeBinding receiverType, MethodBinding method, Expression[] arguments, TypeBinding[] argumentTypes, boolean argsContainCast, InvocationSite invocationSite) {
    TypeBinding[] params = method.parameters;
    int paramLength = params.length;
    boolean isRawMemberInvocation = (!method.isStatic()) && 
      (!receiverType.isUnboundWildcard()) && 
      (method.declaringClass.isRawType()) && 
      (method.hasSubstitutedParameters());

    MethodBinding rawOriginalGenericMethod = null;
    if ((!isRawMemberInvocation) && 
      ((method instanceof ParameterizedGenericMethodBinding))) {
      ParameterizedGenericMethodBinding paramMethod = (ParameterizedGenericMethodBinding)method;
      if ((paramMethod.isUnchecked) || ((paramMethod.isRaw) && (method.hasSubstitutedParameters()))) {
        rawOriginalGenericMethod = method.original();
      }
    }

    int invocationStatus = 0;
    if (arguments == null) {
      if (method.isVarargs()) {
        TypeBinding parameterType = ((ArrayBinding)params[(paramLength - 1)]).elementsType();
        if (!parameterType.isReifiable())
          scope.problemReporter().unsafeGenericArrayForVarargs(parameterType, (ASTNode)invocationSite);
      }
    }
    else {
      if (method.isVarargs())
      {
        int lastIndex = paramLength - 1;
        for (int i = 0; i < lastIndex; i++) {
          TypeBinding originalRawParam = rawOriginalGenericMethod == null ? null : rawOriginalGenericMethod.parameters[i];
          invocationStatus |= checkInvocationArgument(scope, arguments[i], params[i], argumentTypes[i], originalRawParam);
        }
        int argLength = arguments.length;
        if (lastIndex < argLength) {
          TypeBinding parameterType = params[lastIndex];
          TypeBinding originalRawParam = null;

          if ((paramLength != argLength) || (parameterType.dimensions() != argumentTypes[lastIndex].dimensions())) {
            parameterType = ((ArrayBinding)parameterType).elementsType();
            if (!parameterType.isReifiable()) {
              scope.problemReporter().unsafeGenericArrayForVarargs(parameterType, (ASTNode)invocationSite);
            }
            originalRawParam = rawOriginalGenericMethod == null ? null : ((ArrayBinding)rawOriginalGenericMethod.parameters[lastIndex]).elementsType();
          }
          for (int i = lastIndex; i < argLength; i++) {
            invocationStatus |= checkInvocationArgument(scope, arguments[i], parameterType, argumentTypes[i], originalRawParam);
          }
        }

        if (paramLength == argumentTypes.length) {
          int varargsIndex = paramLength - 1;
          ArrayBinding varargsType = (ArrayBinding)params[varargsIndex];
          TypeBinding lastArgType = argumentTypes[varargsIndex];

          if (lastArgType == TypeBinding.NULL) {
            if ((!varargsType.leafComponentType().isBaseType()) || (varargsType.dimensions() != 1))
              scope.problemReporter().varargsArgumentNeedCast(method, lastArgType, invocationSite);
          }
          else
          {
            int dimensions;
            if (varargsType.dimensions <= (dimensions = lastArgType.dimensions())) {
              if (lastArgType.leafComponentType().isBaseType()) {
                dimensions--;
              }
              if (varargsType.dimensions < dimensions)
                scope.problemReporter().varargsArgumentNeedCast(method, lastArgType, invocationSite);
              else if ((varargsType.dimensions == dimensions) && 
                (lastArgType != varargsType) && 
                (lastArgType.leafComponentType().erasure() != varargsType.leafComponentType.erasure()) && 
                (lastArgType.isCompatibleWith(varargsType.elementsType())) && 
                (lastArgType.isCompatibleWith(varargsType)))
                scope.problemReporter().varargsArgumentNeedCast(method, lastArgType, invocationSite);
            }
          }
        }
      } else {
        for (int i = 0; i < paramLength; i++) {
          TypeBinding originalRawParam = rawOriginalGenericMethod == null ? null : rawOriginalGenericMethod.parameters[i];
          invocationStatus |= checkInvocationArgument(scope, arguments[i], params[i], argumentTypes[i], originalRawParam);
        }
      }
      if (argsContainCast) {
        CastExpression.checkNeedForArgumentCasts(scope, receiver, receiverType, method, arguments, argumentTypes, invocationSite);
      }
    }
    if ((invocationStatus & 0x2) != 0)
      scope.problemReporter().wildcardInvocation((ASTNode)invocationSite, receiverType, method, argumentTypes);
    else if ((!method.isStatic()) && (!receiverType.isUnboundWildcard()) && (method.declaringClass.isRawType()) && (method.hasSubstitutedParameters()))
      scope.problemReporter().unsafeRawInvocation((ASTNode)invocationSite, method);
    else if (rawOriginalGenericMethod != null)
      scope.problemReporter().unsafeRawGenericMethodInvocation((ASTNode)invocationSite, method);
  }

  public ASTNode concreteStatement() {
    return this;
  }

  public final boolean isFieldUseDeprecated(FieldBinding field, Scope scope, boolean isStrictlyAssigned)
  {
    if ((!isStrictlyAssigned) && ((field.isPrivate()) || ((field.declaringClass != null) && (field.declaringClass.isLocalType()))) && (!scope.isDefinedInField(field)))
    {
      field.original().modifiers |= 134217728;
    }

    if ((field.modifiers & 0x40000) != 0) {
      AccessRestriction restriction = 
        scope.environment().getAccessRestriction(field.declaringClass.erasure());
      if (restriction != null) {
        scope.problemReporter().forbiddenReference(field, this, 
          restriction.getFieldAccessMessageTemplate(), restriction.getProblemId());
      }
    }

    if (!field.isViewedAsDeprecated()) return false;

    if (scope.isDefinedInSameUnit(field.declaringClass)) return false;

    if ((!scope.compilerOptions().reportDeprecationInsideDeprecatedCode) && (scope.isInsideDeprecatedCode())) return false;
    return true;
  }

  public boolean isImplicitThis()
  {
    return false;
  }

  public final boolean isMethodUseDeprecated(MethodBinding method, Scope scope, boolean isExplicitUse)
  {
    if (((method.isPrivate()) || (method.declaringClass.isLocalType())) && (!scope.isDefinedInMethod(method)))
    {
      method.original().modifiers |= 134217728;
    }

    if ((isExplicitUse) && ((method.modifiers & 0x40000) != 0))
    {
      AccessRestriction restriction = 
        scope.environment().getAccessRestriction(method.declaringClass.erasure());
      if (restriction != null) {
        if (method.isConstructor()) {
          scope.problemReporter().forbiddenReference(method, this, 
            restriction.getConstructorAccessMessageTemplate(), 
            restriction.getProblemId());
        }
        else {
          scope.problemReporter().forbiddenReference(method, this, 
            restriction.getMethodAccessMessageTemplate(), 
            restriction.getProblemId());
        }
      }
    }

    if (!method.isViewedAsDeprecated()) return false;

    if (scope.isDefinedInSameUnit(method.declaringClass)) return false;

    if ((!isExplicitUse) && 
      ((method.modifiers & 0x100000) == 0)) {
      return false;
    }

    if ((!scope.compilerOptions().reportDeprecationInsideDeprecatedCode) && (scope.isInsideDeprecatedCode())) return false;
    return true;
  }

  public boolean isSuper()
  {
    return false;
  }

  public boolean isThis()
  {
    return false;
  }

  public final boolean isTypeUseDeprecated(TypeBinding type, Scope scope)
  {
    if (type.isArrayType())
      type = ((ArrayBinding)type).leafComponentType;
    if (type.isBaseType()) {
      return false;
    }
    ReferenceBinding refType = (ReferenceBinding)type;

    if (((refType.isPrivate()) || (refType.isLocalType())) && (!scope.isDefinedInType(refType)))
    {
      ((ReferenceBinding)refType.erasure()).modifiers |= 134217728;
    }

    if (refType.hasRestrictedAccess()) {
      AccessRestriction restriction = scope.environment().getAccessRestriction(type.erasure());
      if (restriction != null) {
        scope.problemReporter().forbiddenReference(type, this, restriction.getMessageTemplate(), restriction.getProblemId());
      }

    }

    refType.initializeDeprecatedAnnotationTagBits();

    if (!refType.isViewedAsDeprecated()) return false;

    if (scope.isDefinedInSameUnit(refType)) return false;

    if ((!scope.compilerOptions().reportDeprecationInsideDeprecatedCode) && (scope.isInsideDeprecatedCode())) return false;
    return true;
  }

  public abstract StringBuffer print(int paramInt, StringBuffer paramStringBuffer);

  public static StringBuffer printAnnotations(Annotation[] annotations, StringBuffer output) {
    int length = annotations.length;
    for (int i = 0; i < length; i++) {
      annotations[i].print(0, output);
      output.append(" ");
    }
    return output;
  }

  public static StringBuffer printIndent(int indent, StringBuffer output)
  {
    for (int i = indent; i > 0; i--) output.append("  ");
    return output;
  }

  public static StringBuffer printModifiers(int modifiers, StringBuffer output)
  {
    if ((modifiers & 0x1) != 0)
      output.append("public ");
    if ((modifiers & 0x2) != 0)
      output.append("private ");
    if ((modifiers & 0x4) != 0)
      output.append("protected ");
    if ((modifiers & 0x8) != 0)
      output.append("static ");
    if ((modifiers & 0x10) != 0)
      output.append("final ");
    if ((modifiers & 0x20) != 0)
      output.append("synchronized ");
    if ((modifiers & 0x40) != 0)
      output.append("volatile ");
    if ((modifiers & 0x80) != 0)
      output.append("transient ");
    if ((modifiers & 0x100) != 0)
      output.append("native ");
    if ((modifiers & 0x400) != 0)
      output.append("abstract ");
    return output;
  }

  public static void resolveAnnotations(BlockScope scope, Annotation[] annotations, Binding recipient)
  {
    AnnotationBinding[] instances = (AnnotationBinding[])null;
    int length = annotations == null ? 0 : annotations.length;
    if (recipient != null) {
      switch (recipient.kind()) {
      case 16:
        PackageBinding packageBinding = (PackageBinding)recipient;
        if ((packageBinding.tagBits & 0x0) != 0L) return;
        packageBinding.tagBits |= 25769803776L;
        break;
      case 4:
      case 2052:
        ReferenceBinding type = (ReferenceBinding)recipient;
        if ((type.tagBits & 0x0) != 0L) return;
        type.tagBits |= 25769803776L;
        if (length > 0) {
          instances = new AnnotationBinding[length];
          type.setAnnotations(instances);
        }
        break;
      case 8:
        MethodBinding method = (MethodBinding)recipient;
        if ((method.tagBits & 0x0) != 0L) return;
        method.tagBits |= 25769803776L;
        if (length > 0) {
          instances = new AnnotationBinding[length];
          method.setAnnotations(instances);
        }
        break;
      case 1:
        FieldBinding field = (FieldBinding)recipient;
        if ((field.tagBits & 0x0) != 0L) return;
        field.tagBits |= 25769803776L;
        if (length > 0) {
          instances = new AnnotationBinding[length];
          field.setAnnotations(instances);
        }
        break;
      case 2:
        LocalVariableBinding local = (LocalVariableBinding)recipient;
        if ((local.tagBits & 0x0) != 0L) return;
        local.tagBits |= 25769803776L;
        if (length > 0) {
          instances = new AnnotationBinding[length];
          local.setAnnotations(instances);
        }
        break;
      default:
        return;
      }
    }
    if (annotations == null)
      return;
    TypeBinding[] annotationTypes = new TypeBinding[length];
    for (int i = 0; i < length; i++) {
      Annotation annotation = annotations[i];
      Binding annotationRecipient = annotation.recipient;
      if ((annotationRecipient != null) && (recipient != null))
      {
        switch (recipient.kind()) {
        case 1:
          FieldBinding field = (FieldBinding)recipient;
          field.tagBits = ((FieldBinding)annotationRecipient).tagBits;
          break;
        case 2:
          LocalVariableBinding local = (LocalVariableBinding)recipient;
          local.tagBits = ((LocalVariableBinding)annotationRecipient).tagBits;
        }

        if (instances != null)
        {
          instances[0] = annotation.getCompilerAnnotation();
          for (int j = 1; j < length; j++) {
            Annotation annot = annotations[j];
            instances[j] = annot.getCompilerAnnotation();
          }
        }
        return;
      }
      annotation.recipient = recipient;
      annotationTypes[i] = annotation.resolveType(scope);

      if (instances != null) {
        instances[i] = annotation.getCompilerAnnotation();
      }

    }

    for (int i = 0; i < length; i++) {
      TypeBinding annotationType = annotationTypes[i];
      if (annotationType != null) {
        boolean foundDuplicate = false;
        for (int j = i + 1; j < length; j++) {
          if (annotationTypes[j] == annotationType) {
            foundDuplicate = true;
            annotationTypes[j] = null;
            scope.problemReporter().duplicateAnnotation(annotations[j]);
          }
        }
        if (foundDuplicate)
          scope.problemReporter().duplicateAnnotation(annotations[i]);
      }
    }
  }

  public static void resolveDeprecatedAnnotations(BlockScope scope, Annotation[] annotations, Binding recipient)
  {
    if (recipient != null) {
      int kind = recipient.kind();
      if (annotations != null)
      {
        int length;
        if ((length = annotations.length) >= 0) {
          switch (kind) {
          case 16:
            PackageBinding packageBinding = (PackageBinding)recipient;
            if ((packageBinding.tagBits & 0x0) != 0L) return;
            break;
          case 4:
          case 2052:
            ReferenceBinding type = (ReferenceBinding)recipient;
            if ((type.tagBits & 0x0) != 0L) return;
            break;
          case 8:
            MethodBinding method = (MethodBinding)recipient;
            if ((method.tagBits & 0x0) != 0L) return;
            break;
          case 1:
            FieldBinding field = (FieldBinding)recipient;
            if ((field.tagBits & 0x0) != 0L) return;
            break;
          case 2:
            LocalVariableBinding local = (LocalVariableBinding)recipient;
            if ((local.tagBits & 0x0) != 0L) return;
            break;
          default:
            return;
          }
          for (int i = 0; i < length; i++) {
            TypeReference annotationTypeRef = annotations[i].type;

            if (!CharOperation.equals(TypeIds.JAVA_LANG_DEPRECATED[2], annotationTypeRef.getLastToken())) return;
            TypeBinding annotationType = annotations[i].type.resolveType(scope);
            if ((annotationType != null) && (annotationType.isValidBinding()) && (annotationType.id == 44)) {
              switch (kind) {
              case 16:
                PackageBinding packageBinding = (PackageBinding)recipient;
                packageBinding.tagBits |= 70385924046848L;
                return;
              case 4:
              case 2052:
              case 4100:
                ReferenceBinding type = (ReferenceBinding)recipient;
                type.tagBits |= 70385924046848L;
                return;
              case 8:
                MethodBinding method = (MethodBinding)recipient;
                method.tagBits |= 70385924046848L;
                return;
              case 1:
                FieldBinding field = (FieldBinding)recipient;
                field.tagBits |= 70385924046848L;
                return;
              case 2:
                LocalVariableBinding local = (LocalVariableBinding)recipient;
                local.tagBits |= 70385924046848L;
                return;
              }
              return;
            }
          }
        }
      }

      switch (kind) {
      case 16:
        PackageBinding packageBinding = (PackageBinding)recipient;
        packageBinding.tagBits |= 17179869184L;
        return;
      case 4:
      case 2052:
      case 4100:
        ReferenceBinding type = (ReferenceBinding)recipient;
        type.tagBits |= 17179869184L;
        return;
      case 8:
        MethodBinding method = (MethodBinding)recipient;
        method.tagBits |= 17179869184L;
        return;
      case 1:
        FieldBinding field = (FieldBinding)recipient;
        field.tagBits |= 17179869184L;
        return;
      case 2:
        LocalVariableBinding local = (LocalVariableBinding)recipient;
        local.tagBits |= 17179869184L;
        return;
      }
      return;
    }
  }

  public int sourceStart()
  {
    return this.sourceStart;
  }
  public int sourceEnd() {
    return this.sourceEnd;
  }

  public String toString() {
    return print(0, new StringBuffer(30)).toString();
  }

  public void traverse(ASTVisitor visitor, BlockScope scope)
  {
  }
}