package org.eclipse.jdt.internal.compiler.ast;

import org.eclipse.jdt.internal.compiler.ASTVisitor;
import org.eclipse.jdt.internal.compiler.codegen.CodeStream;
import org.eclipse.jdt.internal.compiler.impl.FloatConstant;
import org.eclipse.jdt.internal.compiler.lookup.BlockScope;
import org.eclipse.jdt.internal.compiler.lookup.TypeBinding;
import org.eclipse.jdt.internal.compiler.util.FloatUtil;

public class FloatLiteral extends NumberLiteral
{
  float value;
  static final float Float_MIN_VALUE = Float.intBitsToFloat(1);

  public FloatLiteral(char[] token, int s, int e) { super(token, s, e); }

  public void computeConstant()
  {
    try {
      computedValue = Float.valueOf(String.valueOf(this.source));
    }
    catch (NumberFormatException localNumberFormatException1)
    {
      try
      {
        Float computedValue;
        float v = FloatUtil.valueOfHexFloatLiteral(this.source);
        if (v == (1.0F / 1.0F))
        {
          return;
        }
        if (Float.isNaN(v))
        {
          return;
        }
        this.value = v;
        this.constant = FloatConstant.fromValue(v);
      }
      catch (NumberFormatException localNumberFormatException2)
      {
      }
      return;
    }
    Float computedValue;
    float floatValue = computedValue.floatValue();
    if (floatValue > 3.4028235E+38F)
    {
      return;
    }
    if (floatValue < 1.4E-45F)
    {
      boolean isHexaDecimal = false;
      for (int i = 0; i < this.source.length; i++) {
        switch (this.source[i]) {
        case '.':
        case '0':
          break;
        case 'X':
        case 'x':
          isHexaDecimal = true;
          break;
        case 'D':
        case 'E':
        case 'F':
        case 'd':
        case 'e':
        case 'f':
          if (!isHexaDecimal) break label230;
          return;
        case 'P':
        case 'p':
          break;
        default:
          return;
        }
      }
    }
    label230: this.value = floatValue;
    this.constant = FloatConstant.fromValue(this.value);
  }

  public void generateCode(BlockScope currentScope, CodeStream codeStream, boolean valueRequired)
  {
    int pc = codeStream.position;
    if (valueRequired) {
      codeStream.generateConstant(this.constant, this.implicitConversion);
    }
    codeStream.recordPositionsFrom(pc, this.sourceStart);
  }
  public TypeBinding literalType(BlockScope scope) {
    return TypeBinding.FLOAT;
  }
  public void traverse(ASTVisitor visitor, BlockScope scope) {
    visitor.visit(this, scope);
    visitor.endVisit(this, scope);
  }
}