package org.eclipse.jdt.internal.compiler.classfmt;

import org.eclipse.jdt.core.compiler.CharOperation;
import org.eclipse.jdt.internal.compiler.ast.Annotation;
import org.eclipse.jdt.internal.compiler.codegen.ConstantPool;
import org.eclipse.jdt.internal.compiler.env.ClassSignature;
import org.eclipse.jdt.internal.compiler.env.EnumConstantSignature;
import org.eclipse.jdt.internal.compiler.env.IBinaryAnnotation;
import org.eclipse.jdt.internal.compiler.env.IBinaryElementValuePair;
import org.eclipse.jdt.internal.compiler.impl.BooleanConstant;
import org.eclipse.jdt.internal.compiler.impl.ByteConstant;
import org.eclipse.jdt.internal.compiler.impl.CharConstant;
import org.eclipse.jdt.internal.compiler.impl.DoubleConstant;
import org.eclipse.jdt.internal.compiler.impl.FloatConstant;
import org.eclipse.jdt.internal.compiler.impl.IntConstant;
import org.eclipse.jdt.internal.compiler.impl.LongConstant;
import org.eclipse.jdt.internal.compiler.impl.ShortConstant;
import org.eclipse.jdt.internal.compiler.impl.StringConstant;

public class AnnotationInfo extends ClassFileStruct
  implements IBinaryAnnotation
{
  private char[] typename;
  private ElementValuePairInfo[] pairs;
  long standardAnnotationTagBits = 0L;
  int readOffset = 0;

  static Object[] EmptyValueArray = new Object[0];

  AnnotationInfo(byte[] classFileBytes, int[] contantPoolOffsets, int offset) {
    super(classFileBytes, contantPoolOffsets, offset);
  }

  AnnotationInfo(byte[] classFileBytes, int[] contantPoolOffsets, int offset, boolean runtimeVisible, boolean populate)
  {
    this(classFileBytes, contantPoolOffsets, offset);
    if (populate)
      decodeAnnotation();
    else
      this.readOffset = scanAnnotation(0, runtimeVisible, true); 
  }

  private void decodeAnnotation() { this.readOffset = 0;
    int utf8Offset = this.constantPoolOffsets[u2At(0)] - this.structOffset;
    this.typename = utf8At(utf8Offset + 3, u2At(utf8Offset + 1));
    int numberOfPairs = u2At(2);

    this.readOffset += 4;
    this.pairs = (numberOfPairs == 0 ? ElementValuePairInfo.NoMembers : new ElementValuePairInfo[numberOfPairs]);
    for (int i = 0; i < numberOfPairs; i++)
    {
      utf8Offset = this.constantPoolOffsets[u2At(this.readOffset)] - this.structOffset;
      char[] membername = utf8At(utf8Offset + 3, u2At(utf8Offset + 1));
      this.readOffset += 2;
      Object value = decodeDefaultValue();
      this.pairs[i] = new ElementValuePairInfo(membername, value);
    } }

  Object decodeDefaultValue() {
    Object value = null;

    int tag = u1At(this.readOffset);
    this.readOffset += 1;
    int constValueOffset = -1;
    switch (tag) {
    case 90:
      constValueOffset = this.constantPoolOffsets[u2At(this.readOffset)] - this.structOffset;
      value = BooleanConstant.fromValue(i4At(constValueOffset + 1) == 1);
      this.readOffset += 2;
      break;
    case 73:
      constValueOffset = this.constantPoolOffsets[u2At(this.readOffset)] - this.structOffset;
      value = IntConstant.fromValue(i4At(constValueOffset + 1));
      this.readOffset += 2;
      break;
    case 67:
      constValueOffset = this.constantPoolOffsets[u2At(this.readOffset)] - this.structOffset;
      value = CharConstant.fromValue((char)i4At(constValueOffset + 1));
      this.readOffset += 2;
      break;
    case 66:
      constValueOffset = this.constantPoolOffsets[u2At(this.readOffset)] - this.structOffset;
      value = ByteConstant.fromValue((byte)i4At(constValueOffset + 1));
      this.readOffset += 2;
      break;
    case 83:
      constValueOffset = this.constantPoolOffsets[u2At(this.readOffset)] - this.structOffset;
      value = ShortConstant.fromValue((short)i4At(constValueOffset + 1));
      this.readOffset += 2;
      break;
    case 68:
      constValueOffset = this.constantPoolOffsets[u2At(this.readOffset)] - this.structOffset;
      value = DoubleConstant.fromValue(doubleAt(constValueOffset + 1));
      this.readOffset += 2;
      break;
    case 70:
      constValueOffset = this.constantPoolOffsets[u2At(this.readOffset)] - this.structOffset;
      value = FloatConstant.fromValue(floatAt(constValueOffset + 1));
      this.readOffset += 2;
      break;
    case 74:
      constValueOffset = this.constantPoolOffsets[u2At(this.readOffset)] - this.structOffset;
      value = LongConstant.fromValue(i8At(constValueOffset + 1));
      this.readOffset += 2;
      break;
    case 115:
      constValueOffset = this.constantPoolOffsets[u2At(this.readOffset)] - this.structOffset;
      value = StringConstant.fromValue(String.valueOf(utf8At(constValueOffset + 3, u2At(constValueOffset + 1))));
      this.readOffset += 2;
      break;
    case 101:
      constValueOffset = this.constantPoolOffsets[u2At(this.readOffset)] - this.structOffset;
      char[] typeName = utf8At(constValueOffset + 3, u2At(constValueOffset + 1));
      this.readOffset += 2;
      constValueOffset = this.constantPoolOffsets[u2At(this.readOffset)] - this.structOffset;
      char[] constName = utf8At(constValueOffset + 3, u2At(constValueOffset + 1));
      this.readOffset += 2;
      value = new EnumConstantSignature(typeName, constName);
      break;
    case 99:
      constValueOffset = this.constantPoolOffsets[u2At(this.readOffset)] - this.structOffset;
      char[] className = utf8At(constValueOffset + 3, u2At(constValueOffset + 1));
      value = new ClassSignature(className);
      this.readOffset += 2;
      break;
    case 64:
      value = new AnnotationInfo(this.reference, this.constantPoolOffsets, this.readOffset + this.structOffset, false, true);
      this.readOffset += ((AnnotationInfo)value).readOffset;
      break;
    case 91:
      int numberOfValues = u2At(this.readOffset);
      this.readOffset += 2;
      if (numberOfValues == 0) {
        value = EmptyValueArray;
      } else {
        Object[] arrayElements = new Object[numberOfValues];
        value = arrayElements;
        for (int i = 0; i < numberOfValues; i++)
          arrayElements[i] = decodeDefaultValue();
      }
      break;
    default:
      throw new IllegalStateException("Unrecognized tag " + (char)tag);
    }
    return value;
  }
  public IBinaryElementValuePair[] getElementValuePairs() {
    if (this.pairs == null)
      initialize();
    return this.pairs;
  }
  public char[] getTypeName() {
    return this.typename;
  }
  void initialize() {
    if (this.pairs == null)
      decodeAnnotation(); 
  }

  private int readRetentionPolicy(int offset) { int currentOffset = offset;
    int tag = u1At(currentOffset);
    currentOffset++;
    switch (tag) {
    case 101:
      int utf8Offset = this.constantPoolOffsets[u2At(currentOffset)] - this.structOffset;
      char[] typeName = utf8At(utf8Offset + 3, u2At(utf8Offset + 1));
      currentOffset += 2;
      if ((typeName.length == 38) && (CharOperation.equals(typeName, ConstantPool.JAVA_LANG_ANNOTATION_RETENTIONPOLICY))) {
        utf8Offset = this.constantPoolOffsets[u2At(currentOffset)] - this.structOffset;
        char[] constName = utf8At(utf8Offset + 3, u2At(utf8Offset + 1));
        this.standardAnnotationTagBits |= Annotation.getRetentionPolicy(constName);
      }
      currentOffset += 2;
      break;
    case 66:
    case 67:
    case 68:
    case 70:
    case 73:
    case 74:
    case 83:
    case 90:
    case 99:
    case 115:
      currentOffset += 2;
      break;
    case 64:
      currentOffset = scanAnnotation(currentOffset, false, false);
      break;
    case 91:
      int numberOfValues = u2At(currentOffset);
      currentOffset += 2;
      for (int i = 0; i < numberOfValues; i++)
        currentOffset = scanElementValue(currentOffset);
      break;
    default:
      throw new IllegalStateException();
    }
    return currentOffset; }

  private int readTargetValue(int offset) {
    int currentOffset = offset;
    int tag = u1At(currentOffset);
    currentOffset++;
    switch (tag) {
    case 101:
      int utf8Offset = this.constantPoolOffsets[u2At(currentOffset)] - this.structOffset;
      char[] typeName = utf8At(utf8Offset + 3, u2At(utf8Offset + 1));
      currentOffset += 2;
      if ((typeName.length == 34) && (CharOperation.equals(typeName, ConstantPool.JAVA_LANG_ANNOTATION_ELEMENTTYPE))) {
        utf8Offset = this.constantPoolOffsets[u2At(currentOffset)] - this.structOffset;
        char[] constName = utf8At(utf8Offset + 3, u2At(utf8Offset + 1));
        this.standardAnnotationTagBits |= Annotation.getTargetElementType(constName);
      }
      currentOffset += 2;
      break;
    case 66:
    case 67:
    case 68:
    case 70:
    case 73:
    case 74:
    case 83:
    case 90:
    case 99:
    case 115:
      currentOffset += 2;
      break;
    case 64:
      currentOffset = scanAnnotation(currentOffset, false, false);
      break;
    case 91:
      int numberOfValues = u2At(currentOffset);
      currentOffset += 2;
      if (numberOfValues == 0)
        this.standardAnnotationTagBits |= 34359738368L;
      else {
        for (int i = 0; i < numberOfValues; i++)
          currentOffset = readTargetValue(currentOffset);
      }
      break;
    default:
      throw new IllegalStateException();
    }
    return currentOffset;
  }

  private int scanAnnotation(int offset, boolean expectRuntimeVisibleAnno, boolean toplevel)
  {
    int currentOffset = offset;
    int utf8Offset = this.constantPoolOffsets[u2At(offset)] - this.structOffset;
    char[] typeName = utf8At(utf8Offset + 3, u2At(utf8Offset + 1));
    if (toplevel)
      this.typename = typeName;
    int numberOfPairs = u2At(offset + 2);

    currentOffset += 4;
    if ((expectRuntimeVisibleAnno) && (toplevel)) {
      switch (typeName.length) {
      case 22:
        if (CharOperation.equals(typeName, ConstantPool.JAVA_LANG_DEPRECATED)) {
          this.standardAnnotationTagBits |= 70368744177664L;
          return currentOffset;
        }
        break;
      case 29:
        if (CharOperation.equals(typeName, ConstantPool.JAVA_LANG_ANNOTATION_TARGET)) {
          currentOffset += 2;
          return readTargetValue(currentOffset);
        }
        break;
      case 33:
        if (CharOperation.equals(typeName, ConstantPool.JAVA_LANG_ANNOTATION_DOCUMENTED)) {
          this.standardAnnotationTagBits |= 140737488355328L;
          return currentOffset;
        }
        break;
      case 32:
        if (CharOperation.equals(typeName, ConstantPool.JAVA_LANG_ANNOTATION_RETENTION)) {
          currentOffset += 2;
          return readRetentionPolicy(currentOffset);
        }
        if (CharOperation.equals(typeName, ConstantPool.JAVA_LANG_ANNOTATION_INHERITED)) {
          this.standardAnnotationTagBits |= 281474976710656L;
          return currentOffset;
        }
        break;
      }
    }
    for (int i = 0; i < numberOfPairs; i++)
    {
      currentOffset += 2;
      currentOffset = scanElementValue(currentOffset);
    }
    return currentOffset;
  }

  private int scanElementValue(int offset)
  {
    int currentOffset = offset;
    int tag = u1At(currentOffset);
    currentOffset++;
    switch (tag) {
    case 66:
    case 67:
    case 68:
    case 70:
    case 73:
    case 74:
    case 83:
    case 90:
    case 99:
    case 115:
      currentOffset += 2;
      break;
    case 101:
      currentOffset += 4;
      break;
    case 64:
      currentOffset = scanAnnotation(currentOffset, false, false);
      break;
    case 91:
      int numberOfValues = u2At(currentOffset);
      currentOffset += 2;
      for (int i = 0; i < numberOfValues; i++)
        currentOffset = scanElementValue(currentOffset);
      break;
    default:
      throw new IllegalStateException();
    }
    return currentOffset;
  }
  public String toString() {
    StringBuffer buffer = new StringBuffer();
    buffer.append('@');
    buffer.append(this.typename);
    if (this.pairs != null) {
      buffer.append('(');
      buffer.append("\n\t");
      int i = 0; for (int len = this.pairs.length; i < len; i++) {
        if (i > 0)
          buffer.append(",\n\t");
        buffer.append(this.pairs[i]);
      }
      buffer.append(')');
    }
    return buffer.toString();
  }
}