package org.eclipse.jdt.internal.compiler.codegen;

import org.eclipse.jdt.core.compiler.CharOperation;
import org.eclipse.jdt.internal.compiler.ClassFile;
import org.eclipse.jdt.internal.compiler.CompilationResult;
import org.eclipse.jdt.internal.compiler.ast.ASTNode;
import org.eclipse.jdt.internal.compiler.ast.AbstractMethodDeclaration;
import org.eclipse.jdt.internal.compiler.ast.AllocationExpression;
import org.eclipse.jdt.internal.compiler.ast.CompilationUnitDeclaration;
import org.eclipse.jdt.internal.compiler.ast.ExplicitConstructorCall;
import org.eclipse.jdt.internal.compiler.ast.Expression;
import org.eclipse.jdt.internal.compiler.ast.TypeDeclaration;
import org.eclipse.jdt.internal.compiler.impl.CompilerOptions;
import org.eclipse.jdt.internal.compiler.impl.Constant;
import org.eclipse.jdt.internal.compiler.lookup.ArrayBinding;
import org.eclipse.jdt.internal.compiler.lookup.Binding;
import org.eclipse.jdt.internal.compiler.lookup.BlockScope;
import org.eclipse.jdt.internal.compiler.lookup.ClassScope;
import org.eclipse.jdt.internal.compiler.lookup.FieldBinding;
import org.eclipse.jdt.internal.compiler.lookup.LocalVariableBinding;
import org.eclipse.jdt.internal.compiler.lookup.MethodBinding;
import org.eclipse.jdt.internal.compiler.lookup.MethodScope;
import org.eclipse.jdt.internal.compiler.lookup.NestedTypeBinding;
import org.eclipse.jdt.internal.compiler.lookup.ReferenceBinding;
import org.eclipse.jdt.internal.compiler.lookup.Scope;
import org.eclipse.jdt.internal.compiler.lookup.SourceTypeBinding;
import org.eclipse.jdt.internal.compiler.lookup.SyntheticArgumentBinding;
import org.eclipse.jdt.internal.compiler.lookup.SyntheticMethodBinding;
import org.eclipse.jdt.internal.compiler.lookup.TypeBinding;
import org.eclipse.jdt.internal.compiler.lookup.TypeConstants;
import org.eclipse.jdt.internal.compiler.lookup.VariableBinding;
import org.eclipse.jdt.internal.compiler.problem.AbortMethod;
import org.eclipse.jdt.internal.compiler.problem.ProblemReporter;
import org.eclipse.jdt.internal.compiler.util.Util;

public class CodeStream
{
  public static final boolean DEBUG = false;
  public static FieldBinding[] ImplicitThis = new FieldBinding[0];
  public static final int LABELS_INCREMENT = 5;
  public static final int LOCALS_INCREMENT = 10;
  static ExceptionLabel[] noExceptionHandlers = new ExceptionLabel[5];
  static BranchLabel[] noLabels = new BranchLabel[5];
  static LocalVariableBinding[] noLocals = new LocalVariableBinding[10];
  static LocalVariableBinding[] noVisibleLocals = new LocalVariableBinding[10];
  public static final CompilationResult RESTART_IN_WIDE_MODE = new CompilationResult(null, 0, 0, 0);
  public int allLocalsCounter;
  public byte[] bCodeStream;
  public ClassFile classFile;
  public int classFileOffset;
  public ConstantPool constantPool;
  public int countLabels;
  public ExceptionLabel[] exceptionLabels = new ExceptionLabel[5];
  public int exceptionLabelsCounter;
  public int generateAttributes;
  static final int L_UNKNOWN = 0;
  static final int L_OPTIMIZABLE = 2;
  static final int L_CANNOT_OPTIMIZE = 4;
  public BranchLabel[] labels = new BranchLabel[5];
  public int lastEntryPC;
  public int lastAbruptCompletion;
  public int[] lineSeparatorPositions;
  public LocalVariableBinding[] locals = new LocalVariableBinding[10];
  public int maxFieldCount;
  public int maxLocals;
  public AbstractMethodDeclaration methodDeclaration;
  public int[] pcToSourceMap = new int[24];
  public int pcToSourceMapSize;
  public int position;
  public boolean preserveUnusedLocals;
  public int stackDepth;
  public int stackMax;
  public int startingClassFileOffset;
  private long targetLevel;
  public LocalVariableBinding[] visibleLocals = new LocalVariableBinding[10];
  int visibleLocalsCount;
  public boolean wideMode = false;

  public static int insertionIndex(int[] pcToSourceMap, int length, int pc)
  {
    int g = 0;
    int d = length - 2;
    int m = 0;
    while (g <= d) {
      m = (g + d) / 2;

      if ((m & 0x1) != 0)
        m--;
      int currentPC = pcToSourceMap[m];
      if (pc < currentPC) {
        d = m - 2;
      }
      else if (pc > currentPC)
        g = m + 2;
      else {
        return -1;
      }
    }
    if (pc < pcToSourceMap[m])
      return m;
    return m + 2;
  }
  public static final void sort(int[] tab, int lo0, int hi0, int[] result) {
    int lo = lo0;
    int hi = hi0;

    if (hi0 > lo0)
    {
      int mid = tab[(lo0 + (hi0 - lo0) / 2)];

      for (goto 86; lo <= hi; )
      {
        do
        {
          if (lo >= hi0) break;  } while (tab[lo] < mid);

        while ((hi > lo0) && (tab[hi] > mid)) {
          hi--;
        }
        if (lo <= hi) {
          swap(tab, lo, hi, result);
          lo++;
          hi--;
        }

      }

      if (lo0 < hi) {
        sort(tab, lo0, hi, result);
      }

      if (lo < hi0)
        sort(tab, lo, hi0, result);
    }
  }

  private static final void swap(int[] a, int i, int j, int[] result) {
    int T = a[i];
    a[i] = a[j];
    a[j] = T;
    T = result[j];
    result[j] = result[i];
    result[i] = T;
  }

  public CodeStream(ClassFile givenClassFile)
  {
    this.targetLevel = givenClassFile.targetJDK;
    this.generateAttributes = givenClassFile.produceAttributes;
    if ((givenClassFile.produceAttributes & 0x2) != 0)
      this.lineSeparatorPositions = givenClassFile.referenceBinding.scope.referenceCompilationUnit().compilationResult.getLineSeparatorPositions();
  }

  public void aaload()
  {
    this.countLabels = 0;
    this.stackDepth -= 1;
    if (this.classFileOffset >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 1;
    this.bCodeStream[(this.classFileOffset++)] = 50;
  }

  public void aastore() {
    this.countLabels = 0;
    this.stackDepth -= 3;
    if (this.classFileOffset >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 1;
    this.bCodeStream[(this.classFileOffset++)] = 83;
  }

  public void aconst_null() {
    this.countLabels = 0;
    this.stackDepth += 1;
    if (this.stackDepth > this.stackMax) {
      this.stackMax = this.stackDepth;
    }
    if (this.classFileOffset >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 1;
    this.bCodeStream[(this.classFileOffset++)] = 1;
  }

  public void addDefinitelyAssignedVariables(Scope scope, int initStateIndex) {
    if ((this.generateAttributes & 0xC) == 0)
      return;
    for (int i = 0; i < this.visibleLocalsCount; i++) {
      LocalVariableBinding localBinding = this.visibleLocals[i];
      if (localBinding != null)
      {
        if ((isDefinitelyAssigned(scope, initStateIndex, localBinding)) && (
          (localBinding.initializationCount == 0) || (localBinding.initializationPCs[((localBinding.initializationCount - 1 << 1) + 1)] != -1)))
        {
          localBinding.recordInitializationStartPC(this.position);
        }
      }
    }
  }

  public void addLabel(BranchLabel aLabel) {
    if (this.countLabels == this.labels.length)
      System.arraycopy(this.labels, 0, this.labels = new BranchLabel[this.countLabels + 5], 0, this.countLabels);
    this.labels[(this.countLabels++)] = aLabel;
  }
  public void addVisibleLocalVariable(LocalVariableBinding localBinding) {
    if ((this.generateAttributes & 0xC) == 0) {
      return;
    }
    if (this.visibleLocalsCount >= this.visibleLocals.length)
      System.arraycopy(this.visibleLocals, 0, this.visibleLocals = new LocalVariableBinding[this.visibleLocalsCount * 2], 0, this.visibleLocalsCount);
    this.visibleLocals[(this.visibleLocalsCount++)] = localBinding;
  }

  public void addVariable(LocalVariableBinding localBinding)
  {
  }

  public void aload(int iArg) {
    this.countLabels = 0;
    this.stackDepth += 1;
    if (this.stackDepth > this.stackMax)
      this.stackMax = this.stackDepth;
    if (this.maxLocals <= iArg) {
      this.maxLocals = (iArg + 1);
    }
    if (iArg > 255) {
      if (this.classFileOffset + 3 >= this.bCodeStream.length) {
        resizeByteArray();
      }
      this.position += 2;
      this.bCodeStream[(this.classFileOffset++)] = -60;
      this.bCodeStream[(this.classFileOffset++)] = 25;
      writeUnsignedShort(iArg);
    }
    else {
      if (this.classFileOffset + 1 >= this.bCodeStream.length) {
        resizeByteArray();
      }
      this.position += 2;
      this.bCodeStream[(this.classFileOffset++)] = 25;
      this.bCodeStream[(this.classFileOffset++)] = ((byte)iArg);
    }
  }

  public void aload_0() {
    this.countLabels = 0;
    this.stackDepth += 1;
    if (this.stackDepth > this.stackMax) {
      this.stackMax = this.stackDepth;
    }
    if (this.maxLocals == 0) {
      this.maxLocals = 1;
    }
    if (this.classFileOffset >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 1;
    this.bCodeStream[(this.classFileOffset++)] = 42;
  }

  public void aload_1() {
    this.countLabels = 0;
    this.stackDepth += 1;
    if (this.stackDepth > this.stackMax)
      this.stackMax = this.stackDepth;
    if (this.maxLocals <= 1) {
      this.maxLocals = 2;
    }
    if (this.classFileOffset >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 1;
    this.bCodeStream[(this.classFileOffset++)] = 43;
  }

  public void aload_2() {
    this.countLabels = 0;
    this.stackDepth += 1;
    if (this.stackDepth > this.stackMax)
      this.stackMax = this.stackDepth;
    if (this.maxLocals <= 2) {
      this.maxLocals = 3;
    }
    if (this.classFileOffset >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 1;
    this.bCodeStream[(this.classFileOffset++)] = 44;
  }

  public void aload_3() {
    this.countLabels = 0;
    this.stackDepth += 1;
    if (this.stackDepth > this.stackMax)
      this.stackMax = this.stackDepth;
    if (this.maxLocals <= 3) {
      this.maxLocals = 4;
    }
    if (this.classFileOffset >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 1;
    this.bCodeStream[(this.classFileOffset++)] = 45;
  }

  public void anewarray(TypeBinding typeBinding) {
    this.countLabels = 0;
    if (this.classFileOffset + 2 >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 1;
    this.bCodeStream[(this.classFileOffset++)] = -67;
    writeUnsignedShort(this.constantPool.literalIndexForType(typeBinding));
  }

  public void areturn() {
    this.countLabels = 0;
    this.stackDepth -= 1;

    if (this.classFileOffset >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 1;
    this.bCodeStream[(this.classFileOffset++)] = -80;
    this.lastAbruptCompletion = this.position;
  }
  public void arrayAt(int typeBindingID) {
    switch (typeBindingID) {
    case 10:
      iaload();
      break;
    case 3:
    case 5:
      baload();
      break;
    case 4:
      saload();
      break;
    case 2:
      caload();
      break;
    case 7:
      laload();
      break;
    case 9:
      faload();
      break;
    case 8:
      daload();
      break;
    case 6:
    default:
      aaload();
    }
  }

  public void arrayAtPut(int elementTypeID, boolean valueRequired) { switch (elementTypeID) {
    case 10:
      if (valueRequired)
        dup_x2();
      iastore();
      break;
    case 3:
    case 5:
      if (valueRequired)
        dup_x2();
      bastore();
      break;
    case 4:
      if (valueRequired)
        dup_x2();
      sastore();
      break;
    case 2:
      if (valueRequired)
        dup_x2();
      castore();
      break;
    case 7:
      if (valueRequired)
        dup2_x2();
      lastore();
      break;
    case 9:
      if (valueRequired)
        dup_x2();
      fastore();
      break;
    case 8:
      if (valueRequired)
        dup2_x2();
      dastore();
      break;
    case 6:
    default:
      if (valueRequired)
        dup_x2();
      aastore();
    } }

  public void arraylength()
  {
    this.countLabels = 0;
    if (this.classFileOffset >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 1;
    this.bCodeStream[(this.classFileOffset++)] = -66;
  }

  public void astore(int iArg) {
    this.countLabels = 0;
    this.stackDepth -= 1;
    if (this.maxLocals <= iArg) {
      this.maxLocals = (iArg + 1);
    }
    if (iArg > 255) {
      if (this.classFileOffset + 3 >= this.bCodeStream.length) {
        resizeByteArray();
      }
      this.position += 2;
      this.bCodeStream[(this.classFileOffset++)] = -60;
      this.bCodeStream[(this.classFileOffset++)] = 58;
      writeUnsignedShort(iArg);
    } else {
      if (this.classFileOffset + 1 >= this.bCodeStream.length) {
        resizeByteArray();
      }
      this.position += 2;
      this.bCodeStream[(this.classFileOffset++)] = 58;
      this.bCodeStream[(this.classFileOffset++)] = ((byte)iArg);
    }
  }

  public void astore_0() {
    this.countLabels = 0;
    this.stackDepth -= 1;
    if (this.maxLocals == 0) {
      this.maxLocals = 1;
    }
    if (this.classFileOffset >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 1;
    this.bCodeStream[(this.classFileOffset++)] = 75;
  }

  public void astore_1() {
    this.countLabels = 0;
    this.stackDepth -= 1;
    if (this.maxLocals <= 1) {
      this.maxLocals = 2;
    }
    if (this.classFileOffset >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 1;
    this.bCodeStream[(this.classFileOffset++)] = 76;
  }

  public void astore_2() {
    this.countLabels = 0;
    this.stackDepth -= 1;
    if (this.maxLocals <= 2) {
      this.maxLocals = 3;
    }
    if (this.classFileOffset >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 1;
    this.bCodeStream[(this.classFileOffset++)] = 77;
  }

  public void astore_3() {
    this.countLabels = 0;
    this.stackDepth -= 1;
    if (this.maxLocals <= 3) {
      this.maxLocals = 4;
    }
    if (this.classFileOffset >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 1;
    this.bCodeStream[(this.classFileOffset++)] = 78;
  }

  public void athrow() {
    this.countLabels = 0;
    this.stackDepth -= 1;
    if (this.classFileOffset >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 1;
    this.bCodeStream[(this.classFileOffset++)] = -65;
    this.lastAbruptCompletion = this.position;
  }

  public void baload() {
    this.countLabels = 0;
    this.stackDepth -= 1;
    if (this.classFileOffset >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 1;
    this.bCodeStream[(this.classFileOffset++)] = 51;
  }

  public void bastore() {
    this.countLabels = 0;
    this.stackDepth -= 3;
    if (this.classFileOffset >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 1;
    this.bCodeStream[(this.classFileOffset++)] = 84;
  }

  public void bipush(byte b) {
    this.countLabels = 0;
    this.stackDepth += 1;
    if (this.stackDepth > this.stackMax)
      this.stackMax = this.stackDepth;
    if (this.classFileOffset + 1 >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 2;
    this.bCodeStream[(this.classFileOffset++)] = 16;
    this.bCodeStream[(this.classFileOffset++)] = b;
  }

  public void caload() {
    this.countLabels = 0;
    this.stackDepth -= 1;
    if (this.classFileOffset >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 1;
    this.bCodeStream[(this.classFileOffset++)] = 52;
  }

  public void castore() {
    this.countLabels = 0;
    this.stackDepth -= 3;
    if (this.classFileOffset >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 1;
    this.bCodeStream[(this.classFileOffset++)] = 85;
  }
  public void checkcast(int baseId) {
    this.countLabels = 0;
    if (this.classFileOffset + 2 >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 1;
    this.bCodeStream[(this.classFileOffset++)] = -64;
    switch (baseId) {
    case 3:
      writeUnsignedShort(this.constantPool.literalIndexForType(ConstantPool.JavaLangByteConstantPoolName));
      break;
    case 4:
      writeUnsignedShort(this.constantPool.literalIndexForType(ConstantPool.JavaLangShortConstantPoolName));
      break;
    case 2:
      writeUnsignedShort(this.constantPool.literalIndexForType(ConstantPool.JavaLangCharacterConstantPoolName));
      break;
    case 10:
      writeUnsignedShort(this.constantPool.literalIndexForType(ConstantPool.JavaLangIntegerConstantPoolName));
      break;
    case 7:
      writeUnsignedShort(this.constantPool.literalIndexForType(ConstantPool.JavaLangLongConstantPoolName));
      break;
    case 9:
      writeUnsignedShort(this.constantPool.literalIndexForType(ConstantPool.JavaLangFloatConstantPoolName));
      break;
    case 8:
      writeUnsignedShort(this.constantPool.literalIndexForType(ConstantPool.JavaLangDoubleConstantPoolName));
      break;
    case 5:
      writeUnsignedShort(this.constantPool.literalIndexForType(ConstantPool.JavaLangBooleanConstantPoolName));
    case 6:
    }
  }

  public void checkcast(TypeBinding typeBinding) { this.countLabels = 0;
    if (this.classFileOffset + 2 >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 1;
    this.bCodeStream[(this.classFileOffset++)] = -64;
    writeUnsignedShort(this.constantPool.literalIndexForType(typeBinding)); }

  public void d2f()
  {
    this.countLabels = 0;
    this.stackDepth -= 1;
    if (this.classFileOffset >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 1;
    this.bCodeStream[(this.classFileOffset++)] = -112;
  }

  public void d2i() {
    this.countLabels = 0;
    this.stackDepth -= 1;
    if (this.classFileOffset >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 1;
    this.bCodeStream[(this.classFileOffset++)] = -114;
  }

  public void d2l() {
    this.countLabels = 0;
    if (this.classFileOffset >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 1;
    this.bCodeStream[(this.classFileOffset++)] = -113;
  }

  public void dadd() {
    this.countLabels = 0;
    this.stackDepth -= 2;
    if (this.classFileOffset >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 1;
    this.bCodeStream[(this.classFileOffset++)] = 99;
  }

  public void daload() {
    this.countLabels = 0;
    if (this.classFileOffset >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 1;
    this.bCodeStream[(this.classFileOffset++)] = 49;
  }

  public void dastore() {
    this.countLabels = 0;
    this.stackDepth -= 4;
    if (this.classFileOffset >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 1;
    this.bCodeStream[(this.classFileOffset++)] = 82;
  }

  public void dcmpg() {
    this.countLabels = 0;
    this.stackDepth -= 3;
    if (this.classFileOffset >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 1;
    this.bCodeStream[(this.classFileOffset++)] = -104;
  }

  public void dcmpl() {
    this.countLabels = 0;
    this.stackDepth -= 3;
    if (this.classFileOffset >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 1;
    this.bCodeStream[(this.classFileOffset++)] = -105;
  }

  public void dconst_0() {
    this.countLabels = 0;
    this.stackDepth += 2;
    if (this.stackDepth > this.stackMax)
      this.stackMax = this.stackDepth;
    if (this.classFileOffset >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 1;
    this.bCodeStream[(this.classFileOffset++)] = 14;
  }

  public void dconst_1() {
    this.countLabels = 0;
    this.stackDepth += 2;
    if (this.stackDepth > this.stackMax)
      this.stackMax = this.stackDepth;
    if (this.classFileOffset >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 1;
    this.bCodeStream[(this.classFileOffset++)] = 15;
  }

  public void ddiv() {
    this.countLabels = 0;
    this.stackDepth -= 2;
    if (this.classFileOffset >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 1;
    this.bCodeStream[(this.classFileOffset++)] = 111;
  }
  public void decrStackSize(int offset) {
    this.stackDepth -= offset;
  }

  public void dload(int iArg) {
    this.countLabels = 0;
    this.stackDepth += 2;
    if (this.stackDepth > this.stackMax)
      this.stackMax = this.stackDepth;
    if (this.maxLocals < iArg + 2) {
      this.maxLocals = (iArg + 2);
    }
    if (iArg > 255) {
      if (this.classFileOffset + 3 >= this.bCodeStream.length) {
        resizeByteArray();
      }
      this.position += 2;
      this.bCodeStream[(this.classFileOffset++)] = -60;
      this.bCodeStream[(this.classFileOffset++)] = 24;
      writeUnsignedShort(iArg);
    }
    else {
      if (this.classFileOffset + 1 >= this.bCodeStream.length) {
        resizeByteArray();
      }
      this.position += 2;
      this.bCodeStream[(this.classFileOffset++)] = 24;
      this.bCodeStream[(this.classFileOffset++)] = ((byte)iArg);
    }
  }

  public void dload_0() {
    this.countLabels = 0;
    this.stackDepth += 2;
    if (this.stackDepth > this.stackMax)
      this.stackMax = this.stackDepth;
    if (this.maxLocals < 2) {
      this.maxLocals = 2;
    }
    if (this.classFileOffset >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 1;
    this.bCodeStream[(this.classFileOffset++)] = 38;
  }

  public void dload_1() {
    this.countLabels = 0;
    this.stackDepth += 2;
    if (this.stackDepth > this.stackMax)
      this.stackMax = this.stackDepth;
    if (this.maxLocals < 3) {
      this.maxLocals = 3;
    }
    if (this.classFileOffset >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 1;
    this.bCodeStream[(this.classFileOffset++)] = 39;
  }

  public void dload_2() {
    this.countLabels = 0;
    this.stackDepth += 2;
    if (this.stackDepth > this.stackMax)
      this.stackMax = this.stackDepth;
    if (this.maxLocals < 4) {
      this.maxLocals = 4;
    }
    if (this.classFileOffset >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 1;
    this.bCodeStream[(this.classFileOffset++)] = 40;
  }

  public void dload_3() {
    this.countLabels = 0;
    this.stackDepth += 2;
    if (this.stackDepth > this.stackMax)
      this.stackMax = this.stackDepth;
    if (this.maxLocals < 5) {
      this.maxLocals = 5;
    }
    if (this.classFileOffset >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 1;
    this.bCodeStream[(this.classFileOffset++)] = 41;
  }

  public void dmul() {
    this.countLabels = 0;
    this.stackDepth -= 2;
    if (this.classFileOffset >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 1;
    this.bCodeStream[(this.classFileOffset++)] = 107;
  }

  public void dneg() {
    this.countLabels = 0;
    if (this.classFileOffset >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 1;
    this.bCodeStream[(this.classFileOffset++)] = 119;
  }

  public void drem() {
    this.countLabels = 0;
    this.stackDepth -= 2;
    if (this.classFileOffset >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 1;
    this.bCodeStream[(this.classFileOffset++)] = 115;
  }

  public void dreturn() {
    this.countLabels = 0;
    this.stackDepth -= 2;

    if (this.classFileOffset >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 1;
    this.bCodeStream[(this.classFileOffset++)] = -81;
    this.lastAbruptCompletion = this.position;
  }

  public void dstore(int iArg) {
    this.countLabels = 0;
    this.stackDepth -= 2;
    if (this.maxLocals <= iArg + 1) {
      this.maxLocals = (iArg + 2);
    }
    if (iArg > 255) {
      if (this.classFileOffset + 3 >= this.bCodeStream.length) {
        resizeByteArray();
      }
      this.position += 2;
      this.bCodeStream[(this.classFileOffset++)] = -60;
      this.bCodeStream[(this.classFileOffset++)] = 57;
      writeUnsignedShort(iArg);
    } else {
      if (this.classFileOffset + 1 >= this.bCodeStream.length) {
        resizeByteArray();
      }
      this.position += 2;
      this.bCodeStream[(this.classFileOffset++)] = 57;
      this.bCodeStream[(this.classFileOffset++)] = ((byte)iArg);
    }
  }

  public void dstore_0() {
    this.countLabels = 0;
    this.stackDepth -= 2;
    if (this.maxLocals < 2) {
      this.maxLocals = 2;
    }
    if (this.classFileOffset >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 1;
    this.bCodeStream[(this.classFileOffset++)] = 71;
  }

  public void dstore_1() {
    this.countLabels = 0;
    this.stackDepth -= 2;
    if (this.maxLocals < 3) {
      this.maxLocals = 3;
    }
    if (this.classFileOffset >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 1;
    this.bCodeStream[(this.classFileOffset++)] = 72;
  }

  public void dstore_2() {
    this.countLabels = 0;
    this.stackDepth -= 2;
    if (this.maxLocals < 4) {
      this.maxLocals = 4;
    }
    if (this.classFileOffset >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 1;
    this.bCodeStream[(this.classFileOffset++)] = 73;
  }

  public void dstore_3() {
    this.countLabels = 0;
    this.stackDepth -= 2;
    if (this.maxLocals < 5) {
      this.maxLocals = 5;
    }
    if (this.classFileOffset >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 1;
    this.bCodeStream[(this.classFileOffset++)] = 74;
  }

  public void dsub() {
    this.countLabels = 0;
    this.stackDepth -= 2;
    if (this.classFileOffset >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 1;
    this.bCodeStream[(this.classFileOffset++)] = 103;
  }

  public void dup() {
    this.countLabels = 0;
    this.stackDepth += 1;
    if (this.stackDepth > this.stackMax) {
      this.stackMax = this.stackDepth;
    }
    if (this.classFileOffset >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 1;
    this.bCodeStream[(this.classFileOffset++)] = 89;
  }

  public void dup_x1() {
    this.countLabels = 0;
    this.stackDepth += 1;
    if (this.stackDepth > this.stackMax)
      this.stackMax = this.stackDepth;
    if (this.classFileOffset >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 1;
    this.bCodeStream[(this.classFileOffset++)] = 90;
  }

  public void dup_x2() {
    this.countLabels = 0;
    this.stackDepth += 1;
    if (this.stackDepth > this.stackMax)
      this.stackMax = this.stackDepth;
    if (this.classFileOffset >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 1;
    this.bCodeStream[(this.classFileOffset++)] = 91;
  }

  public void dup2() {
    this.countLabels = 0;
    this.stackDepth += 2;
    if (this.stackDepth > this.stackMax)
      this.stackMax = this.stackDepth;
    if (this.classFileOffset >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 1;
    this.bCodeStream[(this.classFileOffset++)] = 92;
  }

  public void dup2_x1() {
    this.countLabels = 0;
    this.stackDepth += 2;
    if (this.stackDepth > this.stackMax)
      this.stackMax = this.stackDepth;
    if (this.classFileOffset >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 1;
    this.bCodeStream[(this.classFileOffset++)] = 93;
  }

  public void dup2_x2() {
    this.countLabels = 0;
    this.stackDepth += 2;
    if (this.stackDepth > this.stackMax)
      this.stackMax = this.stackDepth;
    if (this.classFileOffset >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 1;
    this.bCodeStream[(this.classFileOffset++)] = 94;
  }

  public void exitUserScope(BlockScope currentScope)
  {
    if ((this.generateAttributes & 0xC) == 0)
      return;
    int index = this.visibleLocalsCount - 1;
    while (index >= 0) {
      LocalVariableBinding visibleLocal = this.visibleLocals[index];
      if ((visibleLocal == null) || (visibleLocal.declaringScope != currentScope))
      {
        index--;
      }
      else
      {
        if (visibleLocal.initializationCount > 0) {
          visibleLocal.recordInitializationEndPC(this.position);
        }
        this.visibleLocals[(index--)] = null;
      }
    }
  }

  public void exitUserScope(BlockScope currentScope, LocalVariableBinding binding) { if ((this.generateAttributes & 0xC) == 0)
      return;
    int index = this.visibleLocalsCount - 1;
    while (index >= 0) {
      LocalVariableBinding visibleLocal = this.visibleLocals[index];
      if ((visibleLocal == null) || (visibleLocal.declaringScope != currentScope) || (visibleLocal == binding))
      {
        index--;
      }
      else
      {
        if (visibleLocal.initializationCount > 0) {
          visibleLocal.recordInitializationEndPC(this.position);
        }
        this.visibleLocals[(index--)] = null;
      }
    } }

  public void f2d() {
    this.countLabels = 0;
    this.stackDepth += 1;
    if (this.stackDepth > this.stackMax)
      this.stackMax = this.stackDepth;
    if (this.classFileOffset >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 1;
    this.bCodeStream[(this.classFileOffset++)] = -115;
  }

  public void f2i() {
    this.countLabels = 0;
    if (this.classFileOffset >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 1;
    this.bCodeStream[(this.classFileOffset++)] = -117;
  }

  public void f2l() {
    this.countLabels = 0;
    this.stackDepth += 1;
    if (this.stackDepth > this.stackMax)
      this.stackMax = this.stackDepth;
    if (this.classFileOffset >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 1;
    this.bCodeStream[(this.classFileOffset++)] = -116;
  }

  public void fadd() {
    this.countLabels = 0;
    this.stackDepth -= 1;
    if (this.classFileOffset >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 1;
    this.bCodeStream[(this.classFileOffset++)] = 98;
  }

  public void faload() {
    this.countLabels = 0;
    this.stackDepth -= 1;
    if (this.classFileOffset >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 1;
    this.bCodeStream[(this.classFileOffset++)] = 48;
  }

  public void fastore() {
    this.countLabels = 0;
    this.stackDepth -= 3;
    if (this.classFileOffset >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 1;
    this.bCodeStream[(this.classFileOffset++)] = 81;
  }

  public void fcmpg() {
    this.countLabels = 0;
    this.stackDepth -= 1;
    if (this.classFileOffset >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 1;
    this.bCodeStream[(this.classFileOffset++)] = -106;
  }

  public void fcmpl() {
    this.countLabels = 0;
    this.stackDepth -= 1;
    if (this.classFileOffset >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 1;
    this.bCodeStream[(this.classFileOffset++)] = -107;
  }

  public void fconst_0() {
    this.countLabels = 0;
    this.stackDepth += 1;
    if (this.stackDepth > this.stackMax)
      this.stackMax = this.stackDepth;
    if (this.classFileOffset >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 1;
    this.bCodeStream[(this.classFileOffset++)] = 11;
  }

  public void fconst_1() {
    this.countLabels = 0;
    this.stackDepth += 1;
    if (this.stackDepth > this.stackMax)
      this.stackMax = this.stackDepth;
    if (this.classFileOffset >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 1;
    this.bCodeStream[(this.classFileOffset++)] = 12;
  }

  public void fconst_2() {
    this.countLabels = 0;
    this.stackDepth += 1;
    if (this.stackDepth > this.stackMax)
      this.stackMax = this.stackDepth;
    if (this.classFileOffset >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 1;
    this.bCodeStream[(this.classFileOffset++)] = 13;
  }

  public void fdiv() {
    this.countLabels = 0;
    this.stackDepth -= 1;
    if (this.classFileOffset >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 1;
    this.bCodeStream[(this.classFileOffset++)] = 110;
  }

  public void fload(int iArg) {
    this.countLabels = 0;
    this.stackDepth += 1;
    if (this.maxLocals <= iArg) {
      this.maxLocals = (iArg + 1);
    }
    if (this.stackDepth > this.stackMax)
      this.stackMax = this.stackDepth;
    if (iArg > 255) {
      if (this.classFileOffset + 3 >= this.bCodeStream.length) {
        resizeByteArray();
      }
      this.position += 2;
      this.bCodeStream[(this.classFileOffset++)] = -60;
      this.bCodeStream[(this.classFileOffset++)] = 23;
      writeUnsignedShort(iArg);
    } else {
      if (this.classFileOffset + 1 >= this.bCodeStream.length) {
        resizeByteArray();
      }
      this.position += 2;
      this.bCodeStream[(this.classFileOffset++)] = 23;
      this.bCodeStream[(this.classFileOffset++)] = ((byte)iArg);
    }
  }

  public void fload_0() {
    this.countLabels = 0;
    this.stackDepth += 1;
    if (this.maxLocals == 0) {
      this.maxLocals = 1;
    }
    if (this.stackDepth > this.stackMax)
      this.stackMax = this.stackDepth;
    if (this.classFileOffset >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 1;
    this.bCodeStream[(this.classFileOffset++)] = 34;
  }

  public void fload_1() {
    this.countLabels = 0;
    this.stackDepth += 1;
    if (this.maxLocals <= 1) {
      this.maxLocals = 2;
    }
    if (this.stackDepth > this.stackMax)
      this.stackMax = this.stackDepth;
    if (this.classFileOffset >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 1;
    this.bCodeStream[(this.classFileOffset++)] = 35;
  }

  public void fload_2() {
    this.countLabels = 0;
    this.stackDepth += 1;
    if (this.maxLocals <= 2) {
      this.maxLocals = 3;
    }
    if (this.stackDepth > this.stackMax)
      this.stackMax = this.stackDepth;
    if (this.classFileOffset >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 1;
    this.bCodeStream[(this.classFileOffset++)] = 36;
  }

  public void fload_3() {
    this.countLabels = 0;
    this.stackDepth += 1;
    if (this.maxLocals <= 3) {
      this.maxLocals = 4;
    }
    if (this.stackDepth > this.stackMax)
      this.stackMax = this.stackDepth;
    if (this.classFileOffset >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 1;
    this.bCodeStream[(this.classFileOffset++)] = 37;
  }

  public void fmul() {
    this.countLabels = 0;
    this.stackDepth -= 1;
    if (this.classFileOffset >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 1;
    this.bCodeStream[(this.classFileOffset++)] = 106;
  }

  public void fneg() {
    this.countLabels = 0;
    if (this.classFileOffset >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 1;
    this.bCodeStream[(this.classFileOffset++)] = 118;
  }

  public void frem() {
    this.countLabels = 0;
    this.stackDepth -= 1;
    if (this.classFileOffset >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 1;
    this.bCodeStream[(this.classFileOffset++)] = 114;
  }

  public void freturn() {
    this.countLabels = 0;
    this.stackDepth -= 1;

    if (this.classFileOffset >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 1;
    this.bCodeStream[(this.classFileOffset++)] = -82;
    this.lastAbruptCompletion = this.position;
  }

  public void fstore(int iArg) {
    this.countLabels = 0;
    this.stackDepth -= 1;
    if (this.maxLocals <= iArg) {
      this.maxLocals = (iArg + 1);
    }
    if (iArg > 255) {
      if (this.classFileOffset + 3 >= this.bCodeStream.length) {
        resizeByteArray();
      }
      this.position += 2;
      this.bCodeStream[(this.classFileOffset++)] = -60;
      this.bCodeStream[(this.classFileOffset++)] = 56;
      writeUnsignedShort(iArg);
    } else {
      if (this.classFileOffset + 1 >= this.bCodeStream.length) {
        resizeByteArray();
      }
      this.position += 2;
      this.bCodeStream[(this.classFileOffset++)] = 56;
      this.bCodeStream[(this.classFileOffset++)] = ((byte)iArg);
    }
  }

  public void fstore_0() {
    this.countLabels = 0;
    this.stackDepth -= 1;
    if (this.maxLocals == 0) {
      this.maxLocals = 1;
    }
    if (this.classFileOffset >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 1;
    this.bCodeStream[(this.classFileOffset++)] = 67;
  }

  public void fstore_1() {
    this.countLabels = 0;
    this.stackDepth -= 1;
    if (this.maxLocals <= 1) {
      this.maxLocals = 2;
    }
    if (this.classFileOffset >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 1;
    this.bCodeStream[(this.classFileOffset++)] = 68;
  }

  public void fstore_2() {
    this.countLabels = 0;
    this.stackDepth -= 1;
    if (this.maxLocals <= 2) {
      this.maxLocals = 3;
    }
    if (this.classFileOffset >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 1;
    this.bCodeStream[(this.classFileOffset++)] = 69;
  }

  public void fstore_3() {
    this.countLabels = 0;
    this.stackDepth -= 1;
    if (this.maxLocals <= 3) {
      this.maxLocals = 4;
    }
    if (this.classFileOffset >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 1;
    this.bCodeStream[(this.classFileOffset++)] = 70;
  }

  public void fsub()
  {
    this.countLabels = 0;
    this.stackDepth -= 1;
    if (this.classFileOffset >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 1;
    this.bCodeStream[(this.classFileOffset++)] = 102;
  }
  public void generateBoxingConversion(int unboxedTypeID) {
    switch (unboxedTypeID) {
    case 3:
      if (this.targetLevel >= 3211264L)
      {
        invoke(
          -72, 
          1, 
          1, 
          ConstantPool.JavaLangByteConstantPoolName, 
          ConstantPool.ValueOf, 
          ConstantPool.byteByteSignature);
      }
      else
      {
        newWrapperFor(unboxedTypeID);
        dup_x1();
        swap();
        invoke(
          -73, 
          1, 
          0, 
          ConstantPool.JavaLangByteConstantPoolName, 
          ConstantPool.Init, 
          ConstantPool.ByteConstrSignature);
      }
      break;
    case 4:
      if (this.targetLevel >= 3211264L)
      {
        invoke(
          -72, 
          1, 
          1, 
          ConstantPool.JavaLangShortConstantPoolName, 
          ConstantPool.ValueOf, 
          ConstantPool.shortShortSignature);
      }
      else
      {
        newWrapperFor(unboxedTypeID);
        dup_x1();
        swap();
        invoke(
          -73, 
          1, 
          0, 
          ConstantPool.JavaLangShortConstantPoolName, 
          ConstantPool.Init, 
          ConstantPool.ShortConstrSignature);
      }
      break;
    case 2:
      if (this.targetLevel >= 3211264L)
      {
        invoke(
          -72, 
          1, 
          1, 
          ConstantPool.JavaLangCharacterConstantPoolName, 
          ConstantPool.ValueOf, 
          ConstantPool.charCharacterSignature);
      }
      else
      {
        newWrapperFor(unboxedTypeID);
        dup_x1();
        swap();
        invoke(
          -73, 
          1, 
          0, 
          ConstantPool.JavaLangCharacterConstantPoolName, 
          ConstantPool.Init, 
          ConstantPool.CharConstrSignature);
      }
      break;
    case 10:
      if (this.targetLevel >= 3211264L)
      {
        invoke(
          -72, 
          1, 
          1, 
          ConstantPool.JavaLangIntegerConstantPoolName, 
          ConstantPool.ValueOf, 
          ConstantPool.IntIntegerSignature);
      }
      else
      {
        newWrapperFor(unboxedTypeID);
        dup_x1();
        swap();
        invoke(
          -73, 
          1, 
          0, 
          ConstantPool.JavaLangIntegerConstantPoolName, 
          ConstantPool.Init, 
          ConstantPool.IntConstrSignature);
      }
      break;
    case 7:
      if (this.targetLevel >= 3211264L)
      {
        invoke(
          -72, 
          2, 
          1, 
          ConstantPool.JavaLangLongConstantPoolName, 
          ConstantPool.ValueOf, 
          ConstantPool.longLongSignature);
      }
      else
      {
        newWrapperFor(unboxedTypeID);
        dup_x2();
        dup_x2();
        pop();
        invoke(
          -73, 
          2, 
          0, 
          ConstantPool.JavaLangLongConstantPoolName, 
          ConstantPool.Init, 
          ConstantPool.LongConstrSignature);
      }
      break;
    case 9:
      if (this.targetLevel >= 3211264L)
      {
        invoke(
          -72, 
          1, 
          1, 
          ConstantPool.JavaLangFloatConstantPoolName, 
          ConstantPool.ValueOf, 
          ConstantPool.floatFloatSignature);
      }
      else
      {
        newWrapperFor(unboxedTypeID);
        dup_x1();
        swap();
        invoke(
          -73, 
          1, 
          0, 
          ConstantPool.JavaLangFloatConstantPoolName, 
          ConstantPool.Init, 
          ConstantPool.FloatConstrSignature);
      }
      break;
    case 8:
      if (this.targetLevel >= 3211264L)
      {
        invoke(
          -72, 
          2, 
          1, 
          ConstantPool.JavaLangDoubleConstantPoolName, 
          ConstantPool.ValueOf, 
          ConstantPool.doubleDoubleSignature);
      }
      else
      {
        newWrapperFor(unboxedTypeID);
        dup_x2();
        dup_x2();
        pop();

        invoke(
          -73, 
          2, 
          0, 
          ConstantPool.JavaLangDoubleConstantPoolName, 
          ConstantPool.Init, 
          ConstantPool.DoubleConstrSignature);
      }

      break;
    case 5:
      if (this.targetLevel >= 3211264L)
      {
        invoke(
          -72, 
          1, 
          1, 
          ConstantPool.JavaLangBooleanConstantPoolName, 
          ConstantPool.ValueOf, 
          ConstantPool.booleanBooleanSignature);
      }
      else
      {
        newWrapperFor(unboxedTypeID);
        dup_x1();
        swap();
        invoke(
          -73, 
          1, 
          0, 
          ConstantPool.JavaLangBooleanConstantPoolName, 
          ConstantPool.Init, 
          ConstantPool.BooleanConstrSignature);
      }
      break;
    case 6:
    }
  }

  public void generateClassLiteralAccessForType(TypeBinding accessedType, FieldBinding syntheticFieldBinding)
  {
    if ((accessedType.isBaseType()) && (accessedType != TypeBinding.NULL)) {
      getTYPE(accessedType.id);
      return;
    }

    if (this.targetLevel >= 3211264L)
    {
      ldc(accessedType);
    } else {
      BranchLabel endLabel = new BranchLabel(this);
      if (syntheticFieldBinding != null) {
        getstatic(syntheticFieldBinding);
        dup();
        ifnonnull(endLabel);
        pop();
      }

      ExceptionLabel anyExceptionHandler = new ExceptionLabel(this, TypeBinding.NULL);
      anyExceptionHandler.placeStart();
      ldc(accessedType == TypeBinding.NULL ? "java.lang.Object" : String.valueOf(accessedType.constantPoolName()).replace('/', '.'));
      invokeClassForName();

      anyExceptionHandler.placeEnd();

      if (syntheticFieldBinding != null) {
        dup();
        putstatic(syntheticFieldBinding);
      }
      goto_(endLabel);

      int saveStackSize = this.stackDepth;
      this.stackDepth = 1;

      anyExceptionHandler.place();

      newNoClassDefFoundError();
      dup_x1();
      swap();

      invokeThrowableGetMessage();

      invokeNoClassDefFoundErrorStringConstructor();
      athrow();
      this.stackDepth = saveStackSize;
      endLabel.place();
    }
  }

  public final void generateCodeAttributeForProblemMethod(String problemMessage)
  {
    newJavaLangError();
    dup();
    ldc(problemMessage);
    invokeJavaLangErrorConstructor();
    athrow();
  }
  public void generateConstant(Constant constant, int implicitConversionCode) {
    int targetTypeID = (implicitConversionCode & 0xFF) >> 4;
    if (targetTypeID == 0) targetTypeID = constant.typeID();
    switch (targetTypeID) {
    case 5:
      generateInlinedValue(constant.booleanValue());
      break;
    case 2:
      generateInlinedValue(constant.charValue());
      break;
    case 3:
      generateInlinedValue(constant.byteValue());
      break;
    case 4:
      generateInlinedValue(constant.shortValue());
      break;
    case 10:
      generateInlinedValue(constant.intValue());
      break;
    case 7:
      generateInlinedValue(constant.longValue());
      break;
    case 9:
      generateInlinedValue(constant.floatValue());
      break;
    case 8:
      generateInlinedValue(constant.doubleValue());
      break;
    case 11:
      ldc(constant.stringValue());
    case 6:
    }if ((implicitConversionCode & 0x200) != 0)
    {
      generateBoxingConversion(targetTypeID);
    }
  }

  public void generateEmulatedReadAccessForField(FieldBinding fieldBinding) { generateEmulationForField(fieldBinding);

    swap();
    invokeJavaLangReflectFieldGetter(fieldBinding.type.id);
    if (!fieldBinding.type.isBaseType())
      checkcast(fieldBinding.type); }

  public void generateEmulatedWriteAccessForField(FieldBinding fieldBinding)
  {
    invokeJavaLangReflectFieldSetter(fieldBinding.type.id);
  }

  public void generateEmulationForConstructor(Scope scope, MethodBinding methodBinding) {
    ldc(String.valueOf(methodBinding.declaringClass.constantPoolName()).replace('/', '.'));
    invokeClassForName();
    int paramLength = methodBinding.parameters.length;
    generateInlinedValue(paramLength);
    newArray(scope.createArrayType(scope.getType(TypeConstants.JAVA_LANG_CLASS, 3), 1));
    if (paramLength > 0) {
      dup();
      for (int i = 0; i < paramLength; i++) {
        generateInlinedValue(i);
        TypeBinding parameter = methodBinding.parameters[i];
        if (parameter.isBaseType()) {
          getTYPE(parameter.id);
        } else if (parameter.isArrayType()) {
          ArrayBinding array = (ArrayBinding)parameter;
          if (array.leafComponentType.isBaseType()) {
            getTYPE(array.leafComponentType.id);
          } else {
            ldc(String.valueOf(array.leafComponentType.constantPoolName()).replace('/', '.'));
            invokeClassForName();
          }
          int dimensions = array.dimensions;
          generateInlinedValue(dimensions);
          newarray(10);
          invokeArrayNewInstance();
          invokeObjectGetClass();
        }
        else {
          ldc(String.valueOf(methodBinding.declaringClass.constantPoolName()).replace('/', '.'));
          invokeClassForName();
        }
        aastore();
        if (i < paramLength - 1) {
          dup();
        }
      }
    }
    invokeClassGetDeclaredConstructor();
    dup();
    iconst_1();
    invokeAccessibleObjectSetAccessible();
  }

  public void generateEmulationForField(FieldBinding fieldBinding) {
    ldc(String.valueOf(fieldBinding.declaringClass.constantPoolName()).replace('/', '.'));
    invokeClassForName();
    ldc(String.valueOf(fieldBinding.name));
    invokeClassGetDeclaredField();
    dup();
    iconst_1();
    invokeAccessibleObjectSetAccessible();
  }

  public void generateEmulationForMethod(Scope scope, MethodBinding methodBinding) {
    ldc(String.valueOf(methodBinding.declaringClass.constantPoolName()).replace('/', '.'));
    invokeClassForName();
    ldc(String.valueOf(methodBinding.selector));
    int paramLength = methodBinding.parameters.length;
    generateInlinedValue(paramLength);
    newArray(scope.createArrayType(scope.getType(TypeConstants.JAVA_LANG_CLASS, 3), 1));
    if (paramLength > 0) {
      dup();
      for (int i = 0; i < paramLength; i++) {
        generateInlinedValue(i);
        TypeBinding parameter = methodBinding.parameters[i];
        if (parameter.isBaseType()) {
          getTYPE(parameter.id);
        } else if (parameter.isArrayType()) {
          ArrayBinding array = (ArrayBinding)parameter;
          if (array.leafComponentType.isBaseType()) {
            getTYPE(array.leafComponentType.id);
          } else {
            ldc(String.valueOf(array.leafComponentType.constantPoolName()).replace('/', '.'));
            invokeClassForName();
          }
          int dimensions = array.dimensions;
          generateInlinedValue(dimensions);
          newarray(10);
          invokeArrayNewInstance();
          invokeObjectGetClass();
        }
        else {
          ldc(String.valueOf(methodBinding.declaringClass.constantPoolName()).replace('/', '.'));
          invokeClassForName();
        }
        aastore();
        if (i < paramLength - 1) {
          dup();
        }
      }
    }
    invokeClassGetDeclaredMethod();
    dup();
    iconst_1();
    invokeAccessibleObjectSetAccessible();
  }
  private void generateFieldAccess(byte opcode, int returnTypeSize, char[] declaringClass, char[] name, char[] signature) {
    this.countLabels = 0;
    switch (opcode) {
    case -76:
      if (returnTypeSize == 2) {
        this.stackDepth += 1;
      }
      break;
    case -78:
      if (returnTypeSize == 2)
        this.stackDepth += 2;
      else {
        this.stackDepth += 1;
      }
      break;
    case -75:
      if (returnTypeSize == 2)
        this.stackDepth -= 3;
      else {
        this.stackDepth -= 2;
      }
      break;
    case -77:
      if (returnTypeSize == 2)
        this.stackDepth -= 2;
      else
        this.stackDepth -= 1;
      break;
    }
    if (this.stackDepth > this.stackMax) {
      this.stackMax = this.stackDepth;
    }
    if (this.classFileOffset + 2 >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 1;
    this.bCodeStream[(this.classFileOffset++)] = opcode;
    writeUnsignedShort(this.constantPool.literalIndexForField(declaringClass, name, signature));
  }
  private void generateFieldAccess(byte opcode, int returnTypeSize, ReferenceBinding binding, char[] name, TypeBinding type) {
    if (binding.isNestedType()) {
      this.classFile.recordInnerClasses(binding);
    }
    TypeBinding leafComponentType = type.leafComponentType();
    if (leafComponentType.isNestedType()) {
      this.classFile.recordInnerClasses(leafComponentType);
    }
    generateFieldAccess(opcode, returnTypeSize, binding.constantPoolName(), name, type.signature());
  }

  public void generateImplicitConversion(int implicitConversionCode)
  {
    if ((implicitConversionCode & 0x400) != 0) {
      int typeId = implicitConversionCode & 0xF;
      generateUnboxingConversion(typeId);
    }

    switch (implicitConversionCode & 0xFF) {
    case 41:
      f2i();
      i2c();
      break;
    case 40:
      d2i();
      i2c();
      break;
    case 35:
    case 36:
    case 42:
      i2c();
      break;
    case 39:
      l2i();
      i2c();
      break;
    case 146:
    case 147:
    case 148:
    case 154:
      i2f();
      break;
    case 152:
      d2f();
      break;
    case 151:
      l2f();
      break;
    case 57:
      f2i();
      i2b();
      break;
    case 56:
      d2i();
      i2b();
      break;
    case 50:
    case 52:
    case 58:
      i2b();
      break;
    case 55:
      l2i();
      i2b();
      break;
    case 130:
    case 131:
    case 132:
    case 138:
      i2d();
      break;
    case 137:
      f2d();
      break;
    case 135:
      l2d();
      break;
    case 66:
    case 67:
    case 74:
      i2s();
      break;
    case 72:
      d2i();
      i2s();
      break;
    case 71:
      l2i();
      i2s();
      break;
    case 73:
      f2i();
      i2s();
      break;
    case 168:
      d2i();
      break;
    case 169:
      f2i();
      break;
    case 167:
      l2i();
      break;
    case 114:
    case 115:
    case 116:
    case 122:
      i2l();
      break;
    case 120:
      d2l();
      break;
    case 121:
      f2l();
    }
    if ((implicitConversionCode & 0x200) != 0)
    {
      int typeId = (implicitConversionCode & 0xFF) >> 4;
      generateBoxingConversion(typeId);
    }
  }

  public void generateInlinedValue(boolean inlinedValue) { if (inlinedValue)
      iconst_1();
    else
      iconst_0(); }

  public void generateInlinedValue(byte inlinedValue)
  {
    switch (inlinedValue) {
    case -1:
      iconst_m1();
      break;
    case 0:
      iconst_0();
      break;
    case 1:
      iconst_1();
      break;
    case 2:
      iconst_2();
      break;
    case 3:
      iconst_3();
      break;
    case 4:
      iconst_4();
      break;
    case 5:
      iconst_5();
      break;
    default:
      if ((-128 <= inlinedValue) && (inlinedValue <= 127)) { bipush(inlinedValue);
        return; }
      break;
    }
  }

  public void generateInlinedValue(char inlinedValue) {
    switch (inlinedValue) {
    case '\000':
      iconst_0();
      break;
    case '\001':
      iconst_1();
      break;
    case '\002':
      iconst_2();
      break;
    case '\003':
      iconst_3();
      break;
    case '\004':
      iconst_4();
      break;
    case '\005':
      iconst_5();
      break;
    default:
      if (('\006' <= inlinedValue) && (inlinedValue <= '')) {
        bipush((byte)inlinedValue);
        return;
      }
      if (('' <= inlinedValue) && (inlinedValue <= '翿')) {
        sipush(inlinedValue);
        return;
      }
      ldc(inlinedValue);
    }
  }

  public void generateInlinedValue(double inlinedValue) { if (inlinedValue == 0.0D) {
      if (Double.doubleToLongBits(inlinedValue) != 0L)
        ldc2_w(inlinedValue);
      else
        dconst_0();
      return;
    }
    if (inlinedValue == 1.0D) {
      dconst_1();
      return;
    }
    ldc2_w(inlinedValue); }

  public void generateInlinedValue(float inlinedValue) {
    if (inlinedValue == 0.0F) {
      if (Float.floatToIntBits(inlinedValue) != 0)
        ldc(inlinedValue);
      else
        fconst_0();
      return;
    }
    if (inlinedValue == 1.0F) {
      fconst_1();
      return;
    }
    if (inlinedValue == 2.0F) {
      fconst_2();
      return;
    }
    ldc(inlinedValue);
  }
  public void generateInlinedValue(int inlinedValue) {
    switch (inlinedValue) {
    case -1:
      iconst_m1();
      break;
    case 0:
      iconst_0();
      break;
    case 1:
      iconst_1();
      break;
    case 2:
      iconst_2();
      break;
    case 3:
      iconst_3();
      break;
    case 4:
      iconst_4();
      break;
    case 5:
      iconst_5();
      break;
    default:
      if ((-128 <= inlinedValue) && (inlinedValue <= 127)) {
        bipush((byte)inlinedValue);
        return;
      }
      if ((-32768 <= inlinedValue) && (inlinedValue <= 32767)) {
        sipush(inlinedValue);
        return;
      }
      ldc(inlinedValue);
    }
  }

  public void generateInlinedValue(long inlinedValue) { if (inlinedValue == 0L) {
      lconst_0();
      return;
    }
    if (inlinedValue == 1L) {
      lconst_1();
      return;
    }
    ldc2_w(inlinedValue); }

  public void generateInlinedValue(short inlinedValue) {
    switch (inlinedValue) {
    case -1:
      iconst_m1();
      break;
    case 0:
      iconst_0();
      break;
    case 1:
      iconst_1();
      break;
    case 2:
      iconst_2();
      break;
    case 3:
      iconst_3();
      break;
    case 4:
      iconst_4();
      break;
    case 5:
      iconst_5();
      break;
    default:
      if ((-128 <= inlinedValue) && (inlinedValue <= 127)) {
        bipush((byte)inlinedValue);
        return;
      }
      sipush(inlinedValue);
    }
  }

  public void generateOuterAccess(Object[] mappingSequence, ASTNode invocationSite, Binding target, Scope scope) { if (mappingSequence == null) {
      if ((target instanceof LocalVariableBinding))
        scope.problemReporter().needImplementation();
      else {
        scope.problemReporter().noSuchEnclosingInstance((ReferenceBinding)target, invocationSite, false);
      }
      return;
    }
    if (mappingSequence == BlockScope.NoEnclosingInstanceInConstructorCall) {
      scope.problemReporter().noSuchEnclosingInstance((ReferenceBinding)target, invocationSite, true);
      return;
    }if (mappingSequence == BlockScope.NoEnclosingInstanceInStaticContext) {
      scope.problemReporter().noSuchEnclosingInstance((ReferenceBinding)target, invocationSite, false);
      return;
    }

    if (mappingSequence == BlockScope.EmulationPathToImplicitThis) {
      aload_0();
      return;
    }if ((mappingSequence[0] instanceof FieldBinding)) {
      FieldBinding fieldBinding = (FieldBinding)mappingSequence[0];
      aload_0();
      getfield(fieldBinding);
    } else {
      load((LocalVariableBinding)mappingSequence[0]);
    }
    int i = 1; for (int length = mappingSequence.length; i < length; i++)
      if ((mappingSequence[i] instanceof FieldBinding)) {
        FieldBinding fieldBinding = (FieldBinding)mappingSequence[i];
        getfield(fieldBinding);
      } else {
        invokestatic((MethodBinding)mappingSequence[i]);
      }
  }

  public void generateReturnBytecode(Expression expression)
  {
    if (expression == null) {
      return_();
    } else {
      int implicitConversion = expression.implicitConversion;
      if ((implicitConversion & 0x200) != 0) {
        areturn();
        return;
      }
      int runtimeType = (implicitConversion & 0xFF) >> 4;
      switch (runtimeType) {
      case 5:
      case 10:
        ireturn();
        break;
      case 9:
        freturn();
        break;
      case 7:
        lreturn();
        break;
      case 8:
        dreturn();
        break;
      case 6:
      default:
        areturn();
      }
    }
  }

  public void generateStringConcatenationAppend(BlockScope blockScope, Expression oper1, Expression oper2)
  {
    if (oper1 == null)
    {
      newStringContatenation();
      dup_x1();
      swap();

      invokeStringValueOf(1);
      invokeStringConcatenationStringConstructor();
    } else {
      int pc = this.position;
      oper1.generateOptimizedStringConcatenationCreation(blockScope, this, oper1.implicitConversion & 0xF);
      recordPositionsFrom(pc, oper1.sourceStart);
    }
    int pc = this.position;
    oper2.generateOptimizedStringConcatenation(blockScope, this, oper2.implicitConversion & 0xF);
    recordPositionsFrom(pc, oper2.sourceStart);
    invokeStringConcatenationToString();
  }

  public void generateSyntheticBodyForConstructorAccess(SyntheticMethodBinding accessBinding)
  {
    initializeMaxLocals(accessBinding);

    MethodBinding constructorBinding = accessBinding.targetMethod;
    TypeBinding[] parameters = constructorBinding.parameters;
    int length = parameters.length;
    int resolvedPosition = 1;
    aload_0();

    TypeBinding declaringClass = constructorBinding.declaringClass;
    if ((declaringClass.erasure().id == 41) || (declaringClass.isEnum())) {
      aload_1();
      iload_2();
      resolvedPosition += 2;
    }
    if (declaringClass.isNestedType()) {
      NestedTypeBinding nestedType = (NestedTypeBinding)declaringClass;
      SyntheticArgumentBinding[] syntheticArguments = nestedType.syntheticEnclosingInstances();
      for (int i = 0; i < (syntheticArguments == null ? 0 : syntheticArguments.length); i++)
      {
        TypeBinding type;
        load(type = syntheticArguments[i].type, resolvedPosition);
        if ((type == TypeBinding.DOUBLE) || (type == TypeBinding.LONG))
          resolvedPosition += 2;
        else
          resolvedPosition++;
      }
    }
    for (int i = 0; i < length; i++) {
      load(parameters[i], resolvedPosition);
      if ((parameters[i] == TypeBinding.DOUBLE) || (parameters[i] == TypeBinding.LONG))
        resolvedPosition += 2;
      else {
        resolvedPosition++;
      }
    }
    if (declaringClass.isNestedType()) {
      NestedTypeBinding nestedType = (NestedTypeBinding)declaringClass;
      SyntheticArgumentBinding[] syntheticArguments = nestedType.syntheticOuterLocalVariables();
      for (int i = 0; i < (syntheticArguments == null ? 0 : syntheticArguments.length); i++)
      {
        TypeBinding type;
        load(type = syntheticArguments[i].type, resolvedPosition);
        if ((type == TypeBinding.DOUBLE) || (type == TypeBinding.LONG))
          resolvedPosition += 2;
        else
          resolvedPosition++;
      }
    }
    invokespecial(constructorBinding);
    return_();
  }

  public void generateSyntheticBodyForEnumValueOf(SyntheticMethodBinding methodBinding)
  {
    initializeMaxLocals(methodBinding);
    ReferenceBinding declaringClass = methodBinding.declaringClass;
    ldc(declaringClass);
    aload_0();
    invokeJavaLangEnumvalueOf(declaringClass);
    checkcast(declaringClass);
    areturn();
  }

  public void generateSyntheticBodyForEnumValues(SyntheticMethodBinding methodBinding)
  {
    ClassScope scope = ((SourceTypeBinding)methodBinding.declaringClass).scope;
    FieldBinding enumValuesSyntheticfield = scope.referenceContext.enumValuesSyntheticfield;
    initializeMaxLocals(methodBinding);
    TypeBinding enumArray = methodBinding.returnType;

    getstatic(enumValuesSyntheticfield);
    dup();
    astore_0();
    iconst_0();
    aload_0();
    arraylength();
    dup();
    istore_1();
    newArray((ArrayBinding)enumArray);
    dup();
    astore_2();
    iconst_0();
    iload_1();
    invokeSystemArraycopy();
    aload_2();
    areturn();
  }
  public void generateSyntheticBodyForFieldReadAccess(SyntheticMethodBinding accessBinding) {
    initializeMaxLocals(accessBinding);
    FieldBinding fieldBinding = accessBinding.targetReadField;
    if (fieldBinding.isStatic()) {
      getstatic(fieldBinding);
    } else {
      aload_0();
      getfield(fieldBinding);
    }
    switch (fieldBinding.type.id)
    {
    case 2:
    case 3:
    case 4:
    case 5:
    case 10:
      ireturn();
      break;
    case 7:
      lreturn();
      break;
    case 9:
      freturn();
      break;
    case 8:
      dreturn();
      break;
    case 6:
    default:
      areturn();
    }
  }

  public void generateSyntheticBodyForFieldWriteAccess(SyntheticMethodBinding accessBinding) { initializeMaxLocals(accessBinding);
    FieldBinding fieldBinding = accessBinding.targetWriteField;
    if (fieldBinding.isStatic()) {
      load(fieldBinding.type, 0);
      putstatic(fieldBinding);
    } else {
      aload_0();
      load(fieldBinding.type, 1);
      putfield(fieldBinding);
    }
    return_(); }

  public void generateSyntheticBodyForMethodAccess(SyntheticMethodBinding accessMethod)
  {
    initializeMaxLocals(accessMethod);
    MethodBinding targetMethod = accessMethod.targetMethod;
    TypeBinding[] parameters = targetMethod.parameters;
    int length = parameters.length;
    TypeBinding[] arguments = accessMethod.kind == 6 ? 
      accessMethod.parameters : 
      null;
    int resolvedPosition;
    int resolvedPosition;
    if (targetMethod.isStatic()) {
      resolvedPosition = 0;
    } else {
      aload_0();
      resolvedPosition = 1;
    }
    for (int i = 0; i < length; i++) {
      TypeBinding parameter = parameters[i];
      if (arguments != null) {
        TypeBinding argument = arguments[i];
        load(argument, resolvedPosition);
        if (argument != parameter)
          checkcast(parameter);
      } else {
        load(parameter, resolvedPosition);
      }
      if ((parameter == TypeBinding.DOUBLE) || (parameter == TypeBinding.LONG))
        resolvedPosition += 2;
      else
        resolvedPosition++;
    }
    if (targetMethod.isStatic()) {
      invokestatic(targetMethod);
    }
    else if ((targetMethod.isConstructor()) || 
      (targetMethod.isPrivate()) || 
      (accessMethod.kind == 5)) {
      invokespecial(targetMethod);
    }
    else if (targetMethod.declaringClass.isInterface())
      invokeinterface(targetMethod);
    else {
      invokevirtual(targetMethod);
    }

    switch (targetMethod.returnType.id) {
    case 6:
      return_();
      break;
    case 2:
    case 3:
    case 4:
    case 5:
    case 10:
      ireturn();
      break;
    case 7:
      lreturn();
      break;
    case 9:
      freturn();
      break;
    case 8:
      dreturn();
      break;
    default:
      TypeBinding accessErasure = accessMethod.returnType.erasure();
      if (!targetMethod.returnType.isCompatibleWith(accessErasure))
        checkcast(accessErasure);
      areturn();
    }
  }

  public void generateSyntheticBodyForSwitchTable(SyntheticMethodBinding methodBinding) { ClassScope scope = ((SourceTypeBinding)methodBinding.declaringClass).scope;
    initializeMaxLocals(methodBinding);
    BranchLabel nullLabel = new BranchLabel(this);
    FieldBinding syntheticFieldBinding = methodBinding.targetReadField;

    getstatic(syntheticFieldBinding);
    dup();
    ifnull(nullLabel);
    areturn();
    pushOnStack(syntheticFieldBinding.type);
    nullLabel.place();
    pop();
    ReferenceBinding enumBinding = (ReferenceBinding)methodBinding.targetEnumType;
    ArrayBinding arrayBinding = scope.createArrayType(enumBinding, 1);
    invokeJavaLangEnumValues(enumBinding, arrayBinding);
    arraylength();
    newarray(10);
    astore_0();
    FieldBinding[] fields = enumBinding.fields();
    if (fields != null) {
      int i = 0; for (int max = fields.length; i < max; i++) {
        FieldBinding fieldBinding = fields[i];
        if ((fieldBinding.getAccessFlags() & 0x4000) != 0) {
          BranchLabel endLabel = new BranchLabel(this);
          ExceptionLabel anyExceptionHandler = new ExceptionLabel(this, TypeBinding.LONG);
          anyExceptionHandler.placeStart();
          aload_0();
          getstatic(fieldBinding);
          invokeEnumOrdinal(enumBinding.constantPoolName());
          generateInlinedValue(fieldBinding.id + 1);
          iastore();
          anyExceptionHandler.placeEnd();
          goto_(endLabel);

          pushOnStack(scope.getJavaLangThrowable());
          anyExceptionHandler.place();
          pop();
          endLabel.place();
        }
      }
    }
    aload_0();
    dup();
    putstatic(syntheticFieldBinding);
    areturn();
  }

  public void generateSyntheticEnclosingInstanceValues(BlockScope currentScope, ReferenceBinding targetType, Expression enclosingInstance, ASTNode invocationSite)
  {
    ReferenceBinding checkedTargetType = targetType.isAnonymousType() ? (ReferenceBinding)targetType.superclass().erasure() : targetType;
    boolean hasExtraEnclosingInstance = enclosingInstance != null;
    if ((hasExtraEnclosingInstance) && (
      (!checkedTargetType.isNestedType()) || (checkedTargetType.isStatic()))) {
      currentScope.problemReporter().unnecessaryEnclosingInstanceSpecification(enclosingInstance, checkedTargetType);
      return;
    }
    ReferenceBinding[] syntheticArgumentTypes;
    if ((syntheticArgumentTypes = targetType.syntheticEnclosingInstanceTypes()) != null)
    {
      ReferenceBinding targetEnclosingType = checkedTargetType.enclosingType();
      long compliance = currentScope.compilerOptions().complianceLevel;
      boolean denyEnclosingArgInConstructorCall;
      boolean denyEnclosingArgInConstructorCall;
      if (compliance <= 3080192L) {
        denyEnclosingArgInConstructorCall = invocationSite instanceof AllocationExpression;
      }
      else
      {
        boolean denyEnclosingArgInConstructorCall;
        if (compliance == 3145728L) {
          denyEnclosingArgInConstructorCall = ((invocationSite instanceof AllocationExpression)) || (
            ((invocationSite instanceof ExplicitConstructorCall)) && (((ExplicitConstructorCall)invocationSite).isSuperAccess()));
        }
        else {
          denyEnclosingArgInConstructorCall = (((invocationSite instanceof AllocationExpression)) || (
            ((invocationSite instanceof ExplicitConstructorCall)) && (((ExplicitConstructorCall)invocationSite).isSuperAccess()))) && 
            (!targetType.isLocalType());
        }
      }
      boolean complyTo14 = compliance >= 3145728L;
      int i = 0; for (int max = syntheticArgumentTypes.length; i < max; i++) {
        ReferenceBinding syntheticArgType = syntheticArgumentTypes[i];
        if ((hasExtraEnclosingInstance) && (syntheticArgType == targetEnclosingType)) {
          hasExtraEnclosingInstance = false;
          enclosingInstance.generateCode(currentScope, this, true);
          if (complyTo14) {
            dup();
            invokeObjectGetClass();
            pop();
          }
        } else {
          Object[] emulationPath = currentScope.getEmulationPath(
            syntheticArgType, 
            false, 
            denyEnclosingArgInConstructorCall);
          generateOuterAccess(emulationPath, invocationSite, syntheticArgType, currentScope);
        }
      }
      if (hasExtraEnclosingInstance)
        currentScope.problemReporter().unnecessaryEnclosingInstanceSpecification(enclosingInstance, checkedTargetType);
    }
  }

  public void generateSyntheticOuterArgumentValues(BlockScope currentScope, ReferenceBinding targetType, ASTNode invocationSite)
  {
    SyntheticArgumentBinding[] syntheticArguments;
    if ((syntheticArguments = targetType.syntheticOuterLocalVariables()) != null) {
      int i = 0; for (int max = syntheticArguments.length; i < max; i++) {
        LocalVariableBinding targetVariable = syntheticArguments[i].actualOuterLocalVariable;
        VariableBinding[] emulationPath = currentScope.getEmulationPath(targetVariable);
        generateOuterAccess(emulationPath, invocationSite, targetVariable, currentScope);
      }
    }
  }

  public void generateUnboxingConversion(int unboxedTypeID) { switch (unboxedTypeID)
    {
    case 3:
      invoke(
        -74, 
        0, 
        1, 
        ConstantPool.JavaLangByteConstantPoolName, 
        ConstantPool.BYTEVALUE_BYTE_METHOD_NAME, 
        ConstantPool.BYTEVALUE_BYTE_METHOD_SIGNATURE);
      break;
    case 4:
      invoke(
        -74, 
        0, 
        1, 
        ConstantPool.JavaLangShortConstantPoolName, 
        ConstantPool.SHORTVALUE_SHORT_METHOD_NAME, 
        ConstantPool.SHORTVALUE_SHORT_METHOD_SIGNATURE);
      break;
    case 2:
      invoke(
        -74, 
        0, 
        1, 
        ConstantPool.JavaLangCharacterConstantPoolName, 
        ConstantPool.CHARVALUE_CHARACTER_METHOD_NAME, 
        ConstantPool.CHARVALUE_CHARACTER_METHOD_SIGNATURE);
      break;
    case 10:
      invoke(
        -74, 
        0, 
        1, 
        ConstantPool.JavaLangIntegerConstantPoolName, 
        ConstantPool.INTVALUE_INTEGER_METHOD_NAME, 
        ConstantPool.INTVALUE_INTEGER_METHOD_SIGNATURE);
      break;
    case 7:
      invoke(
        -74, 
        0, 
        2, 
        ConstantPool.JavaLangLongConstantPoolName, 
        ConstantPool.LONGVALUE_LONG_METHOD_NAME, 
        ConstantPool.LONGVALUE_LONG_METHOD_SIGNATURE);
      break;
    case 9:
      invoke(
        -74, 
        0, 
        1, 
        ConstantPool.JavaLangFloatConstantPoolName, 
        ConstantPool.FLOATVALUE_FLOAT_METHOD_NAME, 
        ConstantPool.FLOATVALUE_FLOAT_METHOD_SIGNATURE);
      break;
    case 8:
      invoke(
        -74, 
        0, 
        2, 
        ConstantPool.JavaLangDoubleConstantPoolName, 
        ConstantPool.DOUBLEVALUE_DOUBLE_METHOD_NAME, 
        ConstantPool.DOUBLEVALUE_DOUBLE_METHOD_SIGNATURE);
      break;
    case 5:
      invoke(
        -74, 
        0, 
        1, 
        ConstantPool.JavaLangBooleanConstantPoolName, 
        ConstantPool.BOOLEANVALUE_BOOLEAN_METHOD_NAME, 
        ConstantPool.BOOLEANVALUE_BOOLEAN_METHOD_SIGNATURE);
    case 6:
    }
  }

  public void generateWideRevertedConditionalBranch(byte revertedOpcode, BranchLabel wideTarget)
  {
    BranchLabel intermediate = new BranchLabel(this);
    if (this.classFileOffset >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 1;
    this.bCodeStream[(this.classFileOffset++)] = revertedOpcode;
    intermediate.branch();
    goto_w(wideTarget);
    intermediate.place();
  }
  public void getBaseTypeValue(int baseTypeID) {
    switch (baseTypeID)
    {
    case 3:
      invoke(
        -74, 
        0, 
        1, 
        ConstantPool.JavaLangByteConstantPoolName, 
        ConstantPool.BYTEVALUE_BYTE_METHOD_NAME, 
        ConstantPool.BYTEVALUE_BYTE_METHOD_SIGNATURE);
      break;
    case 4:
      invoke(
        -74, 
        0, 
        1, 
        ConstantPool.JavaLangShortConstantPoolName, 
        ConstantPool.SHORTVALUE_SHORT_METHOD_NAME, 
        ConstantPool.SHORTVALUE_SHORT_METHOD_SIGNATURE);
      break;
    case 2:
      invoke(
        -74, 
        0, 
        1, 
        ConstantPool.JavaLangCharacterConstantPoolName, 
        ConstantPool.CHARVALUE_CHARACTER_METHOD_NAME, 
        ConstantPool.CHARVALUE_CHARACTER_METHOD_SIGNATURE);
      break;
    case 10:
      invoke(
        -74, 
        0, 
        1, 
        ConstantPool.JavaLangIntegerConstantPoolName, 
        ConstantPool.INTVALUE_INTEGER_METHOD_NAME, 
        ConstantPool.INTVALUE_INTEGER_METHOD_SIGNATURE);
      break;
    case 7:
      invoke(
        -74, 
        0, 
        2, 
        ConstantPool.JavaLangLongConstantPoolName, 
        ConstantPool.LONGVALUE_LONG_METHOD_NAME, 
        ConstantPool.LONGVALUE_LONG_METHOD_SIGNATURE);
      break;
    case 9:
      invoke(
        -74, 
        0, 
        1, 
        ConstantPool.JavaLangFloatConstantPoolName, 
        ConstantPool.FLOATVALUE_FLOAT_METHOD_NAME, 
        ConstantPool.FLOATVALUE_FLOAT_METHOD_SIGNATURE);
      break;
    case 8:
      invoke(
        -74, 
        0, 
        2, 
        ConstantPool.JavaLangDoubleConstantPoolName, 
        ConstantPool.DOUBLEVALUE_DOUBLE_METHOD_NAME, 
        ConstantPool.DOUBLEVALUE_DOUBLE_METHOD_SIGNATURE);
      break;
    case 5:
      invoke(
        -74, 
        0, 
        1, 
        ConstantPool.JavaLangBooleanConstantPoolName, 
        ConstantPool.BOOLEANVALUE_BOOLEAN_METHOD_NAME, 
        ConstantPool.BOOLEANVALUE_BOOLEAN_METHOD_SIGNATURE);
    case 6:
    }
  }

  public final byte[] getContents()
  {
    byte[] contents;
    System.arraycopy(this.bCodeStream, 0, contents = new byte[this.position], 0, this.position);
    return contents;
  }

  public void getfield(FieldBinding fieldBinding) {
    int returnTypeSize = 1;
    if ((fieldBinding.type.id == 8) || (fieldBinding.type.id == 7)) {
      returnTypeSize = 2;
    }
    generateFieldAccess(
      (byte)-76, 
      returnTypeSize, 
      fieldBinding.declaringClass, 
      fieldBinding.name, 
      fieldBinding.type);
  }
  protected int getPosition() {
    return this.position;
  }

  public void getstatic(FieldBinding fieldBinding) {
    int returnTypeSize = 1;
    if ((fieldBinding.type.id == 8) || (fieldBinding.type.id == 7)) {
      returnTypeSize = 2;
    }
    generateFieldAccess(
      (byte)-78, 
      returnTypeSize, 
      fieldBinding.declaringClass, 
      fieldBinding.name, 
      fieldBinding.type);
  }
  public void getTYPE(int baseTypeID) {
    this.countLabels = 0;
    switch (baseTypeID)
    {
    case 3:
      generateFieldAccess(
        (byte)-78, 
        1, 
        ConstantPool.JavaLangByteConstantPoolName, 
        ConstantPool.TYPE, 
        ConstantPool.JavaLangClassSignature);
      break;
    case 4:
      generateFieldAccess(
        (byte)-78, 
        1, 
        ConstantPool.JavaLangShortConstantPoolName, 
        ConstantPool.TYPE, 
        ConstantPool.JavaLangClassSignature);
      break;
    case 2:
      generateFieldAccess(
        (byte)-78, 
        1, 
        ConstantPool.JavaLangCharacterConstantPoolName, 
        ConstantPool.TYPE, 
        ConstantPool.JavaLangClassSignature);
      break;
    case 10:
      generateFieldAccess(
        (byte)-78, 
        1, 
        ConstantPool.JavaLangIntegerConstantPoolName, 
        ConstantPool.TYPE, 
        ConstantPool.JavaLangClassSignature);
      break;
    case 7:
      generateFieldAccess(
        (byte)-78, 
        1, 
        ConstantPool.JavaLangLongConstantPoolName, 
        ConstantPool.TYPE, 
        ConstantPool.JavaLangClassSignature);
      break;
    case 9:
      generateFieldAccess(
        (byte)-78, 
        1, 
        ConstantPool.JavaLangFloatConstantPoolName, 
        ConstantPool.TYPE, 
        ConstantPool.JavaLangClassSignature);
      break;
    case 8:
      generateFieldAccess(
        (byte)-78, 
        1, 
        ConstantPool.JavaLangDoubleConstantPoolName, 
        ConstantPool.TYPE, 
        ConstantPool.JavaLangClassSignature);
      break;
    case 5:
      generateFieldAccess(
        (byte)-78, 
        1, 
        ConstantPool.JavaLangBooleanConstantPoolName, 
        ConstantPool.TYPE, 
        ConstantPool.JavaLangClassSignature);
      break;
    case 6:
      generateFieldAccess(
        (byte)-78, 
        1, 
        ConstantPool.JavaLangVoidConstantPoolName, 
        ConstantPool.TYPE, 
        ConstantPool.JavaLangClassSignature);
    }
  }

  public void goto_(BranchLabel label)
  {
    if (this.wideMode) {
      goto_w(label);
      return;
    }

    if (this.classFileOffset >= this.bCodeStream.length) {
      resizeByteArray();
    }
    boolean chained = inlineForwardReferencesFromLabelsTargeting(label, this.position);

    if ((chained) && (this.lastAbruptCompletion == this.position)) {
      if (label.position != -1) {
        int[] forwardRefs = label.forwardReferences();
        int i = 0; for (int max = label.forwardReferenceCount(); i < max; i++) {
          writePosition(label, forwardRefs[i]);
        }
        this.countLabels = 0;
      }

      return;
    }
    this.position += 1;
    this.bCodeStream[(this.classFileOffset++)] = -89;
    label.branch();
    this.lastAbruptCompletion = this.position;
  }

  public void goto_w(BranchLabel label) {
    if (this.classFileOffset >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 1;
    this.bCodeStream[(this.classFileOffset++)] = -56;
    label.branchWide();
    this.lastAbruptCompletion = this.position;
  }

  public void i2b() {
    this.countLabels = 0;
    if (this.classFileOffset >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 1;
    this.bCodeStream[(this.classFileOffset++)] = -111;
  }

  public void i2c() {
    this.countLabels = 0;
    if (this.classFileOffset >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 1;
    this.bCodeStream[(this.classFileOffset++)] = -110;
  }

  public void i2d() {
    this.countLabels = 0;
    this.stackDepth += 1;
    if (this.stackDepth > this.stackMax)
      this.stackMax = this.stackDepth;
    if (this.classFileOffset >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 1;
    this.bCodeStream[(this.classFileOffset++)] = -121;
  }

  public void i2f() {
    this.countLabels = 0;
    if (this.classFileOffset >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 1;
    this.bCodeStream[(this.classFileOffset++)] = -122;
  }

  public void i2l() {
    this.countLabels = 0;
    this.stackDepth += 1;
    if (this.stackDepth > this.stackMax)
      this.stackMax = this.stackDepth;
    if (this.classFileOffset >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 1;
    this.bCodeStream[(this.classFileOffset++)] = -123;
  }

  public void i2s() {
    this.countLabels = 0;
    if (this.classFileOffset >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 1;
    this.bCodeStream[(this.classFileOffset++)] = -109;
  }

  public void iadd() {
    this.countLabels = 0;
    this.stackDepth -= 1;
    if (this.classFileOffset >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 1;
    this.bCodeStream[(this.classFileOffset++)] = 96;
  }

  public void iaload() {
    this.countLabels = 0;
    this.stackDepth -= 1;
    if (this.classFileOffset >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 1;
    this.bCodeStream[(this.classFileOffset++)] = 46;
  }

  public void iand() {
    this.countLabels = 0;
    this.stackDepth -= 1;
    if (this.classFileOffset >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 1;
    this.bCodeStream[(this.classFileOffset++)] = 126;
  }

  public void iastore() {
    this.countLabels = 0;
    this.stackDepth -= 3;
    if (this.classFileOffset >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 1;
    this.bCodeStream[(this.classFileOffset++)] = 79;
  }

  public void iconst_0() {
    this.countLabels = 0;
    this.stackDepth += 1;
    if (this.stackDepth > this.stackMax)
      this.stackMax = this.stackDepth;
    if (this.classFileOffset >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 1;
    this.bCodeStream[(this.classFileOffset++)] = 3;
  }

  public void iconst_1() {
    this.countLabels = 0;
    this.stackDepth += 1;
    if (this.stackDepth > this.stackMax)
      this.stackMax = this.stackDepth;
    if (this.classFileOffset >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 1;
    this.bCodeStream[(this.classFileOffset++)] = 4;
  }

  public void iconst_2() {
    this.countLabels = 0;
    this.stackDepth += 1;
    if (this.stackDepth > this.stackMax)
      this.stackMax = this.stackDepth;
    if (this.classFileOffset >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 1;
    this.bCodeStream[(this.classFileOffset++)] = 5;
  }

  public void iconst_3() {
    this.countLabels = 0;
    this.stackDepth += 1;
    if (this.stackDepth > this.stackMax)
      this.stackMax = this.stackDepth;
    if (this.classFileOffset >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 1;
    this.bCodeStream[(this.classFileOffset++)] = 6;
  }

  public void iconst_4() {
    this.countLabels = 0;
    this.stackDepth += 1;
    if (this.stackDepth > this.stackMax)
      this.stackMax = this.stackDepth;
    if (this.classFileOffset >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 1;
    this.bCodeStream[(this.classFileOffset++)] = 7;
  }

  public void iconst_5() {
    this.countLabels = 0;
    this.stackDepth += 1;
    if (this.stackDepth > this.stackMax)
      this.stackMax = this.stackDepth;
    if (this.classFileOffset >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 1;
    this.bCodeStream[(this.classFileOffset++)] = 8;
  }

  public void iconst_m1() {
    this.countLabels = 0;
    this.stackDepth += 1;
    if (this.stackDepth > this.stackMax)
      this.stackMax = this.stackDepth;
    if (this.classFileOffset >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 1;
    this.bCodeStream[(this.classFileOffset++)] = 2;
  }

  public void idiv() {
    this.countLabels = 0;
    this.stackDepth -= 1;
    if (this.classFileOffset >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 1;
    this.bCodeStream[(this.classFileOffset++)] = 108;
  }

  public void if_acmpeq(BranchLabel lbl) {
    this.countLabels = 0;
    this.stackDepth -= 2;
    if (this.wideMode) {
      generateWideRevertedConditionalBranch((byte)-90, lbl);
    } else {
      if (this.classFileOffset >= this.bCodeStream.length) {
        resizeByteArray();
      }
      this.position += 1;
      this.bCodeStream[(this.classFileOffset++)] = -91;
      lbl.branch();
    }
  }

  public void if_acmpne(BranchLabel lbl) {
    this.countLabels = 0;
    this.stackDepth -= 2;
    if (this.wideMode) {
      generateWideRevertedConditionalBranch((byte)-91, lbl);
    } else {
      if (this.classFileOffset >= this.bCodeStream.length) {
        resizeByteArray();
      }
      this.position += 1;
      this.bCodeStream[(this.classFileOffset++)] = -90;
      lbl.branch();
    }
  }

  public void if_icmpeq(BranchLabel lbl) {
    this.countLabels = 0;
    this.stackDepth -= 2;
    if (this.wideMode) {
      generateWideRevertedConditionalBranch((byte)-96, lbl);
    } else {
      if (this.classFileOffset >= this.bCodeStream.length) {
        resizeByteArray();
      }
      this.position += 1;
      this.bCodeStream[(this.classFileOffset++)] = -97;
      lbl.branch();
    }
  }

  public void if_icmpge(BranchLabel lbl) {
    this.countLabels = 0;
    this.stackDepth -= 2;
    if (this.wideMode) {
      generateWideRevertedConditionalBranch((byte)-95, lbl);
    } else {
      if (this.classFileOffset >= this.bCodeStream.length) {
        resizeByteArray();
      }
      this.position += 1;
      this.bCodeStream[(this.classFileOffset++)] = -94;
      lbl.branch();
    }
  }

  public void if_icmpgt(BranchLabel lbl) {
    this.countLabels = 0;
    this.stackDepth -= 2;
    if (this.wideMode) {
      generateWideRevertedConditionalBranch((byte)-92, lbl);
    } else {
      if (this.classFileOffset >= this.bCodeStream.length) {
        resizeByteArray();
      }
      this.position += 1;
      this.bCodeStream[(this.classFileOffset++)] = -93;
      lbl.branch();
    }
  }

  public void if_icmple(BranchLabel lbl) {
    this.countLabels = 0;
    this.stackDepth -= 2;
    if (this.wideMode) {
      generateWideRevertedConditionalBranch((byte)-93, lbl);
    } else {
      if (this.classFileOffset >= this.bCodeStream.length) {
        resizeByteArray();
      }
      this.position += 1;
      this.bCodeStream[(this.classFileOffset++)] = -92;
      lbl.branch();
    }
  }

  public void if_icmplt(BranchLabel lbl) {
    this.countLabels = 0;
    this.stackDepth -= 2;
    if (this.wideMode) {
      generateWideRevertedConditionalBranch((byte)-94, lbl);
    } else {
      if (this.classFileOffset >= this.bCodeStream.length) {
        resizeByteArray();
      }
      this.position += 1;
      this.bCodeStream[(this.classFileOffset++)] = -95;
      lbl.branch();
    }
  }

  public void if_icmpne(BranchLabel lbl) {
    this.countLabels = 0;
    this.stackDepth -= 2;
    if (this.wideMode) {
      generateWideRevertedConditionalBranch((byte)-97, lbl);
    } else {
      if (this.classFileOffset >= this.bCodeStream.length) {
        resizeByteArray();
      }
      this.position += 1;
      this.bCodeStream[(this.classFileOffset++)] = -96;
      lbl.branch();
    }
  }

  public void ifeq(BranchLabel lbl) {
    this.countLabels = 0;
    this.stackDepth -= 1;
    if (this.wideMode) {
      generateWideRevertedConditionalBranch((byte)-102, lbl);
    } else {
      if (this.classFileOffset >= this.bCodeStream.length) {
        resizeByteArray();
      }
      this.position += 1;
      this.bCodeStream[(this.classFileOffset++)] = -103;
      lbl.branch();
    }
  }

  public void ifge(BranchLabel lbl) {
    this.countLabels = 0;
    this.stackDepth -= 1;
    if (this.wideMode) {
      generateWideRevertedConditionalBranch((byte)-101, lbl);
    } else {
      if (this.classFileOffset >= this.bCodeStream.length) {
        resizeByteArray();
      }
      this.position += 1;
      this.bCodeStream[(this.classFileOffset++)] = -100;
      lbl.branch();
    }
  }

  public void ifgt(BranchLabel lbl) {
    this.countLabels = 0;
    this.stackDepth -= 1;
    if (this.wideMode) {
      generateWideRevertedConditionalBranch((byte)-98, lbl);
    } else {
      if (this.classFileOffset >= this.bCodeStream.length) {
        resizeByteArray();
      }
      this.position += 1;
      this.bCodeStream[(this.classFileOffset++)] = -99;
      lbl.branch();
    }
  }

  public void ifle(BranchLabel lbl) {
    this.countLabels = 0;
    this.stackDepth -= 1;
    if (this.wideMode) {
      generateWideRevertedConditionalBranch((byte)-99, lbl);
    } else {
      if (this.classFileOffset >= this.bCodeStream.length) {
        resizeByteArray();
      }
      this.position += 1;
      this.bCodeStream[(this.classFileOffset++)] = -98;
      lbl.branch();
    }
  }

  public void iflt(BranchLabel lbl) {
    this.countLabels = 0;
    this.stackDepth -= 1;
    if (this.wideMode) {
      generateWideRevertedConditionalBranch((byte)-100, lbl);
    } else {
      if (this.classFileOffset >= this.bCodeStream.length) {
        resizeByteArray();
      }
      this.position += 1;
      this.bCodeStream[(this.classFileOffset++)] = -101;
      lbl.branch();
    }
  }

  public void ifne(BranchLabel lbl) {
    this.countLabels = 0;
    this.stackDepth -= 1;
    if (this.wideMode) {
      generateWideRevertedConditionalBranch((byte)-103, lbl);
    } else {
      if (this.classFileOffset >= this.bCodeStream.length) {
        resizeByteArray();
      }
      this.position += 1;
      this.bCodeStream[(this.classFileOffset++)] = -102;
      lbl.branch();
    }
  }

  public void ifnonnull(BranchLabel lbl) {
    this.countLabels = 0;
    this.stackDepth -= 1;
    if (this.wideMode) {
      generateWideRevertedConditionalBranch((byte)-58, lbl);
    } else {
      if (this.classFileOffset >= this.bCodeStream.length) {
        resizeByteArray();
      }
      this.position += 1;
      this.bCodeStream[(this.classFileOffset++)] = -57;
      lbl.branch();
    }
  }

  public void ifnull(BranchLabel lbl) {
    this.countLabels = 0;
    this.stackDepth -= 1;
    if (this.wideMode) {
      generateWideRevertedConditionalBranch((byte)-57, lbl);
    } else {
      if (this.classFileOffset >= this.bCodeStream.length) {
        resizeByteArray();
      }
      this.position += 1;
      this.bCodeStream[(this.classFileOffset++)] = -58;
      lbl.branch();
    }
  }

  public final void iinc(int index, int value) {
    this.countLabels = 0;
    if ((index > 255) || (value < -128) || (value > 127)) {
      if (this.classFileOffset + 3 >= this.bCodeStream.length) {
        resizeByteArray();
      }
      this.position += 2;
      this.bCodeStream[(this.classFileOffset++)] = -60;
      this.bCodeStream[(this.classFileOffset++)] = -124;
      writeUnsignedShort(index);
      writeSignedShort(value);
    } else {
      if (this.classFileOffset + 2 >= this.bCodeStream.length) {
        resizeByteArray();
      }
      this.position += 3;
      this.bCodeStream[(this.classFileOffset++)] = -124;
      this.bCodeStream[(this.classFileOffset++)] = ((byte)index);
      this.bCodeStream[(this.classFileOffset++)] = ((byte)value);
    }
  }

  public void iload(int iArg) {
    this.countLabels = 0;
    this.stackDepth += 1;
    if (this.maxLocals <= iArg) {
      this.maxLocals = (iArg + 1);
    }
    if (this.stackDepth > this.stackMax)
      this.stackMax = this.stackDepth;
    if (iArg > 255) {
      if (this.classFileOffset + 3 >= this.bCodeStream.length) {
        resizeByteArray();
      }
      this.position += 2;
      this.bCodeStream[(this.classFileOffset++)] = -60;
      this.bCodeStream[(this.classFileOffset++)] = 21;
      writeUnsignedShort(iArg);
    } else {
      if (this.classFileOffset + 1 >= this.bCodeStream.length) {
        resizeByteArray();
      }
      this.position += 2;
      this.bCodeStream[(this.classFileOffset++)] = 21;
      this.bCodeStream[(this.classFileOffset++)] = ((byte)iArg);
    }
  }

  public void iload_0() {
    this.countLabels = 0;
    this.stackDepth += 1;
    if (this.maxLocals <= 0) {
      this.maxLocals = 1;
    }
    if (this.stackDepth > this.stackMax)
      this.stackMax = this.stackDepth;
    if (this.classFileOffset >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 1;
    this.bCodeStream[(this.classFileOffset++)] = 26;
  }

  public void iload_1() {
    this.countLabels = 0;
    this.stackDepth += 1;
    if (this.maxLocals <= 1) {
      this.maxLocals = 2;
    }
    if (this.stackDepth > this.stackMax)
      this.stackMax = this.stackDepth;
    if (this.classFileOffset >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 1;
    this.bCodeStream[(this.classFileOffset++)] = 27;
  }

  public void iload_2() {
    this.countLabels = 0;
    this.stackDepth += 1;
    if (this.maxLocals <= 2) {
      this.maxLocals = 3;
    }
    if (this.stackDepth > this.stackMax)
      this.stackMax = this.stackDepth;
    if (this.classFileOffset >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 1;
    this.bCodeStream[(this.classFileOffset++)] = 28;
  }

  public void iload_3() {
    this.countLabels = 0;
    this.stackDepth += 1;
    if (this.maxLocals <= 3) {
      this.maxLocals = 4;
    }
    if (this.stackDepth > this.stackMax)
      this.stackMax = this.stackDepth;
    if (this.classFileOffset >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 1;
    this.bCodeStream[(this.classFileOffset++)] = 29;
  }

  public void imul() {
    this.countLabels = 0;
    this.stackDepth -= 1;
    if (this.classFileOffset >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 1;
    this.bCodeStream[(this.classFileOffset++)] = 104;
  }
  public int indexOfSameLineEntrySincePC(int pc, int line) {
    int index = pc; for (int max = this.pcToSourceMapSize; index < max; index += 2) {
      if (this.pcToSourceMap[(index + 1)] == line)
        return index;
    }
    return -1;
  }

  public void ineg() {
    this.countLabels = 0;
    if (this.classFileOffset >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 1;
    this.bCodeStream[(this.classFileOffset++)] = 116;
  }

  public boolean inlineForwardReferencesFromLabelsTargeting(BranchLabel targetLabel, int gotoLocation)
  {
    if (targetLabel.delegate != null) return false;
    int chaining = 0;
    for (int i = this.countLabels - 1; i >= 0; i--) {
      BranchLabel currentLabel = this.labels[i];
      if (currentLabel.position != gotoLocation) break;
      if (currentLabel == targetLabel) {
        chaining |= 4;
      }
      else if (currentLabel.isStandardLabel()) {
        if (currentLabel.delegate == null) {
          targetLabel.becomeDelegateFor(currentLabel);
          chaining |= 2;
        }
      }
      else
        chaining |= 4;
    }
    return (chaining & 0x6) == 2;
  }
  public void init(ClassFile targetClassFile) {
    this.classFile = targetClassFile;
    this.constantPool = targetClassFile.constantPool;
    this.bCodeStream = targetClassFile.contents;
    this.classFileOffset = targetClassFile.contentsOffset;
    this.startingClassFileOffset = this.classFileOffset;
    this.pcToSourceMapSize = 0;
    this.lastEntryPC = 0;
    int length = this.visibleLocals.length;
    if (noVisibleLocals.length < length) {
      noVisibleLocals = new LocalVariableBinding[length];
    }
    System.arraycopy(noVisibleLocals, 0, this.visibleLocals, 0, length);
    this.visibleLocalsCount = 0;

    length = this.locals.length;
    if (noLocals.length < length) {
      noLocals = new LocalVariableBinding[length];
    }
    System.arraycopy(noLocals, 0, this.locals, 0, length);
    this.allLocalsCounter = 0;

    length = this.exceptionLabels.length;
    if (noExceptionHandlers.length < length) {
      noExceptionHandlers = new ExceptionLabel[length];
    }
    System.arraycopy(noExceptionHandlers, 0, this.exceptionLabels, 0, length);
    this.exceptionLabelsCounter = 0;

    length = this.labels.length;
    if (noLabels.length < length) {
      noLabels = new BranchLabel[length];
    }
    System.arraycopy(noLabels, 0, this.labels, 0, length);
    this.countLabels = 0;
    this.lastAbruptCompletion = -1;

    this.stackMax = 0;
    this.stackDepth = 0;
    this.maxLocals = 0;
    this.position = 0;
  }

  public void initializeMaxLocals(MethodBinding methodBinding)
  {
    if (methodBinding == null) {
      this.maxLocals = 0;
      return;
    }

    this.maxLocals = (methodBinding.isStatic() ? 0 : 1);

    if ((methodBinding.isConstructor()) && (methodBinding.declaringClass.isEnum())) {
      this.maxLocals += 2;
    }

    if ((methodBinding.isConstructor()) && (methodBinding.declaringClass.isNestedType()))
    {
      ReferenceBinding[] enclosingInstanceTypes;
      if ((enclosingInstanceTypes = methodBinding.declaringClass.syntheticEnclosingInstanceTypes()) != null) {
        int i = 0; for (int max = enclosingInstanceTypes.length; i < max; i++)
          this.maxLocals += 1;
      }
      SyntheticArgumentBinding[] syntheticArguments;
      if ((syntheticArguments = methodBinding.declaringClass.syntheticOuterLocalVariables()) != null) {
        int i = 0; for (int max = syntheticArguments.length; i < max; i++)
        {
          TypeBinding argType;
          if (((argType = syntheticArguments[i].type) == TypeBinding.LONG) || (argType == TypeBinding.DOUBLE))
            this.maxLocals += 2;
          else
            this.maxLocals += 1;
        }
      }
    }
    TypeBinding[] arguments;
    if ((arguments = methodBinding.parameters) != null) {
      int i = 0; for (int max = arguments.length; i < max; i++)
      {
        TypeBinding argType;
        if (((argType = arguments[i]) == TypeBinding.LONG) || (argType == TypeBinding.DOUBLE))
          this.maxLocals += 2;
        else
          this.maxLocals += 1;
      }
    }
  }

  public void instance_of(TypeBinding typeBinding)
  {
    this.countLabels = 0;
    if (this.classFileOffset + 2 >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 1;
    this.bCodeStream[(this.classFileOffset++)] = -63;
    writeUnsignedShort(this.constantPool.literalIndexForType(typeBinding));
  }
  protected void invoke(int opcode, int argsSize, int returnTypeSize, char[] declaringClass, char[] selector, char[] signature) {
    this.countLabels = 0;
    int argCount = argsSize;
    switch (opcode) {
    case -71:
      if (this.classFileOffset + 4 >= this.bCodeStream.length) {
        resizeByteArray();
      }
      this.position += 3;
      this.bCodeStream[(this.classFileOffset++)] = -71;
      writeUnsignedShort(this.constantPool.literalIndexForMethod(declaringClass, selector, signature, true));
      argCount++;
      this.bCodeStream[(this.classFileOffset++)] = ((byte)argCount);
      this.bCodeStream[(this.classFileOffset++)] = 0;
      break;
    case -74:
    case -73:
      if (this.classFileOffset + 2 >= this.bCodeStream.length) {
        resizeByteArray();
      }
      this.position += 1;
      this.bCodeStream[(this.classFileOffset++)] = ((byte)opcode);
      writeUnsignedShort(this.constantPool.literalIndexForMethod(declaringClass, selector, signature, false));
      argCount++;
      break;
    case -72:
      if (this.classFileOffset + 2 >= this.bCodeStream.length) {
        resizeByteArray();
      }
      this.position += 1;
      this.bCodeStream[(this.classFileOffset++)] = -72;
      writeUnsignedShort(this.constantPool.literalIndexForMethod(declaringClass, selector, signature, false));
    }this.stackDepth = 
      (this.stackDepth + (
      returnTypeSize - argCount));
    if (this.stackDepth > this.stackMax)
      this.stackMax = this.stackDepth;
  }

  protected void invokeAccessibleObjectSetAccessible()
  {
    invoke(
      -74, 
      1, 
      0, 
      ConstantPool.JAVALANGREFLECTACCESSIBLEOBJECT_CONSTANTPOOLNAME, 
      ConstantPool.SETACCESSIBLE_NAME, 
      ConstantPool.SETACCESSIBLE_SIGNATURE);
  }

  protected void invokeArrayNewInstance() {
    invoke(
      -72, 
      2, 
      1, 
      ConstantPool.JAVALANGREFLECTARRAY_CONSTANTPOOLNAME, 
      ConstantPool.NewInstance, 
      ConstantPool.NewInstanceSignature);
  }

  public void invokeClassForName()
  {
    invoke(
      -72, 
      1, 
      1, 
      ConstantPool.JavaLangClassConstantPoolName, 
      ConstantPool.ForName, 
      ConstantPool.ForNameSignature);
  }

  protected void invokeClassGetDeclaredConstructor() {
    invoke(
      -74, 
      1, 
      1, 
      ConstantPool.JavaLangClassConstantPoolName, 
      ConstantPool.GETDECLAREDCONSTRUCTOR_NAME, 
      ConstantPool.GETDECLAREDCONSTRUCTOR_SIGNATURE);
  }

  protected void invokeClassGetDeclaredField() {
    invoke(
      -74, 
      1, 
      1, 
      ConstantPool.JavaLangClassConstantPoolName, 
      ConstantPool.GETDECLAREDFIELD_NAME, 
      ConstantPool.GETDECLAREDFIELD_SIGNATURE);
  }

  protected void invokeClassGetDeclaredMethod() {
    invoke(
      -74, 
      2, 
      1, 
      ConstantPool.JavaLangClassConstantPoolName, 
      ConstantPool.GETDECLAREDMETHOD_NAME, 
      ConstantPool.GETDECLAREDMETHOD_SIGNATURE);
  }

  public void invokeEnumOrdinal(char[] enumTypeConstantPoolName)
  {
    invoke(
      -74, 
      0, 
      1, 
      enumTypeConstantPoolName, 
      ConstantPool.Ordinal, 
      ConstantPool.OrdinalSignature);
  }

  public void invokeinterface(MethodBinding methodBinding) {
    this.countLabels = 0;

    int argCount = 1;

    if (this.classFileOffset + 4 >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 3;
    this.bCodeStream[(this.classFileOffset++)] = -71;
    writeUnsignedShort(
      this.constantPool.literalIndexForMethod(
      methodBinding.constantPoolDeclaringClass(), 
      methodBinding.selector, 
      methodBinding.signature(this.classFile), 
      true));
    for (int i = methodBinding.parameters.length - 1; i >= 0; i--)
    {
      int id;
      if (((id = methodBinding.parameters[i].id) == 8) || (id == 7))
        argCount += 2;
      else
        argCount++; 
    }
    this.bCodeStream[(this.classFileOffset++)] = ((byte)argCount);

    this.bCodeStream[(this.classFileOffset++)] = 0;
    int id;
    if (((id = methodBinding.returnType.id) == 8) || (id == 7)) {
      this.stackDepth += 2 - argCount;
    }
    else if (id == 6)
      this.stackDepth -= argCount;
    else {
      this.stackDepth += 1 - argCount;
    }

    if (this.stackDepth > this.stackMax)
      this.stackMax = this.stackDepth;
  }

  public void invokeJavaLangAssertionErrorConstructor(int typeBindingID)
  {
    int argCount = 1;
    char[] signature = (char[])null;
    switch (typeBindingID) {
    case 3:
    case 4:
    case 10:
      signature = ConstantPool.IntConstrSignature;
      break;
    case 7:
      signature = ConstantPool.LongConstrSignature;
      argCount = 2;
      break;
    case 9:
      signature = ConstantPool.FloatConstrSignature;
      break;
    case 8:
      signature = ConstantPool.DoubleConstrSignature;
      argCount = 2;
      break;
    case 2:
      signature = ConstantPool.CharConstrSignature;
      break;
    case 5:
      signature = ConstantPool.BooleanConstrSignature;
      break;
    case 1:
    case 11:
    case 12:
      signature = ConstantPool.ObjectConstrSignature;
    case 6:
    }
    invoke(
      -73, 
      argCount, 
      0, 
      ConstantPool.JavaLangAssertionErrorConstantPoolName, 
      ConstantPool.Init, 
      signature);
  }

  public void invokeJavaLangAssertionErrorDefaultConstructor()
  {
    invoke(
      -73, 
      0, 
      0, 
      ConstantPool.JavaLangAssertionErrorConstantPoolName, 
      ConstantPool.Init, 
      ConstantPool.DefaultConstructorSignature);
  }

  public void invokeJavaLangClassDesiredAssertionStatus()
  {
    invoke(
      -74, 
      0, 
      1, 
      ConstantPool.JavaLangClassConstantPoolName, 
      ConstantPool.DesiredAssertionStatus, 
      ConstantPool.DesiredAssertionStatusSignature);
  }

  public void invokeJavaLangEnumvalueOf(ReferenceBinding binding)
  {
    invoke(
      -72, 
      2, 
      1, 
      ConstantPool.JavaLangEnumConstantPoolName, 
      ConstantPool.ValueOf, 
      ConstantPool.ValueOfStringClassSignature);
  }
  public void invokeJavaLangEnumValues(TypeBinding enumBinding, ArrayBinding arrayBinding) {
    char[] signature = "()".toCharArray();
    signature = CharOperation.concat(signature, arrayBinding.constantPoolName());
    invoke(-72, 0, 1, enumBinding.constantPoolName(), TypeConstants.VALUES, signature);
  }

  public void invokeJavaLangErrorConstructor()
  {
    invoke(
      -73, 
      1, 
      0, 
      ConstantPool.JavaLangErrorConstantPoolName, 
      ConstantPool.Init, 
      ConstantPool.StringConstructorSignature);
  }

  public void invokeJavaLangReflectConstructorNewInstance() {
    invoke(
      -74, 
      1, 
      1, 
      ConstantPool.JavaLangReflectConstructorConstantPoolName, 
      ConstantPool.NewInstance, 
      ConstantPool.JavaLangReflectConstructorNewInstanceSignature);
  }
  protected void invokeJavaLangReflectFieldGetter(int typeID) {
    int returnTypeSize = 1;
    char[] signature = (char[])null;
    char[] selector = (char[])null;
    switch (typeID) {
    case 10:
      selector = ConstantPool.GET_INT_METHOD_NAME;
      signature = ConstantPool.GET_INT_METHOD_SIGNATURE;
      break;
    case 3:
      selector = ConstantPool.GET_BYTE_METHOD_NAME;
      signature = ConstantPool.GET_BYTE_METHOD_SIGNATURE;
      break;
    case 4:
      selector = ConstantPool.GET_SHORT_METHOD_NAME;
      signature = ConstantPool.GET_SHORT_METHOD_SIGNATURE;
      break;
    case 7:
      selector = ConstantPool.GET_LONG_METHOD_NAME;
      signature = ConstantPool.GET_LONG_METHOD_SIGNATURE;
      returnTypeSize = 2;
      break;
    case 9:
      selector = ConstantPool.GET_FLOAT_METHOD_NAME;
      signature = ConstantPool.GET_FLOAT_METHOD_SIGNATURE;
      break;
    case 8:
      selector = ConstantPool.GET_DOUBLE_METHOD_NAME;
      signature = ConstantPool.GET_DOUBLE_METHOD_SIGNATURE;
      returnTypeSize = 2;
      break;
    case 2:
      selector = ConstantPool.GET_CHAR_METHOD_NAME;
      signature = ConstantPool.GET_CHAR_METHOD_SIGNATURE;
      break;
    case 5:
      selector = ConstantPool.GET_BOOLEAN_METHOD_NAME;
      signature = ConstantPool.GET_BOOLEAN_METHOD_SIGNATURE;
      break;
    case 6:
    default:
      selector = ConstantPool.GET_OBJECT_METHOD_NAME;
      signature = ConstantPool.GET_OBJECT_METHOD_SIGNATURE;
    }

    invoke(
      -74, 
      1, 
      returnTypeSize, 
      ConstantPool.JAVALANGREFLECTFIELD_CONSTANTPOOLNAME, 
      selector, 
      signature);
  }
  protected void invokeJavaLangReflectFieldSetter(int typeID) {
    int argCount = 2;
    char[] signature = (char[])null;
    char[] selector = (char[])null;
    switch (typeID) {
    case 10:
      selector = ConstantPool.SET_INT_METHOD_NAME;
      signature = ConstantPool.SET_INT_METHOD_SIGNATURE;
      break;
    case 3:
      selector = ConstantPool.SET_BYTE_METHOD_NAME;
      signature = ConstantPool.SET_BYTE_METHOD_SIGNATURE;
      break;
    case 4:
      selector = ConstantPool.SET_SHORT_METHOD_NAME;
      signature = ConstantPool.SET_SHORT_METHOD_SIGNATURE;
      break;
    case 7:
      selector = ConstantPool.SET_LONG_METHOD_NAME;
      signature = ConstantPool.SET_LONG_METHOD_SIGNATURE;
      argCount = 3;
      break;
    case 9:
      selector = ConstantPool.SET_FLOAT_METHOD_NAME;
      signature = ConstantPool.SET_FLOAT_METHOD_SIGNATURE;
      break;
    case 8:
      selector = ConstantPool.SET_DOUBLE_METHOD_NAME;
      signature = ConstantPool.SET_DOUBLE_METHOD_SIGNATURE;
      argCount = 3;
      break;
    case 2:
      selector = ConstantPool.SET_CHAR_METHOD_NAME;
      signature = ConstantPool.SET_CHAR_METHOD_SIGNATURE;
      break;
    case 5:
      selector = ConstantPool.SET_BOOLEAN_METHOD_NAME;
      signature = ConstantPool.SET_BOOLEAN_METHOD_SIGNATURE;
      break;
    case 6:
    default:
      selector = ConstantPool.SET_OBJECT_METHOD_NAME;
      signature = ConstantPool.SET_OBJECT_METHOD_SIGNATURE;
    }

    invoke(
      -74, 
      argCount, 
      0, 
      ConstantPool.JAVALANGREFLECTFIELD_CONSTANTPOOLNAME, 
      selector, 
      signature);
  }

  public void invokeJavaLangReflectMethodInvoke() {
    invoke(
      -74, 
      2, 
      1, 
      ConstantPool.JAVALANGREFLECTMETHOD_CONSTANTPOOLNAME, 
      ConstantPool.INVOKE_METHOD_METHOD_NAME, 
      ConstantPool.INVOKE_METHOD_METHOD_SIGNATURE);
  }

  public void invokeJavaUtilIteratorHasNext()
  {
    invoke(
      -71, 
      0, 
      1, 
      ConstantPool.JavaUtilIteratorConstantPoolName, 
      ConstantPool.HasNext, 
      ConstantPool.HasNextSignature);
  }

  public void invokeJavaUtilIteratorNext()
  {
    invoke(
      -71, 
      0, 
      1, 
      ConstantPool.JavaUtilIteratorConstantPoolName, 
      ConstantPool.Next, 
      ConstantPool.NextSignature);
  }

  public void invokeNoClassDefFoundErrorStringConstructor()
  {
    invoke(
      -73, 
      1, 
      0, 
      ConstantPool.JavaLangNoClassDefFoundErrorConstantPoolName, 
      ConstantPool.Init, 
      ConstantPool.StringConstructorSignature);
  }

  public void invokeObjectGetClass()
  {
    invoke(
      -74, 
      0, 
      1, 
      ConstantPool.JavaLangObjectConstantPoolName, 
      ConstantPool.GetClass, 
      ConstantPool.GetClassSignature);
  }

  public void invokespecial(MethodBinding methodBinding) {
    this.countLabels = 0;

    int argCount = 1;

    if (this.classFileOffset + 2 >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 1;
    this.bCodeStream[(this.classFileOffset++)] = -73;
    writeUnsignedShort(
      this.constantPool.literalIndexForMethod(
      methodBinding.constantPoolDeclaringClass(), 
      methodBinding.selector, 
      methodBinding.signature(this.classFile), 
      false));
    if (methodBinding.isConstructor()) {
      ReferenceBinding declaringClass = methodBinding.declaringClass;
      if (declaringClass.isNestedType())
      {
        TypeBinding[] syntheticArgumentTypes = declaringClass.syntheticEnclosingInstanceTypes();
        if (syntheticArgumentTypes != null) {
          int i = 0; for (int max = syntheticArgumentTypes.length; i < max; i++)
          {
            int id;
            if (((id = syntheticArgumentTypes[i].id) == 8) || (id == 7))
              argCount += 2;
            else {
              argCount++;
            }
          }
        }

        SyntheticArgumentBinding[] syntheticArguments = declaringClass.syntheticOuterLocalVariables();
        if (syntheticArguments != null) {
          int i = 0; for (int max = syntheticArguments.length; i < max; i++)
          {
            int id;
            if (((id = syntheticArguments[i].type.id) == 8) || (id == 7))
              argCount += 2;
            else {
              argCount++;
            }
          }
        }
      }
      if (declaringClass.isEnum())
      {
        argCount += 2;
      }
    }
    for (int i = methodBinding.parameters.length - 1; i >= 0; i--)
    {
      int id;
      if (((id = methodBinding.parameters[i].id) == 8) || (id == 7))
        argCount += 2;
      else
        argCount++;
    }
    int id;
    if (((id = methodBinding.returnType.id) == 8) || (id == 7)) {
      this.stackDepth += 2 - argCount;
    }
    else if (id == 6)
      this.stackDepth -= argCount;
    else
      this.stackDepth += 1 - argCount;
    if (this.stackDepth > this.stackMax)
      this.stackMax = this.stackDepth;
  }

  public void invokestatic(MethodBinding methodBinding)
  {
    this.countLabels = 0;
    int argCount = 0;

    if (this.classFileOffset + 2 >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 1;
    this.bCodeStream[(this.classFileOffset++)] = -72;
    writeUnsignedShort(
      this.constantPool.literalIndexForMethod(
      methodBinding.constantPoolDeclaringClass(), 
      methodBinding.selector, 
      methodBinding.signature(this.classFile), 
      false));
    for (int i = methodBinding.parameters.length - 1; i >= 0; i--)
    {
      int id;
      if (((id = methodBinding.parameters[i].id) == 8) || (id == 7))
        argCount += 2;
      else
        argCount++;
    }
    int id;
    if (((id = methodBinding.returnType.id) == 8) || (id == 7)) {
      this.stackDepth += 2 - argCount;
    }
    else if (id == 6)
      this.stackDepth -= argCount;
    else
      this.stackDepth += 1 - argCount;
    if (this.stackDepth > this.stackMax)
      this.stackMax = this.stackDepth;
  }

  public void invokeStringConcatenationAppendForType(int typeID)
  {
    int argCount = 1;
    int returnType = 1;
    char[] declaringClass = (char[])null;
    char[] selector = ConstantPool.Append;
    char[] signature = (char[])null;
    switch (typeID) {
    case 3:
    case 4:
    case 10:
      if (this.targetLevel >= 3211264L) {
        declaringClass = ConstantPool.JavaLangStringBuilderConstantPoolName;
        signature = ConstantPool.StringBuilderAppendIntSignature;
      } else {
        declaringClass = ConstantPool.JavaLangStringBufferConstantPoolName;
        signature = ConstantPool.StringBufferAppendIntSignature;
      }
      break;
    case 7:
      if (this.targetLevel >= 3211264L) {
        declaringClass = ConstantPool.JavaLangStringBuilderConstantPoolName;
        signature = ConstantPool.StringBuilderAppendLongSignature;
      } else {
        declaringClass = ConstantPool.JavaLangStringBufferConstantPoolName;
        signature = ConstantPool.StringBufferAppendLongSignature;
      }
      argCount = 2;
      break;
    case 9:
      if (this.targetLevel >= 3211264L) {
        declaringClass = ConstantPool.JavaLangStringBuilderConstantPoolName;
        signature = ConstantPool.StringBuilderAppendFloatSignature;
      } else {
        declaringClass = ConstantPool.JavaLangStringBufferConstantPoolName;
        signature = ConstantPool.StringBufferAppendFloatSignature;
      }
      break;
    case 8:
      if (this.targetLevel >= 3211264L) {
        declaringClass = ConstantPool.JavaLangStringBuilderConstantPoolName;
        signature = ConstantPool.StringBuilderAppendDoubleSignature;
      } else {
        declaringClass = ConstantPool.JavaLangStringBufferConstantPoolName;
        signature = ConstantPool.StringBufferAppendDoubleSignature;
      }
      argCount = 2;
      break;
    case 2:
      if (this.targetLevel >= 3211264L) {
        declaringClass = ConstantPool.JavaLangStringBuilderConstantPoolName;
        signature = ConstantPool.StringBuilderAppendCharSignature;
      } else {
        declaringClass = ConstantPool.JavaLangStringBufferConstantPoolName;
        signature = ConstantPool.StringBufferAppendCharSignature;
      }
      break;
    case 5:
      if (this.targetLevel >= 3211264L) {
        declaringClass = ConstantPool.JavaLangStringBuilderConstantPoolName;
        signature = ConstantPool.StringBuilderAppendBooleanSignature;
      } else {
        declaringClass = ConstantPool.JavaLangStringBufferConstantPoolName;
        signature = ConstantPool.StringBufferAppendBooleanSignature;
      }
      break;
    case 0:
    case 1:
    case 12:
      if (this.targetLevel >= 3211264L) {
        declaringClass = ConstantPool.JavaLangStringBuilderConstantPoolName;
        signature = ConstantPool.StringBuilderAppendObjectSignature;
      } else {
        declaringClass = ConstantPool.JavaLangStringBufferConstantPoolName;
        signature = ConstantPool.StringBufferAppendObjectSignature;
      }
      break;
    case 11:
      if (this.targetLevel >= 3211264L) {
        declaringClass = ConstantPool.JavaLangStringBuilderConstantPoolName;
        signature = ConstantPool.StringBuilderAppendStringSignature;
      } else {
        declaringClass = ConstantPool.JavaLangStringBufferConstantPoolName;
        signature = ConstantPool.StringBufferAppendStringSignature;
      }break;
    case 6:
    }
    invoke(
      -74, 
      argCount, 
      returnType, 
      declaringClass, 
      selector, 
      signature);
  }

  public void invokeStringConcatenationDefaultConstructor()
  {
    char[] declaringClass = ConstantPool.JavaLangStringBufferConstantPoolName;
    if (this.targetLevel >= 3211264L) {
      declaringClass = ConstantPool.JavaLangStringBuilderConstantPoolName;
    }
    invoke(
      -73, 
      0, 
      0, 
      declaringClass, 
      ConstantPool.Init, 
      ConstantPool.DefaultConstructorSignature);
  }

  public void invokeStringConcatenationStringConstructor()
  {
    char[] declaringClass = ConstantPool.JavaLangStringBufferConstantPoolName;
    if (this.targetLevel >= 3211264L) {
      declaringClass = ConstantPool.JavaLangStringBuilderConstantPoolName;
    }
    invoke(
      -73, 
      1, 
      0, 
      declaringClass, 
      ConstantPool.Init, 
      ConstantPool.StringConstructorSignature);
  }

  public void invokeStringConcatenationToString()
  {
    char[] declaringClass = ConstantPool.JavaLangStringBufferConstantPoolName;
    if (this.targetLevel >= 3211264L) {
      declaringClass = ConstantPool.JavaLangStringBuilderConstantPoolName;
    }
    invoke(
      -74, 
      0, 
      1, 
      declaringClass, 
      ConstantPool.ToString, 
      ConstantPool.ToStringSignature);
  }

  public void invokeStringIntern()
  {
    invoke(
      -74, 
      0, 
      1, 
      ConstantPool.JavaLangStringConstantPoolName, 
      ConstantPool.Intern, 
      ConstantPool.InternSignature);
  }

  public void invokeStringValueOf(int typeID)
  {
    int argCount = 1;
    char[] signature = (char[])null;
    switch (typeID) {
    case 3:
    case 4:
    case 10:
      signature = ConstantPool.ValueOfIntSignature;
      break;
    case 7:
      signature = ConstantPool.ValueOfLongSignature;
      argCount = 2;
      break;
    case 9:
      signature = ConstantPool.ValueOfFloatSignature;
      break;
    case 8:
      signature = ConstantPool.ValueOfDoubleSignature;
      argCount = 2;
      break;
    case 2:
      signature = ConstantPool.ValueOfCharSignature;
      break;
    case 5:
      signature = ConstantPool.ValueOfBooleanSignature;
      break;
    case 0:
    case 1:
    case 11:
    case 12:
      signature = ConstantPool.ValueOfObjectSignature;
    case 6:
    }
    invoke(
      -72, 
      argCount, 
      1, 
      ConstantPool.JavaLangStringConstantPoolName, 
      ConstantPool.ValueOf, 
      signature);
  }

  public void invokeSystemArraycopy()
  {
    invoke(
      -72, 
      5, 
      0, 
      ConstantPool.JavaLangSystemConstantPoolName, 
      ConstantPool.ArrayCopy, 
      ConstantPool.ArrayCopySignature);
  }

  public void invokeThrowableGetMessage()
  {
    invoke(
      -74, 
      0, 
      1, 
      ConstantPool.JavaLangThrowableConstantPoolName, 
      ConstantPool.GetMessage, 
      ConstantPool.GetMessageSignature);
  }

  public void invokevirtual(MethodBinding methodBinding) {
    this.countLabels = 0;

    int argCount = 1;

    if (this.classFileOffset + 2 >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 1;
    this.bCodeStream[(this.classFileOffset++)] = -74;
    writeUnsignedShort(
      this.constantPool.literalIndexForMethod(
      methodBinding.constantPoolDeclaringClass(), 
      methodBinding.selector, 
      methodBinding.signature(this.classFile), 
      false));
    for (int i = methodBinding.parameters.length - 1; i >= 0; i--)
    {
      int id;
      if (((id = methodBinding.parameters[i].id) == 8) || (id == 7))
        argCount += 2;
      else
        argCount++;
    }
    int id;
    if (((id = methodBinding.returnType.id) == 8) || (id == 7)) {
      this.stackDepth += 2 - argCount;
    }
    else if (id == 6)
      this.stackDepth -= argCount;
    else
      this.stackDepth += 1 - argCount;
    if (this.stackDepth > this.stackMax)
      this.stackMax = this.stackDepth;
  }

  public void ior() {
    this.countLabels = 0;
    this.stackDepth -= 1;
    if (this.classFileOffset >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 1;
    this.bCodeStream[(this.classFileOffset++)] = -128;
  }

  public void irem() {
    this.countLabels = 0;
    this.stackDepth -= 1;
    if (this.classFileOffset >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 1;
    this.bCodeStream[(this.classFileOffset++)] = 112;
  }

  public void ireturn() {
    this.countLabels = 0;
    this.stackDepth -= 1;

    if (this.classFileOffset >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 1;
    this.bCodeStream[(this.classFileOffset++)] = -84;
    this.lastAbruptCompletion = this.position;
  }

  public boolean isDefinitelyAssigned(Scope scope, int initStateIndex, LocalVariableBinding local) {
    if ((local.tagBits & 0x400) != 0L) {
      return true;
    }
    if (initStateIndex == -1)
      return false;
    int localPosition = local.id + this.maxFieldCount;
    MethodScope methodScope = scope.methodScope();

    if (localPosition < 64) {
      return (methodScope.definiteInits[initStateIndex] & 1L << localPosition) != 0L;
    }

    long[] extraInits = methodScope.extraDefiniteInits[initStateIndex];
    if (extraInits == null)
      return false;
    int vectorIndex;
    if ((vectorIndex = localPosition / 64 - 1) >= extraInits.length)
      return false;
    return (extraInits[vectorIndex] & 1L << localPosition % 64) != 0L;
  }

  public void ishl() {
    this.countLabels = 0;
    this.stackDepth -= 1;
    if (this.classFileOffset >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 1;
    this.bCodeStream[(this.classFileOffset++)] = 120;
  }

  public void ishr() {
    this.countLabels = 0;
    this.stackDepth -= 1;
    if (this.classFileOffset >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 1;
    this.bCodeStream[(this.classFileOffset++)] = 122;
  }

  public void istore(int iArg) {
    this.countLabels = 0;
    this.stackDepth -= 1;
    if (this.maxLocals <= iArg) {
      this.maxLocals = (iArg + 1);
    }
    if (iArg > 255) {
      if (this.classFileOffset + 3 >= this.bCodeStream.length) {
        resizeByteArray();
      }
      this.position += 2;
      this.bCodeStream[(this.classFileOffset++)] = -60;
      this.bCodeStream[(this.classFileOffset++)] = 54;
      writeUnsignedShort(iArg);
    } else {
      if (this.classFileOffset + 1 >= this.bCodeStream.length) {
        resizeByteArray();
      }
      this.position += 2;
      this.bCodeStream[(this.classFileOffset++)] = 54;
      this.bCodeStream[(this.classFileOffset++)] = ((byte)iArg);
    }
  }

  public void istore_0() {
    this.countLabels = 0;
    this.stackDepth -= 1;
    if (this.maxLocals == 0) {
      this.maxLocals = 1;
    }
    if (this.classFileOffset >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 1;
    this.bCodeStream[(this.classFileOffset++)] = 59;
  }

  public void istore_1() {
    this.countLabels = 0;
    this.stackDepth -= 1;
    if (this.maxLocals <= 1) {
      this.maxLocals = 2;
    }
    if (this.classFileOffset >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 1;
    this.bCodeStream[(this.classFileOffset++)] = 60;
  }

  public void istore_2() {
    this.countLabels = 0;
    this.stackDepth -= 1;
    if (this.maxLocals <= 2) {
      this.maxLocals = 3;
    }
    if (this.classFileOffset >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 1;
    this.bCodeStream[(this.classFileOffset++)] = 61;
  }

  public void istore_3() {
    this.countLabels = 0;
    this.stackDepth -= 1;
    if (this.maxLocals <= 3) {
      this.maxLocals = 4;
    }
    if (this.classFileOffset >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 1;
    this.bCodeStream[(this.classFileOffset++)] = 62;
  }

  public void isub() {
    this.countLabels = 0;
    this.stackDepth -= 1;
    if (this.classFileOffset >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 1;
    this.bCodeStream[(this.classFileOffset++)] = 100;
  }

  public void iushr() {
    this.countLabels = 0;
    this.stackDepth -= 1;
    if (this.classFileOffset >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 1;
    this.bCodeStream[(this.classFileOffset++)] = 124;
  }

  public void ixor() {
    this.countLabels = 0;
    this.stackDepth -= 1;
    if (this.classFileOffset >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 1;
    this.bCodeStream[(this.classFileOffset++)] = -126;
  }
  public final void jsr(BranchLabel lbl) {
    if (this.wideMode) {
      jsr_w(lbl);
      return;
    }

    this.countLabels = 0;
    if (this.classFileOffset >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 1;
    this.bCodeStream[(this.classFileOffset++)] = -88;
    lbl.branch();
  }

  public final void jsr_w(BranchLabel lbl) {
    this.countLabels = 0;
    if (this.classFileOffset >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 1;
    this.bCodeStream[(this.classFileOffset++)] = -55;
    lbl.branchWide();
  }

  public void l2d() {
    this.countLabels = 0;
    if (this.classFileOffset >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 1;
    this.bCodeStream[(this.classFileOffset++)] = -118;
  }

  public void l2f() {
    this.countLabels = 0;
    this.stackDepth -= 1;
    if (this.classFileOffset >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 1;
    this.bCodeStream[(this.classFileOffset++)] = -119;
  }

  public void l2i() {
    this.countLabels = 0;
    this.stackDepth -= 1;
    if (this.classFileOffset >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 1;
    this.bCodeStream[(this.classFileOffset++)] = -120;
  }

  public void ladd() {
    this.countLabels = 0;
    this.stackDepth -= 2;
    if (this.classFileOffset >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 1;
    this.bCodeStream[(this.classFileOffset++)] = 97;
  }

  public void laload() {
    this.countLabels = 0;
    if (this.classFileOffset >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 1;
    this.bCodeStream[(this.classFileOffset++)] = 47;
  }

  public void land() {
    this.countLabels = 0;
    this.stackDepth -= 2;
    if (this.classFileOffset >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 1;
    this.bCodeStream[(this.classFileOffset++)] = 127;
  }

  public void lastore() {
    this.countLabels = 0;
    this.stackDepth -= 4;
    if (this.classFileOffset >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 1;
    this.bCodeStream[(this.classFileOffset++)] = 80;
  }

  public void lcmp() {
    this.countLabels = 0;
    this.stackDepth -= 3;
    if (this.classFileOffset >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 1;
    this.bCodeStream[(this.classFileOffset++)] = -108;
  }

  public void lconst_0() {
    this.countLabels = 0;
    this.stackDepth += 2;
    if (this.stackDepth > this.stackMax)
      this.stackMax = this.stackDepth;
    if (this.classFileOffset >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 1;
    this.bCodeStream[(this.classFileOffset++)] = 9;
  }

  public void lconst_1() {
    this.countLabels = 0;
    this.stackDepth += 2;
    if (this.stackDepth > this.stackMax)
      this.stackMax = this.stackDepth;
    if (this.classFileOffset >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 1;
    this.bCodeStream[(this.classFileOffset++)] = 10;
  }
  public void ldc(float constant) {
    this.countLabels = 0;
    int index = this.constantPool.literalIndex(constant);
    this.stackDepth += 1;
    if (this.stackDepth > this.stackMax)
      this.stackMax = this.stackDepth;
    if (index > 255)
    {
      if (this.classFileOffset + 2 >= this.bCodeStream.length) {
        resizeByteArray();
      }
      this.position += 1;
      this.bCodeStream[(this.classFileOffset++)] = 19;
      writeUnsignedShort(index);
    }
    else
    {
      if (this.classFileOffset + 1 >= this.bCodeStream.length) {
        resizeByteArray();
      }
      this.position += 2;
      this.bCodeStream[(this.classFileOffset++)] = 18;
      this.bCodeStream[(this.classFileOffset++)] = ((byte)index);
    }
  }

  public void ldc(int constant) { this.countLabels = 0;
    int index = this.constantPool.literalIndex(constant);
    this.stackDepth += 1;
    if (this.stackDepth > this.stackMax)
      this.stackMax = this.stackDepth;
    if (index > 255)
    {
      if (this.classFileOffset + 2 >= this.bCodeStream.length) {
        resizeByteArray();
      }
      this.position += 1;
      this.bCodeStream[(this.classFileOffset++)] = 19;
      writeUnsignedShort(index);
    }
    else
    {
      if (this.classFileOffset + 1 >= this.bCodeStream.length) {
        resizeByteArray();
      }
      this.position += 2;
      this.bCodeStream[(this.classFileOffset++)] = 18;
      this.bCodeStream[(this.classFileOffset++)] = ((byte)index);
    } }

  public void ldc(String constant) {
    this.countLabels = 0;
    int currentCodeStreamPosition = this.position;
    char[] constantChars = constant.toCharArray();
    int index = this.constantPool.literalIndexForLdc(constantChars);
    if (index > 0)
    {
      ldcForIndex(index, constantChars);
    }
    else
    {
      this.position = currentCodeStreamPosition;
      int i = 0;
      int length = 0;
      int constantLength = constant.length();
      byte[] utf8encoding = new byte[Math.min(constantLength + 100, 65535)];
      int utf8encodingLength = 0;
      while ((length < 65532) && (i < constantLength)) {
        char current = constantChars[i];

        if (length + 3 > (utf8encodingLength = utf8encoding.length)) {
          System.arraycopy(utf8encoding, 0, utf8encoding = new byte[Math.min(utf8encodingLength + 100, 65535)], 0, length);
        }
        if ((current >= '\001') && (current <= ''))
        {
          utf8encoding[(length++)] = ((byte)current);
        }
        else if (current > '߿')
        {
          utf8encoding[(length++)] = ((byte)(0xE0 | current >> '\f' & 0xF));
          utf8encoding[(length++)] = ((byte)(0x80 | current >> '\006' & 0x3F));
          utf8encoding[(length++)] = ((byte)(0x80 | current & 0x3F));
        }
        else
        {
          utf8encoding[(length++)] = ((byte)(0xC0 | current >> '\006' & 0x1F));
          utf8encoding[(length++)] = ((byte)(0x80 | current & 0x3F));
        }

        i++;
      }

      newStringContatenation();
      dup();

      char[] subChars = new char[i];
      System.arraycopy(constantChars, 0, subChars, 0, i);
      System.arraycopy(utf8encoding, 0, utf8encoding = new byte[length], 0, length);
      index = this.constantPool.literalIndex(subChars, utf8encoding);
      ldcForIndex(index, subChars);

      invokeStringConcatenationStringConstructor();
      while (i < constantLength) {
        length = 0;
        utf8encoding = new byte[Math.min(constantLength - i + 100, 65535)];
        int startIndex = i;
        while ((length < 65532) && (i < constantLength)) {
          char current = constantChars[i];

          if (length + 3 > (utf8encodingLength = utf8encoding.length)) {
            System.arraycopy(utf8encoding, 0, utf8encoding = new byte[Math.min(utf8encodingLength + 100, 65535)], 0, length);
          }
          if ((current >= '\001') && (current <= ''))
          {
            utf8encoding[(length++)] = ((byte)current);
          }
          else if (current > '߿')
          {
            utf8encoding[(length++)] = ((byte)(0xE0 | current >> '\f' & 0xF));
            utf8encoding[(length++)] = ((byte)(0x80 | current >> '\006' & 0x3F));
            utf8encoding[(length++)] = ((byte)(0x80 | current & 0x3F));
          }
          else
          {
            utf8encoding[(length++)] = ((byte)(0xC0 | current >> '\006' & 0x1F));
            utf8encoding[(length++)] = ((byte)(0x80 | current & 0x3F));
          }

          i++;
        }

        int newCharLength = i - startIndex;
        subChars = new char[newCharLength];
        System.arraycopy(constantChars, startIndex, subChars, 0, newCharLength);
        System.arraycopy(utf8encoding, 0, utf8encoding = new byte[length], 0, length);
        index = this.constantPool.literalIndex(subChars, utf8encoding);
        ldcForIndex(index, subChars);

        invokeStringConcatenationAppendForType(11);
      }
      invokeStringConcatenationToString();
      invokeStringIntern();
    }
  }

  public void ldc(TypeBinding typeBinding) { this.countLabels = 0;
    int index = this.constantPool.literalIndexForType(typeBinding);
    this.stackDepth += 1;
    if (this.stackDepth > this.stackMax)
      this.stackMax = this.stackDepth;
    if (index > 255)
    {
      if (this.classFileOffset + 2 >= this.bCodeStream.length) {
        resizeByteArray();
      }
      this.position += 1;
      this.bCodeStream[(this.classFileOffset++)] = 19;
      writeUnsignedShort(index);
    }
    else
    {
      if (this.classFileOffset + 1 >= this.bCodeStream.length) {
        resizeByteArray();
      }
      this.position += 2;
      this.bCodeStream[(this.classFileOffset++)] = 18;
      this.bCodeStream[(this.classFileOffset++)] = ((byte)index);
    } }

  public void ldc2_w(double constant)
  {
    this.countLabels = 0;
    int index = this.constantPool.literalIndex(constant);
    this.stackDepth += 2;
    if (this.stackDepth > this.stackMax) {
      this.stackMax = this.stackDepth;
    }
    if (this.classFileOffset + 2 >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 1;
    this.bCodeStream[(this.classFileOffset++)] = 20;
    writeUnsignedShort(index);
  }

  public void ldc2_w(long constant) {
    this.countLabels = 0;
    int index = this.constantPool.literalIndex(constant);
    this.stackDepth += 2;
    if (this.stackDepth > this.stackMax) {
      this.stackMax = this.stackDepth;
    }
    if (this.classFileOffset + 2 >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 1;
    this.bCodeStream[(this.classFileOffset++)] = 20;
    writeUnsignedShort(index);
  }
  public void ldcForIndex(int index, char[] constant) {
    this.stackDepth += 1;
    if (this.stackDepth > this.stackMax) {
      this.stackMax = this.stackDepth;
    }
    if (index > 255)
    {
      if (this.classFileOffset + 2 >= this.bCodeStream.length) {
        resizeByteArray();
      }
      this.position += 1;
      this.bCodeStream[(this.classFileOffset++)] = 19;
      writeUnsignedShort(index);
    }
    else
    {
      if (this.classFileOffset + 1 >= this.bCodeStream.length) {
        resizeByteArray();
      }
      this.position += 2;
      this.bCodeStream[(this.classFileOffset++)] = 18;
      this.bCodeStream[(this.classFileOffset++)] = ((byte)index);
    }
  }

  public void ldiv() {
    this.countLabels = 0;
    this.stackDepth -= 2;
    if (this.classFileOffset >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 1;
    this.bCodeStream[(this.classFileOffset++)] = 109;
  }

  public void lload(int iArg) {
    this.countLabels = 0;
    this.stackDepth += 2;
    if (this.maxLocals <= iArg + 1) {
      this.maxLocals = (iArg + 2);
    }
    if (this.stackDepth > this.stackMax)
      this.stackMax = this.stackDepth;
    if (iArg > 255) {
      if (this.classFileOffset + 3 >= this.bCodeStream.length) {
        resizeByteArray();
      }
      this.position += 2;
      this.bCodeStream[(this.classFileOffset++)] = -60;
      this.bCodeStream[(this.classFileOffset++)] = 22;
      writeUnsignedShort(iArg);
    } else {
      if (this.classFileOffset + 1 >= this.bCodeStream.length) {
        resizeByteArray();
      }
      this.position += 2;
      this.bCodeStream[(this.classFileOffset++)] = 22;
      this.bCodeStream[(this.classFileOffset++)] = ((byte)iArg);
    }
  }

  public void lload_0() {
    this.countLabels = 0;
    this.stackDepth += 2;
    if (this.maxLocals < 2) {
      this.maxLocals = 2;
    }
    if (this.stackDepth > this.stackMax)
      this.stackMax = this.stackDepth;
    if (this.classFileOffset >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 1;
    this.bCodeStream[(this.classFileOffset++)] = 30;
  }

  public void lload_1() {
    this.countLabels = 0;
    this.stackDepth += 2;
    if (this.maxLocals < 3) {
      this.maxLocals = 3;
    }
    if (this.stackDepth > this.stackMax)
      this.stackMax = this.stackDepth;
    if (this.classFileOffset >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 1;
    this.bCodeStream[(this.classFileOffset++)] = 31;
  }

  public void lload_2() {
    this.countLabels = 0;
    this.stackDepth += 2;
    if (this.maxLocals < 4) {
      this.maxLocals = 4;
    }
    if (this.stackDepth > this.stackMax)
      this.stackMax = this.stackDepth;
    if (this.classFileOffset >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 1;
    this.bCodeStream[(this.classFileOffset++)] = 32;
  }

  public void lload_3() {
    this.countLabels = 0;
    this.stackDepth += 2;
    if (this.maxLocals < 5) {
      this.maxLocals = 5;
    }
    if (this.stackDepth > this.stackMax)
      this.stackMax = this.stackDepth;
    if (this.classFileOffset >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 1;
    this.bCodeStream[(this.classFileOffset++)] = 33;
  }

  public void lmul() {
    this.countLabels = 0;
    this.stackDepth -= 2;
    if (this.classFileOffset >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 1;
    this.bCodeStream[(this.classFileOffset++)] = 105;
  }

  public void lneg() {
    this.countLabels = 0;
    if (this.classFileOffset >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 1;
    this.bCodeStream[(this.classFileOffset++)] = 117;
  }
  public final void load(LocalVariableBinding localBinding) {
    load(localBinding.type, localBinding.resolvedPosition);
  }
  public final void load(TypeBinding typeBinding, int resolvedPosition) {
    this.countLabels = 0;

    switch (typeBinding.id) {
    case 2:
    case 3:
    case 4:
    case 5:
    case 10:
      switch (resolvedPosition) {
      case 0:
        iload_0();
        break;
      case 1:
        iload_1();
        break;
      case 2:
        iload_2();
        break;
      case 3:
        iload_3();
        break;
      default:
        iload(resolvedPosition);
      }
      break;
    case 9:
      switch (resolvedPosition) {
      case 0:
        fload_0();
        break;
      case 1:
        fload_1();
        break;
      case 2:
        fload_2();
        break;
      case 3:
        fload_3();
        break;
      default:
        fload(resolvedPosition);
      }
      break;
    case 7:
      switch (resolvedPosition) {
      case 0:
        lload_0();
        break;
      case 1:
        lload_1();
        break;
      case 2:
        lload_2();
        break;
      case 3:
        lload_3();
        break;
      default:
        lload(resolvedPosition);
      }
      break;
    case 8:
      switch (resolvedPosition) {
      case 0:
        dload_0();
        break;
      case 1:
        dload_1();
        break;
      case 2:
        dload_2();
        break;
      case 3:
        dload_3();
        break;
      default:
        dload(resolvedPosition);
      }
      break;
    case 6:
    default:
      switch (resolvedPosition) {
      case 0:
        aload_0();
        break;
      case 1:
        aload_1();
        break;
      case 2:
        aload_2();
        break;
      case 3:
        aload_3();
        break;
      default:
        aload(resolvedPosition);
      }break;
    }
  }

  public void lookupswitch(CaseLabel defaultLabel, int[] keys, int[] sortedIndexes, CaseLabel[] casesLabel) {
    this.countLabels = 0;
    this.stackDepth -= 1;
    int length = keys.length;
    int pos = this.position;
    defaultLabel.placeInstruction();
    for (int i = 0; i < length; i++) {
      casesLabel[i].placeInstruction();
    }
    if (this.classFileOffset >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 1;
    this.bCodeStream[(this.classFileOffset++)] = -85;
    for (int i = 3 - (pos & 0x3); i > 0; i--) {
      if (this.classFileOffset >= this.bCodeStream.length) {
        resizeByteArray();
      }
      this.position += 1;
      this.bCodeStream[(this.classFileOffset++)] = 0;
    }
    defaultLabel.branch();
    writeSignedWord(length);
    for (int i = 0; i < length; i++) {
      writeSignedWord(keys[sortedIndexes[i]]);
      casesLabel[sortedIndexes[i]].branch();
    }
  }

  public void lor() {
    this.countLabels = 0;
    this.stackDepth -= 2;
    if (this.classFileOffset >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 1;
    this.bCodeStream[(this.classFileOffset++)] = -127;
  }

  public void lrem() {
    this.countLabels = 0;
    this.stackDepth -= 2;
    if (this.classFileOffset >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 1;
    this.bCodeStream[(this.classFileOffset++)] = 113;
  }

  public void lreturn() {
    this.countLabels = 0;
    this.stackDepth -= 2;

    if (this.classFileOffset >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 1;
    this.bCodeStream[(this.classFileOffset++)] = -83;
    this.lastAbruptCompletion = this.position;
  }

  public void lshl() {
    this.countLabels = 0;
    this.stackDepth -= 1;
    if (this.classFileOffset >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 1;
    this.bCodeStream[(this.classFileOffset++)] = 121;
  }

  public void lshr() {
    this.countLabels = 0;
    this.stackDepth -= 1;
    if (this.classFileOffset >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 1;
    this.bCodeStream[(this.classFileOffset++)] = 123;
  }

  public void lstore(int iArg) {
    this.countLabels = 0;
    this.stackDepth -= 2;
    if (this.maxLocals <= iArg + 1) {
      this.maxLocals = (iArg + 2);
    }
    if (iArg > 255) {
      if (this.classFileOffset + 3 >= this.bCodeStream.length) {
        resizeByteArray();
      }
      this.position += 2;
      this.bCodeStream[(this.classFileOffset++)] = -60;
      this.bCodeStream[(this.classFileOffset++)] = 55;
      writeUnsignedShort(iArg);
    } else {
      if (this.classFileOffset + 1 >= this.bCodeStream.length) {
        resizeByteArray();
      }
      this.position += 2;
      this.bCodeStream[(this.classFileOffset++)] = 55;
      this.bCodeStream[(this.classFileOffset++)] = ((byte)iArg);
    }
  }

  public void lstore_0() {
    this.countLabels = 0;
    this.stackDepth -= 2;
    if (this.maxLocals < 2) {
      this.maxLocals = 2;
    }
    if (this.classFileOffset >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 1;
    this.bCodeStream[(this.classFileOffset++)] = 63;
  }

  public void lstore_1() {
    this.countLabels = 0;
    this.stackDepth -= 2;
    if (this.maxLocals < 3) {
      this.maxLocals = 3;
    }
    if (this.classFileOffset >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 1;
    this.bCodeStream[(this.classFileOffset++)] = 64;
  }

  public void lstore_2() {
    this.countLabels = 0;
    this.stackDepth -= 2;
    if (this.maxLocals < 4) {
      this.maxLocals = 4;
    }
    if (this.classFileOffset >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 1;
    this.bCodeStream[(this.classFileOffset++)] = 65;
  }

  public void lstore_3() {
    this.countLabels = 0;
    this.stackDepth -= 2;
    if (this.maxLocals < 5) {
      this.maxLocals = 5;
    }
    if (this.classFileOffset >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 1;
    this.bCodeStream[(this.classFileOffset++)] = 66;
  }

  public void lsub() {
    this.countLabels = 0;
    this.stackDepth -= 2;
    if (this.classFileOffset >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 1;
    this.bCodeStream[(this.classFileOffset++)] = 101;
  }

  public void lushr() {
    this.countLabels = 0;
    this.stackDepth -= 1;
    if (this.classFileOffset >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 1;
    this.bCodeStream[(this.classFileOffset++)] = 125;
  }

  public void lxor() {
    this.countLabels = 0;
    this.stackDepth -= 2;
    if (this.classFileOffset >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 1;
    this.bCodeStream[(this.classFileOffset++)] = -125;
  }

  public void monitorenter() {
    this.countLabels = 0;
    this.stackDepth -= 1;
    if (this.classFileOffset >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 1;
    this.bCodeStream[(this.classFileOffset++)] = -62;
  }

  public void monitorexit() {
    this.countLabels = 0;
    this.stackDepth -= 1;
    if (this.classFileOffset >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 1;
    this.bCodeStream[(this.classFileOffset++)] = -61;
  }

  public void multianewarray(TypeBinding typeBinding, int dimensions) {
    this.countLabels = 0;
    this.stackDepth += 1 - dimensions;
    if (this.classFileOffset + 3 >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 2;
    this.bCodeStream[(this.classFileOffset++)] = -59;
    writeUnsignedShort(this.constantPool.literalIndexForType(typeBinding));
    this.bCodeStream[(this.classFileOffset++)] = ((byte)dimensions);
  }

  public void new_(TypeBinding typeBinding)
  {
    this.countLabels = 0;
    this.stackDepth += 1;
    if (this.stackDepth > this.stackMax)
      this.stackMax = this.stackDepth;
    if (this.classFileOffset + 2 >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 1;
    this.bCodeStream[(this.classFileOffset++)] = -69;
    writeUnsignedShort(this.constantPool.literalIndexForType(typeBinding));
  }

  public void newarray(int array_Type) {
    this.countLabels = 0;
    if (this.classFileOffset + 1 >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 2;
    this.bCodeStream[(this.classFileOffset++)] = -68;
    this.bCodeStream[(this.classFileOffset++)] = ((byte)array_Type);
  }
  public void newArray(ArrayBinding arrayBinding) {
    TypeBinding component = arrayBinding.elementsType();
    switch (component.id) {
    case 10:
      newarray(10);
      break;
    case 3:
      newarray(8);
      break;
    case 5:
      newarray(4);
      break;
    case 4:
      newarray(9);
      break;
    case 2:
      newarray(5);
      break;
    case 7:
      newarray(11);
      break;
    case 9:
      newarray(6);
      break;
    case 8:
      newarray(7);
      break;
    case 6:
    default:
      anewarray(component);
    }
  }

  public void newJavaLangAssertionError()
  {
    this.countLabels = 0;
    this.stackDepth += 1;
    if (this.stackDepth > this.stackMax)
      this.stackMax = this.stackDepth;
    if (this.classFileOffset + 2 >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 1;
    this.bCodeStream[(this.classFileOffset++)] = -69;
    writeUnsignedShort(this.constantPool.literalIndexForType(ConstantPool.JavaLangAssertionErrorConstantPoolName));
  }

  public void newJavaLangError()
  {
    this.countLabels = 0;
    this.stackDepth += 1;
    if (this.stackDepth > this.stackMax)
      this.stackMax = this.stackDepth;
    if (this.classFileOffset + 2 >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 1;
    this.bCodeStream[(this.classFileOffset++)] = -69;
    writeUnsignedShort(this.constantPool.literalIndexForType(ConstantPool.JavaLangErrorConstantPoolName));
  }

  public void newNoClassDefFoundError()
  {
    this.countLabels = 0;
    this.stackDepth += 1;
    if (this.stackDepth > this.stackMax)
      this.stackMax = this.stackDepth;
    if (this.classFileOffset + 2 >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 1;
    this.bCodeStream[(this.classFileOffset++)] = -69;
    writeUnsignedShort(this.constantPool.literalIndexForType(ConstantPool.JavaLangNoClassDefFoundErrorConstantPoolName));
  }

  public void newStringContatenation()
  {
    this.countLabels = 0;
    this.stackDepth += 1;
    if (this.stackDepth > this.stackMax) {
      this.stackMax = this.stackDepth;
    }
    if (this.classFileOffset + 2 >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 1;
    this.bCodeStream[(this.classFileOffset++)] = -69;
    if (this.targetLevel >= 3211264L)
      writeUnsignedShort(this.constantPool.literalIndexForType(ConstantPool.JavaLangStringBuilderConstantPoolName));
    else
      writeUnsignedShort(this.constantPool.literalIndexForType(ConstantPool.JavaLangStringBufferConstantPoolName));
  }

  public void newWrapperFor(int typeID) {
    this.countLabels = 0;
    this.stackDepth += 1;
    if (this.stackDepth > this.stackMax)
      this.stackMax = this.stackDepth;
    if (this.classFileOffset + 2 >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 1;
    this.bCodeStream[(this.classFileOffset++)] = -69;
    switch (typeID)
    {
    case 10:
      writeUnsignedShort(this.constantPool.literalIndexForType(ConstantPool.JavaLangIntegerConstantPoolName));
      break;
    case 5:
      writeUnsignedShort(this.constantPool.literalIndexForType(ConstantPool.JavaLangBooleanConstantPoolName));
      break;
    case 3:
      writeUnsignedShort(this.constantPool.literalIndexForType(ConstantPool.JavaLangByteConstantPoolName));
      break;
    case 2:
      writeUnsignedShort(this.constantPool.literalIndexForType(ConstantPool.JavaLangCharacterConstantPoolName));
      break;
    case 9:
      writeUnsignedShort(this.constantPool.literalIndexForType(ConstantPool.JavaLangFloatConstantPoolName));
      break;
    case 8:
      writeUnsignedShort(this.constantPool.literalIndexForType(ConstantPool.JavaLangDoubleConstantPoolName));
      break;
    case 4:
      writeUnsignedShort(this.constantPool.literalIndexForType(ConstantPool.JavaLangShortConstantPoolName));
      break;
    case 7:
      writeUnsignedShort(this.constantPool.literalIndexForType(ConstantPool.JavaLangLongConstantPoolName));
      break;
    case 6:
      writeUnsignedShort(this.constantPool.literalIndexForType(ConstantPool.JavaLangVoidConstantPoolName));
    }
  }

  public void nop() {
    this.countLabels = 0;
    if (this.classFileOffset >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 1;
    this.bCodeStream[(this.classFileOffset++)] = 0;
  }
  public void optimizeBranch(int oldPosition, BranchLabel lbl) {
    for (int i = 0; i < this.countLabels; i++) {
      BranchLabel label = this.labels[i];
      if (oldPosition == label.position) {
        label.position = this.position;
        if ((label instanceof CaseLabel)) {
          int offset = this.position - ((CaseLabel)label).instructionPosition;
          int[] forwardRefs = label.forwardReferences();
          int j = 0; for (int length = label.forwardReferenceCount(); j < length; j++) {
            int forwardRef = forwardRefs[j];
            writeSignedWord(forwardRef, offset);
          }
        } else {
          int[] forwardRefs = label.forwardReferences();
          int j = 0; for (int length = label.forwardReferenceCount(); j < length; j++) {
            int forwardRef = forwardRefs[j];
            writePosition(lbl, forwardRef);
          }
        }
      }
    }
  }

  public void pop() {
    this.countLabels = 0;
    this.stackDepth -= 1;
    if (this.classFileOffset >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 1;
    this.bCodeStream[(this.classFileOffset++)] = 87;
  }

  public void pop2() {
    this.countLabels = 0;
    this.stackDepth -= 2;
    if (this.classFileOffset >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 1;
    this.bCodeStream[(this.classFileOffset++)] = 88;
  }
  public void pushOnStack(TypeBinding binding) {
    if (++this.stackDepth > this.stackMax)
      this.stackMax = this.stackDepth;
  }

  public void putfield(FieldBinding fieldBinding) {
    int returnTypeSize = 1;
    if ((fieldBinding.type.id == 8) || (fieldBinding.type.id == 7)) {
      returnTypeSize = 2;
    }
    generateFieldAccess(
      (byte)-75, 
      returnTypeSize, 
      fieldBinding.declaringClass, 
      fieldBinding.name, 
      fieldBinding.type);
  }

  public void putstatic(FieldBinding fieldBinding) {
    int returnTypeSize = 1;
    if ((fieldBinding.type.id == 8) || (fieldBinding.type.id == 7)) {
      returnTypeSize = 2;
    }
    generateFieldAccess(
      (byte)-77, 
      returnTypeSize, 
      fieldBinding.declaringClass, 
      fieldBinding.name, 
      fieldBinding.type);
  }
  public void record(LocalVariableBinding local) {
    if ((this.generateAttributes & 0xC) == 0)
      return;
    if (this.allLocalsCounter == this.locals.length)
    {
      System.arraycopy(this.locals, 0, this.locals = new LocalVariableBinding[this.allLocalsCounter + 10], 0, this.allLocalsCounter);
    }
    this.locals[(this.allLocalsCounter++)] = local;
    local.initializationPCs = new int[4];
    local.initializationCount = 0;
  }

  public void recordExpressionType(TypeBinding typeBinding) {
  }

  public void recordPositionsFrom(int startPC, int sourcePos) {
    recordPositionsFrom(startPC, sourcePos, false);
  }

  public void recordPositionsFrom(int startPC, int sourcePos, boolean widen)
  {
    if (((this.generateAttributes & 0x2) == 0) || 
      (sourcePos == 0) || (
      (startPC == this.position) && (!widen))) {
      return;
    }

    if (this.pcToSourceMapSize + 4 > this.pcToSourceMap.length)
    {
      System.arraycopy(this.pcToSourceMap, 0, this.pcToSourceMap = new int[this.pcToSourceMapSize << 1], 0, this.pcToSourceMapSize);
    }
    int lineNumber = Util.getLineNumber(sourcePos, this.lineSeparatorPositions, 0, this.lineSeparatorPositions.length - 1);

    if (this.pcToSourceMapSize > 0)
    {
      if (this.pcToSourceMap[(this.pcToSourceMapSize - 1)] != lineNumber) {
        if (startPC <= this.lastEntryPC)
        {
          int insertionIndex = insertionIndex(this.pcToSourceMap, this.pcToSourceMapSize, startPC);
          if (insertionIndex != -1)
          {
            int existingEntryIndex = indexOfSameLineEntrySincePC(startPC, lineNumber);

            if (existingEntryIndex != -1)
            {
              this.pcToSourceMap[existingEntryIndex] = startPC;
            } else if ((insertionIndex < 1) || (this.pcToSourceMap[(insertionIndex - 1)] != lineNumber))
            {
              System.arraycopy(this.pcToSourceMap, insertionIndex, this.pcToSourceMap, insertionIndex + 2, this.pcToSourceMapSize - insertionIndex);
              this.pcToSourceMap[(insertionIndex++)] = startPC;
              this.pcToSourceMap[insertionIndex] = lineNumber;
              this.pcToSourceMapSize += 2;
            }
          } else if (this.position != this.lastEntryPC) {
            if ((this.lastEntryPC == startPC) || (this.lastEntryPC == this.pcToSourceMap[(this.pcToSourceMapSize - 2)])) {
              this.pcToSourceMap[(this.pcToSourceMapSize - 1)] = lineNumber;
            } else {
              this.pcToSourceMap[(this.pcToSourceMapSize++)] = this.lastEntryPC;
              this.pcToSourceMap[(this.pcToSourceMapSize++)] = lineNumber;
            }
          } else if ((this.pcToSourceMap[(this.pcToSourceMapSize - 1)] < lineNumber) && (widen))
          {
            this.pcToSourceMap[(this.pcToSourceMapSize - 1)] = lineNumber;
          }
        }
        else {
          this.pcToSourceMap[(this.pcToSourceMapSize++)] = startPC;
          this.pcToSourceMap[(this.pcToSourceMapSize++)] = lineNumber;
        }

      }
      else if (startPC < this.pcToSourceMap[(this.pcToSourceMapSize - 2)]) {
        int insertionIndex = insertionIndex(this.pcToSourceMap, this.pcToSourceMapSize, startPC);
        if (insertionIndex != -1)
        {
          if ((insertionIndex <= 1) || (this.pcToSourceMap[(insertionIndex - 1)] != lineNumber)) {
            if ((this.pcToSourceMapSize > 4) && (this.pcToSourceMap[(this.pcToSourceMapSize - 4)] > startPC)) {
              System.arraycopy(this.pcToSourceMap, insertionIndex, this.pcToSourceMap, insertionIndex + 2, this.pcToSourceMapSize - 2 - insertionIndex);
              this.pcToSourceMap[(insertionIndex++)] = startPC;
              this.pcToSourceMap[insertionIndex] = lineNumber;
            } else {
              this.pcToSourceMap[(this.pcToSourceMapSize - 2)] = startPC;
            }
          }
        }
      }

      this.lastEntryPC = this.position;
    }
    else {
      this.pcToSourceMap[(this.pcToSourceMapSize++)] = startPC;
      this.pcToSourceMap[(this.pcToSourceMapSize++)] = lineNumber;
      this.lastEntryPC = this.position;
    }
  }

  public void registerExceptionHandler(ExceptionLabel anExceptionLabel)
  {
    int length;
    if (this.exceptionLabelsCounter == (length = this.exceptionLabels.length))
    {
      System.arraycopy(this.exceptionLabels, 0, this.exceptionLabels = new ExceptionLabel[length + 5], 0, length);
    }

    this.exceptionLabels[(this.exceptionLabelsCounter++)] = anExceptionLabel;
  }

  public void removeNotDefinitelyAssignedVariables(Scope scope, int initStateIndex)
  {
    if ((this.generateAttributes & 0xC) == 0)
      return;
    for (int i = 0; i < this.visibleLocalsCount; i++) {
      LocalVariableBinding localBinding = this.visibleLocals[i];
      if ((localBinding != null) && (!isDefinitelyAssigned(scope, initStateIndex, localBinding)) && (localBinding.initializationCount > 0))
        localBinding.recordInitializationEndPC(this.position);
    }
  }

  public void removeUnusedPcToSourceMapEntries()
  {
    if (this.pcToSourceMapSize != 0)
      while ((this.pcToSourceMapSize >= 2) && (this.pcToSourceMap[(this.pcToSourceMapSize - 2)] > this.position))
        this.pcToSourceMapSize -= 2;
  }

  public void removeVariable(LocalVariableBinding localBinding)
  {
    if (localBinding == null) return;
    if (localBinding.initializationCount > 0) {
      localBinding.recordInitializationEndPC(this.position);
    }
    for (int i = this.visibleLocalsCount - 1; i >= 0; i--) {
      LocalVariableBinding visibleLocal = this.visibleLocals[i];
      if (visibleLocal == localBinding) {
        this.visibleLocals[i] = null;
        return;
      }
    }
  }

  public void reset(AbstractMethodDeclaration referenceMethod, ClassFile targetClassFile)
  {
    init(targetClassFile);
    this.methodDeclaration = referenceMethod;
    this.preserveUnusedLocals = referenceMethod.scope.compilerOptions().preserveAllLocalVariables;
    initializeMaxLocals(referenceMethod.binding);
  }
  public void reset(ClassFile givenClassFile) {
    this.targetLevel = givenClassFile.targetJDK;
    this.generateAttributes = givenClassFile.produceAttributes;
    if ((givenClassFile.produceAttributes & 0x2) != 0)
      this.lineSeparatorPositions = givenClassFile.referenceBinding.scope.referenceCompilationUnit().compilationResult.getLineSeparatorPositions();
  }

  public void resetForProblemClinit(ClassFile targetClassFile)
  {
    init(targetClassFile);
    initializeMaxLocals(null);
  }
  private final void resizeByteArray() {
    int length = this.bCodeStream.length;
    int requiredSize = length + length;
    if (this.classFileOffset >= requiredSize)
    {
      requiredSize = this.classFileOffset + length;
    }
    System.arraycopy(this.bCodeStream, 0, this.bCodeStream = new byte[requiredSize], 0, length);
  }

  public final void ret(int index) {
    this.countLabels = 0;
    if (index > 255) {
      if (this.classFileOffset + 3 >= this.bCodeStream.length) {
        resizeByteArray();
      }
      this.position += 2;
      this.bCodeStream[(this.classFileOffset++)] = -60;
      this.bCodeStream[(this.classFileOffset++)] = -87;
      writeUnsignedShort(index);
    } else {
      if (this.classFileOffset + 1 >= this.bCodeStream.length) {
        resizeByteArray();
      }
      this.position += 2;
      this.bCodeStream[(this.classFileOffset++)] = -87;
      this.bCodeStream[(this.classFileOffset++)] = ((byte)index);
    }
  }

  public void return_() {
    this.countLabels = 0;

    if (this.classFileOffset >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 1;
    this.bCodeStream[(this.classFileOffset++)] = -79;
    this.lastAbruptCompletion = this.position;
  }

  public void saload() {
    this.countLabels = 0;
    this.stackDepth -= 1;
    if (this.classFileOffset >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 1;
    this.bCodeStream[(this.classFileOffset++)] = 53;
  }

  public void sastore() {
    this.countLabels = 0;
    this.stackDepth -= 3;
    if (this.classFileOffset >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 1;
    this.bCodeStream[(this.classFileOffset++)] = 86;
  }

  public void sendOperator(int operatorConstant, int type_ID)
  {
    switch (type_ID) {
    case 2:
    case 3:
    case 4:
    case 5:
    case 10:
      switch (operatorConstant) {
      case 14:
        iadd();
        break;
      case 13:
        isub();
        break;
      case 15:
        imul();
        break;
      case 9:
        idiv();
        break;
      case 16:
        irem();
        break;
      case 10:
        ishl();
        break;
      case 17:
        ishr();
        break;
      case 19:
        iushr();
        break;
      case 2:
        iand();
        break;
      case 3:
        ior();
        break;
      case 8:
        ixor();
      case 4:
      case 5:
      case 6:
      case 7:
      case 11:
      case 12:
      case 18: } break;
    case 7:
      switch (operatorConstant) {
      case 14:
        ladd();
        break;
      case 13:
        lsub();
        break;
      case 15:
        lmul();
        break;
      case 9:
        ldiv();
        break;
      case 16:
        lrem();
        break;
      case 10:
        lshl();
        break;
      case 17:
        lshr();
        break;
      case 19:
        lushr();
        break;
      case 2:
        land();
        break;
      case 3:
        lor();
        break;
      case 8:
        lxor();
      case 4:
      case 5:
      case 6:
      case 7:
      case 11:
      case 12:
      case 18: } break;
    case 9:
      switch (operatorConstant) {
      case 14:
        fadd();
        break;
      case 13:
        fsub();
        break;
      case 15:
        fmul();
        break;
      case 9:
        fdiv();
        break;
      case 16:
        frem();
      case 10:
      case 11:
      case 12: } break;
    case 8:
      switch (operatorConstant) {
      case 14:
        dadd();
        break;
      case 13:
        dsub();
        break;
      case 15:
        dmul();
        break;
      case 9:
        ddiv();
        break;
      case 16:
        drem();
      case 10:
      case 11:
      case 12: } break;
    case 6:
    }
  }
  public void sipush(int s) { this.countLabels = 0;
    this.stackDepth += 1;
    if (this.stackDepth > this.stackMax)
      this.stackMax = this.stackDepth;
    if (this.classFileOffset >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 1;
    this.bCodeStream[(this.classFileOffset++)] = 17;
    writeSignedShort(s); }

  public void store(LocalVariableBinding localBinding, boolean valueRequired) {
    int localPosition = localBinding.resolvedPosition;

    switch (localBinding.type.id) {
    case 2:
    case 3:
    case 4:
    case 5:
    case 10:
      if (valueRequired)
        dup();
      switch (localPosition) {
      case 0:
        istore_0();
        break;
      case 1:
        istore_1();
        break;
      case 2:
        istore_2();
        break;
      case 3:
        istore_3();
        break;
      default:
        istore(localPosition);
      }
      break;
    case 9:
      if (valueRequired)
        dup();
      switch (localPosition) {
      case 0:
        fstore_0();
        break;
      case 1:
        fstore_1();
        break;
      case 2:
        fstore_2();
        break;
      case 3:
        fstore_3();
        break;
      default:
        fstore(localPosition);
      }
      break;
    case 8:
      if (valueRequired)
        dup2();
      switch (localPosition) {
      case 0:
        dstore_0();
        break;
      case 1:
        dstore_1();
        break;
      case 2:
        dstore_2();
        break;
      case 3:
        dstore_3();
        break;
      default:
        dstore(localPosition);
      }
      break;
    case 7:
      if (valueRequired)
        dup2();
      switch (localPosition) {
      case 0:
        lstore_0();
        break;
      case 1:
        lstore_1();
        break;
      case 2:
        lstore_2();
        break;
      case 3:
        lstore_3();
        break;
      default:
        lstore(localPosition);
      }
      break;
    case 6:
    default:
      if (valueRequired)
        dup();
      switch (localPosition) {
      case 0:
        astore_0();
        break;
      case 1:
        astore_1();
        break;
      case 2:
        astore_2();
        break;
      case 3:
        astore_3();
        break;
      default:
        astore(localPosition);
      }break;
    }
  }

  public void swap() {
    this.countLabels = 0;
    if (this.classFileOffset >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 1;
    this.bCodeStream[(this.classFileOffset++)] = 95;
  }

  public void tableswitch(CaseLabel defaultLabel, int low, int high, int[] keys, int[] sortedIndexes, CaseLabel[] casesLabel) {
    this.countLabels = 0;
    this.stackDepth -= 1;
    int length = casesLabel.length;
    int pos = this.position;
    defaultLabel.placeInstruction();
    for (int i = 0; i < length; i++)
      casesLabel[i].placeInstruction();
    if (this.classFileOffset >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 1;
    this.bCodeStream[(this.classFileOffset++)] = -86;

    for (int i = 3 - (pos & 0x3); i > 0; i--) {
      if (this.classFileOffset >= this.bCodeStream.length) {
        resizeByteArray();
      }
      this.position += 1;
      this.bCodeStream[(this.classFileOffset++)] = 0;
    }
    defaultLabel.branch();
    writeSignedWord(low);
    writeSignedWord(high);
    int i = low; int j = low;
    while (true)
    {
      int index;
      int key = keys[(index = sortedIndexes[(j - low)])];
      if (key == i) {
        casesLabel[index].branch();
        j++;
        if (i == high) break; 
      }
      else { defaultLabel.branch(); }

      i++;
    }
  }

  public void throwAnyException(LocalVariableBinding anyExceptionVariable) { load(anyExceptionVariable);
    athrow(); }

  public String toString() {
    StringBuffer buffer = new StringBuffer("( position:");
    buffer.append(this.position);
    buffer.append(",\nstackDepth:");
    buffer.append(this.stackDepth);
    buffer.append(",\nmaxStack:");
    buffer.append(this.stackMax);
    buffer.append(",\nmaxLocals:");
    buffer.append(this.maxLocals);
    buffer.append(")");
    return buffer.toString();
  }

  public void updateLastRecordedEndPC(Scope scope, int pos)
  {
    if ((this.generateAttributes & 0x2) != 0) {
      this.lastEntryPC = pos;
    }

    if ((this.generateAttributes & 0xC) != 0) {
      int i = 0; for (int max = this.locals.length; i < max; i++) {
        LocalVariableBinding local = this.locals[i];
        if ((local != null) && (local.declaringScope == scope) && (local.initializationCount > 0) && 
          (local.initializationPCs[((local.initializationCount - 1 << 1) + 1)] == pos))
          local.initializationPCs[((local.initializationCount - 1 << 1) + 1)] = this.position;
      }
    }
  }

  protected void writePosition(BranchLabel label)
  {
    int offset = label.position - this.position + 1;
    if ((Math.abs(offset) > 32767) && (!this.wideMode)) {
      throw new AbortMethod(RESTART_IN_WIDE_MODE, null);
    }
    writeSignedShort(offset);
    int[] forwardRefs = label.forwardReferences();
    int i = 0; for (int max = label.forwardReferenceCount(); i < max; i++)
      writePosition(label, forwardRefs[i]);
  }

  protected void writePosition(BranchLabel label, int forwardReference) {
    int offset = label.position - forwardReference + 1;
    if ((Math.abs(offset) > 32767) && (!this.wideMode)) {
      throw new AbortMethod(RESTART_IN_WIDE_MODE, null);
    }
    if (this.wideMode) {
      if ((label.tagBits & 0x1) != 0)
        writeSignedWord(forwardReference, offset);
      else
        writeSignedShort(forwardReference, offset);
    }
    else
      writeSignedShort(forwardReference, offset);
  }

  private final void writeSignedShort(int value)
  {
    if (this.classFileOffset + 1 >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 2;
    this.bCodeStream[(this.classFileOffset++)] = ((byte)(value >> 8));
    this.bCodeStream[(this.classFileOffset++)] = ((byte)value);
  }
  private final void writeSignedShort(int pos, int value) {
    int currentOffset = this.startingClassFileOffset + pos;
    if (currentOffset + 1 >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.bCodeStream[currentOffset] = ((byte)(value >> 8));
    this.bCodeStream[(currentOffset + 1)] = ((byte)value);
  }

  protected final void writeSignedWord(int value) {
    if (this.classFileOffset + 3 >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.position += 4;
    this.bCodeStream[(this.classFileOffset++)] = ((byte)((value & 0xFF000000) >> 24));
    this.bCodeStream[(this.classFileOffset++)] = ((byte)((value & 0xFF0000) >> 16));
    this.bCodeStream[(this.classFileOffset++)] = ((byte)((value & 0xFF00) >> 8));
    this.bCodeStream[(this.classFileOffset++)] = ((byte)(value & 0xFF));
  }
  protected final void writeSignedWord(int pos, int value) {
    int currentOffset = this.startingClassFileOffset + pos;
    if (currentOffset + 3 >= this.bCodeStream.length) {
      resizeByteArray();
    }
    this.bCodeStream[(currentOffset++)] = ((byte)((value & 0xFF000000) >> 24));
    this.bCodeStream[(currentOffset++)] = ((byte)((value & 0xFF0000) >> 16));
    this.bCodeStream[(currentOffset++)] = ((byte)((value & 0xFF00) >> 8));
    this.bCodeStream[(currentOffset++)] = ((byte)(value & 0xFF));
  }

  private final void writeUnsignedShort(int value)
  {
    this.position += 2;
    this.bCodeStream[(this.classFileOffset++)] = ((byte)(value >>> 8));
    this.bCodeStream[(this.classFileOffset++)] = ((byte)value);
  }
  protected void writeWidePosition(BranchLabel label) {
    int labelPos = label.position;
    int offset = labelPos - this.position + 1;
    writeSignedWord(offset);
    int[] forwardRefs = label.forwardReferences();
    int i = 0; for (int max = label.forwardReferenceCount(); i < max; i++) {
      int forward = forwardRefs[i];
      offset = labelPos - forward + 1;
      writeSignedWord(forward, offset);
    }
  }
}