package org.eclipse.jdt.internal.compiler.lookup;

public class ParameterizedMethodBinding extends MethodBinding
{
  protected MethodBinding originalMethod;

  public ParameterizedMethodBinding(ParameterizedTypeBinding parameterizedDeclaringClass, MethodBinding originalMethod)
  {
    super(originalMethod.modifiers, 
      originalMethod.selector, 
      originalMethod.returnType, 
      originalMethod.parameters, 
      originalMethod.thrownExceptions, 
      parameterizedDeclaringClass);
    this.originalMethod = originalMethod;
    this.tagBits = originalMethod.tagBits;

    TypeVariableBinding[] originalVariables = originalMethod.typeVariables;
    Substitution substitution = null;
    int length = originalVariables.length;
    boolean isStatic = originalMethod.isStatic();
    if (length == 0) {
      this.typeVariables = Binding.NO_TYPE_VARIABLES;
      if (!isStatic) substitution = parameterizedDeclaringClass; 
    }
    else
    {
      TypeVariableBinding[] substitutedVariables = new TypeVariableBinding[length];
      for (int i = 0; i < length; i++) {
        TypeVariableBinding originalVariable = originalVariables[i];
        substitutedVariables[i] = new TypeVariableBinding(originalVariable.sourceName, this, originalVariable.rank);
      }
      this.typeVariables = substitutedVariables;

      substitution = new Substitution() { private final ParameterizedTypeBinding val$parameterizedDeclaringClass;
        private final boolean val$isStatic;
        private final int val$length;
        private final TypeVariableBinding[] val$originalVariables;
        private final TypeVariableBinding[] val$substitutedVariables;

        public LookupEnvironment environment() { return this.val$parameterizedDeclaringClass.environment; }

        public boolean isRawSubstitution() {
          return (!this.val$isStatic) && (this.val$parameterizedDeclaringClass.isRawSubstitution());
        }

        public TypeBinding substitute(TypeVariableBinding typeVariable) {
          if ((typeVariable.rank < this.val$length) && (this.val$originalVariables[typeVariable.rank] == typeVariable)) {
            return this.val$substitutedVariables[typeVariable.rank];
          }
          if (!this.val$isStatic)
            return this.val$parameterizedDeclaringClass.substitute(typeVariable);
          return typeVariable;
        }
      };
      for (int i = 0; i < length; i++) {
        TypeVariableBinding originalVariable = originalVariables[i];
        TypeVariableBinding substitutedVariable = substitutedVariables[i];
        TypeBinding substitutedSuperclass = Scope.substitute(substitution, originalVariable.superclass);
        ReferenceBinding[] substitutedInterfaces = Scope.substitute(substitution, originalVariable.superInterfaces);
        if (originalVariable.firstBound != null) {
          substitutedVariable.firstBound = (originalVariable.firstBound == originalVariable.superclass ? 
            substitutedSuperclass : 
            substitutedInterfaces[0]);
        }
        switch (substitutedSuperclass.kind()) {
        case 68:
          substitutedVariable.superclass = parameterizedDeclaringClass.environment.getResolvedType(TypeConstants.JAVA_LANG_OBJECT, null);
          substitutedVariable.superInterfaces = substitutedInterfaces;
          break;
        default:
          if (substitutedSuperclass.isInterface()) {
            substitutedVariable.superclass = parameterizedDeclaringClass.environment.getResolvedType(TypeConstants.JAVA_LANG_OBJECT, null);
            int interfaceCount = substitutedInterfaces.length;
            System.arraycopy(substitutedInterfaces, 0, substitutedInterfaces = new ReferenceBinding[interfaceCount + 1], 1, interfaceCount);
            substitutedInterfaces[0] = ((ReferenceBinding)substitutedSuperclass);
            substitutedVariable.superInterfaces = substitutedInterfaces;
          } else {
            substitutedVariable.superclass = ((ReferenceBinding)substitutedSuperclass);
            substitutedVariable.superInterfaces = substitutedInterfaces;
          }break;
        }
      }
    }
    if (substitution != null) {
      this.returnType = Scope.substitute(substitution, this.returnType);
      this.parameters = Scope.substitute(substitution, this.parameters);
      this.thrownExceptions = Scope.substitute(substitution, this.thrownExceptions);
    }
  }

  public ParameterizedMethodBinding(ReferenceBinding declaringClass, MethodBinding originalMethod, char[][] alternateParamaterNames, LookupEnvironment environment)
  {
    super(originalMethod.modifiers, 
      originalMethod.selector, 
      originalMethod.returnType, 
      originalMethod.parameters, 
      originalMethod.thrownExceptions, 
      declaringClass);
    this.originalMethod = originalMethod;
    this.tagBits = originalMethod.tagBits;

    TypeVariableBinding[] originalVariables = originalMethod.typeVariables;
    Substitution substitution = null;
    int length = originalVariables.length;
    if (length == 0) {
      this.typeVariables = Binding.NO_TYPE_VARIABLES;
    }
    else {
      TypeVariableBinding[] substitutedVariables = new TypeVariableBinding[length];
      for (int i = 0; i < length; i++) {
        TypeVariableBinding originalVariable = originalVariables[i];
        substitutedVariables[i] = new TypeVariableBinding(
          alternateParamaterNames == null ? 
          originalVariable.sourceName : 
          alternateParamaterNames[i], 
          this, 
          originalVariable.rank);
      }
      this.typeVariables = substitutedVariables;

      substitution = new Substitution() { private final LookupEnvironment val$environment;
        private final int val$length;
        private final TypeVariableBinding[] val$originalVariables;
        private final TypeVariableBinding[] val$substitutedVariables;

        public LookupEnvironment environment() { return this.val$environment; }

        public boolean isRawSubstitution() {
          return false;
        }

        public TypeBinding substitute(TypeVariableBinding typeVariable) {
          if ((typeVariable.rank < this.val$length) && (this.val$originalVariables[typeVariable.rank] == typeVariable)) {
            return this.val$substitutedVariables[typeVariable.rank];
          }
          return typeVariable;
        }
      };
      for (int i = 0; i < length; i++) {
        TypeVariableBinding originalVariable = originalVariables[i];
        TypeVariableBinding substitutedVariable = substitutedVariables[i];
        TypeBinding substitutedSuperclass = Scope.substitute(substitution, originalVariable.superclass);
        ReferenceBinding[] substitutedInterfaces = Scope.substitute(substitution, originalVariable.superInterfaces);
        if (originalVariable.firstBound != null) {
          substitutedVariable.firstBound = (originalVariable.firstBound == originalVariable.superclass ? 
            substitutedSuperclass : 
            substitutedInterfaces[0]);
        }
        switch (substitutedSuperclass.kind()) {
        case 68:
          substitutedVariable.superclass = environment.getResolvedType(TypeConstants.JAVA_LANG_OBJECT, null);
          substitutedVariable.superInterfaces = substitutedInterfaces;
          break;
        default:
          if (substitutedSuperclass.isInterface()) {
            substitutedVariable.superclass = environment.getResolvedType(TypeConstants.JAVA_LANG_OBJECT, null);
            int interfaceCount = substitutedInterfaces.length;
            System.arraycopy(substitutedInterfaces, 0, substitutedInterfaces = new ReferenceBinding[interfaceCount + 1], 1, interfaceCount);
            substitutedInterfaces[0] = ((ReferenceBinding)substitutedSuperclass);
            substitutedVariable.superInterfaces = substitutedInterfaces;
          } else {
            substitutedVariable.superclass = ((ReferenceBinding)substitutedSuperclass);
            substitutedVariable.superInterfaces = substitutedInterfaces;
          }break;
        }
      }
    }
    if (substitution != null) {
      this.returnType = Scope.substitute(substitution, this.returnType);
      this.parameters = Scope.substitute(substitution, this.parameters);
      this.thrownExceptions = Scope.substitute(substitution, this.thrownExceptions);
    }
  }

  public ParameterizedMethodBinding()
  {
  }

  public static ParameterizedMethodBinding instantiateGetClass(TypeBinding receiverType, MethodBinding originalMethod, Scope scope)
  {
    ParameterizedMethodBinding method = new ParameterizedMethodBinding();
    method.modifiers = originalMethod.modifiers;
    method.selector = originalMethod.selector;
    method.declaringClass = originalMethod.declaringClass;
    method.typeVariables = Binding.NO_TYPE_VARIABLES;
    method.originalMethod = originalMethod;
    method.parameters = originalMethod.parameters;
    method.thrownExceptions = originalMethod.thrownExceptions;
    ReferenceBinding genericClassType = scope.getJavaLangClass();
    LookupEnvironment environment = scope.environment();
    TypeBinding rawType = environment.convertToRawType(receiverType.erasure());
    method.returnType = environment
      .createParameterizedType(genericClassType, 
      new TypeBinding[] { environment.createWildcard(genericClassType, 0, rawType, null, 1) }, 
      null);
    return method;
  }

  public boolean hasSubstitutedParameters()
  {
    return this.parameters != this.originalMethod.parameters;
  }

  public boolean hasSubstitutedReturnType()
  {
    return this.returnType != this.originalMethod.returnType;
  }

  public MethodBinding original()
  {
    return this.originalMethod.original();
  }
}