package org.eclipse.jdt.internal.compiler.lookup;

import org.eclipse.jdt.core.compiler.CharOperation;

public class RawTypeBinding extends ParameterizedTypeBinding
{
  public RawTypeBinding(ReferenceBinding type, ReferenceBinding enclosingType, LookupEnvironment environment)
  {
    super(type, null, enclosingType, environment);
    if ((enclosingType == null) || ((enclosingType.modifiers & 0x40000000) == 0))
      this.modifiers &= -1073741825;
  }

  public char[] computeUniqueKey(boolean isLeaf) {
    StringBuffer sig = new StringBuffer(10);
    if ((isMemberType()) && (enclosingType().isParameterizedType())) {
      char[] typeSig = enclosingType().computeUniqueKey(false);
      for (int i = 0; i < typeSig.length - 1; i++) sig.append(typeSig[i]);
      sig.append('.').append(sourceName()).append('<').append('>').append(';');
    } else {
      sig.append(genericType().computeUniqueKey(false));
      sig.insert(sig.length() - 1, "<>");
    }

    int sigLength = sig.length();
    char[] uniqueKey = new char[sigLength];
    sig.getChars(0, sigLength, uniqueKey, 0);
    return uniqueKey;
  }

  public ParameterizedMethodBinding createParameterizedMethod(MethodBinding originalMethod)
  {
    if ((originalMethod.typeVariables == Binding.NO_TYPE_VARIABLES) || (originalMethod.isStatic())) {
      return super.createParameterizedMethod(originalMethod);
    }
    return this.environment.createParameterizedGenericMethod(originalMethod, this);
  }

  public int kind() {
    return 1028;
  }

  public String debugName()
  {
    StringBuffer nameBuffer = new StringBuffer(10);
    nameBuffer.append(actualType().sourceName()).append("#RAW");
    return nameBuffer.toString();
  }

  public char[] genericTypeSignature()
  {
    if (this.genericTypeSignature == null) {
      StringBuffer sig = new StringBuffer(10);
      if ((isMemberType()) && (enclosingType().isParameterizedType())) {
        char[] typeSig = enclosingType().genericTypeSignature();
        for (int i = 0; i < typeSig.length - 1; i++) sig.append(typeSig[i]);
        sig.append('.').append(sourceName()).append(';');
        int sigLength = sig.length();
        this.genericTypeSignature = new char[sigLength];
        sig.getChars(0, sigLength, this.genericTypeSignature, 0);
      } else {
        this.genericTypeSignature = genericType().signature();
      }
    }
    return this.genericTypeSignature;
  }

  public boolean isEquivalentTo(TypeBinding otherType) {
    if (this == otherType)
      return true;
    if (otherType == null)
      return false;
    switch (otherType.kind())
    {
    case 516:
      return ((WildcardBinding)otherType).boundCheck(this);
    case 260:
    case 1028:
    case 2052:
      return erasure() == otherType.erasure();
    }
    return false;
  }

  public boolean isIntersectingWith(TypeBinding otherType) {
    if (this == otherType)
      return true;
    if (otherType == null)
      return false;
    switch (otherType.kind())
    {
    case 260:
    case 1028:
    case 2052:
      return erasure() == otherType.erasure();
    }
    return false;
  }

  public boolean isParameterizedType()
  {
    return false;
  }
  public boolean isRawType() {
    return true;
  }

  protected void initializeArguments() {
    TypeVariableBinding[] typeVariables = genericType().typeVariables();
    int length = typeVariables.length;
    TypeBinding[] typeArguments = new TypeBinding[length];
    for (int i = 0; i < length; i++)
    {
      typeArguments[i] = this.environment.convertToRawType(typeVariables[i].erasure());
    }
    this.arguments = typeArguments;
  }

  public char[] readableName()
  {
    char[] readableName;
    char[] readableName;
    if (isMemberType())
      readableName = CharOperation.concat(enclosingType().readableName(), this.sourceName, '.');
    else {
      readableName = CharOperation.concatWith(actualType().compoundName, '.');
    }
    return readableName;
  }

  public char[] shortReadableName()
  {
    char[] shortReadableName;
    char[] shortReadableName;
    if (isMemberType())
      shortReadableName = CharOperation.concat(enclosingType().shortReadableName(), this.sourceName, '.');
    else {
      shortReadableName = actualType().sourceName;
    }
    return shortReadableName;
  }
}