package org.eclipse.jdt.internal.compiler.lookup;

import org.eclipse.jdt.core.compiler.CharOperation;
import org.eclipse.jdt.internal.compiler.ast.AbstractMethodDeclaration;
import org.eclipse.jdt.internal.compiler.ast.FieldDeclaration;
import org.eclipse.jdt.internal.compiler.ast.TypeDeclaration;

public class SyntheticMethodBinding extends MethodBinding
{
  public FieldBinding targetReadField;
  public FieldBinding targetWriteField;
  public MethodBinding targetMethod;
  public TypeBinding targetEnumType;
  public int kind;
  public static final int FieldReadAccess = 1;
  public static final int FieldWriteAccess = 2;
  public static final int MethodAccess = 3;
  public static final int ConstructorAccess = 4;
  public static final int SuperMethodAccess = 5;
  public static final int BridgeMethod = 6;
  public static final int EnumValues = 7;
  public static final int EnumValueOf = 8;
  public static final int SwitchTable = 9;
  public int sourceStart = 0;
  public int index;

  public SyntheticMethodBinding(FieldBinding targetField, boolean isReadAccess, ReferenceBinding declaringClass)
  {
    this.modifiers = 4104;
    this.tagBits |= 25769803776L;
    SourceTypeBinding declaringSourceType = (SourceTypeBinding)declaringClass;
    SyntheticMethodBinding[] knownAccessMethods = declaringSourceType.syntheticMethods();
    int methodId = knownAccessMethods == null ? 0 : knownAccessMethods.length;
    this.index = methodId;
    this.selector = CharOperation.concat(TypeConstants.SYNTHETIC_ACCESS_METHOD_PREFIX, String.valueOf(methodId).toCharArray());
    if (isReadAccess) {
      this.returnType = targetField.type;
      if (targetField.isStatic()) {
        this.parameters = Binding.NO_PARAMETERS;
      } else {
        this.parameters = new TypeBinding[1];
        this.parameters[0] = declaringSourceType;
      }
      this.targetReadField = targetField;
      this.kind = 1;
    } else {
      this.returnType = TypeBinding.VOID;
      if (targetField.isStatic()) {
        this.parameters = new TypeBinding[1];
        this.parameters[0] = targetField.type;
      } else {
        this.parameters = new TypeBinding[2];
        this.parameters[0] = declaringSourceType;
        this.parameters[1] = targetField.type;
      }
      this.targetWriteField = targetField;
      this.kind = 2;
    }
    this.thrownExceptions = Binding.NO_EXCEPTIONS;
    this.declaringClass = declaringSourceType;
    boolean needRename;
    label419: 
    do {
      needRename = false;

      MethodBinding[] methods = declaringSourceType.methods();
      long range;
      if ((range = ReferenceBinding.binarySearch(this.selector, methods)) >= 0L) {
        int paramCount = this.parameters.length;
        int imethod = (int)range; for (int end = (int)(range >> 32); imethod <= end; imethod++) {
          MethodBinding method = methods[imethod];
          if (method.parameters.length == paramCount) {
            TypeBinding[] toMatch = method.parameters;
            int i = 0;
            while (toMatch[i] == this.parameters[i])
            {
              i++; if (i >= paramCount)
              {
                needRename = true;
                break label419;
              }
            }
          }
        }
      }
      if (knownAccessMethods != null) {
        int i = 0; for (int length = knownAccessMethods.length; i < length; i++) {
          if ((knownAccessMethods[i] != null) && 
            (CharOperation.equals(this.selector, knownAccessMethods[i].selector)) && (areParametersEqual(methods[i]))) {
            needRename = true;
            break;
          }
        }
      }

      if (needRename)
        setSelector(CharOperation.concat(TypeConstants.SYNTHETIC_ACCESS_METHOD_PREFIX, String.valueOf(++methodId).toCharArray()));
    }
    while (
      needRename);

    FieldDeclaration[] fieldDecls = declaringSourceType.scope.referenceContext.fields;
    if (fieldDecls != null) {
      int i = 0; for (int max = fieldDecls.length; i < max; i++) {
        if (fieldDecls[i].binding == targetField) {
          this.sourceStart = fieldDecls[i].sourceStart;
          return;
        }

      }

    }

    this.sourceStart = declaringSourceType.scope.referenceContext.sourceStart;
  }

  public SyntheticMethodBinding(FieldBinding targetField, ReferenceBinding declaringClass, TypeBinding enumBinding, char[] selector) {
    this.modifiers = 4104;
    this.tagBits |= 25769803776L;
    SourceTypeBinding declaringSourceType = (SourceTypeBinding)declaringClass;
    SyntheticMethodBinding[] knownAccessMethods = declaringSourceType.syntheticMethods();
    int methodId = knownAccessMethods == null ? 0 : knownAccessMethods.length;
    this.index = methodId;
    this.selector = selector;
    this.returnType = declaringSourceType.scope.createArrayType(TypeBinding.INT, 1);
    this.parameters = Binding.NO_PARAMETERS;
    this.targetReadField = targetField;
    this.targetEnumType = enumBinding;
    this.kind = 9;
    this.thrownExceptions = Binding.NO_EXCEPTIONS;
    this.declaringClass = declaringSourceType;

    if (declaringSourceType.isStrictfp()) {
      this.modifiers |= 2048;
    }
    boolean needRename;
    label337: 
    do
    {
      needRename = false;

      MethodBinding[] methods = declaringSourceType.methods();
      long range;
      if ((range = ReferenceBinding.binarySearch(this.selector, methods)) >= 0L) {
        int paramCount = this.parameters.length;
        int imethod = (int)range; for (int end = (int)(range >> 32); imethod <= end; imethod++) {
          MethodBinding method = methods[imethod];
          if (method.parameters.length == paramCount) {
            TypeBinding[] toMatch = method.parameters;
            int i = 0;
            while (toMatch[i] == this.parameters[i])
            {
              i++; if (i >= paramCount)
              {
                needRename = true;
                break label337;
              }
            }
          }
        }
      }
      if (knownAccessMethods != null) {
        int i = 0; for (int length = knownAccessMethods.length; i < length; i++) {
          if ((knownAccessMethods[i] != null) && 
            (CharOperation.equals(this.selector, knownAccessMethods[i].selector)) && (areParametersEqual(methods[i]))) {
            needRename = true;
            break;
          }
        }
      }

      if (needRename)
        setSelector(CharOperation.concat(selector, String.valueOf(++methodId).toCharArray()));
    }
    while (
      needRename);

    this.sourceStart = declaringSourceType.scope.referenceContext.sourceStart;
  }

  public SyntheticMethodBinding(MethodBinding targetMethod, boolean isSuperAccess, ReferenceBinding receiverType)
  {
    if (targetMethod.isConstructor())
      initializeConstructorAccessor(targetMethod);
    else
      initializeMethodAccessor(targetMethod, isSuperAccess, receiverType);
  }

  public SyntheticMethodBinding(MethodBinding overridenMethodToBridge, MethodBinding targetMethod, SourceTypeBinding declaringClass)
  {
    this.declaringClass = declaringClass;
    this.selector = overridenMethodToBridge.selector;

    this.modifiers = ((targetMethod.modifiers | 0x40 | 0x1000) & 0xBFFFFAEF);
    this.tagBits |= 25769803776L;
    this.returnType = overridenMethodToBridge.returnType;
    this.parameters = overridenMethodToBridge.parameters;
    this.thrownExceptions = overridenMethodToBridge.thrownExceptions;
    this.targetMethod = targetMethod;
    this.kind = 6;
    SyntheticMethodBinding[] knownAccessMethods = declaringClass.syntheticMethods();
    int methodId = knownAccessMethods == null ? 0 : knownAccessMethods.length;
    this.index = methodId;
  }

  public SyntheticMethodBinding(SourceTypeBinding declaringEnum, char[] selector)
  {
    this.declaringClass = declaringEnum;
    this.selector = selector;
    this.modifiers = 9;
    this.tagBits |= 25769803776L;
    this.thrownExceptions = Binding.NO_EXCEPTIONS;
    if (selector == TypeConstants.VALUES) {
      this.returnType = declaringEnum.scope.createArrayType(declaringEnum, 1);
      this.parameters = Binding.NO_PARAMETERS;
      this.kind = 7;
    } else if (selector == TypeConstants.VALUEOF) {
      this.returnType = declaringEnum;
      this.parameters = new TypeBinding[] { declaringEnum.scope.getJavaLangString() };
      this.kind = 8;
    }
    SyntheticMethodBinding[] knownAccessMethods = ((SourceTypeBinding)this.declaringClass).syntheticMethods();
    int methodId = knownAccessMethods == null ? 0 : knownAccessMethods.length;
    this.index = methodId;
    if (declaringEnum.isStrictfp())
      this.modifiers |= 2048;  } 
  public void initializeConstructorAccessor(MethodBinding accessedConstructor) { this.targetMethod = accessedConstructor;
    this.modifiers = 4096;
    this.tagBits |= 25769803776L;
    SourceTypeBinding sourceType = (SourceTypeBinding)accessedConstructor.declaringClass;
    SyntheticMethodBinding[] knownSyntheticMethods = 
      sourceType.syntheticMethods();
    this.index = (knownSyntheticMethods == null ? 0 : knownSyntheticMethods.length);

    this.selector = accessedConstructor.selector;
    this.returnType = accessedConstructor.returnType;
    this.kind = 4;
    this.parameters = new TypeBinding[accessedConstructor.parameters.length + 1];
    System.arraycopy(
      accessedConstructor.parameters, 
      0, 
      this.parameters, 
      0, 
      accessedConstructor.parameters.length);
    this.parameters[accessedConstructor.parameters.length] = 
      accessedConstructor.declaringClass;
    this.thrownExceptions = accessedConstructor.thrownExceptions;
    this.declaringClass = sourceType;
    boolean needRename;
    do { needRename = false;

      MethodBinding[] methods = sourceType.methods();
      int i = 0; for (int length = methods.length; i < length; i++) {
        if ((CharOperation.equals(this.selector, methods[i].selector)) && 
          (areParametersEqual(methods[i]))) {
          needRename = true;
          break;
        }
      }

      if (knownSyntheticMethods != null) {
        int i = 0; for (int length = knownSyntheticMethods.length; i < length; i++) {
          if (knownSyntheticMethods[i] != null)
          {
            if ((CharOperation.equals(this.selector, knownSyntheticMethods[i].selector)) && 
              (areParametersEqual(knownSyntheticMethods[i]))) {
              needRename = true;
              break;
            }
          }
        }
      }
      if (needRename) {
        int length = this.parameters.length;
        System.arraycopy(
          this.parameters, 
          0, 
          this.parameters = new TypeBinding[length + 1], 
          0, 
          length);
        this.parameters[length] = this.declaringClass;
      }
    }
    while (
      needRename);

    AbstractMethodDeclaration[] methodDecls = 
      sourceType.scope.referenceContext.methods;
    if (methodDecls != null) {
      int i = 0; for (int length = methodDecls.length; i < length; i++)
        if (methodDecls[i].binding == accessedConstructor) {
          this.sourceStart = methodDecls[i].sourceStart;
          return;
        }
    }
  }

  public void initializeMethodAccessor(MethodBinding accessedMethod, boolean isSuperAccess, ReferenceBinding receiverType)
  {
    this.targetMethod = accessedMethod;
    this.modifiers = 4104;
    this.tagBits |= 25769803776L;
    SourceTypeBinding declaringSourceType = (SourceTypeBinding)receiverType;
    SyntheticMethodBinding[] knownAccessMethods = declaringSourceType.syntheticMethods();
    int methodId = knownAccessMethods == null ? 0 : knownAccessMethods.length;
    this.index = methodId;

    this.selector = CharOperation.concat(TypeConstants.SYNTHETIC_ACCESS_METHOD_PREFIX, String.valueOf(methodId).toCharArray());
    this.returnType = accessedMethod.returnType;
    this.kind = (isSuperAccess ? 5 : 3);

    if (accessedMethod.isStatic()) {
      this.parameters = accessedMethod.parameters;
    } else {
      this.parameters = new TypeBinding[accessedMethod.parameters.length + 1];
      this.parameters[0] = declaringSourceType;
      System.arraycopy(accessedMethod.parameters, 0, this.parameters, 1, accessedMethod.parameters.length);
    }
    this.thrownExceptions = accessedMethod.thrownExceptions;
    this.declaringClass = declaringSourceType;
    boolean needRename;
    do
    {
      needRename = false;

      MethodBinding[] methods = declaringSourceType.methods();
      int i = 0; for (int length = methods.length; i < length; i++) {
        if ((CharOperation.equals(this.selector, methods[i].selector)) && (areParametersEqual(methods[i]))) {
          needRename = true;
          break;
        }
      }

      if (knownAccessMethods != null) {
        int i = 0; for (int length = knownAccessMethods.length; i < length; i++) {
          if ((knownAccessMethods[i] != null) && 
            (CharOperation.equals(this.selector, knownAccessMethods[i].selector)) && (areParametersEqual(knownAccessMethods[i]))) {
            needRename = true;
            break;
          }
        }
      }

      if (needRename)
        setSelector(CharOperation.concat(TypeConstants.SYNTHETIC_ACCESS_METHOD_PREFIX, String.valueOf(++methodId).toCharArray()));
    }
    while (
      needRename);

    AbstractMethodDeclaration[] methodDecls = declaringSourceType.scope.referenceContext.methods;
    if (methodDecls != null) {
      int i = 0; for (int length = methodDecls.length; i < length; i++)
        if (methodDecls[i].binding == accessedMethod) {
          this.sourceStart = methodDecls[i].sourceStart;
          return;
        }
    }
  }

  protected boolean isConstructorRelated()
  {
    return this.kind == 4;
  }
}