package org.eclipse.jdt.internal.compiler.parser;

import org.eclipse.jdt.internal.compiler.ast.ASTNode;
import org.eclipse.jdt.internal.compiler.ast.ArrayQualifiedTypeReference;
import org.eclipse.jdt.internal.compiler.ast.ArrayTypeReference;
import org.eclipse.jdt.internal.compiler.ast.Expression;
import org.eclipse.jdt.internal.compiler.ast.LocalDeclaration;
import org.eclipse.jdt.internal.compiler.ast.Statement;

public class RecoveredLocalVariable extends RecoveredStatement
{
  public LocalDeclaration localDeclaration;
  boolean alreadyCompletedLocalInitialization;

  public RecoveredLocalVariable(LocalDeclaration localDeclaration, RecoveredElement parent, int bracketBalance)
  {
    super(localDeclaration, parent, bracketBalance);
    this.localDeclaration = localDeclaration;
    this.alreadyCompletedLocalInitialization = (localDeclaration.initialization != null);
  }

  public RecoveredElement add(Statement stmt, int bracketBalanceValue)
  {
    if ((this.alreadyCompletedLocalInitialization) || (!(stmt instanceof Expression))) {
      return super.add(stmt, bracketBalanceValue);
    }
    this.alreadyCompletedLocalInitialization = true;
    this.localDeclaration.initialization = ((Expression)stmt);
    this.localDeclaration.declarationSourceEnd = stmt.sourceEnd;
    this.localDeclaration.declarationEnd = stmt.sourceEnd;
    return this;
  }

  public ASTNode parseTree()
  {
    return this.localDeclaration;
  }

  public int sourceEnd()
  {
    return this.localDeclaration.declarationSourceEnd;
  }
  public String toString(int tab) {
    return tabString(tab) + "Recovered local variable:\n" + this.localDeclaration.print(tab + 1, new StringBuffer(10));
  }
  public Statement updatedStatement() {
    return this.localDeclaration;
  }

  public RecoveredElement updateOnClosingBrace(int braceStart, int braceEnd)
  {
    if (this.bracketBalance > 0) {
      this.bracketBalance -= 1;
      if (this.bracketBalance == 0) this.alreadyCompletedLocalInitialization = true;
      return this;
    }
    if (this.parent != null) {
      return this.parent.updateOnClosingBrace(braceStart, braceEnd);
    }
    return this;
  }

  public RecoveredElement updateOnOpeningBrace(int braceStart, int braceEnd)
  {
    if ((this.localDeclaration.declarationSourceEnd == 0) && 
      (((this.localDeclaration.type instanceof ArrayTypeReference)) || ((this.localDeclaration.type instanceof ArrayQualifiedTypeReference))) && 
      (!this.alreadyCompletedLocalInitialization)) {
      this.bracketBalance += 1;
      return null;
    }

    updateSourceEndIfNecessary(braceStart - 1, braceEnd - 1);
    return this.parent.updateOnOpeningBrace(braceStart, braceEnd);
  }
  public void updateParseTree() {
    updatedStatement();
  }

  public void updateSourceEndIfNecessary(int bodyStart, int bodyEnd)
  {
    if (this.localDeclaration.declarationSourceEnd == 0) {
      this.localDeclaration.declarationSourceEnd = bodyEnd;
      this.localDeclaration.declarationEnd = bodyEnd;
    }
  }
}