package org.eclipse.jdt.internal.compiler.util;

public class FloatUtil
{
  private static final int DOUBLE_FRACTION_WIDTH = 52;
  private static final int DOUBLE_PRECISION = 53;
  private static final int MAX_DOUBLE_EXPONENT = 1023;
  private static final int MIN_NORMALIZED_DOUBLE_EXPONENT = -1022;
  private static final int MIN_UNNORMALIZED_DOUBLE_EXPONENT = -1075;
  private static final int DOUBLE_EXPONENT_BIAS = 1023;
  private static final int DOUBLE_EXPONENT_SHIFT = 52;
  private static final int SINGLE_FRACTION_WIDTH = 23;
  private static final int SINGLE_PRECISION = 24;
  private static final int MAX_SINGLE_EXPONENT = 127;
  private static final int MIN_NORMALIZED_SINGLE_EXPONENT = -126;
  private static final int MIN_UNNORMALIZED_SINGLE_EXPONENT = -150;
  private static final int SINGLE_EXPONENT_BIAS = 127;
  private static final int SINGLE_EXPONENT_SHIFT = 23;

  public static float valueOfHexFloatLiteral(char[] source)
  {
    long bits = convertHexFloatingPointLiteralToBits(source);
    return Float.intBitsToFloat((int)bits);
  }

  public static double valueOfHexDoubleLiteral(char[] source)
  {
    long bits = convertHexFloatingPointLiteralToBits(source);
    return Double.longBitsToDouble(bits);
  }

  private static long convertHexFloatingPointLiteralToBits(char[] source)
  {
    int length = source.length;
    long mantissa = 0L;

    int next = 0;
    char nextChar = source[next];
    nextChar = source[next];
    if (nextChar == '0')
      next++;
    else {
      throw new NumberFormatException();
    }
    nextChar = source[next];
    if ((nextChar == 'X') || (nextChar == 'x'))
      next++;
    else {
      throw new NumberFormatException();
    }

    int binaryPointPosition = -1;
    while (true) {
      nextChar = source[next];
      switch (nextChar) {
      case '0':
        next++;
        break;
      case '.':
        binaryPointPosition = next;
        next++;
      case '/':
      }

    }

    int mantissaBits = 0;
    int leadingDigitPosition = -1;
    while (true) {
      nextChar = source[next];
      int hexdigit;
      int hexdigit;
      int hexdigit;
      switch (nextChar) {
      case '0':
      case '1':
      case '2':
      case '3':
      case '4':
      case '5':
      case '6':
      case '7':
      case '8':
      case '9':
        hexdigit = nextChar - '0';
        break;
      case 'a':
      case 'b':
      case 'c':
      case 'd':
      case 'e':
      case 'f':
        hexdigit = nextChar - 'a' + 10;
        break;
      case 'A':
      case 'B':
      case 'C':
      case 'D':
      case 'E':
      case 'F':
        hexdigit = nextChar - 'A' + 10;
        break;
      case '.':
        binaryPointPosition = next;
        next++;
        break;
      case '/':
      case ':':
      case ';':
      case '<':
      case '=':
      case '>':
      case '?':
      case '@':
      case 'G':
      case 'H':
      case 'I':
      case 'J':
      case 'K':
      case 'L':
      case 'M':
      case 'N':
      case 'O':
      case 'P':
      case 'Q':
      case 'R':
      case 'S':
      case 'T':
      case 'U':
      case 'V':
      case 'W':
      case 'X':
      case 'Y':
      case 'Z':
      case '[':
      case '\\':
      case ']':
      case '^':
      case '_':
      case '`':
      default:
        if (binaryPointPosition >= 0)
          break label487;
        binaryPointPosition = next;

        break label487;
        int hexdigit;
        if (mantissaBits == 0)
        {
          leadingDigitPosition = next;
          mantissa = hexdigit;
          mantissaBits = 4;
        } else if (mantissaBits < 60)
        {
          mantissa <<= 4;
          mantissa |= hexdigit;
          mantissaBits += 4;
        }

        next++;
      }

    }

    label487: nextChar = source[next];
    if ((nextChar == 'P') || (nextChar == 'p'))
      next++;
    else {
      throw new NumberFormatException();
    }

    int exponent = 0;
    int exponentSign = 1;
    while (next < length) {
      nextChar = source[next];
      switch (nextChar) {
      case '+':
        exponentSign = 1;
        next++;
        break;
      case '-':
        exponentSign = -1;
        next++;
        break;
      case '0':
      case '1':
      case '2':
      case '3':
      case '4':
      case '5':
      case '6':
      case '7':
      case '8':
      case '9':
        int digit = nextChar - '0';
        exponent = exponent * 10 + digit;
        next++;
        break;
      case ',':
      case '.':
      case '/':
      default:
        break label662;
      }

    }

    label662: boolean doublePrecision = true;
    if (next < length) {
      nextChar = source[next];
      switch (nextChar) {
      case 'F':
      case 'f':
        doublePrecision = false;
        next++;
        break;
      case 'D':
      case 'd':
        doublePrecision = true;
        next++;
        break;
      default:
        throw new NumberFormatException();
      }

    }

    if (mantissa == 0L) {
      return 0L;
    }

    int scaleFactorCompensation = 0;
    long top = mantissa >>> mantissaBits - 4;
    if ((top & 0x8) == 0L) {
      mantissaBits--;
      scaleFactorCompensation++;
      if ((top & 0x4) == 0L) {
        mantissaBits--;
        scaleFactorCompensation++;
        if ((top & 0x2) == 0L) {
          mantissaBits--;
          scaleFactorCompensation++;
        }
      }

    }

    long result = 0L;
    if (doublePrecision)
    {
      long fraction;
      if (mantissaBits > 53)
      {
        int extraBits = mantissaBits - 53;

        long fraction = mantissa >>> extraBits - 1;
        long lowBit = fraction & 1L;
        fraction += lowBit;
        fraction >>>= 1;
        if ((fraction & 0x0) != 0L) {
          fraction >>>= 1;
          scaleFactorCompensation--;
        }
      }
      else {
        fraction = mantissa << 53 - mantissaBits;
      }

      int scaleFactor = 0;
      if (mantissaBits > 0) {
        if (leadingDigitPosition < binaryPointPosition)
        {
          scaleFactor = 4 * (binaryPointPosition - leadingDigitPosition);

          scaleFactor -= scaleFactorCompensation;
        }
        else {
          scaleFactor = -4 * (
            leadingDigitPosition - binaryPointPosition - 1);

          scaleFactor -= scaleFactorCompensation;
        }
      }

      int e = exponentSign * exponent + scaleFactor;
      if (e - 1 > 1023)
      {
        result = Double.doubleToLongBits((1.0D / 0.0D));
      } else if (e - 1 >= -1022)
      {
        long biasedExponent = e - 1 + 1023;
        result = fraction & 0xFFFFFFFF;
        result |= biasedExponent << 52;
      } else if (e - 1 > -1075)
      {
        long biasedExponent = 0L;
        result = fraction >>> -1022 - e + 1;
        result |= biasedExponent << 52;
      }
      else {
        result = Double.doubleToLongBits((0.0D / 0.0D));
      }
      return result;
    }
    long fraction;
    if (mantissaBits > 24)
    {
      int extraBits = mantissaBits - 24;

      long fraction = mantissa >>> extraBits - 1;
      long lowBit = fraction & 1L;
      fraction += lowBit;
      fraction >>>= 1;
      if ((fraction & 0x1000000) != 0L) {
        fraction >>>= 1;
        scaleFactorCompensation--;
      }
    }
    else {
      fraction = mantissa << 24 - mantissaBits;
    }

    int scaleFactor = 0;
    if (mantissaBits > 0) {
      if (leadingDigitPosition < binaryPointPosition)
      {
        scaleFactor = 4 * (binaryPointPosition - leadingDigitPosition);

        scaleFactor -= scaleFactorCompensation;
      }
      else {
        scaleFactor = -4 * (
          leadingDigitPosition - binaryPointPosition - 1);

        scaleFactor -= scaleFactorCompensation;
      }
    }

    int e = exponentSign * exponent + scaleFactor;
    if (e - 1 > 127)
    {
      result = Float.floatToIntBits((1.0F / 1.0F));
    } else if (e - 1 >= -126)
    {
      long biasedExponent = e - 1 + 127;
      result = fraction & 0xFF7FFFFF;
      result |= biasedExponent << 23;
    } else if (e - 1 > -150)
    {
      long biasedExponent = 0L;
      result = fraction >>> -126 - e + 1;
      result |= biasedExponent << 23;
    }
    else {
      result = Float.floatToIntBits((0.0F / 0.0F));
    }
    return result;
  }
}