package org.eclipse.jdt.internal.compiler.util;

import org.eclipse.jdt.core.compiler.CharOperation;
import org.eclipse.jdt.internal.compiler.lookup.PackageBinding;

public final class HashtableOfPackage
{
  public char[][] keyTable;
  public PackageBinding[] valueTable;
  public int elementSize;
  int threshold;

  public HashtableOfPackage()
  {
    this(3);
  }
  public HashtableOfPackage(int size) {
    this.elementSize = 0;
    this.threshold = size;
    int extraRoom = (int)(size * 1.75F);
    if (this.threshold == extraRoom)
      extraRoom++;
    this.keyTable = new char[extraRoom][];
    this.valueTable = new PackageBinding[extraRoom];
  }
  public boolean containsKey(char[] key) {
    int length = this.keyTable.length;
    int index = CharOperation.hashCode(key) % length;
    int keyLength = key.length;
    char[] currentKey;
    while ((currentKey = this.keyTable[index]) != null)
    {
      char[] currentKey;
      if ((currentKey.length == keyLength) && (CharOperation.equals(currentKey, key)))
        return true;
      index++; if (index == length) {
        index = 0;
      }
    }
    return false;
  }
  public PackageBinding get(char[] key) {
    int length = this.keyTable.length;
    int index = CharOperation.hashCode(key) % length;
    int keyLength = key.length;
    char[] currentKey;
    while ((currentKey = this.keyTable[index]) != null)
    {
      char[] currentKey;
      if ((currentKey.length == keyLength) && (CharOperation.equals(currentKey, key)))
        return this.valueTable[index];
      index++; if (index == length) {
        index = 0;
      }
    }
    return null;
  }
  public PackageBinding put(char[] key, PackageBinding value) {
    int length = this.keyTable.length;
    int index = CharOperation.hashCode(key) % length;
    int keyLength = key.length;
    char[] currentKey;
    while ((currentKey = this.keyTable[index]) != null)
    {
      char[] currentKey;
      if ((currentKey.length == keyLength) && (CharOperation.equals(currentKey, key)))
        return this.valueTable[index] =  = value;
      index++; if (index == length) {
        index = 0;
      }
    }
    this.keyTable[index] = key;
    this.valueTable[index] = value;

    if (++this.elementSize > this.threshold)
      rehash();
    return value;
  }
  private void rehash() {
    HashtableOfPackage newHashtable = new HashtableOfPackage(this.elementSize * 2);

    int i = this.keyTable.length;
    do
    {
      char[] currentKey;
      if ((currentKey = this.keyTable[i]) != null)
        newHashtable.put(currentKey, this.valueTable[i]);
      i--; } while (i >= 0);

    this.keyTable = newHashtable.keyTable;
    this.valueTable = newHashtable.valueTable;
    this.threshold = newHashtable.threshold;
  }
  public int size() {
    return this.elementSize;
  }
  public String toString() {
    String s = "";

    int i = 0; for (int length = this.valueTable.length; i < length; i++)
    {
      PackageBinding pkg;
      if ((pkg = this.valueTable[i]) != null)
        s = s + pkg.toString() + "\n"; 
    }
    return s;
  }
}