package org.jvnet.jaxbw.intellij;

import java.io.File;
import java.util.StringTokenizer;

public class FilePathInfo
{
  private String fullPath;
  private String relativePath;
  private String name;

  public static String normalize(String filename)
  {
    if (filename.startsWith("file:/"))
      filename = filename.substring(6);
    String result = filename.replace("/", File.separator).replace("\\", File.separator);

    return result.replace("%20", " ");
  }

  private static String getDirectoryPath(String fullPath)
  {
    int lastSlashIndex = fullPath.lastIndexOf(File.separator);
    if (lastSlashIndex < 0) {
      return "";
    }
    return fullPath.substring(0, lastSlashIndex);
  }

  public FilePathInfo(String fullPath)
  {
    this.fullPath = normalize(fullPath);

    int lastSlashIndex = this.fullPath.lastIndexOf(File.separator);
    if (lastSlashIndex < 0)
      this.name = this.fullPath;
    else {
      this.name = this.fullPath.substring(lastSlashIndex + 1);
    }
    this.relativePath = this.name;
  }

  public FilePathInfo(String referencePath, String relativePath)
  {
    relativePath = normalize(relativePath);

    StringTokenizer tokenizer = new StringTokenizer(relativePath, "/\\", false);

    String currName = getDirectoryPath(referencePath);

    String lastToken = "";
    while (tokenizer.hasMoreTokens()) {
      lastToken = tokenizer.nextToken();
      if (lastToken.compareTo(".") != 0)
      {
        if (lastToken.compareTo("..") == 0)
        {
          int slInd = currName.lastIndexOf(File.separator);
          if (slInd < 0) {
            if (currName.length() == 0) {
              throw new IllegalArgumentException("Too many .. in '" + relativePath + "' for '" + referencePath + "'");
            }

            currName = "";
          }
          else {
            currName = currName.substring(0, slInd);
          }
        }
        else {
          if (currName.length() > 0) {
            currName = currName + File.separator;
          }
          currName = currName + lastToken;
        }
      }
    }
    this.fullPath = currName;
    this.relativePath = relativePath;

    int lastSlashIndex = this.fullPath.lastIndexOf(File.separator);
    if (lastSlashIndex < 0)
      this.name = this.fullPath;
    else
      this.name = this.fullPath.substring(lastSlashIndex + 1);
  }

  public boolean isSame(FilePathInfo path2)
  {
    return this.fullPath.equals(path2.fullPath);
  }

  public String getFullPath()
  {
    return this.fullPath;
  }

  public String getName()
  {
    return this.name;
  }

  public String getRelativePath()
  {
    return this.relativePath;
  }
}