package org.jvnet.jaxbw.intellij;

import com.intellij.openapi.actionSystem.AnAction;
import com.intellij.openapi.actionSystem.AnActionEvent;
import com.intellij.openapi.actionSystem.DataContext;
import com.intellij.openapi.actionSystem.Presentation;
import com.intellij.openapi.application.Application;
import com.intellij.openapi.application.ApplicationManager;
import com.intellij.openapi.progress.ProcessCanceledException;
import com.intellij.openapi.progress.ProgressIndicator;
import com.intellij.openapi.progress.ProgressManager;
import com.intellij.openapi.project.Project;
import com.intellij.openapi.ui.Messages;
import com.intellij.openapi.vfs.VirtualFile;
import com.sun.tools.xjc.Driver;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Map;
import java.util.logging.Logger;
import org.jvnet.flamingo.wizard.WizardMainFrame.WizardMode;
import org.jvnet.flamingo.wizard.WizardStep;
import org.jvnet.jaxbw.intellij.wizard.XjcSettings;
import org.jvnet.jaxbw.intellij.wizard.XjcWizard;

public class XjcAction extends AnAction
{
  private static final Logger _logger = Logger.getLogger(XjcAction.class.getName());

  public void actionPerformed(AnActionEvent e)
  {
    VirtualFile vf = (VirtualFile)e.getDataContext().getData("virtualFile");
    Project proj = (Project)e.getDataContext().getData("project");
    if (vf != null) {
      XjcWizard xjcWizard = new XjcWizard(600, 200, WizardMainFrame.WizardMode.SEMI_MANUAL);
      xjcWizard.setFinishListener(new FinishActionListener(xjcWizard, vf, proj));
      XjcSettings initialSettings = new XjcSettings();
      initialSettings.setPackageName(XjcWizard.getSetting("xjc.package.name"));
      initialSettings.setRootOutputDirectory(XjcWizard.getSetting("xjc.output.dir"));
      initialSettings.setDoAllowVendorExtensions(getBooleanProperty(XjcWizard.getSettings(), "xjc.package.name", false));

      initialSettings.setExternalEntityCatalogFiles(XjcWizard.getSetting("xjc.catalog.files"));
      initialSettings.setDoCreateReadOnlyFiles(getBooleanProperty(XjcWizard.getSettings(), "xjc.create.ro.files", false));

      initialSettings.setHttpProxy(XjcWizard.getSetting("xjc.http.proxy"));
      initialSettings.setDoStrictValidation(getBooleanProperty(XjcWizard.getSettings(), "xjc.strict.validation", true));

      initialSettings.setUserClasspath(XjcWizard.getSetting("xjc.user.cp"));
      xjcWizard.start(initialSettings);

      xjcWizard.setLocationRelativeTo(null);
      xjcWizard.setVisible(true);
    }
  }

  protected static boolean getBooleanProperty(Map<String, String> settings, String propertyName, boolean defaultValue)
  {
    String val = (String)settings.get(propertyName);
    if (val == null)
      return defaultValue;
    if (val.length() == 0)
      return defaultValue;
    return Boolean.valueOf((String)settings.get(propertyName)).booleanValue();
  }

  public void update(AnActionEvent e) {
    VirtualFile vf = (VirtualFile)e.getDataContext().getData("virtualFile");
    boolean isXsdFile = false;
    if ((vf != null) && 
      ("xsd".equals(vf.getExtension()))) {
      isXsdFile = true;
    }
    e.getPresentation().setVisible(isXsdFile);
  }

  private static final class FinishActionListener
    implements ActionListener
  {
    private final XjcWizard xjcWizard;
    private VirtualFile vf;
    private Project proj;

    public FinishActionListener(XjcWizard xjcWizard, VirtualFile vf, Project proj)
    {
      this.xjcWizard = xjcWizard;
      this.vf = vf;
      this.proj = proj;
    }

    public final void actionPerformed(ActionEvent e)
    {
      XjcSettings xjcSettings = new XjcSettings();
      for (WizardStep step : this.xjcWizard.getSteps()) {
        step.exportSettings(xjcSettings);
      }
      xjcSettings.setMainSchemaName(this.vf.getPath());

      final String[] settings = xjcSettings.getDriverCommandLineArgs();
      XjcWizard.setSetting("xjc.package.name", xjcSettings.getPackageName());
      XjcWizard.setSetting("xjc.output.dir", xjcSettings.getRootOutputDirectory());
      XjcWizard.setSetting("xjc.user.cp", xjcSettings.getUserClasspath());
      XjcWizard.setSetting("xjc.http.proxy", xjcSettings.getHttpProxy());
      XjcWizard.setSetting("xjc.catalog.files", xjcSettings.getExternalEntityCatalogFiles());
      XjcWizard.setSetting("xjc.allow.extensions", "" + xjcSettings.isDoAllowVendorExtensions());
      XjcWizard.setSetting("xjc.create.ro.files", "" + xjcSettings.isDoCreateReadOnlyFiles());
      XjcWizard.setSetting("xjc.strict.validation", "" + xjcSettings.isDoStrictValidation());
      this.xjcWizard.dispose();
      try {
        ApplicationManager.getApplication().runProcessWithProgressSynchronously(new Runnable()
        {
          public void run() {
            ProgressIndicator indicator = ProgressManager.getInstance().getProgressIndicator();
            indicator.setIndeterminate(true);
            XjcHarvesterListener listener = new XjcHarvesterListener(indicator);
            try {
              Driver.run(settings, listener);
            } catch (Exception exc) {
              exc.printStackTrace();
              Messages.showErrorDialog("XJC failure", exc.getMessage());
            }
          }
        }
        , "JABX 2.0 XJC generator", true, this.proj);
      }
      catch (ProcessCanceledException pce)
      {
        ProgressManager.getInstance().getProgressIndicator().stop();
      }
    }
  }
}