package org.jvnet.jaxbw.intellij.wizard;

import java.awt.Color;
import java.awt.FlowLayout;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import javax.swing.Icon;
import javax.swing.JComponent;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.SwingUtilities;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import javax.swing.text.Document;
import org.jvnet.flamingo.wizard.WizardSettings;
import org.jvnet.flamingo.wizard.WizardStep;

public final class InputPackageStep
  implements WizardStep
{
  private final XjcWizard owner;
  private final JComponent component;
  private final JTextField packageTf;
  private String packageName;
  private final Pattern packagePattern;

  public InputPackageStep(XjcWizard owner)
  {
    this.owner = owner;

    JPanel panel = new JPanel();
    panel.setLayout(new FlowLayout(1));

    this.packageName = new String("org.example");
    this.packageTf = new JTextField(this.packageName);
    this.packageTf.setColumns(30);
    this.packageTf.getDocument().addDocumentListener(new ChangeDocumentListener(this));

    panel.add(this.packageTf);
    panel.setOpaque(false);

    this.packagePattern = Pattern.compile("[_$a-zA-Z][_$0-9a-zA-Z]*(.[_$a-zA-Z][_$0-9a-zA-Z]*)*");

    this.component = panel;
  }

  private void update()
  {
    this.packageName = this.packageTf.getText();
    if (canProceed())
      this.packageTf.setForeground(Color.black);
    else {
      this.packageTf.setForeground(Color.red);
    }

    SwingUtilities.invokeLater(new Runnable() {
      public void run() {
        InputPackageStep.this.owner.update();

        InputPackageStep.this.packageTf.requestFocus();
      }
    });
  }

  public final boolean canProceed()
  {
    if (this.packageName == null) {
      return false;
    }
    Matcher m = this.packagePattern.matcher(this.packageName);
    return m.matches();
  }

  public void exportSettings(WizardSettings wSettings)
  {
    XjcSettings settings = (XjcSettings)wSettings;
    settings.setPackageName(this.packageName);
  }

  public void importSettings(WizardSettings wSettings)
  {
    XjcSettings settings = (XjcSettings)wSettings;
    if (settings.getPackageName() != null) {
      this.packageName = settings.getPackageName();
      this.packageTf.setText(this.packageName);
    }
  }

  public final JComponent getComponent()
  {
    return this.component;
  }

  public final String getTitle()
  {
    return "Specify package name";
  }

  public final String getCurrentDirectory()
    throws UnsupportedOperationException
  {
    throw new UnsupportedOperationException();
  }

  public final boolean isSupportsDirectory()
  {
    return false;
  }

  public final void setCurrentDirectory(String str)
    throws UnsupportedOperationException
  {
    throw new UnsupportedOperationException();
  }

  public final String getDescription()
  {
    return "Specify the root package of the generated classes. All generated classes will reside in this package";
  }

  public Icon getStepIcon(int width, int height)
  {
    return ImageCreator.getWizardImage(width, height);
  }

  private static final class ChangeDocumentListener
    implements DocumentListener
  {
    private final InputPackageStep step;

    public ChangeDocumentListener(InputPackageStep step)
    {
      this.step = step;
    }

    public final void insertUpdate(DocumentEvent e) {
      this.step.update();
    }

    public final void removeUpdate(DocumentEvent e) {
      this.step.update();
    }

    public final void changedUpdate(DocumentEvent e) {
      this.step.update();
    }
  }
}