package com.netapp.filer.aggr;

import com.netapp.common.zapi.annotation.Element;
import com.netapp.common.zapi.annotation.Optional;
import com.netapp.common.zapi.annotation.Range;
import com.netapp.common.zapi.annotation.Type;
import com.netapp.common.zapi.annotation.Typedef;
import com.netapp.filer.volume.PlexInfo;
import java.io.Serializable;
import java.math.BigInteger;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

@Typedef("aggr-info")
public class AggrInfo
  implements Serializable
{
  private static final long serialVersionUID = 1L;
  public String checksumStatus;
  public String checksumStyle;
  public Integer diskCount;
  public BigInteger filesTotal;
  public BigInteger filesUsed;
  public Boolean hasLocalRoot;
  public Boolean hasPartnerRoot;
  public Boolean checksumEnabled;
  public Boolean inconsistent;
  public Boolean snaplock;
  public String mirrorStatus;
  public String name;
  public Integer plexCount;
  public List<PlexInfo> plexes;
  public Integer raidSize;
  public String raidStatus;
  public BigInteger sizeAvailable;
  public Integer sizePercentageUsed;
  public BigInteger sizeTotal;
  public BigInteger sizeUsed;
  public String snaplockType;
  public String state;
  public String type;
  public String uuid;
  public Integer volumeCount;
  public Integer volumeCountStripedDv;
  public Integer volumeCountStripedMdv;
  public List<ContainedVolumeInfo> volumes;

  @Element("checksum-status")
  @Type("string")
  public void setChecksumStatus(String paramString)
  {
    this.checksumStatus = paramString;
  }

  public String getChecksumStatus()
  {
    return this.checksumStatus;
  }

  public AggrInfo withChecksumStatus(String paramString)
  {
    this.checksumStatus = paramString;
    return this;
  }

  @Element("checksum-style")
  @Type("string")
  public void setChecksumStyle(String paramString)
  {
    this.checksumStyle = paramString;
  }

  public String getChecksumStyle()
  {
    return this.checksumStyle;
  }

  public AggrInfo withChecksumStyle(String paramString)
  {
    this.checksumStyle = paramString;
    return this;
  }

  @Element("disk-count")
  @Type("integer")
  @Range(min="0", max="2^31-1")
  public void setDiskCount(Integer paramInteger)
  {
    this.diskCount = paramInteger;
  }

  public Integer getDiskCount()
  {
    return this.diskCount;
  }

  public AggrInfo withDiskCount(Integer paramInteger)
  {
    this.diskCount = paramInteger;
    return this;
  }

  @Element("files-total")
  @Type("integer")
  @Range(min="0", max="2^64-1")
  public void setFilesTotal(BigInteger paramBigInteger)
  {
    this.filesTotal = paramBigInteger;
  }

  public BigInteger getFilesTotal()
  {
    return this.filesTotal;
  }

  public AggrInfo withFilesTotal(BigInteger paramBigInteger)
  {
    this.filesTotal = paramBigInteger;
    return this;
  }

  @Element("files-used")
  @Type("integer")
  @Range(min="0", max="2^64-1")
  public void setFilesUsed(BigInteger paramBigInteger)
  {
    this.filesUsed = paramBigInteger;
  }

  public BigInteger getFilesUsed()
  {
    return this.filesUsed;
  }

  public AggrInfo withFilesUsed(BigInteger paramBigInteger)
  {
    this.filesUsed = paramBigInteger;
    return this;
  }

  @Element("has-local-root")
  @Type("boolean")
  public void setHasLocalRoot(Boolean paramBoolean)
  {
    this.hasLocalRoot = paramBoolean;
  }

  public Boolean isHasLocalRoot()
  {
    return this.hasLocalRoot;
  }

  public AggrInfo withHasLocalRoot(Boolean paramBoolean)
  {
    this.hasLocalRoot = paramBoolean;
    return this;
  }

  @Element("has-partner-root")
  @Type("boolean")
  public void setHasPartnerRoot(Boolean paramBoolean)
  {
    this.hasPartnerRoot = paramBoolean;
  }

  public Boolean isHasPartnerRoot()
  {
    return this.hasPartnerRoot;
  }

  public AggrInfo withHasPartnerRoot(Boolean paramBoolean)
  {
    this.hasPartnerRoot = paramBoolean;
    return this;
  }

  @Element("is-checksum-enabled")
  @Type("boolean")
  public void setChecksumEnabled(Boolean paramBoolean)
  {
    this.checksumEnabled = paramBoolean;
  }

  public Boolean isChecksumEnabled()
  {
    return this.checksumEnabled;
  }

  public AggrInfo withChecksumEnabled(Boolean paramBoolean)
  {
    this.checksumEnabled = paramBoolean;
    return this;
  }

  @Element("is-inconsistent")
  @Type("boolean")
  public void setInconsistent(Boolean paramBoolean)
  {
    this.inconsistent = paramBoolean;
  }

  public Boolean isInconsistent()
  {
    return this.inconsistent;
  }

  public AggrInfo withInconsistent(Boolean paramBoolean)
  {
    this.inconsistent = paramBoolean;
    return this;
  }

  @Element("is-snaplock")
  @Type("boolean")
  public void setSnaplock(Boolean paramBoolean)
  {
    this.snaplock = paramBoolean;
  }

  public Boolean isSnaplock()
  {
    return this.snaplock;
  }

  public AggrInfo withSnaplock(Boolean paramBoolean)
  {
    this.snaplock = paramBoolean;
    return this;
  }

  @Element("mirror-status")
  @Type("string")
  public void setMirrorStatus(String paramString)
  {
    this.mirrorStatus = paramString;
  }

  public String getMirrorStatus()
  {
    return this.mirrorStatus;
  }

  public AggrInfo withMirrorStatus(String paramString)
  {
    this.mirrorStatus = paramString;
    return this;
  }

  @Element("name")
  @Type("string")
  public void setName(String paramString)
  {
    this.name = paramString;
  }

  public String getName()
  {
    return this.name;
  }

  public AggrInfo withName(String paramString)
  {
    this.name = paramString;
    return this;
  }

  @Element("plex-count")
  @Type("integer")
  @Range(min="0", max="2^31-1")
  public void setPlexCount(Integer paramInteger)
  {
    this.plexCount = paramInteger;
  }

  public Integer getPlexCount()
  {
    return this.plexCount;
  }

  public AggrInfo withPlexCount(Integer paramInteger)
  {
    this.plexCount = paramInteger;
    return this;
  }

  @Element("plexes")
  @Type("plex-info[]")
  public void setPlexes(List<PlexInfo> paramList)
  {
    this.plexes = paramList;
  }

  public List<PlexInfo> getPlexes()
  {
    return this.plexes;
  }

  public AggrInfo withPlexes(List<PlexInfo> paramList)
  {
    this.plexes = paramList;
    return this;
  }

  public AggrInfo withPlexes(PlexInfo[] paramArrayOfPlexInfo)
  {
    this.plexes = new ArrayList(Arrays.asList(paramArrayOfPlexInfo));
    return this;
  }

  @Element("raid-size")
  @Type("integer")
  @Range(min="0", max="2^31-1")
  public void setRaidSize(Integer paramInteger)
  {
    this.raidSize = paramInteger;
  }

  public Integer getRaidSize()
  {
    return this.raidSize;
  }

  public AggrInfo withRaidSize(Integer paramInteger)
  {
    this.raidSize = paramInteger;
    return this;
  }

  @Element("raid-status")
  @Type("string")
  public void setRaidStatus(String paramString)
  {
    this.raidStatus = paramString;
  }

  public String getRaidStatus()
  {
    return this.raidStatus;
  }

  public AggrInfo withRaidStatus(String paramString)
  {
    this.raidStatus = paramString;
    return this;
  }

  @Element("size-available")
  @Type("integer")
  @Range(min="0", max="2^64-1")
  public void setSizeAvailable(BigInteger paramBigInteger)
  {
    this.sizeAvailable = paramBigInteger;
  }

  public BigInteger getSizeAvailable()
  {
    return this.sizeAvailable;
  }

  public AggrInfo withSizeAvailable(BigInteger paramBigInteger)
  {
    this.sizeAvailable = paramBigInteger;
    return this;
  }

  @Element("size-percentage-used")
  @Type("integer")
  @Optional
  @Range(min="0", max="100")
  public void setSizePercentageUsed(Integer paramInteger)
  {
    this.sizePercentageUsed = paramInteger;
  }

  public Integer getSizePercentageUsed()
  {
    return this.sizePercentageUsed;
  }

  public AggrInfo withSizePercentageUsed(Integer paramInteger)
  {
    this.sizePercentageUsed = paramInteger;
    return this;
  }

  @Element("size-total")
  @Type("integer")
  @Range(min="0", max="2^64-1")
  public void setSizeTotal(BigInteger paramBigInteger)
  {
    this.sizeTotal = paramBigInteger;
  }

  public BigInteger getSizeTotal()
  {
    return this.sizeTotal;
  }

  public AggrInfo withSizeTotal(BigInteger paramBigInteger)
  {
    this.sizeTotal = paramBigInteger;
    return this;
  }

  @Element("size-used")
  @Type("integer")
  @Range(min="0", max="2^64-1")
  public void setSizeUsed(BigInteger paramBigInteger)
  {
    this.sizeUsed = paramBigInteger;
  }

  public BigInteger getSizeUsed()
  {
    return this.sizeUsed;
  }

  public AggrInfo withSizeUsed(BigInteger paramBigInteger)
  {
    this.sizeUsed = paramBigInteger;
    return this;
  }

  @Element("snaplock-type")
  @Type("string")
  @Optional
  public void setSnaplockType(String paramString)
  {
    this.snaplockType = paramString;
  }

  public String getSnaplockType()
  {
    return this.snaplockType;
  }

  public AggrInfo withSnaplockType(String paramString)
  {
    this.snaplockType = paramString;
    return this;
  }

  @Element("state")
  @Type("string")
  public void setState(String paramString)
  {
    this.state = paramString;
  }

  public String getState()
  {
    return this.state;
  }

  public AggrInfo withState(String paramString)
  {
    this.state = paramString;
    return this;
  }

  @Element("type")
  @Type("string")
  public void setType(String paramString)
  {
    this.type = paramString;
  }

  public String getType()
  {
    return this.type;
  }

  public AggrInfo withType(String paramString)
  {
    this.type = paramString;
    return this;
  }

  @Element("uuid")
  @Type("string")
  public void setUuid(String paramString)
  {
    this.uuid = paramString;
  }

  public String getUuid()
  {
    return this.uuid;
  }

  public AggrInfo withUuid(String paramString)
  {
    this.uuid = paramString;
    return this;
  }

  @Element("volume-count")
  @Type("integer")
  @Range(min="0", max="200")
  public void setVolumeCount(Integer paramInteger)
  {
    this.volumeCount = paramInteger;
  }

  public Integer getVolumeCount()
  {
    return this.volumeCount;
  }

  public AggrInfo withVolumeCount(Integer paramInteger)
  {
    this.volumeCount = paramInteger;
    return this;
  }

  @Element("volume-count-striped-dv")
  @Type("integer")
  @Optional
  @Range(min="0", max="200")
  public void setVolumeCountStripedDv(Integer paramInteger)
  {
    this.volumeCountStripedDv = paramInteger;
  }

  public Integer getVolumeCountStripedDv()
  {
    return this.volumeCountStripedDv;
  }

  public AggrInfo withVolumeCountStripedDv(Integer paramInteger)
  {
    this.volumeCountStripedDv = paramInteger;
    return this;
  }

  @Element("volume-count-striped-mdv")
  @Type("integer")
  @Optional
  @Range(min="0", max="200")
  public void setVolumeCountStripedMdv(Integer paramInteger)
  {
    this.volumeCountStripedMdv = paramInteger;
  }

  public Integer getVolumeCountStripedMdv()
  {
    return this.volumeCountStripedMdv;
  }

  public AggrInfo withVolumeCountStripedMdv(Integer paramInteger)
  {
    this.volumeCountStripedMdv = paramInteger;
    return this;
  }

  @Element("volumes")
  @Type("contained-volume-info[]")
  public void setVolumes(List<ContainedVolumeInfo> paramList)
  {
    this.volumes = paramList;
  }

  public List<ContainedVolumeInfo> getVolumes()
  {
    return this.volumes;
  }

  public AggrInfo withVolumes(List<ContainedVolumeInfo> paramList)
  {
    this.volumes = paramList;
    return this;
  }

  public AggrInfo withVolumes(ContainedVolumeInfo[] paramArrayOfContainedVolumeInfo)
  {
    this.volumes = new ArrayList(Arrays.asList(paramArrayOfContainedVolumeInfo));
    return this;
  }
}