package com.netapp.filer.aggr;

import com.netapp.common.zapi.annotation.Api;
import com.netapp.common.zapi.annotation.Errno;
import com.netapp.common.zapi.annotation.Family;
import com.netapp.common.zapi.annotation.Input;
import com.netapp.common.zapi.annotation.Optional;
import com.netapp.common.zapi.annotation.Output;
import com.netapp.common.zapi.annotation.Status;
import com.netapp.common.zapi.annotation.StatusEnum;
import com.netapp.common.zapi.annotation.Type;
import com.netapp.filer.volume.DiskInfo;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

@Api("aggr-mirror")
@Status(StatusEnum.PUBLIC)
@Errno({"EINTERNALERROR", "EINVALIDINPUTERROR", "EVOLUMEDISKSIZEINVALID", "EVOLUMENAMEINVALID"})
@Family({"ontap-c", "ontap-d"})
public class AggrMirror
  implements Serializable
{
  private static final long serialVersionUID = 1L;
  public String aggregate;
  public List<DiskInfo> badDisks;
  public Boolean force;
  public List<DiskInfo> mirrorDisks;
  public String victimAggregate;

  @Input("aggregate")
  @Type("string")
  public void setAggregate(String paramString)
  {
    this.aggregate = paramString;
  }

  public String getAggregate()
  {
    return this.aggregate;
  }

  public AggrMirror withAggregate(String paramString)
  {
    this.aggregate = paramString;
    return this;
  }

  @Output("bad-disks")
  @Type("disk-info[]")
  @Optional
  public void setBadDisks(List<DiskInfo> paramList)
  {
    this.badDisks = paramList;
  }

  public List<DiskInfo> getBadDisks()
  {
    return this.badDisks;
  }

  public AggrMirror withBadDisks(List<DiskInfo> paramList)
  {
    this.badDisks = paramList;
    return this;
  }

  public AggrMirror withBadDisks(DiskInfo[] paramArrayOfDiskInfo)
  {
    this.badDisks = new ArrayList(Arrays.asList(paramArrayOfDiskInfo));
    return this;
  }

  @Input("force")
  @Type("boolean")
  @Optional
  public void setForce(Boolean paramBoolean)
  {
    this.force = paramBoolean;
  }

  public Boolean isForce()
  {
    return this.force;
  }

  public AggrMirror withForce(Boolean paramBoolean)
  {
    this.force = paramBoolean;
    return this;
  }

  @Input("mirror-disks")
  @Type("disk-info[]")
  @Optional
  public void setMirrorDisks(List<DiskInfo> paramList)
  {
    this.mirrorDisks = paramList;
  }

  public List<DiskInfo> getMirrorDisks()
  {
    return this.mirrorDisks;
  }

  public AggrMirror withMirrorDisks(List<DiskInfo> paramList)
  {
    this.mirrorDisks = paramList;
    return this;
  }

  public AggrMirror withMirrorDisks(DiskInfo[] paramArrayOfDiskInfo)
  {
    this.mirrorDisks = new ArrayList(Arrays.asList(paramArrayOfDiskInfo));
    return this;
  }

  @Input("victim-aggregate")
  @Type("string")
  @Optional
  public void setVictimAggregate(String paramString)
  {
    this.victimAggregate = paramString;
  }

  public String getVictimAggregate()
  {
    return this.victimAggregate;
  }

  public AggrMirror withVictimAggregate(String paramString)
  {
    this.victimAggregate = paramString;
    return this;
  }
}