package com.netapp.filer.cifs;

import com.netapp.common.zapi.annotation.Api;
import com.netapp.common.zapi.annotation.Errno;
import com.netapp.common.zapi.annotation.Family;
import com.netapp.common.zapi.annotation.Optional;
import com.netapp.common.zapi.annotation.Output;
import com.netapp.common.zapi.annotation.Status;
import com.netapp.common.zapi.annotation.StatusEnum;
import com.netapp.common.zapi.annotation.Type;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

@Api("cifs-list-config")
@Status(StatusEnum.PUBLIC)
@Errno({"ECIFSNOTCONFIGURED", "EDNSNOTENABLED", "EHOSTNOTFOUND", "ELDAPSVRNOTFOUND", "EONTAPI_ENOMEM"})
@Family({"ontap-c", "vfiler"})
public class CifsListConfig
  implements Serializable
{
  private static final long serialVersionUID = 1L;
  public String adSite;
  public List<ConnectionInfo> dcConnection;
  public String dnsDomainname;
  public List<ConnectionInfo> ldapConnection;
  public String netbiosDomainname;
  public String netbiosServername;
  public String windowsType;
  public String authType;
  public String securityStyle;

  @Output("AD-site")
  @Type("string")
  @Optional
  public void setAdSite(String paramString)
  {
    this.adSite = paramString;
  }

  public String getAdSite()
  {
    return this.adSite;
  }

  public CifsListConfig withAdSite(String paramString)
  {
    this.adSite = paramString;
    return this;
  }

  @Output("DC-connection")
  @Type("connection-info[]")
  @Optional
  public void setDcConnection(List<ConnectionInfo> paramList)
  {
    this.dcConnection = paramList;
  }

  public List<ConnectionInfo> getDcConnection()
  {
    return this.dcConnection;
  }

  public CifsListConfig withDcConnection(List<ConnectionInfo> paramList)
  {
    this.dcConnection = paramList;
    return this;
  }

  public CifsListConfig withDcConnection(ConnectionInfo[] paramArrayOfConnectionInfo)
  {
    this.dcConnection = new ArrayList(Arrays.asList(paramArrayOfConnectionInfo));
    return this;
  }

  @Output("DNS-domainname")
  @Type("string")
  @Optional
  public void setDnsDomainname(String paramString)
  {
    this.dnsDomainname = paramString;
  }

  public String getDnsDomainname()
  {
    return this.dnsDomainname;
  }

  public CifsListConfig withDnsDomainname(String paramString)
  {
    this.dnsDomainname = paramString;
    return this;
  }

  @Output("LDAP-connection")
  @Type("connection-info[]")
  @Optional
  public void setLdapConnection(List<ConnectionInfo> paramList)
  {
    this.ldapConnection = paramList;
  }

  public List<ConnectionInfo> getLdapConnection()
  {
    return this.ldapConnection;
  }

  public CifsListConfig withLdapConnection(List<ConnectionInfo> paramList)
  {
    this.ldapConnection = paramList;
    return this;
  }

  public CifsListConfig withLdapConnection(ConnectionInfo[] paramArrayOfConnectionInfo)
  {
    this.ldapConnection = new ArrayList(Arrays.asList(paramArrayOfConnectionInfo));
    return this;
  }

  @Output("NetBIOS-domainname")
  @Type("string")
  @Optional
  public void setNetbiosDomainname(String paramString)
  {
    this.netbiosDomainname = paramString;
  }

  public String getNetbiosDomainname()
  {
    return this.netbiosDomainname;
  }

  public CifsListConfig withNetbiosDomainname(String paramString)
  {
    this.netbiosDomainname = paramString;
    return this;
  }

  @Output("NetBIOS-servername")
  @Type("string")
  public void setNetbiosServername(String paramString)
  {
    this.netbiosServername = paramString;
  }

  public String getNetbiosServername()
  {
    return this.netbiosServername;
  }

  public CifsListConfig withNetbiosServername(String paramString)
  {
    this.netbiosServername = paramString;
    return this;
  }

  @Output("Windows-type")
  @Type("string")
  @Optional
  public void setWindowsType(String paramString)
  {
    this.windowsType = paramString;
  }

  public String getWindowsType()
  {
    return this.windowsType;
  }

  public CifsListConfig withWindowsType(String paramString)
  {
    this.windowsType = paramString;
    return this;
  }

  @Output("auth-type")
  @Type("string")
  public void setAuthType(String paramString)
  {
    this.authType = paramString;
  }

  public String getAuthType()
  {
    return this.authType;
  }

  public CifsListConfig withAuthType(String paramString)
  {
    this.authType = paramString;
    return this;
  }

  @Output("security-style")
  @Type("string")
  public void setSecurityStyle(String paramString)
  {
    this.securityStyle = paramString;
  }

  public String getSecurityStyle()
  {
    return this.securityStyle;
  }

  public CifsListConfig withSecurityStyle(String paramString)
  {
    this.securityStyle = paramString;
    return this;
  }
}