package com.netapp.filer.cifs;

import com.netapp.common.zapi.annotation.Api;
import com.netapp.common.zapi.annotation.Errno;
import com.netapp.common.zapi.annotation.Family;
import com.netapp.common.zapi.annotation.Input;
import com.netapp.common.zapi.annotation.Optional;
import com.netapp.common.zapi.annotation.Output;
import com.netapp.common.zapi.annotation.Status;
import com.netapp.common.zapi.annotation.StatusEnum;
import com.netapp.common.zapi.annotation.Type;
import java.io.Serializable;

@Api("cifs-setup-verify-name")
@Status(StatusEnum.PUBLIC)
@Errno({"EAPILICENSE", "ECIFS_AD_CLOCK_SKEW", "ECIFS_AD_RESET_REQUIRED", "ECIFS_BIND_FAILED", "ECIFS_DC_CONNECT_FAILED", "ECIFS_DNS_REQUIRED", "ECIFS_KRB_CONFLICT", "ECIFS_LIST_UNAVAILABLE", "ECIFS_LOGIN_FAILED", "EINTERNALERROR", "EINVALIDINPUTERROR", "ESVCDISABLED", "ESVCNOTAVAIL"})
@Family({"ontap-c", "vfiler"})
public class CifsSetupVerifyName
  implements Serializable
{
  private static final long serialVersionUID = 1L;
  public String authType;
  public String domainName;
  public Boolean inDomain;
  public Boolean inUse;
  public String loginPassword;
  public String loginUser;
  public String pdcIpAddress;
  public String serverName;

  @Input("auth-type")
  @Type("string")
  public void setAuthType(String paramString)
  {
    this.authType = paramString;
  }

  public String getAuthType()
  {
    return this.authType;
  }

  public CifsSetupVerifyName withAuthType(String paramString)
  {
    this.authType = paramString;
    return this;
  }

  @Input("domain-name")
  @Type("string")
  public void setDomainName(String paramString)
  {
    this.domainName = paramString;
  }

  public String getDomainName()
  {
    return this.domainName;
  }

  public CifsSetupVerifyName withDomainName(String paramString)
  {
    this.domainName = paramString;
    return this;
  }

  @Output("is-in-domain")
  @Type("boolean")
  public void setInDomain(Boolean paramBoolean)
  {
    this.inDomain = paramBoolean;
  }

  public Boolean isInDomain()
  {
    return this.inDomain;
  }

  public CifsSetupVerifyName withInDomain(Boolean paramBoolean)
  {
    this.inDomain = paramBoolean;
    return this;
  }

  @Output("is-in-use")
  @Type("boolean")
  public void setInUse(Boolean paramBoolean)
  {
    this.inUse = paramBoolean;
  }

  public Boolean isInUse()
  {
    return this.inUse;
  }

  public CifsSetupVerifyName withInUse(Boolean paramBoolean)
  {
    this.inUse = paramBoolean;
    return this;
  }

  @Input("login-password")
  @Type("string")
  @Optional
  public void setLoginPassword(String paramString)
  {
    this.loginPassword = paramString;
  }

  public String getLoginPassword()
  {
    return this.loginPassword;
  }

  public CifsSetupVerifyName withLoginPassword(String paramString)
  {
    this.loginPassword = paramString;
    return this;
  }

  @Input("login-user")
  @Type("string")
  @Optional
  public void setLoginUser(String paramString)
  {
    this.loginUser = paramString;
  }

  public String getLoginUser()
  {
    return this.loginUser;
  }

  public CifsSetupVerifyName withLoginUser(String paramString)
  {
    this.loginUser = paramString;
    return this;
  }

  @Input("pdc-ip-address")
  @Type("ip-address")
  @Optional
  public void setPdcIpAddress(String paramString)
  {
    this.pdcIpAddress = paramString;
  }

  public String getPdcIpAddress()
  {
    return this.pdcIpAddress;
  }

  public CifsSetupVerifyName withPdcIpAddress(String paramString)
  {
    this.pdcIpAddress = paramString;
    return this;
  }

  @Input("server-name")
  @Type("string")
  public void setServerName(String paramString)
  {
    this.serverName = paramString;
  }

  public String getServerName()
  {
    return this.serverName;
  }

  public CifsSetupVerifyName withServerName(String paramString)
  {
    this.serverName = paramString;
    return this;
  }
}