package com.netapp.filer.clone;

import com.netapp.common.zapi.annotation.Api;
import com.netapp.common.zapi.annotation.Errno;
import com.netapp.common.zapi.annotation.Family;
import com.netapp.common.zapi.annotation.Input;
import com.netapp.common.zapi.annotation.Optional;
import com.netapp.common.zapi.annotation.Output;
import com.netapp.common.zapi.annotation.Status;
import com.netapp.common.zapi.annotation.StatusEnum;
import com.netapp.common.zapi.annotation.Type;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

@Api("clone-start")
@Status(StatusEnum.PUBLIC)
@Errno({"EDENSE_CLONE_CFILE_ATTR", "EDENSE_CLONE_CFILE_NONEXIST", "EDENSE_CLONE_CHANGELOG_OFF", "EDENSE_CLONE_CREATE", "EDENSE_CLONE_DEST_OUTOF_RANGE", "EDENSE_CLONE_DIFF_VOLUME", "EDENSE_CLONE_DIR_FH", "EDENSE_CLONE_EXIST", "EDENSE_CLONE_FH", "EDENSE_CLONE_FILE_FOLD_INPROG", "EDENSE_CLONE_INVALID_CLONE", "EDENSE_CLONE_INVALID_DIR", "EDENSE_CLONE_INVALID_RANGE", "EDENSE_CLONE_INVALID_SOURCE", "EDENSE_CLONE_LARGE_INFO", "EDENSE_CLONE_LICENSE_EXPIRED", "EDENSE_CLONE_MAX_PROC", "EDENSE_CLONE_METAFILE", "EDENSE_CLONE_METAFILE_ATTR", "EDENSE_CLONE_METAFILE_FH", "EDENSE_CLONE_METAFILE_READ", "EDENSE_CLONE_METAFILE_WRITE", "EDENSE_CLONE_NOT_LICENSED", "EDENSE_CLONE_NOT_QTREE_ROOT", "EDENSE_CLONE_NOT_SUPP", "EDENSE_CLONE_NO_SLOT", "EDENSE_CLONE_OVERLAP_RANGE", "EDENSE_CLONE_PENDING_SNAP", "EDENSE_CLONE_PROC", "EDENSE_CLONE_SAME_FILE", "EDENSE_CLONE_SETATTR", "EDENSE_CLONE_SFILE_ATTR", "EDENSE_CLONE_SFILE_NONEXIST", "EDENSE_CLONE_SFSR_INPROG", "EDENSE_CLONE_SMALL_DATA", "EDENSE_CLONE_SNAPNAME", "EDENSE_CLONE_SNAPPATH", "EDENSE_CLONE_SNAP_ATTR", "EDENSE_CLONE_SNAP_FH", "EDENSE_CLONE_SOURCE_FH", "EDENSE_CLONE_SPLITTING", "EDENSE_CLONE_SPLIT_MAP", "EDENSE_CLONE_SRC_OUTOF_RANGE", "EDENSE_CLONE_TYPE_SUPP", "EDENSE_CLONE_VOL_REVERTING", "EDENSE_CLONE_WORM_VOL", "EDENSE_FAILOVER", "EDENSE_IRON_RUNNING", "EDENSE_NLCSR_INPROG", "EDENSE_NON_SUPPORTED_PLATFORM", "EDENSE_REVERTING", "EDENSE_SHUTDOWN", "EDENSE_SNAPSHOT_CREATE", "EDENSE_SNAPSHOT_NONEXIST", "EDENSE_STALE_CLONE_DIR_INODE", "EDENSE_STALE_DINODE", "EDENSE_STALE_RINODE", "EDENSE_UNSUPPORTED_VOLTYPE", "EDENSE_VDISK_DST_INVALID_BLOCK_RANGE", "EDENSE_VDISK_INTERNAL_ERROR", "EDENSE_VDISK_INVALID_FH", "EDENSE_VDISK_INVALID_NAME", "EDENSE_VDISK_LBA_OFFSET_MISALIGNED", "EDENSE_VDISK_NOT_IN_QTREE_ROOT", "EDENSE_VDISK_NO_SUCH_VDISK", "EDENSE_VDISK_SRC_INVALID_BLOCK_RANGE", "EDENSE_VDISK_SRC_LUN_CLONE", "EDENSE_VDISK_VOL_SPARSE_WO_SCSI", "EDENSE_VOL_NONEXIST", "EDENSE_VOL_NOTSUP", "EDENSE_VOL_OFFLINE", "EDENSE_VOL_RDONLY", "EDENSE_VOL_RESTRICTED", "EDENSE_VOL_TRANSITING"})
@Family({"ontap-c"})
public class CloneStart
  implements Serializable
{
  private static final long serialVersionUID = 1L;
  public List<BlockRange> blockRanges;
  public Boolean changeLog;
  public CloneIdInfo cloneId;
  public String destinationPath;
  public Boolean noSnap;
  public String sourcePath;

  @Input("block-ranges")
  @Type("block-range[]")
  @Optional
  public void setBlockRanges(List<BlockRange> paramList)
  {
    this.blockRanges = paramList;
  }

  public List<BlockRange> getBlockRanges()
  {
    return this.blockRanges;
  }

  public CloneStart withBlockRanges(List<BlockRange> paramList)
  {
    this.blockRanges = paramList;
    return this;
  }

  public CloneStart withBlockRanges(BlockRange[] paramArrayOfBlockRange)
  {
    this.blockRanges = new ArrayList(Arrays.asList(paramArrayOfBlockRange));
    return this;
  }

  @Input("change-log")
  @Type("boolean")
  @Optional
  public void setChangeLog(Boolean paramBoolean)
  {
    this.changeLog = paramBoolean;
  }

  public Boolean isChangeLog()
  {
    return this.changeLog;
  }

  public CloneStart withChangeLog(Boolean paramBoolean)
  {
    this.changeLog = paramBoolean;
    return this;
  }

  @Output("clone-id")
  @Type("clone-id-info")
  public void setCloneId(CloneIdInfo paramCloneIdInfo)
  {
    this.cloneId = paramCloneIdInfo;
  }

  public CloneIdInfo getCloneId()
  {
    return this.cloneId;
  }

  public CloneStart withCloneId(CloneIdInfo paramCloneIdInfo)
  {
    this.cloneId = paramCloneIdInfo;
    return this;
  }

  @Input("destination-path")
  @Type("string")
  @Optional
  public void setDestinationPath(String paramString)
  {
    this.destinationPath = paramString;
  }

  public String getDestinationPath()
  {
    return this.destinationPath;
  }

  public CloneStart withDestinationPath(String paramString)
  {
    this.destinationPath = paramString;
    return this;
  }

  @Input("no-snap")
  @Type("boolean")
  @Optional
  public void setNoSnap(Boolean paramBoolean)
  {
    this.noSnap = paramBoolean;
  }

  public Boolean isNoSnap()
  {
    return this.noSnap;
  }

  public CloneStart withNoSnap(Boolean paramBoolean)
  {
    this.noSnap = paramBoolean;
    return this;
  }

  @Input("source-path")
  @Type("string")
  public void setSourcePath(String paramString)
  {
    this.sourcePath = paramString;
  }

  public String getSourcePath()
  {
    return this.sourcePath;
  }

  public CloneStart withSourcePath(String paramString)
  {
    this.sourcePath = paramString;
    return this;
  }
}