package com.netapp.filer.coredump;

import com.netapp.common.zapi.annotation.Api;
import com.netapp.common.zapi.annotation.Errno;
import com.netapp.common.zapi.annotation.Family;
import com.netapp.common.zapi.annotation.Optional;
import com.netapp.common.zapi.annotation.Output;
import com.netapp.common.zapi.annotation.Range;
import com.netapp.common.zapi.annotation.Status;
import com.netapp.common.zapi.annotation.StatusEnum;
import com.netapp.common.zapi.annotation.Type;
import java.io.Serializable;

@Api("coredump-list-info")
@Status(StatusEnum.PUBLIC)
@Errno({"ECOREDUMPNOTINITIALIZED", "EINTERNALERROR", "ENOTINMAINTMODE", "ENOTINREADONLYMODE", "ENOTWHILEWAITINGFORGIVEBACK"})
@Family({"ontap-d"})
public class CoredumpListInfo
  implements Serializable
{
  private static final long serialVersionUID = 1L;
  public ConfigDetailInfo configDetails;
  public Long fsAvailBytes;
  public Long fsBytesNeeded;
  public Long fsMinfreeBytes;
  public ProgressDetailInfo progressDetails;
  public String state;

  @Output("config-details")
  @Type("config-detail-info")
  @Optional
  public void setConfigDetails(ConfigDetailInfo paramConfigDetailInfo)
  {
    this.configDetails = paramConfigDetailInfo;
  }

  public ConfigDetailInfo getConfigDetails()
  {
    return this.configDetails;
  }

  public CoredumpListInfo withConfigDetails(ConfigDetailInfo paramConfigDetailInfo)
  {
    this.configDetails = paramConfigDetailInfo;
    return this;
  }

  @Output("fs-avail-bytes")
  @Type("integer")
  @Optional
  @Range(min="-1", max="2^63-1")
  public void setFsAvailBytes(Long paramLong)
  {
    this.fsAvailBytes = paramLong;
  }

  public Long getFsAvailBytes()
  {
    return this.fsAvailBytes;
  }

  public CoredumpListInfo withFsAvailBytes(Long paramLong)
  {
    this.fsAvailBytes = paramLong;
    return this;
  }

  @Output("fs-bytes-needed")
  @Type("integer")
  @Optional
  @Range(min="0", max="2^63-1")
  public void setFsBytesNeeded(Long paramLong)
  {
    this.fsBytesNeeded = paramLong;
  }

  public Long getFsBytesNeeded()
  {
    return this.fsBytesNeeded;
  }

  public CoredumpListInfo withFsBytesNeeded(Long paramLong)
  {
    this.fsBytesNeeded = paramLong;
    return this;
  }

  @Output("fs-minfree-bytes")
  @Type("integer")
  @Optional
  @Range(min="-1", max="2^63-1")
  public void setFsMinfreeBytes(Long paramLong)
  {
    this.fsMinfreeBytes = paramLong;
  }

  public Long getFsMinfreeBytes()
  {
    return this.fsMinfreeBytes;
  }

  public CoredumpListInfo withFsMinfreeBytes(Long paramLong)
  {
    this.fsMinfreeBytes = paramLong;
    return this;
  }

  @Output("progress-details")
  @Type("progress-detail-info")
  @Optional
  public void setProgressDetails(ProgressDetailInfo paramProgressDetailInfo)
  {
    this.progressDetails = paramProgressDetailInfo;
  }

  public ProgressDetailInfo getProgressDetails()
  {
    return this.progressDetails;
  }

  public CoredumpListInfo withProgressDetails(ProgressDetailInfo paramProgressDetailInfo)
  {
    this.progressDetails = paramProgressDetailInfo;
    return this;
  }

  @Output("state")
  @Type("string")
  public void setState(String paramString)
  {
    this.state = paramString;
  }

  public String getState()
  {
    return this.state;
  }

  public CoredumpListInfo withState(String paramString)
  {
    this.state = paramString;
    return this;
  }
}