package com.netapp.filer.disk;

import com.netapp.common.zapi.annotation.Api;
import com.netapp.common.zapi.annotation.Errno;
import com.netapp.common.zapi.annotation.Family;
import com.netapp.common.zapi.annotation.Input;
import com.netapp.common.zapi.annotation.Optional;
import com.netapp.common.zapi.annotation.Output;
import com.netapp.common.zapi.annotation.Status;
import com.netapp.common.zapi.annotation.StatusEnum;
import com.netapp.common.zapi.annotation.Type;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

@Api("disk-maint-start")
@Status(StatusEnum.UNSUPPORTED)
@Errno({"EAPIERROR", "EAPIMISSINGARGUMENT", "EDISKNOTFOUND", "EINTERNALERROR"})
@Family({"ontap-c", "ontap-d"})
public class DiskMaintStart
  implements Serializable
{
  private static final long serialVersionUID = 1L;
  public List<String> disks;
  public Boolean immediate;
  public Long maintCycleCount;
  public List<MaintStartStatusInfo> maintStartStatus;
  public List<Long> maintTestIterations;
  public List<String> maintTests;

  @Input("disks")
  @Type("maint-disk-name[]")
  public void setDisks(List<String> paramList)
  {
    this.disks = paramList;
  }

  public List<String> getDisks()
  {
    return this.disks;
  }

  public DiskMaintStart withDisks(List<String> paramList)
  {
    this.disks = paramList;
    return this;
  }

  public DiskMaintStart withDisks(String[] paramArrayOfString)
  {
    this.disks = new ArrayList(Arrays.asList(paramArrayOfString));
    return this;
  }

  @Input("is-immediate")
  @Type("boolean")
  @Optional
  public void setImmediate(Boolean paramBoolean)
  {
    this.immediate = paramBoolean;
  }

  public Boolean isImmediate()
  {
    return this.immediate;
  }

  public DiskMaintStart withImmediate(Boolean paramBoolean)
  {
    this.immediate = paramBoolean;
    return this;
  }

  @Input("maint-cycle-count")
  @Type("integer")
  @Optional
  public void setMaintCycleCount(Long paramLong)
  {
    this.maintCycleCount = paramLong;
  }

  public Long getMaintCycleCount()
  {
    return this.maintCycleCount;
  }

  public DiskMaintStart withMaintCycleCount(Long paramLong)
  {
    this.maintCycleCount = paramLong;
    return this;
  }

  @Output("maint-start-status")
  @Type("maint-start-status-info[]")
  public void setMaintStartStatus(List<MaintStartStatusInfo> paramList)
  {
    this.maintStartStatus = paramList;
  }

  public List<MaintStartStatusInfo> getMaintStartStatus()
  {
    return this.maintStartStatus;
  }

  public DiskMaintStart withMaintStartStatus(List<MaintStartStatusInfo> paramList)
  {
    this.maintStartStatus = paramList;
    return this;
  }

  public DiskMaintStart withMaintStartStatus(MaintStartStatusInfo[] paramArrayOfMaintStartStatusInfo)
  {
    this.maintStartStatus = new ArrayList(Arrays.asList(paramArrayOfMaintStartStatusInfo));
    return this;
  }

  @Input("maint-test-iterations")
  @Type("maint-test-iteration[]")
  @Optional
  public void setMaintTestIterations(List<Long> paramList)
  {
    this.maintTestIterations = paramList;
  }

  public List<Long> getMaintTestIterations()
  {
    return this.maintTestIterations;
  }

  public DiskMaintStart withMaintTestIterations(List<Long> paramList)
  {
    this.maintTestIterations = paramList;
    return this;
  }

  public DiskMaintStart withMaintTestIterations(Long[] paramArrayOfLong)
  {
    this.maintTestIterations = new ArrayList(Arrays.asList(paramArrayOfLong));
    return this;
  }

  @Input("maint-tests")
  @Type("maint-test-id[]")
  @Optional
  public void setMaintTests(List<String> paramList)
  {
    this.maintTests = paramList;
  }

  public List<String> getMaintTests()
  {
    return this.maintTests;
  }

  public DiskMaintStart withMaintTests(List<String> paramList)
  {
    this.maintTests = paramList;
    return this;
  }

  public DiskMaintStart withMaintTests(String[] paramArrayOfString)
  {
    this.maintTests = new ArrayList(Arrays.asList(paramArrayOfString));
    return this;
  }
}