package com.netapp.filer.dsnapshot;

import com.netapp.common.zapi.annotation.Api;
import com.netapp.common.zapi.annotation.Errno;
import com.netapp.common.zapi.annotation.Family;
import com.netapp.common.zapi.annotation.Input;
import com.netapp.common.zapi.annotation.Optional;
import com.netapp.common.zapi.annotation.Output;
import com.netapp.common.zapi.annotation.Status;
import com.netapp.common.zapi.annotation.StatusEnum;
import com.netapp.common.zapi.annotation.Type;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

@Api("d-snapshot-reclaimable")
@Status(StatusEnum.PUBLIC)
@Errno({"EAGGRDOESNOTEXIST", "EAGGRFAILINGOVER", "EAGGRMISMATCH", "EAGGRNOTONLINE", "EAMBIGUOUS_DSID", "EINTERNALERROR", "EINVALIDDSID", "EINVALIDINPUTERROR", "ESNAPSHOTDOESNOTEXIST", "EVOLUMEOFFLINE", "EVOLUMEQUIESCED"})
@Family({"ontap-d"})
public class DSnapshotReclaimable
  implements Serializable
{
  private static final long serialVersionUID = 1L;
  public String containingAggregateUuid;
  public Long reclaimableSize;
  public List<String> snapIds;
  public String uniqVolumeUuid;

  @Input("containing-aggregate-uuid")
  @Type("string")
  @Optional
  public void setContainingAggregateUuid(String paramString)
  {
    this.containingAggregateUuid = paramString;
  }

  public String getContainingAggregateUuid()
  {
    return this.containingAggregateUuid;
  }

  public DSnapshotReclaimable withContainingAggregateUuid(String paramString)
  {
    this.containingAggregateUuid = paramString;
    return this;
  }

  @Output("reclaimable-size")
  @Type("integer")
  public void setReclaimableSize(Long paramLong)
  {
    this.reclaimableSize = paramLong;
  }

  public Long getReclaimableSize()
  {
    return this.reclaimableSize;
  }

  public DSnapshotReclaimable withReclaimableSize(Long paramLong)
  {
    this.reclaimableSize = paramLong;
    return this;
  }

  @Input("snap-ids")
  @Type("d-snapshot-id[]")
  public void setSnapIds(List<String> paramList)
  {
    this.snapIds = paramList;
  }

  public List<String> getSnapIds()
  {
    return this.snapIds;
  }

  public DSnapshotReclaimable withSnapIds(List<String> paramList)
  {
    this.snapIds = paramList;
    return this;
  }

  public DSnapshotReclaimable withSnapIds(String[] paramArrayOfString)
  {
    this.snapIds = new ArrayList(Arrays.asList(paramArrayOfString));
    return this;
  }

  @Input("uniq-volume-uuid")
  @Type("string")
  @Optional
  public void setUniqVolumeUuid(String paramString)
  {
    this.uniqVolumeUuid = paramString;
  }

  public String getUniqVolumeUuid()
  {
    return this.uniqVolumeUuid;
  }

  public DSnapshotReclaimable withUniqVolumeUuid(String paramString)
  {
    this.uniqVolumeUuid = paramString;
    return this;
  }
}