package com.netapp.filer.ems;

import com.netapp.common.zapi.annotation.Api;
import com.netapp.common.zapi.annotation.Errno;
import com.netapp.common.zapi.annotation.Input;
import com.netapp.common.zapi.annotation.Optional;
import com.netapp.common.zapi.annotation.Status;
import com.netapp.common.zapi.annotation.StatusEnum;
import com.netapp.common.zapi.annotation.Type;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

@Api("ems-invoke")
@Status(StatusEnum.PUBLIC)
@Errno({"EEMS_INVOKE_BAD_PARAM", "EEMS_INVOKE_FAILED", "EEMS_INVOKE_ID_BAD", "EEMS_INVOKE_NVRAM_TOO_BIG", "EEMS_INVOKE_PARAMS_INSUFFICIENT", "EEMS_INVOKE_QUEUE_FULL", "EEMS_INVOKE_SEVERITY_INVALID", "EEMS_INVOKE_SEVERITY_NOT_VARIABLE", "EEMS_INVOKE_SEVERITY_REQUIRED", "EEMS_INVOKE_SUPRESS_DUP", "EEMS_INVOKE_SUPRESS_TIMER", "EEMS_INVOKE_VERSION_INVALID"})
public class EmsInvoke
  implements Serializable
{
  private static final long serialVersionUID = 1L;
  public String eventId;
  public Long eventVersion;
  public List<String> params;
  public String severity;
  public Boolean snmp;
  public Boolean syslog;

  @Input("event-id")
  @Type("string")
  public void setEventId(String paramString)
  {
    this.eventId = paramString;
  }

  public String getEventId()
  {
    return this.eventId;
  }

  public EmsInvoke withEventId(String paramString)
  {
    this.eventId = paramString;
    return this;
  }

  @Input("event-version")
  @Type("integer")
  @Optional
  public void setEventVersion(Long paramLong)
  {
    this.eventVersion = paramLong;
  }

  public Long getEventVersion()
  {
    return this.eventVersion;
  }

  public EmsInvoke withEventVersion(Long paramLong)
  {
    this.eventVersion = paramLong;
    return this;
  }

  @Input("params")
  @Type("param[]")
  @Optional
  public void setParams(List<String> paramList)
  {
    this.params = paramList;
  }

  public List<String> getParams()
  {
    return this.params;
  }

  public EmsInvoke withParams(List<String> paramList)
  {
    this.params = paramList;
    return this;
  }

  public EmsInvoke withParams(String[] paramArrayOfString)
  {
    this.params = new ArrayList(Arrays.asList(paramArrayOfString));
    return this;
  }

  @Input("severity")
  @Type("string")
  @Optional
  public void setSeverity(String paramString)
  {
    this.severity = paramString;
  }

  public String getSeverity()
  {
    return this.severity;
  }

  public EmsInvoke withSeverity(String paramString)
  {
    this.severity = paramString;
    return this;
  }

  @Input("snmp")
  @Type("boolean")
  @Optional
  public void setSnmp(Boolean paramBoolean)
  {
    this.snmp = paramBoolean;
  }

  public Boolean isSnmp()
  {
    return this.snmp;
  }

  public EmsInvoke withSnmp(Boolean paramBoolean)
  {
    this.snmp = paramBoolean;
    return this;
  }

  @Input("syslog")
  @Type("boolean")
  @Optional
  public void setSyslog(Boolean paramBoolean)
  {
    this.syslog = paramBoolean;
  }

  public Boolean isSyslog()
  {
    return this.syslog;
  }

  public EmsInvoke withSyslog(Boolean paramBoolean)
  {
    this.syslog = paramBoolean;
    return this;
  }
}