package com.netapp.filer.fpolicy;

import com.netapp.common.zapi.annotation.Api;
import com.netapp.common.zapi.annotation.Errno;
import com.netapp.common.zapi.annotation.Family;
import com.netapp.common.zapi.annotation.Input;
import com.netapp.common.zapi.annotation.Optional;
import com.netapp.common.zapi.annotation.Range;
import com.netapp.common.zapi.annotation.Status;
import com.netapp.common.zapi.annotation.StatusEnum;
import com.netapp.common.zapi.annotation.Type;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

@Api("fpolicy-set-policy-options")
@Status(StatusEnum.PUBLIC)
@Errno({"EINTERNALERROR", "EINVALIDINPUTERROR"})
@Family({"ontap-c", "vfiler"})
public class FpolicySetPolicyOptions
  implements Serializable
{
  private static final long serialVersionUID = 1L;
  public Boolean adsMonitored;
  public Boolean cifsDisconnectCheckEnabled;
  public Boolean cifsSetattrEnabled;
  public Boolean required;
  public String policyName;
  public Long reqcancelTimeout;
  public List<SecondaryServerInfo> secondaryServers;
  public Long serverprogressTimeout;

  @Input("is-ads-monitored")
  @Type("boolean")
  @Optional
  public void setAdsMonitored(Boolean paramBoolean)
  {
    this.adsMonitored = paramBoolean;
  }

  public Boolean isAdsMonitored()
  {
    return this.adsMonitored;
  }

  public FpolicySetPolicyOptions withAdsMonitored(Boolean paramBoolean)
  {
    this.adsMonitored = paramBoolean;
    return this;
  }

  @Input("is-cifs-disconnect-check-enabled")
  @Type("boolean")
  @Optional
  public void setCifsDisconnectCheckEnabled(Boolean paramBoolean)
  {
    this.cifsDisconnectCheckEnabled = paramBoolean;
  }

  public Boolean isCifsDisconnectCheckEnabled()
  {
    return this.cifsDisconnectCheckEnabled;
  }

  public FpolicySetPolicyOptions withCifsDisconnectCheckEnabled(Boolean paramBoolean)
  {
    this.cifsDisconnectCheckEnabled = paramBoolean;
    return this;
  }

  @Input("is-cifs-setattr-enabled")
  @Type("boolean")
  @Optional
  public void setCifsSetattrEnabled(Boolean paramBoolean)
  {
    this.cifsSetattrEnabled = paramBoolean;
  }

  public Boolean isCifsSetattrEnabled()
  {
    return this.cifsSetattrEnabled;
  }

  public FpolicySetPolicyOptions withCifsSetattrEnabled(Boolean paramBoolean)
  {
    this.cifsSetattrEnabled = paramBoolean;
    return this;
  }

  @Input("is-required")
  @Type("boolean")
  @Optional
  public void setRequired(Boolean paramBoolean)
  {
    this.required = paramBoolean;
  }

  public Boolean isRequired()
  {
    return this.required;
  }

  public FpolicySetPolicyOptions withRequired(Boolean paramBoolean)
  {
    this.required = paramBoolean;
    return this;
  }

  @Input("policy-name")
  @Type("string")
  public void setPolicyName(String paramString)
  {
    this.policyName = paramString;
  }

  public String getPolicyName()
  {
    return this.policyName;
  }

  public FpolicySetPolicyOptions withPolicyName(String paramString)
  {
    this.policyName = paramString;
    return this;
  }

  @Input("reqcancel-timeout")
  @Type("integer")
  @Optional
  @Range(min="0", max="2^32-1")
  public void setReqcancelTimeout(Long paramLong)
  {
    this.reqcancelTimeout = paramLong;
  }

  public Long getReqcancelTimeout()
  {
    return this.reqcancelTimeout;
  }

  public FpolicySetPolicyOptions withReqcancelTimeout(Long paramLong)
  {
    this.reqcancelTimeout = paramLong;
    return this;
  }

  @Input("secondary-servers")
  @Type("secondary-server-info[]")
  @Optional
  public void setSecondaryServers(List<SecondaryServerInfo> paramList)
  {
    this.secondaryServers = paramList;
  }

  public List<SecondaryServerInfo> getSecondaryServers()
  {
    return this.secondaryServers;
  }

  public FpolicySetPolicyOptions withSecondaryServers(List<SecondaryServerInfo> paramList)
  {
    this.secondaryServers = paramList;
    return this;
  }

  public FpolicySetPolicyOptions withSecondaryServers(SecondaryServerInfo[] paramArrayOfSecondaryServerInfo)
  {
    this.secondaryServers = new ArrayList(Arrays.asList(paramArrayOfSecondaryServerInfo));
    return this;
  }

  @Input("serverprogress-timeout")
  @Type("integer")
  @Optional
  @Range(min="0", max="2^32-1")
  public void setServerprogressTimeout(Long paramLong)
  {
    this.serverprogressTimeout = paramLong;
  }

  public Long getServerprogressTimeout()
  {
    return this.serverprogressTimeout;
  }

  public FpolicySetPolicyOptions withServerprogressTimeout(Long paramLong)
  {
    this.serverprogressTimeout = paramLong;
    return this;
  }
}