package com.netapp.filer.ic;

import com.netapp.common.zapi.annotation.Api;
import com.netapp.common.zapi.annotation.Errno;
import com.netapp.common.zapi.annotation.Family;
import com.netapp.common.zapi.annotation.Input;
import com.netapp.common.zapi.annotation.Optional;
import com.netapp.common.zapi.annotation.Output;
import com.netapp.common.zapi.annotation.Range;
import com.netapp.common.zapi.annotation.Status;
import com.netapp.common.zapi.annotation.StatusEnum;
import com.netapp.common.zapi.annotation.Type;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

@Api("ic-get-error-stats")
@Status(StatusEnum.PUBLIC)
@Errno({"EINTERNALERROR"})
@Family({"ontap-c", "ontap-d"})
public class IcGetErrorStats
  implements Serializable
{
  private static final long serialVersionUID = 1L;
  public Long activeLogicalLink;
  public List<IcCounter> nvramHardwareErrorStats;
  public List<IcCounter> nvramHardwarePerfStats;
  public List<NvramPerPortCountersInfo> nvramPerPortCounters;
  public List<IcCounter> nvramSoftwareCounters;
  public List<IcCounter> peerConnectStats;
  public List<PerLinkParametersInfo> perLinkParameters;
  public String sfoConnectionState;
  public Long sfoNicUsed;
  public Boolean verbose;

  @Output("active-logical-link")
  @Type("integer")
  @Optional
  @Range(min="0", max="2^32-1")
  public void setActiveLogicalLink(Long paramLong)
  {
    this.activeLogicalLink = paramLong;
  }

  public Long getActiveLogicalLink()
  {
    return this.activeLogicalLink;
  }

  public IcGetErrorStats withActiveLogicalLink(Long paramLong)
  {
    this.activeLogicalLink = paramLong;
    return this;
  }

  @Output("nvram-hardware-error-stats")
  @Type("ic-counter[]")
  @Optional
  public void setNvramHardwareErrorStats(List<IcCounter> paramList)
  {
    this.nvramHardwareErrorStats = paramList;
  }

  public List<IcCounter> getNvramHardwareErrorStats()
  {
    return this.nvramHardwareErrorStats;
  }

  public IcGetErrorStats withNvramHardwareErrorStats(List<IcCounter> paramList)
  {
    this.nvramHardwareErrorStats = paramList;
    return this;
  }

  public IcGetErrorStats withNvramHardwareErrorStats(IcCounter[] paramArrayOfIcCounter)
  {
    this.nvramHardwareErrorStats = new ArrayList(Arrays.asList(paramArrayOfIcCounter));
    return this;
  }

  @Output("nvram-hardware-perf-stats")
  @Type("ic-counter[]")
  @Optional
  public void setNvramHardwarePerfStats(List<IcCounter> paramList)
  {
    this.nvramHardwarePerfStats = paramList;
  }

  public List<IcCounter> getNvramHardwarePerfStats()
  {
    return this.nvramHardwarePerfStats;
  }

  public IcGetErrorStats withNvramHardwarePerfStats(List<IcCounter> paramList)
  {
    this.nvramHardwarePerfStats = paramList;
    return this;
  }

  public IcGetErrorStats withNvramHardwarePerfStats(IcCounter[] paramArrayOfIcCounter)
  {
    this.nvramHardwarePerfStats = new ArrayList(Arrays.asList(paramArrayOfIcCounter));
    return this;
  }

  @Output("nvram-per-port-counters")
  @Type("nvram-per-port-counters-info[]")
  @Optional
  public void setNvramPerPortCounters(List<NvramPerPortCountersInfo> paramList)
  {
    this.nvramPerPortCounters = paramList;
  }

  public List<NvramPerPortCountersInfo> getNvramPerPortCounters()
  {
    return this.nvramPerPortCounters;
  }

  public IcGetErrorStats withNvramPerPortCounters(List<NvramPerPortCountersInfo> paramList)
  {
    this.nvramPerPortCounters = paramList;
    return this;
  }

  public IcGetErrorStats withNvramPerPortCounters(NvramPerPortCountersInfo[] paramArrayOfNvramPerPortCountersInfo)
  {
    this.nvramPerPortCounters = new ArrayList(Arrays.asList(paramArrayOfNvramPerPortCountersInfo));
    return this;
  }

  @Output("nvram-software-counters")
  @Type("ic-counter[]")
  @Optional
  public void setNvramSoftwareCounters(List<IcCounter> paramList)
  {
    this.nvramSoftwareCounters = paramList;
  }

  public List<IcCounter> getNvramSoftwareCounters()
  {
    return this.nvramSoftwareCounters;
  }

  public IcGetErrorStats withNvramSoftwareCounters(List<IcCounter> paramList)
  {
    this.nvramSoftwareCounters = paramList;
    return this;
  }

  public IcGetErrorStats withNvramSoftwareCounters(IcCounter[] paramArrayOfIcCounter)
  {
    this.nvramSoftwareCounters = new ArrayList(Arrays.asList(paramArrayOfIcCounter));
    return this;
  }

  @Output("peer-connect-stats")
  @Type("ic-counter[]")
  public void setPeerConnectStats(List<IcCounter> paramList)
  {
    this.peerConnectStats = paramList;
  }

  public List<IcCounter> getPeerConnectStats()
  {
    return this.peerConnectStats;
  }

  public IcGetErrorStats withPeerConnectStats(List<IcCounter> paramList)
  {
    this.peerConnectStats = paramList;
    return this;
  }

  public IcGetErrorStats withPeerConnectStats(IcCounter[] paramArrayOfIcCounter)
  {
    this.peerConnectStats = new ArrayList(Arrays.asList(paramArrayOfIcCounter));
    return this;
  }

  @Output("per-link-parameters")
  @Type("per-link-parameters-info[]")
  @Optional
  public void setPerLinkParameters(List<PerLinkParametersInfo> paramList)
  {
    this.perLinkParameters = paramList;
  }

  public List<PerLinkParametersInfo> getPerLinkParameters()
  {
    return this.perLinkParameters;
  }

  public IcGetErrorStats withPerLinkParameters(List<PerLinkParametersInfo> paramList)
  {
    this.perLinkParameters = paramList;
    return this;
  }

  public IcGetErrorStats withPerLinkParameters(PerLinkParametersInfo[] paramArrayOfPerLinkParametersInfo)
  {
    this.perLinkParameters = new ArrayList(Arrays.asList(paramArrayOfPerLinkParametersInfo));
    return this;
  }

  @Output("sfo-connection-state")
  @Type("string")
  public void setSfoConnectionState(String paramString)
  {
    this.sfoConnectionState = paramString;
  }

  public String getSfoConnectionState()
  {
    return this.sfoConnectionState;
  }

  public IcGetErrorStats withSfoConnectionState(String paramString)
  {
    this.sfoConnectionState = paramString;
    return this;
  }

  @Output("sfo-nic-used")
  @Type("integer")
  @Range(min="0", max="2^32-1")
  public void setSfoNicUsed(Long paramLong)
  {
    this.sfoNicUsed = paramLong;
  }

  public Long getSfoNicUsed()
  {
    return this.sfoNicUsed;
  }

  public IcGetErrorStats withSfoNicUsed(Long paramLong)
  {
    this.sfoNicUsed = paramLong;
    return this;
  }

  @Input("verbose")
  @Type("boolean")
  @Optional
  public void setVerbose(Boolean paramBoolean)
  {
    this.verbose = paramBoolean;
  }

  public Boolean isVerbose()
  {
    return this.verbose;
  }

  public IcGetErrorStats withVerbose(Boolean paramBoolean)
  {
    this.verbose = paramBoolean;
    return this;
  }
}