package com.netapp.filer.lun;

import com.netapp.common.zapi.annotation.Api;
import com.netapp.common.zapi.annotation.Errno;
import com.netapp.common.zapi.annotation.Family;
import com.netapp.common.zapi.annotation.Input;
import com.netapp.common.zapi.annotation.Optional;
import com.netapp.common.zapi.annotation.Output;
import com.netapp.common.zapi.annotation.Range;
import com.netapp.common.zapi.annotation.Status;
import com.netapp.common.zapi.annotation.StatusEnum;
import com.netapp.common.zapi.annotation.Type;
import java.io.Serializable;

@Api("lun-create-by-size")
@Status(StatusEnum.PUBLIC)
@Errno({"EFTYPE", "EINVALIDINPUTERROR", "EISDIR", "ENAMETOOLONG", "EOFFLINE", "EONTAPI_EEXIST", "EONTAPI_EINVAL", "EONTAPI_ENOENT", "EONTAPI_ENOSPC", "EVDISK_ERROR_FILE_IS_SPC_RESERVED", "EVDISK_ERROR_INVALID_SNAPSHOT_PATH", "EVDISK_ERROR_NOT_IN_SAME_VOLUME", "EVDISK_ERROR_NOT_QTREE_ROOT", "EVDISK_ERROR_NO_SUCH_DIRECTORY", "EVDISK_ERROR_NO_SUCH_VOLUME", "EVDISK_ERROR_SIZE_TOO_LARGE", "EVDISK_ERROR_SIZE_TOO_SMALL", "EVDISK_ERROR_SOURCE_IS_VDISK", "EVDISK_ERROR_VOLUME_SPARSE"})
@Family({"ontap-c", "vfiler"})
public class LunCreateBySize
  implements Serializable
{
  private static final long serialVersionUID = 1L;
  public Long actualSize;
  public String ostype;
  public String path;
  public Long size;
  public Boolean spaceReservationEnabled;
  public String type;

  @Output("actual-size")
  @Type("integer")
  @Range(min="0", max="2^63-1")
  public void setActualSize(Long paramLong)
  {
    this.actualSize = paramLong;
  }

  public Long getActualSize()
  {
    return this.actualSize;
  }

  public LunCreateBySize withActualSize(Long paramLong)
  {
    this.actualSize = paramLong;
    return this;
  }

  @Input("ostype")
  @Type("string")
  @Optional
  public void setOstype(String paramString)
  {
    this.ostype = paramString;
  }

  public String getOstype()
  {
    return this.ostype;
  }

  public LunCreateBySize withOstype(String paramString)
  {
    this.ostype = paramString;
    return this;
  }

  @Input("path")
  @Type("string")
  public void setPath(String paramString)
  {
    this.path = paramString;
  }

  public String getPath()
  {
    return this.path;
  }

  public LunCreateBySize withPath(String paramString)
  {
    this.path = paramString;
    return this;
  }

  @Input("size")
  @Type("integer")
  @Range(min="0", max="2^63-1")
  public void setSize(Long paramLong)
  {
    this.size = paramLong;
  }

  public Long getSize()
  {
    return this.size;
  }

  public LunCreateBySize withSize(Long paramLong)
  {
    this.size = paramLong;
    return this;
  }

  @Input("space-reservation-enabled")
  @Type("boolean")
  @Optional
  public void setSpaceReservationEnabled(Boolean paramBoolean)
  {
    this.spaceReservationEnabled = paramBoolean;
  }

  public Boolean isSpaceReservationEnabled()
  {
    return this.spaceReservationEnabled;
  }

  public LunCreateBySize withSpaceReservationEnabled(Boolean paramBoolean)
  {
    this.spaceReservationEnabled = paramBoolean;
    return this;
  }

  @Input("type")
  @Type("string")
  @Optional
  public void setType(String paramString)
  {
    this.type = paramString;
  }

  public String getType()
  {
    return this.type;
  }

  public LunCreateBySize withType(String paramString)
  {
    this.type = paramString;
    return this;
  }
}