package com.netapp.filer.lun;

import com.netapp.common.zapi.annotation.Api;
import com.netapp.common.zapi.annotation.Errno;
import com.netapp.common.zapi.annotation.Family;
import com.netapp.common.zapi.annotation.Input;
import com.netapp.common.zapi.annotation.Optional;
import com.netapp.common.zapi.annotation.Output;
import com.netapp.common.zapi.annotation.Range;
import com.netapp.common.zapi.annotation.Status;
import com.netapp.common.zapi.annotation.StatusEnum;
import com.netapp.common.zapi.annotation.Type;
import java.io.Serializable;
import java.math.BigInteger;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

@Api("lun-lba-hole-range-query")
@Status(StatusEnum.UNSUPPORTED)
@Errno({"EINVALIDINPUTERROR"})
@Family({"ontap-c", "vfiler"})
public class LunLbaHoleRangeQuery
  implements Serializable
{
  private static final long serialVersionUID = 1L;
  public BigInteger endLba;
  public List<LbaHoleRangeInfo> holes;
  public String lunPath;
  public BigInteger startLba;

  @Input("end-lba")
  @Type("integer")
  @Range(min="0", max="2^64-1")
  public void setEndLba(BigInteger paramBigInteger)
  {
    this.endLba = paramBigInteger;
  }

  public BigInteger getEndLba()
  {
    return this.endLba;
  }

  public LunLbaHoleRangeQuery withEndLba(BigInteger paramBigInteger)
  {
    this.endLba = paramBigInteger;
    return this;
  }

  @Output("holes")
  @Type("lba-hole-range-info[]")
  @Optional
  public void setHoles(List<LbaHoleRangeInfo> paramList)
  {
    this.holes = paramList;
  }

  public List<LbaHoleRangeInfo> getHoles()
  {
    return this.holes;
  }

  public LunLbaHoleRangeQuery withHoles(List<LbaHoleRangeInfo> paramList)
  {
    this.holes = paramList;
    return this;
  }

  public LunLbaHoleRangeQuery withHoles(LbaHoleRangeInfo[] paramArrayOfLbaHoleRangeInfo)
  {
    this.holes = new ArrayList(Arrays.asList(paramArrayOfLbaHoleRangeInfo));
    return this;
  }

  @Input("lun-path")
  @Type("string")
  public void setLunPath(String paramString)
  {
    this.lunPath = paramString;
  }

  public String getLunPath()
  {
    return this.lunPath;
  }

  public LunLbaHoleRangeQuery withLunPath(String paramString)
  {
    this.lunPath = paramString;
    return this;
  }

  @Input("start-lba")
  @Type("integer")
  @Range(min="0", max="2^64-1")
  public void setStartLba(BigInteger paramBigInteger)
  {
    this.startLba = paramBigInteger;
  }

  public BigInteger getStartLba()
  {
    return this.startLba;
  }

  public LunLbaHoleRangeQuery withStartLba(BigInteger paramBigInteger)
  {
    this.startLba = paramBigInteger;
    return this;
  }
}