package com.netapp.filer.nfs;

import com.netapp.common.zapi.annotation.Api;
import com.netapp.common.zapi.annotation.Errno;
import com.netapp.common.zapi.annotation.Family;
import com.netapp.common.zapi.annotation.Input;
import com.netapp.common.zapi.annotation.Optional;
import com.netapp.common.zapi.annotation.Output;
import com.netapp.common.zapi.annotation.Status;
import com.netapp.common.zapi.annotation.StatusEnum;
import com.netapp.common.zapi.annotation.Type;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

@Api("nfs-exportfs-append-rules")
@Status(StatusEnum.PUBLIC)
@Errno({"EINTERNALERROR"})
@Family({"ontap-c", "vfiler"})
public class NfsExportfsAppendRules
  implements Serializable
{
  private static final long serialVersionUID = 1L;
  public List<PathnameInfo> exportedPathnames;
  public List<PathnameInfo> loadedPathnames;
  public Boolean persistent;
  public List<ExportsRuleInfo> rules;
  public Boolean verbose;

  @Output("exported-pathnames")
  @Type("pathname-info[]")
  @Optional
  public void setExportedPathnames(List<PathnameInfo> paramList)
  {
    this.exportedPathnames = paramList;
  }

  public List<PathnameInfo> getExportedPathnames()
  {
    return this.exportedPathnames;
  }

  public NfsExportfsAppendRules withExportedPathnames(List<PathnameInfo> paramList)
  {
    this.exportedPathnames = paramList;
    return this;
  }

  public NfsExportfsAppendRules withExportedPathnames(PathnameInfo[] paramArrayOfPathnameInfo)
  {
    this.exportedPathnames = new ArrayList(Arrays.asList(paramArrayOfPathnameInfo));
    return this;
  }

  @Output("loaded-pathnames")
  @Type("pathname-info[]")
  @Optional
  public void setLoadedPathnames(List<PathnameInfo> paramList)
  {
    this.loadedPathnames = paramList;
  }

  public List<PathnameInfo> getLoadedPathnames()
  {
    return this.loadedPathnames;
  }

  public NfsExportfsAppendRules withLoadedPathnames(List<PathnameInfo> paramList)
  {
    this.loadedPathnames = paramList;
    return this;
  }

  public NfsExportfsAppendRules withLoadedPathnames(PathnameInfo[] paramArrayOfPathnameInfo)
  {
    this.loadedPathnames = new ArrayList(Arrays.asList(paramArrayOfPathnameInfo));
    return this;
  }

  @Input("persistent")
  @Type("boolean")
  @Optional
  public void setPersistent(Boolean paramBoolean)
  {
    this.persistent = paramBoolean;
  }

  public Boolean isPersistent()
  {
    return this.persistent;
  }

  public NfsExportfsAppendRules withPersistent(Boolean paramBoolean)
  {
    this.persistent = paramBoolean;
    return this;
  }

  @Input("rules")
  @Type("exports-rule-info[]")
  public void setRules(List<ExportsRuleInfo> paramList)
  {
    this.rules = paramList;
  }

  public List<ExportsRuleInfo> getRules()
  {
    return this.rules;
  }

  public NfsExportfsAppendRules withRules(List<ExportsRuleInfo> paramList)
  {
    this.rules = paramList;
    return this;
  }

  public NfsExportfsAppendRules withRules(ExportsRuleInfo[] paramArrayOfExportsRuleInfo)
  {
    this.rules = new ArrayList(Arrays.asList(paramArrayOfExportsRuleInfo));
    return this;
  }

  @Input("verbose")
  @Type("boolean")
  @Optional
  public void setVerbose(Boolean paramBoolean)
  {
    this.verbose = paramBoolean;
  }

  public Boolean isVerbose()
  {
    return this.verbose;
  }

  public NfsExportfsAppendRules withVerbose(Boolean paramBoolean)
  {
    this.verbose = paramBoolean;
    return this;
  }
}