package com.netapp.filer.nfs;

import com.netapp.common.zapi.annotation.Api;
import com.netapp.common.zapi.annotation.Errno;
import com.netapp.common.zapi.annotation.Family;
import com.netapp.common.zapi.annotation.Input;
import com.netapp.common.zapi.annotation.Optional;
import com.netapp.common.zapi.annotation.Output;
import com.netapp.common.zapi.annotation.Status;
import com.netapp.common.zapi.annotation.StatusEnum;
import com.netapp.common.zapi.annotation.Type;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

@Api("nfs-exportfs-list-rules")
@Status(StatusEnum.PUBLIC)
@Errno({"EEXPORTSINCOMPATIBLE", "EINTERNALERROR"})
@Family({"ontap-c", "vfiler"})
public class NfsExportfsListRules
  implements Serializable
{
  private static final long serialVersionUID = 1L;
  public String pathname;
  public Boolean persistent;
  public List<ExportsRuleInfo> rules;

  @Input("pathname")
  @Type("string")
  @Optional
  public void setPathname(String paramString)
  {
    this.pathname = paramString;
  }

  public String getPathname()
  {
    return this.pathname;
  }

  public NfsExportfsListRules withPathname(String paramString)
  {
    this.pathname = paramString;
    return this;
  }

  @Input("persistent")
  @Type("boolean")
  @Optional
  public void setPersistent(Boolean paramBoolean)
  {
    this.persistent = paramBoolean;
  }

  public Boolean isPersistent()
  {
    return this.persistent;
  }

  public NfsExportfsListRules withPersistent(Boolean paramBoolean)
  {
    this.persistent = paramBoolean;
    return this;
  }

  @Output("rules")
  @Type("exports-rule-info[]")
  public void setRules(List<ExportsRuleInfo> paramList)
  {
    this.rules = paramList;
  }

  public List<ExportsRuleInfo> getRules()
  {
    return this.rules;
  }

  public NfsExportfsListRules withRules(List<ExportsRuleInfo> paramList)
  {
    this.rules = paramList;
    return this;
  }

  public NfsExportfsListRules withRules(ExportsRuleInfo[] paramArrayOfExportsRuleInfo)
  {
    this.rules = new ArrayList(Arrays.asList(paramArrayOfExportsRuleInfo));
    return this;
  }
}