package com.netapp.filer.reallocate;

import com.netapp.common.zapi.annotation.Api;
import com.netapp.common.zapi.annotation.Errno;
import com.netapp.common.zapi.annotation.Input;
import com.netapp.common.zapi.annotation.Optional;
import com.netapp.common.zapi.annotation.Output;
import com.netapp.common.zapi.annotation.Status;
import com.netapp.common.zapi.annotation.StatusEnum;
import com.netapp.common.zapi.annotation.Type;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

@Api("reallocate-list-info")
@Status(StatusEnum.PUBLIC)
@Errno({"EREALLOCATE_BADPATH", "EREALLOCATE_BADVOL", "EREALLOCATE_NOMEM", "EREALLOCATE_NOSCAN", "EREALLOCATE_OFF"})
public class ReallocateListInfo
  implements Serializable
{
  private static final long serialVersionUID = 1L;
  public String globalStatus;
  public String path;
  public List<ReallocateJobInfo> reallocateJobs;
  public Boolean verbose;

  @Output("global-status")
  @Type("string")
  public void setGlobalStatus(String paramString)
  {
    this.globalStatus = paramString;
  }

  public String getGlobalStatus()
  {
    return this.globalStatus;
  }

  public ReallocateListInfo withGlobalStatus(String paramString)
  {
    this.globalStatus = paramString;
    return this;
  }

  @Input("path")
  @Type("string")
  @Optional
  public void setPath(String paramString)
  {
    this.path = paramString;
  }

  public String getPath()
  {
    return this.path;
  }

  public ReallocateListInfo withPath(String paramString)
  {
    this.path = paramString;
    return this;
  }

  @Output("reallocate-jobs")
  @Type("reallocate-job-info[]")
  public void setReallocateJobs(List<ReallocateJobInfo> paramList)
  {
    this.reallocateJobs = paramList;
  }

  public List<ReallocateJobInfo> getReallocateJobs()
  {
    return this.reallocateJobs;
  }

  public ReallocateListInfo withReallocateJobs(List<ReallocateJobInfo> paramList)
  {
    this.reallocateJobs = paramList;
    return this;
  }

  public ReallocateListInfo withReallocateJobs(ReallocateJobInfo[] paramArrayOfReallocateJobInfo)
  {
    this.reallocateJobs = new ArrayList(Arrays.asList(paramArrayOfReallocateJobInfo));
    return this;
  }

  @Input("verbose")
  @Type("boolean")
  @Optional
  public void setVerbose(Boolean paramBoolean)
  {
    this.verbose = paramBoolean;
  }

  public Boolean isVerbose()
  {
    return this.verbose;
  }

  public ReallocateListInfo withVerbose(Boolean paramBoolean)
  {
    this.verbose = paramBoolean;
    return this;
  }
}