package com.netapp.filer.sf;

import com.netapp.common.zapi.annotation.Api;
import com.netapp.common.zapi.annotation.Errno;
import com.netapp.common.zapi.annotation.Family;
import com.netapp.common.zapi.annotation.Optional;
import com.netapp.common.zapi.annotation.Output;
import com.netapp.common.zapi.annotation.Status;
import com.netapp.common.zapi.annotation.StatusEnum;
import com.netapp.common.zapi.annotation.Type;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.List;

@Api("sf-status")
@Status(StatusEnum.PUBLIC)
@Errno({"EINTERNALERROR", "ESERVICENOTINITIALIZED"})
@Family({"ontap-d"})
public class SfStatus
  implements Serializable
{
  private static final long serialVersionUID = 1L;
  public MailboxIoTimesInfo backupIoTimes;
  public List<MailboxStatusInfo> backupMailboxStatus;
  public Long bootingReceived;
  public Boolean controlPartnerAggregates;
  public String currentResourceTableEntry;
  public Long currentResourceTableIndex;
  public Long firmwareReceived;
  public Boolean enabled;
  public Boolean interconnectUp;
  public Boolean killPackets;
  public List<SfDiskInfo> localMailboxDisks;
  public Long logsUnsyncedCount;
  public Long maxResourceTableIndex;
  public String name;
  public Long nvramId;
  public List<SfDiskInfo> partnerMailboxDisks;
  public String partnerName;
  public Long partnerNvramId;
  public MailboxIoTimesInfo primaryIoTimes;
  public List<MailboxStatusInfo> primaryMailboxStatus;
  public List<ResourceTableInfo> resourceTable;
  public SfOptionsInfo sfOptions;
  public String state;
  public List<TakeoverInfo> takeoverByPartnerNotPossibleReason;
  public String takeoverFailureReason;
  public List<TakeoverInfo> takeoverOfPartnerNotPossibleReason;
  public Boolean takeoverPossible;
  public Long timeUntilTakeover;
  public TimeoutInfo timeouts;
  public Date transitEventTime;

  @Output("backup-io-times")
  @Type("mailbox-io-times-info")
  @Optional
  public void setBackupIoTimes(MailboxIoTimesInfo paramMailboxIoTimesInfo)
  {
    this.backupIoTimes = paramMailboxIoTimesInfo;
  }

  public MailboxIoTimesInfo getBackupIoTimes()
  {
    return this.backupIoTimes;
  }

  public SfStatus withBackupIoTimes(MailboxIoTimesInfo paramMailboxIoTimesInfo)
  {
    this.backupIoTimes = paramMailboxIoTimesInfo;
    return this;
  }

  @Output("backup-mailbox-status")
  @Type("mailbox-status-info[]")
  @Optional
  public void setBackupMailboxStatus(List<MailboxStatusInfo> paramList)
  {
    this.backupMailboxStatus = paramList;
  }

  public List<MailboxStatusInfo> getBackupMailboxStatus()
  {
    return this.backupMailboxStatus;
  }

  public SfStatus withBackupMailboxStatus(List<MailboxStatusInfo> paramList)
  {
    this.backupMailboxStatus = paramList;
    return this;
  }

  public SfStatus withBackupMailboxStatus(MailboxStatusInfo[] paramArrayOfMailboxStatusInfo)
  {
    this.backupMailboxStatus = new ArrayList(Arrays.asList(paramArrayOfMailboxStatusInfo));
    return this;
  }

  @Output("booting-received")
  @Type("integer")
  public void setBootingReceived(Long paramLong)
  {
    this.bootingReceived = paramLong;
  }

  public Long getBootingReceived()
  {
    return this.bootingReceived;
  }

  public SfStatus withBootingReceived(Long paramLong)
  {
    this.bootingReceived = paramLong;
    return this;
  }

  @Output("control-partner-aggregates")
  @Type("boolean")
  public void setControlPartnerAggregates(Boolean paramBoolean)
  {
    this.controlPartnerAggregates = paramBoolean;
  }

  public Boolean isControlPartnerAggregates()
  {
    return this.controlPartnerAggregates;
  }

  public SfStatus withControlPartnerAggregates(Boolean paramBoolean)
  {
    this.controlPartnerAggregates = paramBoolean;
    return this;
  }

  @Output("current-resource-table-entry")
  @Type("string")
  @Optional
  public void setCurrentResourceTableEntry(String paramString)
  {
    this.currentResourceTableEntry = paramString;
  }

  public String getCurrentResourceTableEntry()
  {
    return this.currentResourceTableEntry;
  }

  public SfStatus withCurrentResourceTableEntry(String paramString)
  {
    this.currentResourceTableEntry = paramString;
    return this;
  }

  @Output("current-resource-table-index")
  @Type("integer")
  @Optional
  public void setCurrentResourceTableIndex(Long paramLong)
  {
    this.currentResourceTableIndex = paramLong;
  }

  public Long getCurrentResourceTableIndex()
  {
    return this.currentResourceTableIndex;
  }

  public SfStatus withCurrentResourceTableIndex(Long paramLong)
  {
    this.currentResourceTableIndex = paramLong;
    return this;
  }

  @Output("firmware-received")
  @Type("integer")
  public void setFirmwareReceived(Long paramLong)
  {
    this.firmwareReceived = paramLong;
  }

  public Long getFirmwareReceived()
  {
    return this.firmwareReceived;
  }

  public SfStatus withFirmwareReceived(Long paramLong)
  {
    this.firmwareReceived = paramLong;
    return this;
  }

  @Output("is-enabled")
  @Type("boolean")
  public void setEnabled(Boolean paramBoolean)
  {
    this.enabled = paramBoolean;
  }

  public Boolean isEnabled()
  {
    return this.enabled;
  }

  public SfStatus withEnabled(Boolean paramBoolean)
  {
    this.enabled = paramBoolean;
    return this;
  }

  @Output("is-interconnect-up")
  @Type("boolean")
  public void setInterconnectUp(Boolean paramBoolean)
  {
    this.interconnectUp = paramBoolean;
  }

  public Boolean isInterconnectUp()
  {
    return this.interconnectUp;
  }

  public SfStatus withInterconnectUp(Boolean paramBoolean)
  {
    this.interconnectUp = paramBoolean;
    return this;
  }

  @Output("kill-packets")
  @Type("boolean")
  public void setKillPackets(Boolean paramBoolean)
  {
    this.killPackets = paramBoolean;
  }

  public Boolean isKillPackets()
  {
    return this.killPackets;
  }

  public SfStatus withKillPackets(Boolean paramBoolean)
  {
    this.killPackets = paramBoolean;
    return this;
  }

  @Output("local-mailbox-disks")
  @Type("sf-disk-info[]")
  @Optional
  public void setLocalMailboxDisks(List<SfDiskInfo> paramList)
  {
    this.localMailboxDisks = paramList;
  }

  public List<SfDiskInfo> getLocalMailboxDisks()
  {
    return this.localMailboxDisks;
  }

  public SfStatus withLocalMailboxDisks(List<SfDiskInfo> paramList)
  {
    this.localMailboxDisks = paramList;
    return this;
  }

  public SfStatus withLocalMailboxDisks(SfDiskInfo[] paramArrayOfSfDiskInfo)
  {
    this.localMailboxDisks = new ArrayList(Arrays.asList(paramArrayOfSfDiskInfo));
    return this;
  }

  @Output("logs-unsynced-count")
  @Type("integer")
  public void setLogsUnsyncedCount(Long paramLong)
  {
    this.logsUnsyncedCount = paramLong;
  }

  public Long getLogsUnsyncedCount()
  {
    return this.logsUnsyncedCount;
  }

  public SfStatus withLogsUnsyncedCount(Long paramLong)
  {
    this.logsUnsyncedCount = paramLong;
    return this;
  }

  @Output("max-resource-table-index")
  @Type("integer")
  @Optional
  public void setMaxResourceTableIndex(Long paramLong)
  {
    this.maxResourceTableIndex = paramLong;
  }

  public Long getMaxResourceTableIndex()
  {
    return this.maxResourceTableIndex;
  }

  public SfStatus withMaxResourceTableIndex(Long paramLong)
  {
    this.maxResourceTableIndex = paramLong;
    return this;
  }

  @Output("name")
  @Type("string")
  public void setName(String paramString)
  {
    this.name = paramString;
  }

  public String getName()
  {
    return this.name;
  }

  public SfStatus withName(String paramString)
  {
    this.name = paramString;
    return this;
  }

  @Output("nvram-id")
  @Type("integer")
  public void setNvramId(Long paramLong)
  {
    this.nvramId = paramLong;
  }

  public Long getNvramId()
  {
    return this.nvramId;
  }

  public SfStatus withNvramId(Long paramLong)
  {
    this.nvramId = paramLong;
    return this;
  }

  @Output("partner-mailbox-disks")
  @Type("sf-disk-info[]")
  @Optional
  public void setPartnerMailboxDisks(List<SfDiskInfo> paramList)
  {
    this.partnerMailboxDisks = paramList;
  }

  public List<SfDiskInfo> getPartnerMailboxDisks()
  {
    return this.partnerMailboxDisks;
  }

  public SfStatus withPartnerMailboxDisks(List<SfDiskInfo> paramList)
  {
    this.partnerMailboxDisks = paramList;
    return this;
  }

  public SfStatus withPartnerMailboxDisks(SfDiskInfo[] paramArrayOfSfDiskInfo)
  {
    this.partnerMailboxDisks = new ArrayList(Arrays.asList(paramArrayOfSfDiskInfo));
    return this;
  }

  @Output("partner-name")
  @Type("string")
  @Optional
  public void setPartnerName(String paramString)
  {
    this.partnerName = paramString;
  }

  public String getPartnerName()
  {
    return this.partnerName;
  }

  public SfStatus withPartnerName(String paramString)
  {
    this.partnerName = paramString;
    return this;
  }

  @Output("partner-nvram-id")
  @Type("integer")
  @Optional
  public void setPartnerNvramId(Long paramLong)
  {
    this.partnerNvramId = paramLong;
  }

  public Long getPartnerNvramId()
  {
    return this.partnerNvramId;
  }

  public SfStatus withPartnerNvramId(Long paramLong)
  {
    this.partnerNvramId = paramLong;
    return this;
  }

  @Output("primary-io-times")
  @Type("mailbox-io-times-info")
  @Optional
  public void setPrimaryIoTimes(MailboxIoTimesInfo paramMailboxIoTimesInfo)
  {
    this.primaryIoTimes = paramMailboxIoTimesInfo;
  }

  public MailboxIoTimesInfo getPrimaryIoTimes()
  {
    return this.primaryIoTimes;
  }

  public SfStatus withPrimaryIoTimes(MailboxIoTimesInfo paramMailboxIoTimesInfo)
  {
    this.primaryIoTimes = paramMailboxIoTimesInfo;
    return this;
  }

  @Output("primary-mailbox-status")
  @Type("mailbox-status-info[]")
  @Optional
  public void setPrimaryMailboxStatus(List<MailboxStatusInfo> paramList)
  {
    this.primaryMailboxStatus = paramList;
  }

  public List<MailboxStatusInfo> getPrimaryMailboxStatus()
  {
    return this.primaryMailboxStatus;
  }

  public SfStatus withPrimaryMailboxStatus(List<MailboxStatusInfo> paramList)
  {
    this.primaryMailboxStatus = paramList;
    return this;
  }

  public SfStatus withPrimaryMailboxStatus(MailboxStatusInfo[] paramArrayOfMailboxStatusInfo)
  {
    this.primaryMailboxStatus = new ArrayList(Arrays.asList(paramArrayOfMailboxStatusInfo));
    return this;
  }

  @Output("resource-table")
  @Type("resource-table-info[]")
  @Optional
  public void setResourceTable(List<ResourceTableInfo> paramList)
  {
    this.resourceTable = paramList;
  }

  public List<ResourceTableInfo> getResourceTable()
  {
    return this.resourceTable;
  }

  public SfStatus withResourceTable(List<ResourceTableInfo> paramList)
  {
    this.resourceTable = paramList;
    return this;
  }

  public SfStatus withResourceTable(ResourceTableInfo[] paramArrayOfResourceTableInfo)
  {
    this.resourceTable = new ArrayList(Arrays.asList(paramArrayOfResourceTableInfo));
    return this;
  }

  @Output("sf-options")
  @Type("sf-options-info")
  @Optional
  public void setSfOptions(SfOptionsInfo paramSfOptionsInfo)
  {
    this.sfOptions = paramSfOptionsInfo;
  }

  public SfOptionsInfo getSfOptions()
  {
    return this.sfOptions;
  }

  public SfStatus withSfOptions(SfOptionsInfo paramSfOptionsInfo)
  {
    this.sfOptions = paramSfOptionsInfo;
    return this;
  }

  @Output("state")
  @Type("string")
  @Optional
  public void setState(String paramString)
  {
    this.state = paramString;
  }

  public String getState()
  {
    return this.state;
  }

  public SfStatus withState(String paramString)
  {
    this.state = paramString;
    return this;
  }

  @Output("takeover-by-partner-not-possible-reason")
  @Type("takeover-info[]")
  @Optional
  public void setTakeoverByPartnerNotPossibleReason(List<TakeoverInfo> paramList)
  {
    this.takeoverByPartnerNotPossibleReason = paramList;
  }

  public List<TakeoverInfo> getTakeoverByPartnerNotPossibleReason()
  {
    return this.takeoverByPartnerNotPossibleReason;
  }

  public SfStatus withTakeoverByPartnerNotPossibleReason(List<TakeoverInfo> paramList)
  {
    this.takeoverByPartnerNotPossibleReason = paramList;
    return this;
  }

  public SfStatus withTakeoverByPartnerNotPossibleReason(TakeoverInfo[] paramArrayOfTakeoverInfo)
  {
    this.takeoverByPartnerNotPossibleReason = new ArrayList(Arrays.asList(paramArrayOfTakeoverInfo));
    return this;
  }

  @Output("takeover-failure-reason")
  @Type("string")
  @Optional
  public void setTakeoverFailureReason(String paramString)
  {
    this.takeoverFailureReason = paramString;
  }

  public String getTakeoverFailureReason()
  {
    return this.takeoverFailureReason;
  }

  public SfStatus withTakeoverFailureReason(String paramString)
  {
    this.takeoverFailureReason = paramString;
    return this;
  }

  @Output("takeover-of-partner-not-possible-reason")
  @Type("takeover-info[]")
  @Optional
  public void setTakeoverOfPartnerNotPossibleReason(List<TakeoverInfo> paramList)
  {
    this.takeoverOfPartnerNotPossibleReason = paramList;
  }

  public List<TakeoverInfo> getTakeoverOfPartnerNotPossibleReason()
  {
    return this.takeoverOfPartnerNotPossibleReason;
  }

  public SfStatus withTakeoverOfPartnerNotPossibleReason(List<TakeoverInfo> paramList)
  {
    this.takeoverOfPartnerNotPossibleReason = paramList;
    return this;
  }

  public SfStatus withTakeoverOfPartnerNotPossibleReason(TakeoverInfo[] paramArrayOfTakeoverInfo)
  {
    this.takeoverOfPartnerNotPossibleReason = new ArrayList(Arrays.asList(paramArrayOfTakeoverInfo));
    return this;
  }

  @Output("takeover-possible")
  @Type("boolean")
  public void setTakeoverPossible(Boolean paramBoolean)
  {
    this.takeoverPossible = paramBoolean;
  }

  public Boolean isTakeoverPossible()
  {
    return this.takeoverPossible;
  }

  public SfStatus withTakeoverPossible(Boolean paramBoolean)
  {
    this.takeoverPossible = paramBoolean;
    return this;
  }

  @Output("time-until-takeover")
  @Type("integer")
  @Optional
  public void setTimeUntilTakeover(Long paramLong)
  {
    this.timeUntilTakeover = paramLong;
  }

  public Long getTimeUntilTakeover()
  {
    return this.timeUntilTakeover;
  }

  public SfStatus withTimeUntilTakeover(Long paramLong)
  {
    this.timeUntilTakeover = paramLong;
    return this;
  }

  @Output("timeouts")
  @Type("timeout-info")
  public void setTimeouts(TimeoutInfo paramTimeoutInfo)
  {
    this.timeouts = paramTimeoutInfo;
  }

  public TimeoutInfo getTimeouts()
  {
    return this.timeouts;
  }

  public SfStatus withTimeouts(TimeoutInfo paramTimeoutInfo)
  {
    this.timeouts = paramTimeoutInfo;
    return this;
  }

  @Output("transit-event-time")
  @Type("integer")
  public void setTransitEventTime(Date paramDate)
  {
    this.transitEventTime = paramDate;
  }

  public Date getTransitEventTime()
  {
    return this.transitEventTime;
  }

  public SfStatus withTransitEventTime(Date paramDate)
  {
    this.transitEventTime = paramDate;
    return this;
  }
}