package com.netapp.filer.snapshot;

import com.netapp.common.zapi.annotation.Api;
import com.netapp.common.zapi.annotation.Errno;
import com.netapp.common.zapi.annotation.Family;
import com.netapp.common.zapi.annotation.Input;
import com.netapp.common.zapi.annotation.Optional;
import com.netapp.common.zapi.annotation.Output;
import com.netapp.common.zapi.annotation.Range;
import com.netapp.common.zapi.annotation.Status;
import com.netapp.common.zapi.annotation.StatusEnum;
import com.netapp.common.zapi.annotation.Type;
import java.io.Serializable;
import java.util.Date;

@Api("snapshot-delta-info")
@Status(StatusEnum.PUBLIC)
@Errno({"ESNAPSHOTDOESNOTEXIST", "EVOLUMEBUSY", "EVOLUMEDOESNOTEXIST", "EVOLUMEMOUNTING", "EVOLUMEOFFLINE"})
@Family({"ontap-c", "vfiler"})
public class SnapshotDeltaInfo
  implements Serializable
{
  private static final long serialVersionUID = 1L;
  public Long consumedSize;
  public Date elapsedTime;
  public String snapshot1;
  public String snapshot2;
  public String volume;

  @Output("consumed-size")
  @Type("integer")
  @Range(min="0", max="2^63-1")
  public void setConsumedSize(Long paramLong)
  {
    this.consumedSize = paramLong;
  }

  public Long getConsumedSize()
  {
    return this.consumedSize;
  }

  public SnapshotDeltaInfo withConsumedSize(Long paramLong)
  {
    this.consumedSize = paramLong;
    return this;
  }

  @Output("elapsed-time")
  @Type("integer")
  @Range(min="0", max="2^31-1")
  public void setElapsedTime(Date paramDate)
  {
    this.elapsedTime = paramDate;
  }

  public Date getElapsedTime()
  {
    return this.elapsedTime;
  }

  public SnapshotDeltaInfo withElapsedTime(Date paramDate)
  {
    this.elapsedTime = paramDate;
    return this;
  }

  @Input("snapshot1")
  @Type("string")
  public void setSnapshot1(String paramString)
  {
    this.snapshot1 = paramString;
  }

  public String getSnapshot1()
  {
    return this.snapshot1;
  }

  public SnapshotDeltaInfo withSnapshot1(String paramString)
  {
    this.snapshot1 = paramString;
    return this;
  }

  @Input("snapshot2")
  @Type("string")
  @Optional
  public void setSnapshot2(String paramString)
  {
    this.snapshot2 = paramString;
  }

  public String getSnapshot2()
  {
    return this.snapshot2;
  }

  public SnapshotDeltaInfo withSnapshot2(String paramString)
  {
    this.snapshot2 = paramString;
    return this;
  }

  @Input("volume")
  @Type("string")
  public void setVolume(String paramString)
  {
    this.volume = paramString;
  }

  public String getVolume()
  {
    return this.volume;
  }

  public SnapshotDeltaInfo withVolume(String paramString)
  {
    this.volume = paramString;
    return this;
  }
}