package com.netapp.filer.snapshot;

import com.netapp.common.zapi.annotation.Api;
import com.netapp.common.zapi.annotation.Errno;
import com.netapp.common.zapi.annotation.Family;
import com.netapp.common.zapi.annotation.Input;
import com.netapp.common.zapi.annotation.Optional;
import com.netapp.common.zapi.annotation.Output;
import com.netapp.common.zapi.annotation.Status;
import com.netapp.common.zapi.annotation.StatusEnum;
import com.netapp.common.zapi.annotation.Type;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

@Api("snapshot-multicreate")
@Status(StatusEnum.PUBLIC)
@Errno({"ENOMORESNAPS", "EONTAPI_EAGAIN", "EONTAPI_EINVAL", "EONTAPI_ENOMEM", "EONTAPI_ENOSPCSNAP", "EONTAPI_EPERM", "EONTAPI_EROFS", "EONTAPI_EWORMNOCLOCK", "EONTAPI_EWORMPERIOD", "EOPNOTSUPPORTED", "ESNAPSHOTEXISTS", "EVOLUMEDOESNOTEXIST", "EVOLUMEMOUNTING", "EVOLUMEOFFLINE"})
@Family({"ontap-c", "vfiler"})
public class SnapshotMulticreate
  implements Serializable
{
  private static final long serialVersionUID = 1L;
  public Boolean cleanup;
  public String snapshot;
  public List<VolumeIsSnapcreated> volumeIsSnapcreatedList;
  public List<String> volumeNames;

  @Input("cleanup")
  @Type("boolean")
  @Optional
  public void setCleanup(Boolean paramBoolean)
  {
    this.cleanup = paramBoolean;
  }

  public Boolean isCleanup()
  {
    return this.cleanup;
  }

  public SnapshotMulticreate withCleanup(Boolean paramBoolean)
  {
    this.cleanup = paramBoolean;
    return this;
  }

  @Input("snapshot")
  @Type("string")
  public void setSnapshot(String paramString)
  {
    this.snapshot = paramString;
  }

  public String getSnapshot()
  {
    return this.snapshot;
  }

  public SnapshotMulticreate withSnapshot(String paramString)
  {
    this.snapshot = paramString;
    return this;
  }

  @Output("volume-is-snapcreated-list")
  @Type("volume-is-snapcreated[]")
  @Optional
  public void setVolumeIsSnapcreatedList(List<VolumeIsSnapcreated> paramList)
  {
    this.volumeIsSnapcreatedList = paramList;
  }

  public List<VolumeIsSnapcreated> getVolumeIsSnapcreatedList()
  {
    return this.volumeIsSnapcreatedList;
  }

  public SnapshotMulticreate withVolumeIsSnapcreatedList(List<VolumeIsSnapcreated> paramList)
  {
    this.volumeIsSnapcreatedList = paramList;
    return this;
  }

  public SnapshotMulticreate withVolumeIsSnapcreatedList(VolumeIsSnapcreated[] paramArrayOfVolumeIsSnapcreated)
  {
    this.volumeIsSnapcreatedList = new ArrayList(Arrays.asList(paramArrayOfVolumeIsSnapcreated));
    return this;
  }

  @Input("volume-names")
  @Type("volume-name[]")
  public void setVolumeNames(List<String> paramList)
  {
    this.volumeNames = paramList;
  }

  public List<String> getVolumeNames()
  {
    return this.volumeNames;
  }

  public SnapshotMulticreate withVolumeNames(List<String> paramList)
  {
    this.volumeNames = paramList;
    return this;
  }

  public SnapshotMulticreate withVolumeNames(String[] paramArrayOfString)
  {
    this.volumeNames = new ArrayList(Arrays.asList(paramArrayOfString));
    return this;
  }
}