package com.netapp.filer.software;

import com.netapp.common.zapi.annotation.Api;
import com.netapp.common.zapi.annotation.Errno;
import com.netapp.common.zapi.annotation.Input;
import com.netapp.common.zapi.annotation.Optional;
import com.netapp.common.zapi.annotation.Output;
import com.netapp.common.zapi.annotation.Status;
import com.netapp.common.zapi.annotation.StatusEnum;
import com.netapp.common.zapi.annotation.Type;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

@Api("software-extract-metadata")
@Status(StatusEnum.PUBLIC)
@Errno({"EAPIMISSINGARGUMENT", "EAPINOTIMPLEMENTED", "EINTERNALERROR", "ENPMERXMETA", "ENPMINVPLOC", "ENPMNOMETA", "EONTAPI_EAGAIN", "EONTAPI_EBUSY"})
public class SoftwareExtractMetadata
  implements Serializable
{
  private static final long serialVersionUID = 1L;
  public List<NpmMetaElemInfo> npmMetadata;
  public String packageLocation;
  public String packageName;

  @Output("npm-metadata")
  @Type("npm-meta-elem-info[]")
  public void setNpmMetadata(List<NpmMetaElemInfo> paramList)
  {
    this.npmMetadata = paramList;
  }

  public List<NpmMetaElemInfo> getNpmMetadata()
  {
    return this.npmMetadata;
  }

  public SoftwareExtractMetadata withNpmMetadata(List<NpmMetaElemInfo> paramList)
  {
    this.npmMetadata = paramList;
    return this;
  }

  public SoftwareExtractMetadata withNpmMetadata(NpmMetaElemInfo[] paramArrayOfNpmMetaElemInfo)
  {
    this.npmMetadata = new ArrayList(Arrays.asList(paramArrayOfNpmMetaElemInfo));
    return this;
  }

  @Input("package-location")
  @Type("string")
  public void setPackageLocation(String paramString)
  {
    this.packageLocation = paramString;
  }

  public String getPackageLocation()
  {
    return this.packageLocation;
  }

  public SoftwareExtractMetadata withPackageLocation(String paramString)
  {
    this.packageLocation = paramString;
    return this;
  }

  @Input("package-name")
  @Type("string")
  @Optional
  public void setPackageName(String paramString)
  {
    this.packageName = paramString;
  }

  public String getPackageName()
  {
    return this.packageName;
  }

  public SoftwareExtractMetadata withPackageName(String paramString)
  {
    this.packageName = paramString;
    return this;
  }
}