package com.netapp.filer.volume;

import com.netapp.common.zapi.annotation.Element;
import com.netapp.common.zapi.annotation.Optional;
import com.netapp.common.zapi.annotation.Range;
import com.netapp.common.zapi.annotation.Type;
import com.netapp.common.zapi.annotation.Typedef;
import java.io.Serializable;
import java.math.BigInteger;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

@Typedef("volume-info")
public class VolumeInfo
  implements Serializable
{
  private static final long serialVersionUID = 1L;
  public AutosizeInfo autosize;
  public String checksumStyle;
  public List<CloneChildInfo> cloneChildren;
  public List<CloneParentInfo> cloneParent;
  public CompressionInfo compression;
  public String containingAggregate;
  public Integer diskCount;
  public BigInteger expiryDate;
  public BigInteger filesTotal;
  public BigInteger filesUsed;
  public String formattedExpiryDate;
  public Boolean checksumEnabled;
  public Boolean inconsistent;
  public Boolean invalid;
  public Boolean snaplock;
  public Boolean unrecoverable;
  public Boolean wraparound;
  public String mirrorStatus;
  public String name;
  public String owningVfiler;
  public Integer percentageUsed;
  public Integer plexCount;
  public List<PlexInfo> plexes;
  public Integer quotaInit;
  public Integer raidSize;
  public String raidStatus;
  public String remoteLocation;
  public BigInteger reserve;
  public BigInteger reserveRequired;
  public BigInteger reserveUsed;
  public BigInteger reserveUsedActual;
  public SisInfo sis;
  public BigInteger sizeAvailable;
  public BigInteger sizeTotal;
  public BigInteger sizeUsed;
  public SnapAutodeleteInfo snapAutodelete;
  public String snaplockType;
  public BigInteger snapshotBlocksReserved;
  public Integer snapshotPercentReserved;
  public String spaceReserve;
  public Boolean spaceReserveEnabled;
  public String state;
  public String type;
  public String uuid;

  @Element("autosize")
  @Type("autosize-info")
  @Optional
  public void setAutosize(AutosizeInfo paramAutosizeInfo)
  {
    this.autosize = paramAutosizeInfo;
  }

  public AutosizeInfo getAutosize()
  {
    return this.autosize;
  }

  public VolumeInfo withAutosize(AutosizeInfo paramAutosizeInfo)
  {
    this.autosize = paramAutosizeInfo;
    return this;
  }

  @Element("checksum-style")
  @Type("string")
  public void setChecksumStyle(String paramString)
  {
    this.checksumStyle = paramString;
  }

  public String getChecksumStyle()
  {
    return this.checksumStyle;
  }

  public VolumeInfo withChecksumStyle(String paramString)
  {
    this.checksumStyle = paramString;
    return this;
  }

  @Element("clone-children")
  @Type("clone-child-info[]")
  @Optional
  public void setCloneChildren(List<CloneChildInfo> paramList)
  {
    this.cloneChildren = paramList;
  }

  public List<CloneChildInfo> getCloneChildren()
  {
    return this.cloneChildren;
  }

  public VolumeInfo withCloneChildren(List<CloneChildInfo> paramList)
  {
    this.cloneChildren = paramList;
    return this;
  }

  public VolumeInfo withCloneChildren(CloneChildInfo[] paramArrayOfCloneChildInfo)
  {
    this.cloneChildren = new ArrayList(Arrays.asList(paramArrayOfCloneChildInfo));
    return this;
  }

  @Element("clone-parent")
  @Type("clone-parent-info[]")
  @Optional
  public void setCloneParent(List<CloneParentInfo> paramList)
  {
    this.cloneParent = paramList;
  }

  public List<CloneParentInfo> getCloneParent()
  {
    return this.cloneParent;
  }

  public VolumeInfo withCloneParent(List<CloneParentInfo> paramList)
  {
    this.cloneParent = paramList;
    return this;
  }

  public VolumeInfo withCloneParent(CloneParentInfo[] paramArrayOfCloneParentInfo)
  {
    this.cloneParent = new ArrayList(Arrays.asList(paramArrayOfCloneParentInfo));
    return this;
  }

  @Element("compression")
  @Type("compression-info")
  public void setCompression(CompressionInfo paramCompressionInfo)
  {
    this.compression = paramCompressionInfo;
  }

  public CompressionInfo getCompression()
  {
    return this.compression;
  }

  public VolumeInfo withCompression(CompressionInfo paramCompressionInfo)
  {
    this.compression = paramCompressionInfo;
    return this;
  }

  @Element("containing-aggregate")
  @Type("string")
  @Optional
  public void setContainingAggregate(String paramString)
  {
    this.containingAggregate = paramString;
  }

  public String getContainingAggregate()
  {
    return this.containingAggregate;
  }

  public VolumeInfo withContainingAggregate(String paramString)
  {
    this.containingAggregate = paramString;
    return this;
  }

  @Element("disk-count")
  @Type("integer")
  @Range(min="0", max="2^31-1")
  public void setDiskCount(Integer paramInteger)
  {
    this.diskCount = paramInteger;
  }

  public Integer getDiskCount()
  {
    return this.diskCount;
  }

  public VolumeInfo withDiskCount(Integer paramInteger)
  {
    this.diskCount = paramInteger;
    return this;
  }

  @Element("expiry-date")
  @Type("integer")
  @Optional
  @Range(min="0", max="2^64-1")
  public void setExpiryDate(BigInteger paramBigInteger)
  {
    this.expiryDate = paramBigInteger;
  }

  public BigInteger getExpiryDate()
  {
    return this.expiryDate;
  }

  public VolumeInfo withExpiryDate(BigInteger paramBigInteger)
  {
    this.expiryDate = paramBigInteger;
    return this;
  }

  @Element("files-total")
  @Type("integer")
  @Range(min="0", max="2^64-1")
  public void setFilesTotal(BigInteger paramBigInteger)
  {
    this.filesTotal = paramBigInteger;
  }

  public BigInteger getFilesTotal()
  {
    return this.filesTotal;
  }

  public VolumeInfo withFilesTotal(BigInteger paramBigInteger)
  {
    this.filesTotal = paramBigInteger;
    return this;
  }

  @Element("files-used")
  @Type("integer")
  @Range(min="0", max="2^64-1")
  public void setFilesUsed(BigInteger paramBigInteger)
  {
    this.filesUsed = paramBigInteger;
  }

  public BigInteger getFilesUsed()
  {
    return this.filesUsed;
  }

  public VolumeInfo withFilesUsed(BigInteger paramBigInteger)
  {
    this.filesUsed = paramBigInteger;
    return this;
  }

  @Element("formatted-expiry-date")
  @Type("string")
  @Optional
  public void setFormattedExpiryDate(String paramString)
  {
    this.formattedExpiryDate = paramString;
  }

  public String getFormattedExpiryDate()
  {
    return this.formattedExpiryDate;
  }

  public VolumeInfo withFormattedExpiryDate(String paramString)
  {
    this.formattedExpiryDate = paramString;
    return this;
  }

  @Element("is-checksum-enabled")
  @Type("boolean")
  public void setChecksumEnabled(Boolean paramBoolean)
  {
    this.checksumEnabled = paramBoolean;
  }

  public Boolean isChecksumEnabled()
  {
    return this.checksumEnabled;
  }

  public VolumeInfo withChecksumEnabled(Boolean paramBoolean)
  {
    this.checksumEnabled = paramBoolean;
    return this;
  }

  @Element("is-inconsistent")
  @Type("boolean")
  public void setInconsistent(Boolean paramBoolean)
  {
    this.inconsistent = paramBoolean;
  }

  public Boolean isInconsistent()
  {
    return this.inconsistent;
  }

  public VolumeInfo withInconsistent(Boolean paramBoolean)
  {
    this.inconsistent = paramBoolean;
    return this;
  }

  @Element("is-invalid")
  @Type("boolean")
  @Optional
  public void setInvalid(Boolean paramBoolean)
  {
    this.invalid = paramBoolean;
  }

  public Boolean isInvalid()
  {
    return this.invalid;
  }

  public VolumeInfo withInvalid(Boolean paramBoolean)
  {
    this.invalid = paramBoolean;
    return this;
  }

  @Element("is-snaplock")
  @Type("boolean")
  public void setSnaplock(Boolean paramBoolean)
  {
    this.snaplock = paramBoolean;
  }

  public Boolean isSnaplock()
  {
    return this.snaplock;
  }

  public VolumeInfo withSnaplock(Boolean paramBoolean)
  {
    this.snaplock = paramBoolean;
    return this;
  }

  @Element("is-unrecoverable")
  @Type("boolean")
  @Optional
  public void setUnrecoverable(Boolean paramBoolean)
  {
    this.unrecoverable = paramBoolean;
  }

  public Boolean isUnrecoverable()
  {
    return this.unrecoverable;
  }

  public VolumeInfo withUnrecoverable(Boolean paramBoolean)
  {
    this.unrecoverable = paramBoolean;
    return this;
  }

  @Element("is-wraparound")
  @Type("boolean")
  @Optional
  public void setWraparound(Boolean paramBoolean)
  {
    this.wraparound = paramBoolean;
  }

  public Boolean isWraparound()
  {
    return this.wraparound;
  }

  public VolumeInfo withWraparound(Boolean paramBoolean)
  {
    this.wraparound = paramBoolean;
    return this;
  }

  @Element("mirror-status")
  @Type("string")
  public void setMirrorStatus(String paramString)
  {
    this.mirrorStatus = paramString;
  }

  public String getMirrorStatus()
  {
    return this.mirrorStatus;
  }

  public VolumeInfo withMirrorStatus(String paramString)
  {
    this.mirrorStatus = paramString;
    return this;
  }

  @Element("name")
  @Type("string")
  public void setName(String paramString)
  {
    this.name = paramString;
  }

  public String getName()
  {
    return this.name;
  }

  public VolumeInfo withName(String paramString)
  {
    this.name = paramString;
    return this;
  }

  @Element("owning-vfiler")
  @Type("string")
  @Optional
  public void setOwningVfiler(String paramString)
  {
    this.owningVfiler = paramString;
  }

  public String getOwningVfiler()
  {
    return this.owningVfiler;
  }

  public VolumeInfo withOwningVfiler(String paramString)
  {
    this.owningVfiler = paramString;
    return this;
  }

  @Element("percentage-used")
  @Type("integer")
  @Optional
  @Range(min="0", max="100")
  public void setPercentageUsed(Integer paramInteger)
  {
    this.percentageUsed = paramInteger;
  }

  public Integer getPercentageUsed()
  {
    return this.percentageUsed;
  }

  public VolumeInfo withPercentageUsed(Integer paramInteger)
  {
    this.percentageUsed = paramInteger;
    return this;
  }

  @Element("plex-count")
  @Type("integer")
  @Range(min="0", max="2^31-1")
  public void setPlexCount(Integer paramInteger)
  {
    this.plexCount = paramInteger;
  }

  public Integer getPlexCount()
  {
    return this.plexCount;
  }

  public VolumeInfo withPlexCount(Integer paramInteger)
  {
    this.plexCount = paramInteger;
    return this;
  }

  @Element("plexes")
  @Type("plex-info[]")
  public void setPlexes(List<PlexInfo> paramList)
  {
    this.plexes = paramList;
  }

  public List<PlexInfo> getPlexes()
  {
    return this.plexes;
  }

  public VolumeInfo withPlexes(List<PlexInfo> paramList)
  {
    this.plexes = paramList;
    return this;
  }

  public VolumeInfo withPlexes(PlexInfo[] paramArrayOfPlexInfo)
  {
    this.plexes = new ArrayList(Arrays.asList(paramArrayOfPlexInfo));
    return this;
  }

  @Element("quota-init")
  @Type("integer")
  @Range(min="0", max="100")
  public void setQuotaInit(Integer paramInteger)
  {
    this.quotaInit = paramInteger;
  }

  public Integer getQuotaInit()
  {
    return this.quotaInit;
  }

  public VolumeInfo withQuotaInit(Integer paramInteger)
  {
    this.quotaInit = paramInteger;
    return this;
  }

  @Element("raid-size")
  @Type("integer")
  @Range(min="0", max="2^31-1")
  public void setRaidSize(Integer paramInteger)
  {
    this.raidSize = paramInteger;
  }

  public Integer getRaidSize()
  {
    return this.raidSize;
  }

  public VolumeInfo withRaidSize(Integer paramInteger)
  {
    this.raidSize = paramInteger;
    return this;
  }

  @Element("raid-status")
  @Type("string")
  public void setRaidStatus(String paramString)
  {
    this.raidStatus = paramString;
  }

  public String getRaidStatus()
  {
    return this.raidStatus;
  }

  public VolumeInfo withRaidStatus(String paramString)
  {
    this.raidStatus = paramString;
    return this;
  }

  @Element("remote-location")
  @Type("string")
  @Optional
  public void setRemoteLocation(String paramString)
  {
    this.remoteLocation = paramString;
  }

  public String getRemoteLocation()
  {
    return this.remoteLocation;
  }

  public VolumeInfo withRemoteLocation(String paramString)
  {
    this.remoteLocation = paramString;
    return this;
  }

  @Element("reserve")
  @Type("integer")
  @Range(min="0", max="2^64-1")
  public void setReserve(BigInteger paramBigInteger)
  {
    this.reserve = paramBigInteger;
  }

  public BigInteger getReserve()
  {
    return this.reserve;
  }

  public VolumeInfo withReserve(BigInteger paramBigInteger)
  {
    this.reserve = paramBigInteger;
    return this;
  }

  @Element("reserve-required")
  @Type("integer")
  @Range(min="0", max="2^64-1")
  public void setReserveRequired(BigInteger paramBigInteger)
  {
    this.reserveRequired = paramBigInteger;
  }

  public BigInteger getReserveRequired()
  {
    return this.reserveRequired;
  }

  public VolumeInfo withReserveRequired(BigInteger paramBigInteger)
  {
    this.reserveRequired = paramBigInteger;
    return this;
  }

  @Element("reserve-used")
  @Type("integer")
  @Range(min="0", max="2^64-1")
  public void setReserveUsed(BigInteger paramBigInteger)
  {
    this.reserveUsed = paramBigInteger;
  }

  public BigInteger getReserveUsed()
  {
    return this.reserveUsed;
  }

  public VolumeInfo withReserveUsed(BigInteger paramBigInteger)
  {
    this.reserveUsed = paramBigInteger;
    return this;
  }

  @Element("reserve-used-actual")
  @Type("integer")
  @Range(min="0", max="2^64-1")
  public void setReserveUsedActual(BigInteger paramBigInteger)
  {
    this.reserveUsedActual = paramBigInteger;
  }

  public BigInteger getReserveUsedActual()
  {
    return this.reserveUsedActual;
  }

  public VolumeInfo withReserveUsedActual(BigInteger paramBigInteger)
  {
    this.reserveUsedActual = paramBigInteger;
    return this;
  }

  @Element("sis")
  @Type("sis-info")
  @Optional
  public void setSis(SisInfo paramSisInfo)
  {
    this.sis = paramSisInfo;
  }

  public SisInfo getSis()
  {
    return this.sis;
  }

  public VolumeInfo withSis(SisInfo paramSisInfo)
  {
    this.sis = paramSisInfo;
    return this;
  }

  @Element("size-available")
  @Type("integer")
  @Range(min="0", max="2^64-1")
  public void setSizeAvailable(BigInteger paramBigInteger)
  {
    this.sizeAvailable = paramBigInteger;
  }

  public BigInteger getSizeAvailable()
  {
    return this.sizeAvailable;
  }

  public VolumeInfo withSizeAvailable(BigInteger paramBigInteger)
  {
    this.sizeAvailable = paramBigInteger;
    return this;
  }

  @Element("size-total")
  @Type("integer")
  @Range(min="0", max="2^64-1")
  public void setSizeTotal(BigInteger paramBigInteger)
  {
    this.sizeTotal = paramBigInteger;
  }

  public BigInteger getSizeTotal()
  {
    return this.sizeTotal;
  }

  public VolumeInfo withSizeTotal(BigInteger paramBigInteger)
  {
    this.sizeTotal = paramBigInteger;
    return this;
  }

  @Element("size-used")
  @Type("integer")
  @Range(min="0", max="2^64-1")
  public void setSizeUsed(BigInteger paramBigInteger)
  {
    this.sizeUsed = paramBigInteger;
  }

  public BigInteger getSizeUsed()
  {
    return this.sizeUsed;
  }

  public VolumeInfo withSizeUsed(BigInteger paramBigInteger)
  {
    this.sizeUsed = paramBigInteger;
    return this;
  }

  @Element("snap-autodelete")
  @Type("snap-autodelete-info")
  @Optional
  public void setSnapAutodelete(SnapAutodeleteInfo paramSnapAutodeleteInfo)
  {
    this.snapAutodelete = paramSnapAutodeleteInfo;
  }

  public SnapAutodeleteInfo getSnapAutodelete()
  {
    return this.snapAutodelete;
  }

  public VolumeInfo withSnapAutodelete(SnapAutodeleteInfo paramSnapAutodeleteInfo)
  {
    this.snapAutodelete = paramSnapAutodeleteInfo;
    return this;
  }

  @Element("snaplock-type")
  @Type("string")
  @Optional
  public void setSnaplockType(String paramString)
  {
    this.snaplockType = paramString;
  }

  public String getSnaplockType()
  {
    return this.snaplockType;
  }

  public VolumeInfo withSnaplockType(String paramString)
  {
    this.snaplockType = paramString;
    return this;
  }

  @Element("snapshot-blocks-reserved")
  @Type("integer")
  @Range(min="0", max="2^64-1")
  public void setSnapshotBlocksReserved(BigInteger paramBigInteger)
  {
    this.snapshotBlocksReserved = paramBigInteger;
  }

  public BigInteger getSnapshotBlocksReserved()
  {
    return this.snapshotBlocksReserved;
  }

  public VolumeInfo withSnapshotBlocksReserved(BigInteger paramBigInteger)
  {
    this.snapshotBlocksReserved = paramBigInteger;
    return this;
  }

  @Element("snapshot-percent-reserved")
  @Type("integer")
  @Range(min="0", max="100")
  public void setSnapshotPercentReserved(Integer paramInteger)
  {
    this.snapshotPercentReserved = paramInteger;
  }

  public Integer getSnapshotPercentReserved()
  {
    return this.snapshotPercentReserved;
  }

  public VolumeInfo withSnapshotPercentReserved(Integer paramInteger)
  {
    this.snapshotPercentReserved = paramInteger;
    return this;
  }

  @Element("space-reserve")
  @Type("string")
  @Optional
  public void setSpaceReserve(String paramString)
  {
    this.spaceReserve = paramString;
  }

  public String getSpaceReserve()
  {
    return this.spaceReserve;
  }

  public VolumeInfo withSpaceReserve(String paramString)
  {
    this.spaceReserve = paramString;
    return this;
  }

  @Element("space-reserve-enabled")
  @Type("boolean")
  @Optional
  public void setSpaceReserveEnabled(Boolean paramBoolean)
  {
    this.spaceReserveEnabled = paramBoolean;
  }

  public Boolean isSpaceReserveEnabled()
  {
    return this.spaceReserveEnabled;
  }

  public VolumeInfo withSpaceReserveEnabled(Boolean paramBoolean)
  {
    this.spaceReserveEnabled = paramBoolean;
    return this;
  }

  @Element("state")
  @Type("string")
  public void setState(String paramString)
  {
    this.state = paramString;
  }

  public String getState()
  {
    return this.state;
  }

  public VolumeInfo withState(String paramString)
  {
    this.state = paramString;
    return this;
  }

  @Element("type")
  @Type("string")
  public void setType(String paramString)
  {
    this.type = paramString;
  }

  public String getType()
  {
    return this.type;
  }

  public VolumeInfo withType(String paramString)
  {
    this.type = paramString;
    return this;
  }

  @Element("uuid")
  @Type("string")
  public void setUuid(String paramString)
  {
    this.uuid = paramString;
  }

  public String getUuid()
  {
    return this.uuid;
  }

  public VolumeInfo withUuid(String paramString)
  {
    this.uuid = paramString;
    return this;
  }
}