package com.netapp.filer.volume;

import com.netapp.common.zapi.annotation.Api;
import com.netapp.common.zapi.annotation.Errno;
import com.netapp.common.zapi.annotation.Input;
import com.netapp.common.zapi.annotation.Optional;
import com.netapp.common.zapi.annotation.Output;
import com.netapp.common.zapi.annotation.Status;
import com.netapp.common.zapi.annotation.StatusEnum;
import com.netapp.common.zapi.annotation.Type;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

@Api("volume-mirror")
@Status(StatusEnum.PUBLIC)
@Errno({"EINTERNALERROR", "EINVALIDINPUTERROR", "EVOLUMEDISKDUP"})
public class VolumeMirror
  implements Serializable
{
  private static final long serialVersionUID = 1L;
  public List<DiskInfo> badDisks;
  public Boolean force;
  public List<DiskInfo> mirrorDisks;
  public String victimVolume;
  public String volume;

  @Output("bad-disks")
  @Type("disk-info[]")
  @Optional
  public void setBadDisks(List<DiskInfo> paramList)
  {
    this.badDisks = paramList;
  }

  public List<DiskInfo> getBadDisks()
  {
    return this.badDisks;
  }

  public VolumeMirror withBadDisks(List<DiskInfo> paramList)
  {
    this.badDisks = paramList;
    return this;
  }

  public VolumeMirror withBadDisks(DiskInfo[] paramArrayOfDiskInfo)
  {
    this.badDisks = new ArrayList(Arrays.asList(paramArrayOfDiskInfo));
    return this;
  }

  @Input("force")
  @Type("boolean")
  @Optional
  public void setForce(Boolean paramBoolean)
  {
    this.force = paramBoolean;
  }

  public Boolean isForce()
  {
    return this.force;
  }

  public VolumeMirror withForce(Boolean paramBoolean)
  {
    this.force = paramBoolean;
    return this;
  }

  @Input("mirror-disks")
  @Type("disk-info[]")
  @Optional
  public void setMirrorDisks(List<DiskInfo> paramList)
  {
    this.mirrorDisks = paramList;
  }

  public List<DiskInfo> getMirrorDisks()
  {
    return this.mirrorDisks;
  }

  public VolumeMirror withMirrorDisks(List<DiskInfo> paramList)
  {
    this.mirrorDisks = paramList;
    return this;
  }

  public VolumeMirror withMirrorDisks(DiskInfo[] paramArrayOfDiskInfo)
  {
    this.mirrorDisks = new ArrayList(Arrays.asList(paramArrayOfDiskInfo));
    return this;
  }

  @Input("victim-volume")
  @Type("string")
  @Optional
  public void setVictimVolume(String paramString)
  {
    this.victimVolume = paramString;
  }

  public String getVictimVolume()
  {
    return this.victimVolume;
  }

  public VolumeMirror withVictimVolume(String paramString)
  {
    this.victimVolume = paramString;
    return this;
  }

  @Input("volume")
  @Type("string")
  public void setVolume(String paramString)
  {
    this.volume = paramString;
  }

  public String getVolume()
  {
    return this.volume;
  }

  public VolumeMirror withVolume(String paramString)
  {
    this.volume = paramString;
    return this;
  }
}