package devmgr.versioned.jrpc;

import devmgr.trace.Trace;
import java.io.IOException;
import java.net.DatagramSocket;
import java.net.InetAddress;
import java.util.Date;
import java.util.Vector;

public class RPCClientBroadcast extends RPCClientGeneric
{
  private static final int DEFAULT_TIMEOUT = 25;
  private static final long DELAY_BETWEEN_BROADCAST = 1500L;
  private static int m_ThreadCounter = 0;
  private long m_Timeout = 25000L;
  private InetAddress[] m_ServerAddresses;
  private int m_ServerPort;
  private UDPChannel[] m_AllChannels = null;
  private int m_Xid;
  private Vector m_Exceptions = null;

  public RPCClientBroadcast(int paramInt1, int paramInt2, int paramInt3)
    throws RPCError, IOException
  {
    super(paramInt1, paramInt2);
    String str = InetAddress.getLocalHost().getHostName();
    InetAddress[] arrayOfInetAddress = InetAddress.getAllByName(str);
    this.m_Xid = getNextXid();
    this.m_ServerPort = paramInt3;
    this.m_AllChannels = new UDPChannel[arrayOfInetAddress.length];
    if (arrayOfInetAddress.length == 1)
      this.m_AllChannels[0] = new UDPChannel(new DatagramSocket());
    else
      for (int i = 0; i < arrayOfInetAddress.length; i++)
        this.m_AllChannels[i] = new UDPChannel(new DatagramSocket(0, arrayOfInetAddress[i]));
    this.m_ServerAddresses = new InetAddress[1];
    this.m_ServerAddresses[0] = InetAddress.getByName("255.255.255.255");
  }

  protected void finalize()
    throws Throwable
  {
    if (this.m_AllChannels != null)
      for (int i = 0; i < this.m_AllChannels.length; i++)
        if (this.m_AllChannels[i] != null)
          this.m_AllChannels[i].close();
  }

  public void setAddress(InetAddress paramInetAddress)
  {
    this.m_ServerAddresses = new InetAddress[1];
    this.m_ServerAddresses[0] = paramInetAddress;
  }

  public void setAddresses(InetAddress[] paramArrayOfInetAddress)
  {
    this.m_ServerAddresses = paramArrayOfInetAddress;
  }

  public void setTimeout(int paramInt)
    throws IOException
  {
    this.m_Timeout = (paramInt * 1000);
    if (paramInt == 0)
      for (int i = 0; i < this.m_AllChannels.length; i++)
        this.m_AllChannels[i].setTimeout(0);
  }

  public void close()
    throws IOException
  {
    for (int i = 0; i < this.m_AllChannels.length; i++)
      try
      {
        this.m_AllChannels[i].close();
        this.m_AllChannels[i] = null;
      }
      catch (Exception localException)
      {
        Trace.printStackTrace(79, localException);
      }
    this.m_AllChannels = null;
  }

  public InetAddress getLocalAddress()
  {
    return this.m_AllChannels[0].getLocalAddress();
  }

  public XDRMessageIOV transact(XDRMessageIOV paramXDRMessageIOV, int paramInt)
    throws RPCError, IOException
  {
    throw new RPCError("INTERNAL_ERROR");
  }

  public synchronized void broadcast(int paramInt, XDRType paramXDRType1, XDRType paramXDRType2, RPCBroadcastReplyHandler paramRPCBroadcastReplyHandler)
    throws RPCError, IOException
  {
    BroadcastOnSocket[] arrayOfBroadcastOnSocket = null;
    Thread[] arrayOfThread = null;
    String str = null;
    this.m_Exceptions = new Vector();
    if (this.m_AllChannels.length == 1)
    {
      arrayOfBroadcastOnSocket = new BroadcastOnSocket[1];
      arrayOfBroadcastOnSocket[0] = new BroadcastOnSocket(this.m_AllChannels[0], paramInt, paramXDRType1, paramXDRType2, paramRPCBroadcastReplyHandler);
      arrayOfBroadcastOnSocket[0].run();
      if (!this.m_Exceptions.isEmpty())
      {
        if ((this.m_Exceptions.firstElement() instanceof IOException))
          throw ((IOException)this.m_Exceptions.firstElement());
        if ((this.m_Exceptions.firstElement() instanceof RPCError))
          throw ((RPCError)this.m_Exceptions.firstElement());
      }
    }
    else
    {
      arrayOfBroadcastOnSocket = new BroadcastOnSocket[this.m_AllChannels.length];
      arrayOfThread = new Thread[this.m_AllChannels.length];
      for (int i = 0; i < this.m_AllChannels.length; i++)
      {
        arrayOfBroadcastOnSocket[i] = new BroadcastOnSocket(this.m_AllChannels[i], paramInt, paramXDRType1, paramXDRType2, paramRPCBroadcastReplyHandler);
        str = getClass().getName() + "_" + m_ThreadCounter;
        arrayOfThread[i] = new Thread(arrayOfBroadcastOnSocket[i], str);
        m_ThreadCounter += 1;
        arrayOfThread[i].start();
        if (i < this.m_AllChannels.length - 1)
          try
          {
            Thread.currentThread();
            Thread.sleep(1500L);
          }
          catch (InterruptedException localInterruptedException1)
          {
            Trace.printStackTrace(79, localInterruptedException1);
          }
      }
      for (int j = 0; j < this.m_AllChannels.length; j++)
        try
        {
          arrayOfThread[j].join();
        }
        catch (InterruptedException localInterruptedException2)
        {
          Trace.printStackTrace(79, localInterruptedException2);
        }
    }
  }

  public InetAddress getDestinationIP()
  {
    return null;
  }

  class BroadcastOnSocket
    implements Runnable
  {
    private UDPChannel m_UdpChannel;
    private RPCBroadcastReplyHandler m_Handler;
    private XDRType m_Arg;
    private XDRType m_Result;
    private int m_Proc;

    BroadcastOnSocket(UDPChannel paramInt, int paramXDRType1, XDRType paramXDRType2, XDRType paramRPCBroadcastReplyHandler, RPCBroadcastReplyHandler arg6)
    {
      this.m_UdpChannel = paramInt;
      Object localObject;
      this.m_Handler = localObject;
      this.m_Arg = paramXDRType2;
      this.m_Result = paramRPCBroadcastReplyHandler;
      this.m_Proc = paramXDRType1;
    }

    public void run()
    {
      int i = 0;
      XDRMessageIOV localXDRMessageIOV = null;
      UDPMessageBundle localUDPMessageBundle1 = null;
      UDPMessageBundle localUDPMessageBundle2 = null;
      try
      {
        localXDRMessageIOV = RPCClientBroadcast.this.buildRequestMsg(this.m_Proc, RPCClientBroadcast.this.m_Xid, this.m_Arg);
      }
      catch (RPCError localRPCError)
      {
        RPCClientBroadcast.this.m_Exceptions.addElement(localRPCError);
        return;
      }
      for (int j = 0; j < RPCClientBroadcast.this.m_ServerAddresses.length; j++)
      {
        Trace.printf(4, "Sending RPC broadcast to %o\n", RPCClientBroadcast.this.m_ServerAddresses[j]);
        localUDPMessageBundle1 = new UDPMessageBundle(localXDRMessageIOV, RPCClientBroadcast.this.m_ServerAddresses[j], RPCClientBroadcast.this.m_ServerPort);
        try
        {
          this.m_UdpChannel.sendMsg(localUDPMessageBundle1);
        }
        catch (Exception localException1)
        {
          RPCClientBroadcast.this.m_Exceptions.addElement(localException1);
          return;
        }
      }
      Date localDate1 = new Date();
      while (i == 0)
      {
        if (RPCClientBroadcast.this.m_Timeout > 0L)
        {
          Date localDate2 = new Date();
          long l = localDate2.getTime() - localDate1.getTime();
          if (l > RPCClientBroadcast.this.m_Timeout)
            i = 1;
          else
            try
            {
              this.m_UdpChannel.setTimeout((int)(RPCClientBroadcast.this.m_Timeout - l) + 1);
            }
            catch (IOException localIOException)
            {
              Trace.printStackTrace(79, localIOException);
            }
        }
        if (i == 0)
          try
          {
            localUDPMessageBundle2 = this.m_UdpChannel.receiveMsg();
            Trace.printf(4, "Broadcast reply from %o\n", localUDPMessageBundle2.getAddress());
            RPCClientBroadcast.this.processReplyMsg(localUDPMessageBundle2.getMsg(), RPCClientBroadcast.this.m_Xid, this.m_Result);
            this.m_Handler.processReply(localUDPMessageBundle2.getAddress(), this.m_Result);
          }
          catch (Exception localException2)
          {
            Trace.printStackTrace(79, localException2);
          }
      }
    }
  }
}