package devmgr.versioned.jrpc;

import devmgr.trace.Trace;
import java.io.IOException;
import java.net.InetAddress;
import java.util.Date;

public abstract class RPCClientGeneric
{
  private static final int CLIENT_AUTH_OFFSET = 24;
  private int m_Program;
  private int m_Version;
  private int m_NextXid;
  private RPCAuthGenerator m_AuthGenerator;
  private RPCOpaqueAuth m_NullVerifier;

  public RPCClientGeneric(int paramInt1, int paramInt2)
  {
    this.m_Program = paramInt1;
    this.m_Version = paramInt2;
    this.m_NextXid = ((int)(new Date().getTime() / 1000L));
    this.m_AuthGenerator = new RPCNullAuthGenerator();
    this.m_NullVerifier = new RPCOpaqueAuth();
    this.m_NullVerifier.getType().setValue(0);
    this.m_NullVerifier.setInfo(new byte[0]);
  }

  public synchronized void call(int paramInt, XDRType paramXDRType1, XDRType paramXDRType2)
    throws RPCError, IOException
  {
    int i = getNextXid();
    XDRMessageIOV localXDRMessageIOV1 = null;
    XDRMessageIOV localXDRMessageIOV2 = null;
    localXDRMessageIOV1 = buildRequestMsg(paramInt, i, paramXDRType1);
    localXDRMessageIOV2 = transact(localXDRMessageIOV1, i);
    processReplyMsg(localXDRMessageIOV2, i, paramXDRType2);
  }

  synchronized void setAuthGenerator(RPCAuthGenerator paramRPCAuthGenerator)
  {
    this.m_AuthGenerator = paramRPCAuthGenerator;
  }

  public abstract void close()
    throws IOException;

  public abstract void setTimeout(int paramInt)
    throws IOException;

  public abstract InetAddress getLocalAddress();

  public abstract InetAddress getDestinationIP();

  public abstract XDRMessageIOV transact(XDRMessageIOV paramXDRMessageIOV, int paramInt)
    throws RPCError, IOException;

  protected synchronized int getNextXid()
  {
    return this.m_NextXid++;
  }

  protected XDRMessageIOV buildRequestMsg(int paramInt1, int paramInt2, XDRType paramXDRType)
    throws RPCError
  {
    XDRMessageIOV localXDRMessageIOV = null;
    XDROutputStream localXDROutputStream = new XDROutputStream();
    RPCMsg localRPCMsg = new RPCMsg();
    RPCOpaqueAuth localRPCOpaqueAuth1 = null;
    RPCOpaqueAuth localRPCOpaqueAuth2 = null;
    localRPCMsg.setXid(paramInt2);
    localRPCMsg.getBody().getMsgType().setValue(0);
    localRPCMsg.getBody().getCallMsg().setRpcVersion(2);
    localRPCMsg.getBody().getCallMsg().setProgramNo(this.m_Program);
    localRPCMsg.getBody().getCallMsg().setVersionNo(this.m_Version);
    localRPCMsg.getBody().getCallMsg().setProcedure(paramInt1);
    localRPCMsg.getBody().getCallMsg().setVerifier(this.m_NullVerifier);
    localRPCOpaqueAuth1 = this.m_AuthGenerator.getInitialAuthData();
    localRPCMsg.getBody().getCallMsg().setCredential(localRPCOpaqueAuth1);
    localRPCMsg.xdrEncode(localXDROutputStream);
    paramXDRType.xdrEncode(localXDROutputStream);
    localXDRMessageIOV = localXDROutputStream.getMessage();
    localRPCOpaqueAuth2 = this.m_AuthGenerator.getFinalAuthData(localRPCOpaqueAuth1, localXDRMessageIOV);
    localXDROutputStream.setPosition(24);
    localRPCOpaqueAuth2.xdrEncode(localXDROutputStream);
    Trace.printf(4, "Sending call message; xid=%d, proc=%d size=%d\n", new Integer(paramInt2), new Integer(paramInt1), new Integer(localXDRMessageIOV.getMessageSize()));
    if (Trace.isTracing(6))
      RPCDebug.dumpIOV(6, localXDRMessageIOV);
    return localXDRMessageIOV;
  }

  protected void processReplyMsg(XDRMessageIOV paramXDRMessageIOV, int paramInt, XDRType paramXDRType)
    throws RPCError
  {
    XDRInputStream localXDRInputStream = null;
    RPCMsg localRPCMsg = new RPCMsg();
    Trace.printf(4, "Processing reply message for xid=%d size=%d \n", new Integer(paramInt), new Integer(paramXDRMessageIOV.getMessageSize()));
    if (Trace.isTracing(5))
      RPCDebug.dumpIOV(5, paramXDRMessageIOV);
    localXDRInputStream = new XDRInputStream(paramXDRMessageIOV);
    localRPCMsg.xdrDecode(localXDRInputStream);
    if ((localRPCMsg.getXid() != paramInt) || (localRPCMsg.getBody().getMsgType().getValue() != 1))
      throw new RPCError("PROTOCOL_ERROR");
    if (localRPCMsg.getBody().getReplyMsg().getStatus().getValue() != 0)
      throw new RPCError("REQUEST_REJECTED");
    if (localRPCMsg.getBody().getReplyMsg().getAcceptedReply().getVerifier().getType().getValue() != 0)
      throw new RPCError("BAD_VERIFIER");
    switch (localRPCMsg.getBody().getReplyMsg().getAcceptedReply().getBody().getStatus().getValue())
    {
    case 0:
      break;
    case 1:
      throw new RPCError("PROG_UNAVAILABLE");
    case 2:
      throw new RPCError("PROG_MISMATCH");
    case 3:
      throw new RPCError("PROC_UNAVAILABLE");
    case 4:
      throw new RPCError("ARGS_REJECTED");
    case 5:
    default:
      throw new RPCError("SYSTEM_ERROR");
    }
    paramXDRType.xdrDecode(localXDRInputStream);
  }
}