package devmgr.versioned.jrpc;

import java.io.IOException;
import java.net.DatagramSocket;
import java.net.InetAddress;
import java.util.Date;

class RPCClientUDP extends RPCClientGeneric
{
  private static final int DEFAULT_TIMEOUT = 25;
  private long m_Timeout = 25000L;
  private InetAddress m_ServerAddress;
  private int m_ServerPort;
  private UDPChannel m_Channel = null;

  RPCClientUDP(InetAddress paramInetAddress, int paramInt1, int paramInt2, int paramInt3)
    throws RPCError, IOException
  {
    super(paramInt1, paramInt2);
    this.m_ServerAddress = paramInetAddress;
    this.m_ServerPort = paramInt3;
    this.m_Channel = new UDPChannel(new DatagramSocket());
  }

  protected void finalize()
    throws Throwable
  {
    if (this.m_Channel != null)
      this.m_Channel.close();
  }

  public void setTimeout(int paramInt)
    throws IOException
  {
    this.m_Timeout = (paramInt * 1000);
    if (paramInt == 0)
      this.m_Channel.setTimeout(0);
  }

  public void close()
    throws IOException
  {
    this.m_Channel.close();
    this.m_Channel = null;
  }

  public InetAddress getLocalAddress()
  {
    return this.m_Channel.getLocalAddress();
  }

  public XDRMessageIOV transact(XDRMessageIOV paramXDRMessageIOV, int paramInt)
    throws RPCError, IOException
  {
    int i = 0;
    UDPMessageBundle localUDPMessageBundle1 = null;
    UDPMessageBundle localUDPMessageBundle2 = null;
    XDRMessageIOV localXDRMessageIOV = null;
    localUDPMessageBundle1 = new UDPMessageBundle(paramXDRMessageIOV, this.m_ServerAddress, this.m_ServerPort);
    this.m_Channel.sendMsg(localUDPMessageBundle1);
    Date localDate1 = new Date();
    while (i == 0)
    {
      if (this.m_Timeout > 0L)
      {
        Date localDate2 = new Date();
        long l = localDate2.getTime() - localDate1.getTime();
        if (l > this.m_Timeout)
          throw new RPCError("TIMEOUT");
        this.m_Channel.setTimeout((int)(this.m_Timeout - l) + 1);
      }
      localUDPMessageBundle2 = this.m_Channel.receiveMsg();
      localXDRMessageIOV = localUDPMessageBundle2.getMsg();
      if (fetchXid(localXDRMessageIOV) != paramInt)
        localXDRMessageIOV = null;
      else
        i = 1;
    }
    return localXDRMessageIOV;
  }

  private int fetchXid(XDRMessageIOV paramXDRMessageIOV)
  {
    byte[] arrayOfByte = paramXDRMessageIOV.getBuffer(0);
    int i;
    if (arrayOfByte.length < 4)
      i = -1;
    else
      i = (arrayOfByte[0] & 0xFF) << 24 | (arrayOfByte[1] & 0xFF) << 16 | (arrayOfByte[2] & 0xFF) << 8 | (arrayOfByte[3] & 0xFF) << 0;
    return i;
  }

  public InetAddress getDestinationIP()
  {
    return this.m_ServerAddress;
  }
}