package org.eclipse.jdt.internal.compiler.ast;

import org.eclipse.jdt.internal.compiler.codegen.BranchLabel;
import org.eclipse.jdt.internal.compiler.codegen.CodeStream;
import org.eclipse.jdt.internal.compiler.flow.FlowContext;
import org.eclipse.jdt.internal.compiler.flow.FlowInfo;
import org.eclipse.jdt.internal.compiler.impl.Constant;
import org.eclipse.jdt.internal.compiler.lookup.ArrayBinding;
import org.eclipse.jdt.internal.compiler.lookup.BlockScope;
import org.eclipse.jdt.internal.compiler.lookup.MethodBinding;
import org.eclipse.jdt.internal.compiler.lookup.TypeBinding;
import org.eclipse.jdt.internal.compiler.problem.ProblemReporter;

public abstract class Statement extends ASTNode
{
  public abstract FlowInfo analyseCode(BlockScope paramBlockScope, FlowContext paramFlowContext, FlowInfo paramFlowInfo);

  public void branchChainTo(BranchLabel label)
  {
  }

  public boolean complainIfUnreachable(FlowInfo flowInfo, BlockScope scope, boolean didAlreadyComplain)
  {
    if ((flowInfo.reachMode() & 0x1) != 0) {
      this.bits &= 2147483647;
      boolean reported = flowInfo == FlowInfo.DEAD_END;
      if ((!didAlreadyComplain) && (reported)) {
        scope.problemReporter().unreachableCode(this);
      }
      return reported;
    }
    return false;
  }

  public void generateArguments(MethodBinding binding, Expression[] arguments, BlockScope currentScope, CodeStream codeStream)
  {
    if (binding.isVarargs())
    {
      TypeBinding[] params = binding.parameters;
      int paramLength = params.length;
      int varArgIndex = paramLength - 1;
      for (int i = 0; i < varArgIndex; i++) {
        arguments[i].generateCode(currentScope, codeStream, true);
      }

      ArrayBinding varArgsType = (ArrayBinding)params[varArgIndex];
      ArrayBinding codeGenVarArgsType = (ArrayBinding)binding.parameters[varArgIndex].erasure();
      int elementsTypeID = varArgsType.elementsType().id;
      int argLength = arguments == null ? 0 : arguments.length;

      if (argLength > paramLength)
      {
        codeStream.generateInlinedValue(argLength - varArgIndex);
        codeStream.newArray(codeGenVarArgsType);
        for (int i = varArgIndex; i < argLength; i++) {
          codeStream.dup();
          codeStream.generateInlinedValue(i - varArgIndex);
          arguments[i].generateCode(currentScope, codeStream, true);
          codeStream.arrayAtPut(elementsTypeID, false);
        }
      } else if (argLength == paramLength)
      {
        TypeBinding lastType = arguments[varArgIndex].resolvedType;
        if ((lastType == TypeBinding.NULL) || (
          (varArgsType.dimensions() == lastType.dimensions()) && 
          (lastType.isCompatibleWith(varArgsType))))
        {
          arguments[varArgIndex].generateCode(currentScope, codeStream, true);
        }
        else
        {
          codeStream.generateInlinedValue(1);
          codeStream.newArray(codeGenVarArgsType);
          codeStream.dup();
          codeStream.generateInlinedValue(0);
          arguments[varArgIndex].generateCode(currentScope, codeStream, true);
          codeStream.arrayAtPut(elementsTypeID, false);
        }
      }
      else
      {
        codeStream.generateInlinedValue(0);
        codeStream.newArray(codeGenVarArgsType);
      }
    } else if (arguments != null) {
      int i = 0; for (int max = arguments.length; i < max; i++)
        arguments[i].generateCode(currentScope, codeStream, true);
    }
  }

  public abstract void generateCode(BlockScope paramBlockScope, CodeStream paramCodeStream);

  public boolean isEmptyBlock() {
    return false;
  }

  public boolean isValidJavaStatement()
  {
    return true;
  }

  public StringBuffer print(int indent, StringBuffer output) {
    return printStatement(indent, output);
  }

  public abstract StringBuffer printStatement(int paramInt, StringBuffer paramStringBuffer);

  public abstract void resolve(BlockScope paramBlockScope);

  public Constant resolveCase(BlockScope scope, TypeBinding testType, SwitchStatement switchStatement)
  {
    resolve(scope);
    return Constant.NotAConstant;
  }
}